;;
;@brief This file contains the memory functions, some global arrays, and some global variables used thoughout the library.
;
;mostly these functions deal with memory management and global variable definitions, there is a detailed explanation of all the routines in the documentation.
;
;;
INCLUDE GlobalStuff64.inc
;extrn malloc: PROC
;extrn free: PROC
extrn calloc : PROC

	.data
	
			;;@brief This holds all the powers of 2 from 0 to 63 (each one is held at the position of the exponent, so 2 ^ 0 is at 0 2 ^ 1 is at 1 and so on).
			_bits		    qword		64 dup(0)			
			_upper			qword		64 dup(0)		
			_lower			qword		64 dup(0)
			;;@brief Used when converting an array to Binary.
			_tobase2		qword		2 dup(0)
			;;@brief Used when converting an array to Octa-decimal.
			_tobase8		qword		8 dup(0)
			;;@brief Used when converting an array to decimal.
			_tobase10		qword		10 dup(0)
			;;@brief Used when converting an array to Hex-a-decimal.
			_tobase16		qword		16 dup(0)
			;;@brief Used when converting an array to base 32.
			_tobase32		qword		33 dup(0)
			;;@brief Used when converting an array to base 64.
			_tobase64		qword		65 dup(0)
			;;@brief Used when converting a binary string into an array value.
			_frombase2		qword		256 dup(0)
			;;@brief Used when converting an octa-decimal string into an array value.
			_frombase8		qword		256 dup(0)
			;;@brief Used when converting a decimal string into an array value.
			_frombase10		qword		256 dup(0)
			;;@brief Used when converting a hex-a-decimal string into an array value.
			_frombase16		qword		256 dup(0)
			;;@brief Used when converting a base 32 string into an array value.
			_frombase32		qword		256 dup(0)
			;;@brief Used when converting a base 64 string into an array value.
			_frombase64		qword		256 dup(0)
			;;@brief This will hold the radix in array representation.
			_base			qword		3 dup(0)

			;;@brief This is the value of ln in a ABD object.
			_bdln			qword			0
			;;@brief This is the value of pi in a ABD object.
			_bdpi			qword			0
			;;@brief This is the value of pi in a ABF object.
			_bfpi			qword			0
			;;@brief This is the value of e in a ABD object.
			_bde			qword			0
			;;@brief This is the value of e in a ABF object.
			_bfe			qword			0
			;;@brief This is the value to multiply to convert degrees into radians (180/pi) in a ABD object.
			_bdtorad		qword			0
			;;@brief This is the value to multiply to convert degrees into radians (180/pi) in a ABF object.
			_bftorad		qword			0
			;;@brief This holds the square root of 2 in a ABD object.
			_bdsqr2			qword			0
			;;@brief This holds the square root of 2 in a ABF object.
			_bfsqr2			qword			0
			;;@brief This holds the value of phi in a ABD object.
			_bdphi			qword			0
			;;@brief This holds the value of phi in a ABF object.
			_bfphi			qword			0
			;;@brief This holds the value of lambda in a ABD object.
			_bdlambda		qword			0
			;;@brief This holds the value of lanbda in a ABF object.
			_bflambda		qword			0
			;;@brief This holds the golden ratio in a ABD object.
			_bdgoldenratio	qword			0
			;;@brief This holds the golden ratio in a ABF object.
			_bfgoldenratio	qword			0
			;;@brief This holds the value of 2 * sqr(3) in a ABD object.
			_bd2sqr3		qword			0
			;;@brief This holds the value of 2 * sqr(3) in a ABF object.
			_bf2sqr3		qword			0
			;;@brief This holds the value of the square root of 3 in a ABD object.
			_bdsqr3			qword			0
			;;@brief This holds the value of the square root of 3 in a ABF object.
			_bfsqr3			qword			0

			;;@brief Flag to show whether initialization has taken place or not.
			_initialized	qword		0
			_initvals		qword		0
			_initmem		qword		0
			_meminiting		qword		0
			;;@brief This sets the default base to use.
			_tbase			qword		10
			;;@brief This is the maximum value.
			_max			qword		18446744073709551615
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 0.
			_zero			qword		0
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 1.
			_one			qword		1
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 2.
			_two			qword		2
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 3.
			_three			qword		3
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 4.
			_four			qword		4
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 7.
			_seven			qword		7
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 8.
			_eight			qword		8
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 9.
			_nine			qword		9
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 10.
			_ten			qword		10
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 15.
			_fifteen		qword		15
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 16.
			_sixteen		qword		16
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 20.
			_twenty			qword		20
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 24.
			_twentyfour		qword		24
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 31.
			_thirtyone		qword		31
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 32.
			_thirtytwo		qword		32
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 48.
			_fourtyeight	qword		48
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 49.
			_fourtynine		qword		49
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 50.
			_fifty			qword		50
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 51.
			_fifty_one		qword		51
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 52.
			_fiftytwo		qword		52
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 53.
			_fiftythree		qword		53
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 54.
			_fiftyfour		qword		54
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 55.
			_fiftyfive		qword		55
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 56.
			_fiftysix		qword		56
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 57.
			_fiftyseven		qword		57	
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 63.
			_sixtythree		qword		63
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 64.
			_sixtyfour		qword		64
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 256.
			_twofiftysix	qword		256
			;;@brief Used with certain instructions that wont allow an immediate value, this value equals 1024.
			_tentwentyfour	qword		1024

			_counting		qword		0
			_docount		qword		0

			;;@brief When debugging it used to signal to write info to a text file.
			_writetofile	qword		0

			;;@brief This controls the size of blocks that are handed out for mem requests if the internal hanlder is used.
			_allocsize		qword		64
			;;@brief This controls how many chunks of _allocsize will be requested when getting memory for the internal handler to hand out.
			_numberofallocs	qword		65535
			;;@brief This specifies the size of memory that will be requested if using the internal handler.
			_allocreqsize	qword		4194368
			;;@brief This specifies the size of the request to malloc for the table that handles memory requests.
			_tablereqsize	qword		524288	
			_locktable		qword		0
			;;@brief This is the table that keeps track of which memory chunks have been handed out and which havent.
			_memslot		qword		0
			;;@brief This is the table that holds a pointer to each reuqest the internal handler made to malloc for handing out.
			_memptr			qword		0
			_resetcount		qword		50000
			;;@brief Controls whether to use the internal memory handling or use malloc.
			_usemalloc		qword		0
			;;@brief this is a flag to show that a thread is requesting another chunk and initializing it.
			_gm				qword		0
			_fm				qword		0
			;;@
			_defaultcyclecount	qword	25




			_memslotcount	qword		0
			_memslotdelete	qword		0
			_memres			qword		0
			_memfree		qword		0



			_created		qword		1

			_reqcount		qword		0
			_freecount		qword		0
			_sendmessage	qword		0

			_check1			qword		18369900232523579118

			;;@brief This holds the value of -1 in a ABD object.
			_bdnegone		qword		0
			;;@brief This holds the value of 0 in a ABD object.
			_bdzero			qword		0
			;;@brief This holds the value of 1 in a ABD object.
			_bdone			qword		0
			;;@brief This holds the value of 5 in a ABD object.
			_bdfive			qword		0
			;;@brief This holds the value of 10 in a ABD object.
			_bdten			qword		0
			;;@brief This is a temp variable of a ABD object used internally.
			_bdtemp			qword		0

			;;@brief This holds the value of -1 in a ABI object.
			_binegone		qword		0
			;;@brief This holds the value of 0 in a ABI object.
			_bizero			qword		0
			;;@brief This holds the value of 1 in a ABI object.
			_bione			qword		0
			;;@brief This holds the value of 5 in a ABI object.
			_bifive			qword		0
			;;@brief This holds the value of 10 in a ABI object.
			_biten			qword		0
			;;@brief This is a temp variable of a ABI object used internally.
			_bitemp			qword		0

			;;@brief This holds the value of -1 in a ABF object.
			_bfnegone		qword		0
			;;@brief This holds the value of 0 in a ABF object.
			_bfzero			qword		0
			;;@brief This holds the value of 1 in a ABF object.
			_bfone			qword		0
			;;@brief This holds the value of 5 in a ABF object.
			_bffive			qword		0
			;;@brief This holds the value of 10 in a ABF object.
			_bften			qword		0
			;;@brief This is a temp variable of a ABF object used internally.
			_bftemp			qword		0

			;;@brief This is equivalent to the HowToHandleRoundingEnum for FiveAndBetter.
			_FiveAndBetter	qword		5	
			;;@brief This is equivalent to the HowToHandleRoundingEnum for DropRemaining.
			_DropRemaining	qword		8	
			;;@brief This is equivalent to the HowToHandleRoundingEnum for RoundUp.
			_RoundUp		qword		10	

	.code

	GetMemCount	PROC	tsize:qword

		mov		rax, _created

		RET

	GetMemCount  ENDP

	SetResetCount	PROC	tsize:qword

		mov		_resetcount, rcx

		RET

	SetResetCount  ENDP

	GetResetCount	PROC

		mov		rax, _resetcount

		RET

	GetResetCount  ENDP
	;;
	;@brief Returns the size for the memory table.
	;@returns a qword.
	;;
	GetTableReqSize	PROC

		mov		rax, _tablereqsize

		RET

	GetTableReqSize  ENDP
	;;
	;@brief Returns the size of the memory block requested.
	;@returns a qword.
	;;
	GetAllocReqSize	PROC

		mov		rax, _allocreqsize

		RET

	GetAllocReqSize  ENDP
	;;
	;@brief Sets the size of blocks returned by the memory request routine.
	;@param rcx - qword, the size of blocks you want to retrun.
	;;
	SetAllocationSize	PROC	tsize:qword

		cmp		_initialized, 0			;check if everything is allready initialized, if it is then exit without changing the value
		je		DoSet

		RET

DoSet:

		cmp		_locktable, 1
		je		GoOut
		mov		_allocsize, rcx
		mov		rax, rcx
		mul		_numberofallocs
		mov		_allocreqsize, rax

GoOut:

		RET

	SetAllocationSize  ENDP
	;;
	;@brief Returns the size of the blocks returned by the memory request routine.
	;@returns a qword.
	;;
	GetAllocationSize	PROC

		mov		rax, _allocsize

		RET

	GetAllocationSize  ENDP
	;;
	;@brief Sets the number of allocations  each major block request keeps track of.
	;@param rcx - qword representing the number of blocks to keep track of.
	;;
	SetNumberOfAllocations	PROC	tsize:qword

		cmp		_initialized, 0			;check if everything is allready initialized, if it is then exit without changing the value
		je		DoSet

		RET

DoSet:

		cmp		_locktable, 1
		je		GoOut
		cmp		rcx, 1
		jna		GoOut
		mov		_numberofallocs, rcx
		dec		_numberofallocs
		mov		rax, rcx
		mul		_allocsize
		mov		_allocreqsize, rax
		mov		rax, rcx
		mul		_eight
		mov		_tablereqsize, rax

GoOut:

		RET

	SetNumberOfAllocations  ENDP
	;;
	;@brief Gets the number of allocations each major block request keeps track of.
	;@returns a qword.
	;;
	GetNumberOfAllocations	PROC

		mov		rax, _numberofallocs
		inc		rax

		RET

	GetNumberOfAllocations  ENDP
	;;
	;@brief Sets the _usemalloc variable.  0==use internal routine, 1== use malloc
	;@param rcx - qword, value to set _usemalloc
	;;
	UseMalloc	PROC	tsize:qword

		mov		_usemalloc, rcx

		RET

	UseMalloc ENDP
	;;
	;@brief Returns the value in the _usemalloc variable
	;@returns a qword
	;;
	GetMalloc PROC

		mov		rax, _usemalloc

		RET

	GetMalloc ENDP
	;;
	;@brief This is the main memory request routine.
	;
	;  This method works by making sure it is initialized first, then it looks at the _usemalloc variable, if it is 1 then it just calls malloc and returns, otherwise, it moves into my algorithm.
	;
	;  My alg starts by taking the requested amount and dividing that by the _allocsize (which controls the size of blocks to be handed out), then it compares rdx to 0, if its not 0 it increments the rax
	;register, which now contains the number of contiguous blocks the method has to look for.  Then it moves into the search loop, which starts by loading the _memslot variable (this is the array
	;that holds the pointers to the tables that mark whether a block of memory is used or not).  It loads the first _memslot value(at element 1 as element 0 holds the number of pointers the array has), sets
	;the rcx register to 0 and the rdi register to 1 the rsi = rdi then enters the main part of the search loop.  This part first compares the address pointed to by _memslot+rdi*8 to 0.  If it equals 0 it increments
	;the rcx register, compares rcx to the number of blocks its looking for, if it equals it goes to the marking loop, if not it jumps to the head of the main search loop and repeats.  If the memory
	;address is not zero then it moves 	the value at the address of _memslot+rdi*8+2 into rax (this is the number of blocks that the request that marked the blocks requested) then it moves the value
	;at the address of _memslot+rdi*8+6 into rdi (which is the first block that was returned when the request that marked these blocks ended) and adds rax to rdi, sets rsi=rdi, sets rcx=0, so now rdi points to the first block
	;past this set of blocks that is allready marked.  In this way we only check the first block of each request.  Anyways, it sets rcx to 0 and jumps to the head of the search loop and repeats the 
	;process.  If rdi reaches the value of the _numberofallocs variable (which holds how many blocks of _allocsize the program requests from windows when it needs memory) before it finds enough
	;free blocks to fill the request, then it compares which set it is on to the value at the 0 element of _memslot, if its less, it increments then loads the next table in the array, sets rdi=1
	;rsi=rdi, rcx=0, then jumps to the head of the search loop and starts again.  If it searches the last table in _memslot without success, it jumps to the mem request part of the method (explained below)
	;sets rdi=1, rsi=rdi, rcx=0, then jumps to the head of the loop and starts again.
	;
	;  When the algorithm finally finds enough free contiguous blocks to fill the request it jumps to the mark part of the alg.  In here it sets rax and rdx=0, it sets rbx to the number of blocks it found
	;and rcx to rsi (which points to the first free block in this contiguous set).  It then shift both rbx and rcx to the left 16 (so that the compare exchange instruction uses the correct form) then
	;fires the cmpxchg8b instruction with the lock prefix so that this particular thread gets exclusive access to that address.  Then it looks at the zero flag, if its not set then the instruction failed 
	;(another thread marked it before it got to) in which case it shifts the rax and rdx registers to the right by 16 (when this instruction fails it loads the value at the address it was trying to mark in
	;these registers, which will contain the first block that was used when the thread that succeeded in marking the loop requested and how many blocks it is marking) and adds the rax, rdx registers
	;which after the add will point to the first block beyond this segment that is allready owned by a thread, then moves the value into rdi, sets rsi=rdi, rcx=0 then jumps to the head of the search loop
	;to find another set of blocks.  If its successful in marking the first block it will be successfull in marking all the next blocks.  After it is done marking the blocks, it loads the _memptr array
	;it then loads the pointer of the array that _memslot found the blocks on (in other words if it was the third element in the _memslot array that it found the memory in it will load the third element
	;of the _memptrarray), it multiplies the first block of the set it just marked by the _allocsize, adds what was retrieved from the _memptr array puts the value in rax and returns.
	;
	;  If the method runs through all the tables in the _memslot array, it starts by marking the _gm variable with 1 which signals all threads it is creating a new block so that no other threads try to
	;create a new block at the same time,  it then creates a new table and gets a new block from malloc.  It starts by multiplying the _numberofallocs variable by 8 (in 64 bit mode
	;or 4 in 32 bit mode), moves that value into rcx, makes a call to malloc, moves the _memslot pointer into a register, inc the value at element 0, then saves the new request in the first empty slot, then it
	;marks all of the elements in the block it just requested to 0, then it multiplies the _allocsize by the _numberofallocs to get the size of the block that will be pieced out to memory requests, then
	;it loads the _memptr variable into a register, increments the value at element 0, then saves the ptr returned from malloc in the element pointed to be element 0.  It clears the _gm variable, sets rdi=1
	; rsi=rdi, rcx =0, then jumps to the head of the search loop.
	;
	;  After the method creates a new memory set to piece out, it flags the system, no other thread will be able to make a new request for a given set of time.  This gives all the threads enough time to
	;find out a new set was allready created and go to it to start searching.  The time it takes to fullfill a request is mainly controlled by the sizes of _allocsize and _numberofallocs (which are
	;both configurable).  One way I had implemented that made fullfilling the request lightning fast was by marking each free element of the array with the number of contiguous blocks that are free, and the
	;position of the first free block in that contiguous set.  Then it would look at the value in the _memslot array, if it was free it would look at how many contiguous free blocks it had, if it was enough
	;it went to the mark part of the alg, if not it added the number of free blocks to the first block of that set, which would point it at the first block past that contiguous set (which because of the way
	;it was set up, was gauranteed to be taken) so it would load the value at the next block, add the parts together so that it was pointing to the first block past it and try again.  Anyways, with the alg
	;set this way, the requesting part of the alg was very quick, but the free algorithm took a very long time to run since it had to run through its normal routine plus look at the first block before and after
	;the set thats being returned to see if they were free or not and then mark each element with the correct info.
	;
	;@returns a PTR to a memory block.
	;@param rcx - qword, size in bytes requested.
	;;
	GetMem64 PROC _tsize:qword

		;using a management table
		;each table is specific to a memory set
		;each entry in the table will have the following pattern
		;
		;all unused block will have all words _zerod
		;
		;a used block will have words 0 and 1 set to the index of the first block shifted to the left 16
		;words 2 and 3 will hold how many blocks are being used shifted to the left 16

		ENTERFUNC _zero, _one, _zero, _one, _one, _one, _one, _one, _one, _one, _zero, _one, _zero, _zero, _zero
		
		;we check to make sure everything is initialized

		mov		_v1, rcx				;save the amount of memory requested
		cmp		_initialized, 0			;check if everything is allready initialized, if not then run initialization
		je		CallInitialize
		mov		rcx, _v1				;restore the requested amount to rcx

Allready_initialized:


		;now we check if we are supposed to use malloc or not, if we do, then call it and exit

		cmp		_usemalloc, 0			;if the _usemalloc variable is 0 we use this alg, if it is not 0, we use malloc to fullfill the request
		je		UseMine
		call	malloc					;the call to malloc
		cmp		rax, 0					;if rax contains 0 then there was an error in the call
		je		MallocError

		EXITFUNC _zero, _one, _zero, _one, _one, _one, _one, _one, _one, _one, _zero, _one, _zero, _zero, _zero

		RET

UseMine:

		;we start by dividing the requested amount by our allocation size to see how many units to look for (we increment it by one to cover the case it equals zero)
		;then we set everything up by moving _memslot into r13, then moving the first set into r13, set rdi and rsi to 1 rcx to zero (rdi is the current position in the set
		;rsi marks the first block of the set we are looking at and rcx holds the count)

		cmp		_initmem, 1
		je		MemInitialized
		mov		rax, 0					;have to prep the cmpexchg instruction
		mov		rdx, 0					;we want to make sure the _gm global variable is 0 (which means no other thread is currently in this section of code)
		mov		rbx, 1					;if it is 0 we want to change it to 1
		mov		rcx, 0
lock	cmpxchg8b	_meminiting			;lock and fire the instruction
		jnz		WaitForIt				;if the zero flag is not set then another thread is allready in this section of code, so exit this part and wait till it is finished
		call	InitializeMem
		mov		_initmem, 1
		mov		_meminiting, 0
		mov		rcx, _v1
		jmp		MemInitialized

WaitForIt:

		cmp		_meminiting, 1
		je		WaitForIt
		mov		rcx, _v1

MemInitialized:

		mov		rax, rcx				;mov the amount requested into rax for the division
		xor		rdx, rdx				;clear the rdx register for the division
		div		_allocsize				;divide the requested amount by the allocation size, this tells us how many blocks to look for
		cmp		rdx, 0					;check if it was an exact division (no remainder)
		cmovne	rdx, _one				;if there was a remainder then move 1 into rdx
		add		rax, rdx				;this increments the number of blocks we are looking for by one if there was a remainder
		mov		_t1, rax				;move the number of blocks we are looking for into t1
		mov		r13, _memslot			;move the memslot array into r13, this array holds the pointers to the arrays that hold the mapping tables (which tells us which blocks are free and whoch are taken)
		mov		rdi, 1					;we intially set to the first table, this will also be used for the pointer into the table
		mov		rsi, 1					;rsi will keep track of the first available block for when we are ready to start marking
		mov		_t2, 1					;the t2 variable keeps track of which table we are using so that we can return a pointer from the correct memptr table
		mov		r13, qword ptr[r13+rdi*8]	;load the first memslot table into r13
		xor		rcx, rcx				;clear rcx, this keeps track of how many contiguous free blocks we find

SearchLoop:

		;look at the slot, if it is 0 then inc rcx, compare rcx to how many units we are looking for, if its equal then jump to found
		;else we inc rdi make sure rdi isnt pointing past the end, then jump to the top to do it again.
		;if rdi is pointing past the end, then we load _memslot back into r13, load rdi from _t2 (which is keeping track of which set) inc rdi, make sure its pointing to a valid set
		;load r13 with the next set, set rdi and rsi to 1, rcx to 0 and jump tp searchloop
		;if it is pointing past the end of the sets, then we jump to create a new set

		cmp		qword ptr[r13+rdi*8], 0		;check the element pointed to is zero, (0 means the block is free)
		jne		ItsTaken					;if its allready being used go to the next section
		inc		rcx							;if its not taken increment rcx to count another free block
		cmp		rcx, _t1					;compare rcx to t1 (the number of free blocks we are looking for)
		jnb		Found						;if we found enough go to prep for marking the blocks
		inc		rdi							;increment rdi to point to the next block
		cmp		rdi, _numberofallocs		;compare rdi to the number of allocations we keep in the table
		jna		SearchLoop					;if its not above we jump to the head of the search loop to check the next block
		inc		_t2							;if it is above we need to load the next table
		mov		r13, _memslot				;load the memslot pointer into r13
		mov		rax, _t2					;move the index to the next table into rax
		cmp		rax, qword ptr[r13]			;check we are not past the last table in there
		ja		CreateNewSet				;if we are jump to the create new set routine
		mov		r13, qword ptr[r13+rax*8]	;move the next table into r13
		mov		rdi, 1						;set the index to the first entry
		mov		rsi, 1						;set rsi to point to the first entry (rsi holds the index to the first free block)
		xor		rcx, rcx					;clear rcx (the counter)
		jmp		SearchLoop					;jump to the head of the search loop

ItsTaken:

		movzx	rax, word ptr[r13+rdi*8+2]	;since the value wasnt 0 we know the value contains the info we need to jump past the taken blocks, we move the first block in this taken set into rax
		movzx	rdi, word ptr[r13+rdi*8+6]	;we move the number of blocks taken in this set into rdi
		add		rdi, rax					;we add rax to rdi so that now rdi points to the first block past this taken set
		mov		rsi, rdi					;we set rsi = rdi so that rsi keeps track of the first block
		xor		rcx, rcx					;we clear rcx since we need contiguous blocks
		cmp		rdi, _numberofallocs		;make sure wwe are not pointing past the end of the table
		jna		SearchLoop					;if we are good jump to the head of the search loop
		inc		_t2							;if not increment the index into the memslot table
		mov		r13, _memslot				;move the memslot table into r13
		mov		rax, _t2					;move the index into rax
		cmp		rax, qword ptr[r13]			;make sure we are not pointing past the last table
		ja		CreateNewSet				;if we are jump to the create new set routine
		mov		r13, qword ptr[r13+rax*8]	;move the next table into r13
		mov		rdi, 1						;point to the first entry
		mov		rsi, 1						;set rsi to the first entry
		xor		rcx, rcx					;clear the counter
		jmp		SearchLoop					;jump to the head of the search loop

Found:





		mov		_var5, rdi
		mov		_var6, rsi






		;ok we found enough empty units, now we prep to mark them, rax:rdx need to be zero, this is what the cmpxchg command compares against the memory location
		;mov the starting block (rsi) into rcx, and the number of blocks we are reserving into rbx and shift both to the left 16 so that it uses the 32 bit form of the command

		mov		rax, 0					;have to prep the compare and exchange each loop
		mov		rdx, 0
		mov		rbx, _t1				;move the number of blocks into rbx
		mov		rcx, rsi				;move the index of the first block into rsi
		shl		rbx, 16					;we have to shift the value so the correct form of cmpexchg is used
		shl		rcx, 16					;we have to shift the value so the correct form of cmpexchg is used

MarkLoop:
		
		;use the lock prefix so that we have exclusive access to the memory location and execute the command, if the zero flag is not set then that means the location no longer
		;contains zero (another thread must have reserved it allready) so jump to allready marked and find another set of blocks.  If this command executes the first run, it is gauranteed
		;to succeed on all the rest.  After marking all blocks we are using then we load _memptr into r13, and rdi with _t2 (so we point to the correct set) and load that set into r13
		;then we multiply the number of blocks between 0 and the set we started at by the allocation size, add that to r13, and that gives us the begginging address of the
		;memory we just reserved and thats what gets returned

lock	cmpxchg8b	qword ptr[r13+rsi*8]	;lock the memory location and fire the cmpexchg instruction
		jnz		AllreadyMarked				;if the zero flag is not set, then the exchange failed (if its going to fail it will only fail on the first time through)
		inc		rsi							;increment the index
		cmp		rsi, rdi					;compare it against rdi (where to stop)
		jna		MarkLoop					;if its not above jump to the head of the mark loop





		;mov		rdi, _t2
		;mov		r13, _memslotcount
		;mov		r13, qword ptr[r13+rdi*8]
		;mov		rdi, _var6
;
;IncCountLoop:
;
		;inc		qword ptr[r13+rdi*8]
		;inc		rdi
		;cmp		rdi, _var5
		;jna		IncCountLoop
;





		mov		r13, _memptr				;load the memptr table inot r13
		mov		rdi, _t2					;load the index of the memslot table we found the free blocks in to rdi
		mov		r13, qword ptr[r13+rdi*8]	;load the correct memptr into r13 (the one that corresponds to the memslot table where we found the free blocks)
		mov		rax, rcx					;move the index to the first free block into rax
		shr		rax, 16						;we have to shift it back to the right (because we shifted it to the left above)
		mul		_allocsize					;multiply it by the allocation size
		add		rax, r13					;add it to the base pointer (r13) this gives us the pointer to the first address of the memory we will return
		mov		_v1, rax					;save the pointer for when we return

		;this is a new section to protect against the possibility that this alg assigns some memory, then the _usemalloc variable gets changed.  With this set up we will keep track of
		;how many requests that we filled are still out.  If the number is above 0 then the free alg will ALLWAYS start by checking if the memory being returned was filled from the memory
		;we are handing out, if it is then we use our free alg to free it, if its not then it falls through to call the windows free routine.  
		;In order to insure that we allways have the correct count we will use the cmpexchg instruction with the lock prefix, there may be an issue since both this alg and the free alg
		;will be locking and accessing the _fm variable, if it slows it down to much Ill either remove this section or try something different.
		
IncCount:

		mov		r13, 0						;prep to increment the count, set r13 to zero
		mov		rax, _fm					;move the value in _fm to rax
		mov		rdx, rax					;copy rax to rdx
		shr		rdx, 32						;shift rdx to the right by 32 bits (_fm should be looked at like two dwords)
		and		rax, 0ffffffffh				;and rax by 0ffffffffh to keep just the first 32 bits (rdx:rax now holds the value we will compare to)
		mov		rbx, rax					;copy rax to rbx
		mov		rcx, rdx					;copy rdx to rcx
		add		ebx, 1						;add one to ebx
		cmovc	r13, _one					;if the carry flag is set then set r13 to 1
		add		ecx, r13d					;add r13d to ecx (r13d will be either 0 or 1)
lock	cmpxchg8b	_fm						;lock and fire the instruction
		jnz		IncCount					;if the zero flag is not set then another thread beat us to it so jump to IncCount and try again
		mov		rax, _v1					;move the pointer back into rax

		mov		r13, _memres
		mov		rdi, 1
		mov		rsi, 1
		mov		r13, qword ptr[r13+8]

LookForItLoop:

		cmp		qword ptr[r13+rsi*8], 0
		je		OneGood
		inc		rsi
		cmp		rsi, 65535
		jna		LookForItLoop
		mov		r13, _memres
		inc		rdi
		cmp		rdi, qword ptr[r13]
		ja		CreateAnotherOne
		mov		r13, qword ptr[r13+rdi*8]
		mov		rsi, 1
		jmp		LookForItLoop

OneGood:

		mov		qword ptr[r13+rsi*8], rax

		EXITFUNC _zero, _one, _zero, _one, _one, _one, _one, _one, _one, _one, _zero, _one, _zero, _zero, _zero

		RET

CreateAnotherOne:

		mov		_v2, rax
		mov		rcx, _tablereqsize
		call	malloc
		mov		r13,  _memres
		inc		qword ptr[r13]
		mov		rdi, qword ptr[r13]
		mov		qword ptr[r13+rdi*8], rax
		mov		qword ptr[rax], 0
		mov		r13, rax
		mov		rdi, rax				
		mov		rcx, _numberofallocs	
		inc		rcx
		mov		rax, 0					
		pushf							
		cld								
rep		stosq							
		popf		
		mov		r13, _memres
		mov		rdi, qword ptr[r13]
		mov		r13, qword ptr[r13+rdi*8]
		mov		rsi, 1					
		mov		rax, _v2
		jmp		LookForItLoop

CallInitialize:

		call	Initialize					;needs to be initialized so call the initialization routine (done this way because all but the first time the jmp statement above will fall through)
		jmp		Allready_initialized		;jmp back to the beggining

AllreadyMarked:

		;ok our marking of that address failed, on failure the command moves the value of the address into edx:eax, so as above we compute how many blocks to jump
		;make sure it is not past the end of the set, then jump to the search loop again.  If it is past the end, load _memslot into r13, _t2 into rdi, inc rdi
		;make sure it is pointing to a valid set, if it is set rdi and rsi to 1 rcx to 0 and jump to the search loop.  If it is not pointing to a valid set jump to create new set

		shr		edx, 16						;the cmpexchg failed when the instruction fails it loads what it found at the memory location into eax/edx
		shr		eax, 16						;both of which needs to be shifted to the right 16 bits
		xor		rdi, rdi					;then eax gets added to edx and moved into rdi, this will point to the first block past this set
		add		edi, eax
		add		edi, edx
		xor		rcx, rcx					;the count is cleared
		mov		rsi, rdi					;mov rdi into rsi so the first block is kept track of
		cmp		rdi, _numberofallocs		;make sure rdi doesnt point past the end of the table
		jna		SearchLoop					;if its good jump to the head of the search loop
		inc		_t2							;if its not incerement the index into the memslot array
		mov		r13, _memslot				;mov the memslot array into r13
		mov		rax, _t2					;mov the index into rax
		cmp		rax, qword ptr[r13]			;make sure the index isnt pointing past the last table
		ja		CreateNewSet				;if it is create a new set
		mov		r13, qword ptr[r13+rax*8]	;otherwise move the next memslot table into r13
		mov		rdi, 1						;point rdi to the first block
		mov		rsi, 1						;point rsi to the first block
		xor		rcx, rcx					;clear the counter
		jmp		SearchLoop					;jump to the head of the search loop

CreateNewSet:

		;we want to make sure we are the only thread creating a new set so we use the cmpxchg command again but this time on a global variable _gm
		;if we fail jump to start bottom.  If we succeed, get enough memory for another _memptr set and store it in the memptr table (updating the count as we go)
		;then we request enough for another memslot table, we zero out all the elements, then store it in the memslot table (updating the count as we go)
		;then we point _t2 to the new set, set rdi and rsi to 1 rcx to 0 and jump to the search loop setting _gm to zero right before we leave

		mov		rax, 0					;have to prep the cmpexchg instruction
		mov		rdx, 0					;we want to make sure the _gm global variable is 0 (which means no other thread is currently in this section of code)
		mov		rbx, 1					;if it is 0 we want to change it to 1
		mov		rcx, 0
lock	cmpxchg8b	_gm					;lock and fire the instruction
		jnz		StartBottom				;if the zero flag is not set then another thread is allready in this section of code, so exit this part and wait till it is finished
		mov		rcx, _allocreqsize		;move the amount to request for the memptr entry into rcx
		call	malloc					;get the memory from the syste,
		cmp		rax, 0					;make sure there wasnt an error
		je		MallocError				;if there was jump to the error routine
		mov		r13, _memptr			;move the memptr array into r13
		mov		rdi, qword ptr[r13]		;move the array count (element 0) into rdi
		inc		rdi						;increment the element count
		mov		qword ptr[r13], rdi		;store the new count in element 0
		mov		qword ptr[r13+rdi*8], rax	;store the pointer we just requested in the new position








		;mov		rcx, _tablereqsize		
		;call	malloc					
		;cmp		rax, 0					
		;je		MallocError				
		;mov		r13, _memslotcount			
		;mov		rdi, qword ptr[r13]		
		;inc		rdi						
		;mov		qword ptr[r13+rdi*8], rax	
		;mov		qword ptr[r13], rdi		
		;mov		r13, rax				
		;mov		rdi, rax				
		;mov		rcx, _numberofallocs	
		;inc		rcx
		;mov		rax, 0					
		;pushf							
		;cld								
;rep		stosq							
		;popf							
		;mov		rcx, _tablereqsize		
		;call	malloc					
		;cmp		rax, 0					
		;je		MallocError				
		;mov		r13, _memslotdelete			
		;mov		rdi, qword ptr[r13]		
		;inc		rdi						
		;mov		qword ptr[r13+rdi*8], rax	
		;mov		qword ptr[r13], rdi		
		;mov		r13, rax				
		;mov		rdi, rax				
		;mov		rcx, _numberofallocs	
		;inc		rcx
		;mov		rax, 0					
		;pushf							
		;cld								
;rep		stosq							
		;popf							
;














		mov		rcx, _tablereqsize		;move the size of the memslot table into rcx
		call	malloc					;request the memory from the system
		cmp		rax, 0					;make sure there wasnt an error filling the request
		je		MallocError				;if there was jump to the error routine
		mov		r13, _memslot			;move the memslot array into r13
		mov		rdi, qword ptr[r13]		;move the array count into rdi
		inc		rdi						;increment the array coutnt
		mov		qword ptr[r13+rdi*8], rax	;store the memory we just requested in the array
		mov		qword ptr[r13], rdi		;store the new count at element 0
		mov		r13, rax				;move the pointer to the new table into r13
		mov		rdi, rax				;move the pointer into rdi
		mov		rcx, _numberofallocs	;move the number of allocations (table size) into rcx
		inc		rcx
		mov		rax, 0					;move zero into rax (this is what will be stored in all the elements in the new memslot table since none of the blocks have been assigned yet)
		pushf							;save the flags register
		cld								;clear the direction flag
rep		stosq							;store 0 in all the elements of the new table
		popf							;restore the flags register
		mov		_gm, 0					;clear the _gm variable
		inc		_created
		xor		rcx, rcx				;clear the count
		mov		rdi, 1					;point at the first block
		mov		rsi, 1					;set rsi to point to the first block
		jmp		SearchLoop				;jump to the head of the search loop

StartBottom:

		;we didnt get exclusive access to that part of the routine, we know there are not enough blocks in any of the other sets for us, so we wait till
		;the other thread is done creating and initializing everything, at which point we point to the new set, set rdi and rsi to 1 and rcx to 0 then jump to the search loop

		cmp		_gm, 1					;see if the thread that is setting up the new tables is still working on it
		je		StartBottom				;if it is jump to the head of this loop again
		mov		r13, _memslot			;load the memslot array into r13
		mov		rdi, qword ptr[r13]		;get the index of the last table (we woudnt be here unless we allready searched all the other tables)
		mov		r13, qword ptr[r13+rdi*8]	;load the new table into r13
		mov		rdi, 1					;point rdi to the first block
		mov		rsi, 1					;set rsi to point to the first block
		xor		rcx, rcx				;clear the counter
		jmp		SearchLoop				;jump to the head of the search loop
	
		EXITFUNC _zero, _one, _zero, _one, _one, _one, _one, _one, _one, _one, _zero, _one, _zero, _zero, _zero

		RET
	
MallocError:

		mov		rax, rax					

	GetMem64 ENDP
	;;
	;@brief This returns an assigned block back to the memory pool.
	;
	;  This method works by loading the _memptr array, then comparing the value being returned to each value in the array.  If it found one that the item being reutrned was > what was in the _memptr and <
	;what was in the _memptr + (_numberofallocs * _allocsize), then it knows that is the correct one.  It then subtracts the value in the _memptr array from the value being returned and divides that result
	;by _allocsize, the result of this calculation tells you the first block in the _memslot array you need to change, so it pulls the corresponding table out of _memslot, pulls the value out of the block
	;it just calculated, adds the values it pulled from the _memslot table together and decrements the result by one, this now gives you the last block that was marked to fullfill this request.
	;  When clearing the memory pointed to by those blocks, it allways starts from the last block and goes to the first, this was if a thread happens to be looking at those blocks at the same time
	;there wont be any conflicts since it will just check the first block, see its allready taken, then move on to check the first block after it.
	;
	;  If it goes through all the values in the _memptr array and doesnt find one that macthes, it then figures the memory was fullfilled through malloc so it calls free on the ptr being returned.  Doing
	;it this way I dont need to keep track of the relevant info in the memory that was requested, I can also change the _usemalloc value in the middle of an algorithm and everything still works fine.
	;
	;  A few ideas about possible improvements, maybe use a few global variables that point to the first free block so that the request alg could autimaticaly jump to it without searching through
	;everything allready taken before it.  I also should rewrite these methods so that they would work in both 32 bit and 64 bit with the same code.
	;
	;@param rcx - pointer to the block to return.
	;;
	Free64 PROC _ptr:qword

		ENTERFUNC _one, _one, _zero, _one, _one, _one, _one, _one, _one, _one, _zero, _one, _zero, _zero, _zero
		
		;check is we are supposed to use malloc, if we are jump to the free routine

		mov		_var7, 0
		cmp		rcx,  0						;make sure we arent processing crap
		je		JustExit					;exit if we are
		mov		_v1, rcx					;move the pointer into _v1



		mov		r13, _memres
		mov		rdi, 1
		mov		rsi, 1
		mov		r13, qword ptr[r13+8]

FindItLoop:

		cmp		rcx, qword ptr[r13+rsi*8]
		je		FoundIt34
		inc		rsi
		cmp		rsi, 65535
		jna		FindItLoop
		inc		rdi
		mov		r13, _memres
		cmp		rdi, qword ptr[r13]
		ja		NotFound
		mov		r13, qword ptr[r13+rdi*8]
		mov		rsi, 1
		jmp		FindItLoop

NotFound:

		mov		r13, _memfree
		mov		rdi, qword ptr[r13]
		mov		r13, qword ptr[r13+rdi*8]
		cmp		qword ptr[r13], 65536
		jnb		CreateAnotherOneFree

ReturnFromCreateFree:

		inc		qword ptr[r13]
		mov		rdi, qword ptr[r13]
		mov		qword ptr[r13+rdi*8], rcx
		jmp		SavedIt

CreateAnotherOneFree:

		mov		rcx, _tablereqsize
		call	malloc
		mov		r13,  _memfree
		inc		qword ptr[r13]
		mov		rdi, qword ptr[r13]
		mov		qword ptr[r13+rdi*8], rax
		mov		qword ptr[rax], 0
		mov		r13, rax
		jmp		ReturnFromCreateFree

FoundIt34:

		mov		qword ptr[r13+rsi*8], 0

SavedIt:

		mov		rcx, _v1






		mov		r13, _memptr				;otherwise move the memptr array into r13
		mov		rdi, 1						;set the index to 1
		mov		_t1, rdi					;save which index we are working on
		cmp		_fm, 0						;check if we have any memory out that we assigned using our alg
		ja		FindLoop					;if we do then allways start by checking against our alg first
		cmp		_usemalloc, 0				;check if we are using malloc (there actually should be a check to make sure no memory was allocated using our alg before switching the _usemalloc var)
		je		FindLoop					;jump to our alg is not using malloc
		call	free						;call free on the pointer

		EXITFUNC _one, _one, _zero, _one, _one, _one, _one, _one, _one, _one, _zero, _one, _zero, _zero, _zero

		RET

FindLoop:

		;other wise we find which set the memory belongs to by comparing it to the top and bottom of each set till we find one where what was sent is between
		;once found we find out which block it is by subtracting the bottom of the memptr set from the address sent to us and dividing by the allocsize
		;this will give us the first block we requested which we pull from memslot, this will tell us how many blocks we requested and how many we must zero out
		;when zeroing out, we start at the top (since getmem searches from the bottom up) so there isnt a problem.

		mov		rax, qword ptr[r13+rdi*8]	;move the memptr pointed to by rdi into rax
		cmp		_v1, rax					;compare it with what was passed in
		jb		NextSet						;if what was passed in is below rax then it cannot be from this base pointer
		add		rax, _allocreqsize			;add the allocation request size to rax
		cmp		_v1, rax					;compare the value passed in to the new rax value
		jnb		NextSet						;if what was passed in is above the value in rax then it cannot be from this base pointer
		mov		rax, _v1					;if we reached this point then we found the base pointer the memory came from, so move what was passed in to rax
		sub		rax, qword ptr[r13+rdi*8]	;subtract the base pointer from the value passed in
		xor		rdx, rdx					;clear the rdx register to prepare for the division
		div		_allocsize					;divide by the allocation size (this gives us the first block we took)
		mov		rsi, rax					;move the pointer to the first block into rsi
		mov		r13, _memslot				;move the memslot array into r13
		mov		r13, qword ptr[r13+rdi*8]	;move memslot table that corresponds to the memptr we want into r13







		;mov		r11, _memslotdelete
		;mov		r11, qword ptr[r11+rdi*8]
		;mov		rcx, qword ptr[r13+rsi*8]
		;mov		rdi, rsi
;
;IncDeleteLoop:
;
		;cmp		rcx, qword ptr[r13+rdi*8]
		;jne		IncOut
		;inc		qword ptr[r11+rdi*8]
		;inc		rdi
		;jmp		IncDeleteLoop
;
;IncOut:
;









		movzx	rax, word ptr[r13+rsi*8+2]	;move the first blocks index into rax
		movzx	rdi, word ptr[r13+rsi*8+6]	;move the number of blocks that this request took into rdi
		mov		rcx, rax					;move this value into rcx
		add		rax, rdi					;add rax and rdi (this points us to the first block past what this request took)
		dec		rax							;decrement rax so that we have the correct count
		mul		_eight						;multiply it by eight to get the number of bytes
		add		rax, r13					;add rax and r13 so that we are pointing at the last byte of the last block in the memslot table that we took
		mov		rdi, rax					;move rax into rdi
		xor		rax, rax					;clear rax
		pushf								;save the flags register
		std									;set the direction flag so that we start clearing at the last block instead of the first block, this will prevent any conflicts since the getmem routine will only hit the first block of a taken element in the memslot table and jump past the taken blocks if it is taken)
rep		stosq								;set each block that was used to fill this request to 0 (signaling it is no longer used)
		popf								;restore the flags register

		;this is a new section in this alg to keep track of the number of requests we filled using our alg are still out.  We will be using the cmpexchg with the lock prefix to make sure we keep an 
		;accurate count.  The only way to directly access windows free alg is if the _fm variable is 0 and the _usemalloc variable is set to 1.
		
DecCount:

		mov		r13, 0						;prep to decrement the count, set r13 to zero
		mov		rax, _fm					;move the value in _fm to rax
		mov		rdx, rax					;copy rax to rdx
		shr		rdx, 32						;shift rdx to the right by 32 bits (_fm should be looked at like two dwords)
		and		rax, 0ffffffffh				;and rax by 0ffffffffh to keep just the first 32 bits (rdx:rax now holds the value we will compare to)
		mov		rbx, rax					;copy rax to rbx
		mov		rcx, rdx					;copy rdx to rcx
		sub		ebx, 1						;subtract one from ebx
		cmovc	r13, _one					;if the carry flag is set then set r13 to 1
		sub		ecx, r13d					;subtract r13d from ecx (r13d will be either 0 or 1)
lock	cmpxchg8b	_fm						;lock and fire the instruction
		jnz		DecCount					;if the zero flag is not set then another thread beat us to it so jump to DecCount and try again

		EXITFUNC _one, _one, _zero, _one, _one, _one, _one, _one, _one, _one, _zero, _one, _zero, _zero, _zero

		RET

NextSet:

		inc		rdi							;so that was not the correct memptr table, increment the index into the array
		cmp		rdi, qword ptr[r13]			;make sure it is not pointing past the last array
		jna		FindLoop					;jump to the head of the find loop if its all good

		;if we reached this point it is either because none of the base pointers could have filled the pointer passed into this routine, or the _usemalloc variable was set
		;I have it set to fall through so that if the _usemalloc var is changed after my routine was used to fill a request then it could use my free routine to free it
		;but with the way it is set right now, if the _usemalloc is set it will jump stright here even if my getmem routine was used to fill the request.
		;need to change it maybe set a global flag to show if there are any remaining memory pointers filled with my routine are out so that it checks my free routine
		;first until all the memory my alg filled is returned, then it can jump straight here is using malloc

UseFree:

		mov		rcx, _v1					;we either got here because we searched all the memptr tables and didnt find one that this pointer cound of come from, or the _usemalloc var was set and we jumped here
		call	free						;once we move the pointer passed into the routine into rcx we call free on it

JustExit:

		EXITFUNC _one, _one, _zero, _one, _one, _one, _one, _one, _one, _one, _zero, _one, _zero, _zero, _zero

		RET

	Free64 ENDP

	InitializeMem PROC

		ENTERFUNC _one, _one, _one, _one, _one, _one, _one, _one, _one, _one, _zero, _zero, _zero, _one, _zero




		mov		rcx, 9600
		call	malloc
		mov		_memres, rax
		mov		rcx, _tablereqsize
		call	malloc
		mov		r13,  _memres
		mov		qword ptr[r13], 1
		mov		qword ptr[r13+8], rax
		mov		qword ptr[rax], 0
		mov		rdi, rax				
		mov		rcx, _numberofallocs	
		inc		rcx
		mov		rax, 0					
		pushf							
		cld								
rep		stosq							
		popf		
		mov		rcx, 9600
		call	malloc
		mov		_memfree, rax
		mov		rcx, _tablereqsize
		call	malloc
		mov		r13,  _memfree
		mov		qword ptr[r13], 1
		mov		qword ptr[r13+8], rax
		mov		qword ptr[rax], 0

		;mov		rcx, 96
		;call	malloc
		;mov		_memslotcount, rax
		;mov		rcx, _tablereqsize		
		;call	malloc					
		;mov		r13, _memslotcount			
		;mov		qword ptr[r13], 1	
		;mov		qword ptr[r13+8], rax	
		;mov		rdi, rax				
		;mov		rcx, _numberofallocs	
		;inc		rcx
		;mov		rax, 0					
		;pushf							
		;cld								
;rep		stosq							
		;popf							
		;mov		rcx, 96
		;call	malloc
		;mov		_memslotdelete, rax
		;mov		rcx, _tablereqsize		
		;call	malloc					
		;mov		r13, _memslotdelete		
		;mov		qword ptr[r13], 1	
		;mov		qword ptr[r13+8], rax	
		;mov		rdi, rax				
		;mov		rcx, _numberofallocs	
		;inc		rcx
		;mov		rax, 0					
		;pushf							
		;cld								
;rep		stosq							
		;popf							
;











		mov		rcx, 96
		call	malloc
		mov		qword ptr[rax], 1
		mov		_memslot, rax
		mov		rcx, _tablereqsize
		call	malloc
		mov		rcx, _memslot
		mov		qword ptr[rcx+8], rax
		mov		rbx, _numberofallocs
		inc		rbx
		mov		qword ptr[rax], rbx
		mov		rdi, _numberofallocs

MarkMemSlotLoop:

		mov		qword ptr[rax+rdi*8], 0
		sub		rdi, 1
		jnz		MarkMemSlotLoop
		mov		rcx, 96
		call	malloc
		mov		qword ptr[rax], 1
		mov		_memptr, rax
		mov		rcx, _allocreqsize
		call	malloc
		mov		rcx, _memptr
		mov		qword ptr[rcx+8], rax
		cmp		rax, 0
		ja		ItsGood
		mov		rax, rax

ItsGood:

		EXITFUNC _one, _one, _one, _one, _one, _one, _one, _one, _one, _one, _zero, _zero, _zero, _one, _zero

		ret

	InitializeMem endp
	;;
	;@brief Initializes the memory system and global variables
	;;	
	Initialize PROC

		ENTERFUNC _one, _one, _one, _one, _one, _one, _one, _one, _one, _one, _zero, _zero, _zero, _one, _zero

		cmp		_initialized, 0			;check if everything is allready initialized, if not then run initialization
		je		DoInitialize

		EXITFUNC _one, _one, _one, _one, _one, _one, _one, _one, _one, _one, _zero, _zero, _zero, _one, _zero
		
		RET

DoInitialize:

		lea		rcx, _base
		mov		qword ptr[rcx], 2
		mov		qword ptr[rcx+8], 0
		mov		qword ptr[rcx+16], 1
		lea		rcx, _upper
		xor		rsi, rsi
		mov		rdx, 18446744073709551614
		mov		qword ptr[rcx+rsi*8], 18446744073709551615
		mov		rax, 1
		inc		rsi

UpperLoop:

		mov		qword ptr[rcx+rsi*8], rdx
		mul		_two
		sub		rdx, rax
		inc		rsi
		cmp		rsi, 63
		jna		UpperLoop
		lea		rcx, _lower
		xor		rsi, rsi
		mov		qword ptr[rcx+rsi*8], 0
		inc		rsi
		mov		rax, 1
		mov		rdx, 1

LowerLoop:

		mov		qword ptr[rcx+rsi*8], rdx
		mul		_two
		add		rdx, rax
		inc		rsi
		cmp		rsi, 63
		jna		LowerLoop
		lea		rcx, _bits
		xor		rsi, rsi
		mov		rax, 1

BitsLoop:

		mov		qword ptr[rcx+rsi*8], rax
		mul		_two
		inc		rsi
		cmp		rsi, 63
		jna		BitsLoop
		lea		r15, _tobase2
		xor			rdi, rdi
		mov			qword ptr[r15+rdi*8], '0'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '1'
		lea			r15, _tobase8
		xor			rdi, rdi
		mov			qword ptr[r15+rdi*8], '0'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '1'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '2'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '3'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '4'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '5'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '6'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '7'
		lea			r15, _tobase10
		xor			rdi, rdi
		mov			qword ptr[r15+rdi*8], '0'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '1'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '2'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '3'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '4'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '5'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '6'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '7'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '8'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '9'
		lea			r15, _tobase16
		xor			rdi, rdi
		mov			qword ptr[r15+rdi*8], '0'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '1'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '2'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '3'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '4'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '5'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '6'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '7'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '8'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '9'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'A'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'B'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'C'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'D'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'E'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'F'
		lea			r15, _tobase32
		xor			rdi, rdi
		mov			qword ptr[r15+rdi*8], 'A'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'B'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'C'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'D'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'E'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'F'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'G'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'H'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'I'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'J'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'K'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'L'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'M'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'N'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'O'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'P'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'Q'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'R'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'S'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'T'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'U'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'V'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'W'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'X'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'Y'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'Z'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '2'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '3'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '4'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '5'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '6'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '7'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '='
		lea			r15, _tobase64
		xor			rdi, rdi
		mov			qword ptr[r15+rdi*8], 'A'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'B'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'C'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'D'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'E'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'F'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'G'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'H'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'I'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'J'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'K'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'L'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'M'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'N'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'O'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'P'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'Q'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'R'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'S'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'T'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'U'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'V'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'W'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'X'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'Y'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'Z'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'a'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'b'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'c'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'd'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'e'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'f'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'g'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'h'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'i'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'j'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'k'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'l'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'm'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'n'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'o'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'p'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'q'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'r'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 's'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 't'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'u'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'v'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'w'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'x'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'y'
		inc			rdi
		mov			qword ptr[r15+rdi*8], 'z'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '0'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '1'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '2'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '3'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '4'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '5'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '6'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '7'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '8'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '9'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '+'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '/'
		inc			rdi
		mov			qword ptr[r15+rdi*8], '='
		lea			r15, _frombase2
		mov			rdi, 30h
		mov			qword ptr[r15+rdi*8], 0
		inc			rdi
		mov			qword ptr[r15+rdi*8], 1
		lea			r15, _frombase8
		mov			rdi, 30h
		mov			qword ptr[r15+rdi*8], 0
		inc			rdi
		mov			qword ptr[r15+rdi*8], 1
		inc			rdi
		mov			qword ptr[r15+rdi*8], 2
		inc			rdi
		mov			qword ptr[r15+rdi*8], 3
		inc			rdi
		mov			qword ptr[r15+rdi*8], 4
		inc			rdi
		mov			qword ptr[r15+rdi*8], 5
		inc			rdi
		mov			qword ptr[r15+rdi*8], 6
		inc			rdi
		mov			qword ptr[r15+rdi*8], 7
		lea			r15, _frombase10
		mov			rdi, 30h
		mov			qword ptr[r15+rdi*8], 0
		inc			rdi
		mov			qword ptr[r15+rdi*8], 1
		inc			rdi
		mov			qword ptr[r15+rdi*8], 2
		inc			rdi
		mov			qword ptr[r15+rdi*8], 3
		inc			rdi
		mov			qword ptr[r15+rdi*8], 4
		inc			rdi
		mov			qword ptr[r15+rdi*8], 5
		inc			rdi
		mov			qword ptr[r15+rdi*8], 6
		inc			rdi
		mov			qword ptr[r15+rdi*8], 7
		inc			rdi
		mov			qword ptr[r15+rdi*8], 8
		inc			rdi
		mov			qword ptr[r15+rdi*8], 9
		lea			r15, _frombase16
		mov			rdi, 30h
		mov			qword ptr[r15+rdi*8], 0
		inc			rdi
		mov			qword ptr[r15+rdi*8], 1
		inc			rdi
		mov			qword ptr[r15+rdi*8], 2
		inc			rdi
		mov			qword ptr[r15+rdi*8], 3
		inc			rdi
		mov			qword ptr[r15+rdi*8], 4
		inc			rdi
		mov			qword ptr[r15+rdi*8], 5
		inc			rdi
		mov			qword ptr[r15+rdi*8], 6
		inc			rdi
		mov			qword ptr[r15+rdi*8], 7
		inc			rdi
		mov			qword ptr[r15+rdi*8], 8
		inc			rdi
		mov			qword ptr[r15+rdi*8], 9
		mov			rdi, 41h
		mov			qword ptr[r15+rdi*8], 10
		inc			rdi
		mov			qword ptr[r15+rdi*8], 11
		inc			rdi
		mov			qword ptr[r15+rdi*8], 12
		inc			rdi
		mov			qword ptr[r15+rdi*8], 13
		inc			rdi
		mov			qword ptr[r15+rdi*8], 14
		inc			rdi
		mov			qword ptr[r15+rdi*8], 15
		mov			rdi, 61h
		mov			qword ptr[r15+rdi*8], 10
		inc			rdi
		mov			qword ptr[r15+rdi*8], 11
		inc			rdi
		mov			qword ptr[r15+rdi*8], 12
		inc			rdi
		mov			qword ptr[r15+rdi*8], 13
		inc			rdi
		mov			qword ptr[r15+rdi*8], 14
		inc			rdi
		mov			qword ptr[r15+rdi*8], 15
		lea			r15, _frombase32
		mov			rdi, 41h
		mov			qword ptr[r15+rdi*8], 0
		inc			rdi
		mov			qword ptr[r15+rdi*8], 1
		inc			rdi
		mov			qword ptr[r15+rdi*8], 2
		inc			rdi
		mov			qword ptr[r15+rdi*8], 3
		inc			rdi
		mov			qword ptr[r15+rdi*8], 4
		inc			rdi
		mov			qword ptr[r15+rdi*8], 5
		inc			rdi
		mov			qword ptr[r15+rdi*8], 6
		inc			rdi
		mov			qword ptr[r15+rdi*8], 7
		inc			rdi
		mov			qword ptr[r15+rdi*8], 8
		inc			rdi
		mov			qword ptr[r15+rdi*8], 9
		inc			rdi
		mov			qword ptr[r15+rdi*8], 10
		inc			rdi
		mov			qword ptr[r15+rdi*8], 11
		inc			rdi
		mov			qword ptr[r15+rdi*8], 12
		inc			rdi
		mov			qword ptr[r15+rdi*8], 13
		inc			rdi
		mov			qword ptr[r15+rdi*8], 14
		inc			rdi
		mov			qword ptr[r15+rdi*8], 15
		inc			rdi
		mov			qword ptr[r15+rdi*8], 16
		inc			rdi
		mov			qword ptr[r15+rdi*8], 17
		inc			rdi
		mov			qword ptr[r15+rdi*8], 18
		inc			rdi
		mov			qword ptr[r15+rdi*8], 19
		inc			rdi
		mov			qword ptr[r15+rdi*8], 20
		inc			rdi
		mov			qword ptr[r15+rdi*8], 21
		inc			rdi
		mov			qword ptr[r15+rdi*8], 22
		inc			rdi
		mov			qword ptr[r15+rdi*8], 23
		inc			rdi
		mov			qword ptr[r15+rdi*8], 24
		inc			rdi
		mov			qword ptr[r15+rdi*8], 25
		mov			rdi, 32h
		mov			qword ptr[r15+rdi*8], 26
		inc			rdi
		mov			qword ptr[r15+rdi*8], 27
		inc			rdi
		mov			qword ptr[r15+rdi*8], 28
		inc			rdi
		mov			qword ptr[r15+rdi*8], 29
		inc			rdi
		mov			qword ptr[r15+rdi*8], 30
		inc			rdi
		mov			qword ptr[r15+rdi*8], 31
		lea			r15, _frombase64
		mov			rdi, 41h
		mov			qword ptr[r15+rdi*8], 0
		inc			rdi
		mov			qword ptr[r15+rdi*8], 1
		inc			rdi
		mov			qword ptr[r15+rdi*8], 2
		inc			rdi
		mov			qword ptr[r15+rdi*8], 3
		inc			rdi
		mov			qword ptr[r15+rdi*8], 4
		inc			rdi
		mov			qword ptr[r15+rdi*8], 5
		inc			rdi
		mov			qword ptr[r15+rdi*8], 6
		inc			rdi
		mov			qword ptr[r15+rdi*8], 7
		inc			rdi
		mov			qword ptr[r15+rdi*8], 8
		inc			rdi
		mov			qword ptr[r15+rdi*8], 9
		inc			rdi
		mov			qword ptr[r15+rdi*8], 10
		inc			rdi
		mov			qword ptr[r15+rdi*8], 11
		inc			rdi
		mov			qword ptr[r15+rdi*8], 12
		inc			rdi
		mov			qword ptr[r15+rdi*8], 13
		inc			rdi
		mov			qword ptr[r15+rdi*8], 14
		inc			rdi
		mov			qword ptr[r15+rdi*8], 15
		inc			rdi
		mov			qword ptr[r15+rdi*8], 16
		inc			rdi
		mov			qword ptr[r15+rdi*8], 17
		inc			rdi
		mov			qword ptr[r15+rdi*8], 18
		inc			rdi
		mov			qword ptr[r15+rdi*8], 19
		inc			rdi
		mov			qword ptr[r15+rdi*8], 20
		inc			rdi
		mov			qword ptr[r15+rdi*8], 21
		inc			rdi
		mov			qword ptr[r15+rdi*8], 22
		inc			rdi
		mov			qword ptr[r15+rdi*8], 23
		inc			rdi
		mov			qword ptr[r15+rdi*8], 24
		inc			rdi
		mov			qword ptr[r15+rdi*8], 25
		mov			rdi, 61h
		mov			qword ptr[r15+rdi*8], 26
		inc			rdi
		mov			qword ptr[r15+rdi*8], 27
		inc			rdi
		mov			qword ptr[r15+rdi*8], 28
		inc			rdi
		mov			qword ptr[r15+rdi*8], 29
		inc			rdi
		mov			qword ptr[r15+rdi*8], 30
		inc			rdi
		mov			qword ptr[r15+rdi*8], 31
		inc			rdi
		mov			qword ptr[r15+rdi*8], 32
		inc			rdi
		mov			qword ptr[r15+rdi*8], 33
		inc			rdi
		mov			qword ptr[r15+rdi*8], 34
		inc			rdi
		mov			qword ptr[r15+rdi*8], 35
		inc			rdi
		mov			qword ptr[r15+rdi*8], 36
		inc			rdi
		mov			qword ptr[r15+rdi*8], 37
		inc			rdi
		mov			qword ptr[r15+rdi*8], 38
		inc			rdi
		mov			qword ptr[r15+rdi*8], 39
		inc			rdi
		mov			qword ptr[r15+rdi*8], 40
		inc			rdi
		mov			qword ptr[r15+rdi*8], 41
		inc			rdi
		mov			qword ptr[r15+rdi*8], 42
		inc			rdi
		mov			qword ptr[r15+rdi*8], 43
		inc			rdi
		mov			qword ptr[r15+rdi*8], 44
		inc			rdi
		mov			qword ptr[r15+rdi*8], 45
		inc			rdi
		mov			qword ptr[r15+rdi*8], 46
		inc			rdi
		mov			qword ptr[r15+rdi*8], 47
		inc			rdi
		mov			qword ptr[r15+rdi*8], 48
		inc			rdi
		mov			qword ptr[r15+rdi*8], 49
		inc			rdi
		mov			qword ptr[r15+rdi*8], 50
		inc			rdi
		mov			qword ptr[r15+rdi*8], 51
		mov			rdi, 30h
		mov			qword ptr[r15+rdi*8], 52
		inc			rdi
		mov			qword ptr[r15+rdi*8], 53
		inc			rdi
		mov			qword ptr[r15+rdi*8], 54
		inc			rdi
		mov			qword ptr[r15+rdi*8], 55
		inc			rdi
		mov			qword ptr[r15+rdi*8], 56
		inc			rdi
		mov			qword ptr[r15+rdi*8], 57
		inc			rdi
		mov			qword ptr[r15+rdi*8], 58
		inc			rdi
		mov			qword ptr[r15+rdi*8], 59
		inc			rdi
		mov			qword ptr[r15+rdi*8], 60
		inc			rdi
		mov			qword ptr[r15+rdi*8], 61
		mov			rdi, 2bh
		mov			qword ptr[r15+rdi*8], 62
		mov			rdi, 2fh
		mov			qword ptr[r15+rdi*8], 63
		mov		_initialized, 1
		call	InitVals

		EXITFUNC _one, _one, _one, _one, _one, _one, _one, _one, _one, _one, _zero, _zero, _zero, _one, _zero
		
		RET

Initialize ENDP
	;;
	;@brief Initializes some commonly used ABI, ABD, and ABF objects
	;;
InitVals PROC

		ENTERFUNC _one, _one, _one, _one, _one, _one, _one, _one, _one, _one, _zero, _zero, _zero, _zero, _zero

		mov		_initvals, 1
		mov		rcx, 24
		call	malloc
		mov		_var1, rax
		mov		qword ptr[rax], 1
		mov		qword ptr[rax+8], 10
		mov		rcx, 24
		call	malloc
		mov		_var2, rax
		mov		qword ptr[rax], 1
		mov		rcx, 64
		call	malloc
		mov		_t3, rax
		mov		qword ptr[rax], 6
		mov		rbx, 10686169104128798711
		mov		qword ptr[rax+8], rbx
		mov		rbx, 7849733902273087162
		mov		qword ptr[rax+16], rbx
		mov		rbx, 11419148344871743026
		mov		qword ptr[rax+24], rbx
		mov		rbx, 79130234824454320
		mov		qword ptr[rax+32], rbx
		mov		rbx, 16971786432492145963
		mov		qword ptr[rax+40], rbx
		mov		qword ptr[rax+48], 14707
		mov		rcx, rax
		call	CreateBDCopy
		mov		_bdpi, rax
		mov		[rax].ABD._decimalplace, 100
		mov		rcx, _var1
		mov		rdx, _var2
		mov		qword ptr[rdx+8], 100
		call	APow64
		mov		rdx, rax
		mov		rcx, _t3
		call	CreateBFTakeBoth
		mov		_bfpi, rax
		mov		rcx, 64
		call	malloc
		mov		_t3, rax
		mov		qword ptr[rax], 6
		mov		rbx, 2398342449645769106
		mov		qword ptr[rax+8], rbx
		mov		rbx, 6418917767345865382
		mov		qword ptr[rax+16], rbx
		mov		rbx, 12948940461481586887
		mov		qword ptr[rax+24], rbx
		mov		rbx, 5692482523596577000
		mov		qword ptr[rax+32], rbx
		mov		rbx, 2135426198622894028
		mov		qword ptr[rax+40], rbx
		mov		qword ptr[rax+48], 12726
		mov		rcx, rax
		call	CreateBDCopy
		mov		_bde, rax
		mov		[rax].ABD._decimalplace, 100
		mov		rcx, _var1
		mov		rdx, _var2
		mov		qword ptr[rdx+8], 100
		call	APow64
		mov		rdx, rax
		mov		rcx, _t3
		call	CreateBFTakeBoth
		mov		_bfe, rax
		mov		rcx, 32
		call	malloc
		mov		_t3, rax
		mov		qword ptr[rax], 2
		mov		rbx, 9515212105952205938
		mov		qword ptr[rax+8], rbx
		mov		rbx, 9461448973346115
		mov		qword ptr[rax+16], rbx
		mov		rcx, rax
		call	CreateBDCopy
		mov		_bdtorad, rax
		mov		[rax].ABD._decimalplace, 37
		mov		rcx, _var1
		mov		rdx, _var2
		mov		qword ptr[rdx+8], 37
		call	APow64
		mov		rdx, rax
		mov		rcx, _t3
		call	CreateBFTakeBoth
		mov		_bftorad, rax
		mov		rcx, 32
		call	malloc
		mov		_t3, rax
		mov		qword ptr[rax], 2
		mov		rbx, 3583277511020380145
		mov		qword ptr[rax+8], rbx
		mov		rbx, 7666467083416870407
		mov		qword ptr[rax+16], rbx
		mov		rcx, rax
		call	CreateBDCopy
		mov		_bdsqr2, rax
		mov		[rax].ABD._decimalplace, 38
		mov		rcx, _var1
		mov		rdx, _var2
		mov		qword ptr[rdx+8], 38
		call	APow64
		mov		rdx, rax
		mov		rcx, _t3
		call	CreateBFTakeBoth
		mov		_bfsqr2, rax
		mov		rcx, 32
		call	malloc
		mov		_t3, rax
		mov		qword ptr[rax], 3
		mov		rbx, 7251689156681271640
		mov		qword ptr[rax+8], rbx
		mov		rbx, 13926821993062905291
		mov		qword ptr[rax+16], rbx
		mov		qword ptr[rax+24], 4
		mov		rcx, rax
		call	CreateBDCopy
		mov		_bdphi, rax
		mov		[rax].ABD._decimalplace, 39
		mov		rcx, _var1
		mov		rdx, _var2
		mov		qword ptr[rdx+8], 39
		call	APow64
		mov		rdx, rax
		mov		rcx, _t3
		call	CreateBFTakeBoth
		mov		_bfphi, rax
		mov		rcx, 40
		call	malloc
		mov		_t3, rax
		mov		qword ptr[rax], 3
		mov		rbx, 6648110680099582018
		mov		qword ptr[rax+8], rbx
		mov		rbx, 12844179820235791298
		mov		qword ptr[rax+16], rbx
		mov		qword ptr[rax+24], 1
		mov		rcx, rax
		call	CreateBDCopy
		mov		_bdlambda, rax
		mov		[rax].ABD._decimalplace, 39
		mov		rcx, _var1
		mov		rdx, _var2
		mov		qword ptr[rdx+8], 39
		call	APow64
		mov		rdx, rax
		mov		rcx, _t3
		call	CreateBFTakeBoth
		mov		_bflambda, rax
		mov		rcx, 64
		call	malloc
		mov		_t3, rax
		mov		qword ptr[rax], 6
		mov		rbx, 7913502088132746062
		mov		qword ptr[rax+8], rbx
		mov		rbx, 16015517243861033306
		mov		qword ptr[rax+16], rbx
		mov		rbx, 16006184436263609029
		mov		qword ptr[rax+24], rbx
		mov		rbx, 3598788493353623874
		mov		qword ptr[rax+32], rbx
		mov		rbx, 2056188808949124308
		mov		qword ptr[rax+40], rbx
		mov		qword ptr[rax+48], 7575
		mov		rcx, rax
		call	CreateBDCopy
		mov		_bdgoldenratio, rax
		mov		[rax].ABD._decimalplace, 100
		mov		rcx, _var1
		mov		rdx, _var2
		mov		qword ptr[rdx+8], 100
		call	APow64
		mov		rdx, rax
		mov		rcx, _t3
		call	CreateBFTakeBoth
		mov		_bfgoldenratio, rax
		mov		rcx, 32
		call	malloc
		mov		_t3, rax
		mov		qword ptr[rax], 2
		mov		rbx, 14385875712753874157
		mov		qword ptr[rax+8], rbx
		mov		rbx, 145255548274
		mov		qword ptr[rax+16], rbx
		mov		rcx, rax
		call	CreateBDCopy
		mov		_bd2sqr3, rax
		mov		[rax].ABD._decimalplace, 31
		mov		rcx, _var1
		mov		rdx, _var2
		mov		qword ptr[rdx+8], 31
		call	APow64
		mov		rdx, rax
		mov		rcx, _t3
		call	CreateBFTakeBoth
		mov		_bf2sqr3, rax
		mov		rcx, 32
		call	malloc
		mov		_t3, rax
		mov		qword ptr[rax], 2
		mov		rbx, 13366033398513191699
		mov		qword ptr[rax+8], rbx
		mov		rbx, 938946624210
		mov		qword ptr[rax+16], rbx
		mov		rcx, rax
		call	CreateBDCopy
		mov		_bdsqr3, rax
		mov		[rax].ABD._decimalplace, 31
		mov		rcx, _var1
		mov		rdx, _var2
		mov		qword ptr[rdx+8], 31
		call	APow64
		mov		rdx, rax
		mov		rcx, _t2
		call	CreateBFTakeBoth
		mov		_bfsqr3, rax
		call	CreateBI
		mov		_binegone, rax
		mov		[rax].ABI._sign, 1
		mov		rcx, [rax].ABI._num
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 1
		mov		rcx, _binegone
		call	CreateBD
		mov		_bdnegone, rax
		mov		[rax].ABD._sign, 1
		mov		[rax].ABD._decimalplace, 0
		mov		rcx, [rax].ABD._num
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 1
		call	CreateBF
		mov		_bfnegone, rax
		mov		[rax].ABF._sign, 1
		mov		rcx, [rax].ABF._numerator
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 1
		call	CreateBI
		mov		_bizero, rax
		mov		[rax].ABI._sign, 0
		mov		rcx, [rax].ABI._num
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 0
		mov		rcx, _bizero
		call	CreateBD
		mov		_bdzero, rax
		call	CreateBF
		mov		_bfzero, rax
		call	CreateBI
		mov		_bione, rax
		mov		[rax].ABI._sign, 0
		mov		rcx, [rax].ABI._num
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 1
		call	CreateBD
		mov		_bdone, rax
		mov		[rax].ABD._sign, 0
		mov		rcx, [rax].ABD._num
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 1
		call	CreateBF
		mov		_bfone, rax
		mov		[rax].ABF._sign, 0
		mov		rcx, [rax].ABF._numerator
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 1
		call	CreateBI
		mov		_bifive, rax
		mov		[rax].ABI._sign, 0
		mov		rcx, [rax].ABI._num
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 5
		call	CreateBD
		mov		_bdfive, rax
		mov		[rax].ABD._sign, 0
		mov		rcx, [rax].ABD._num
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 5
		call	CreateBF
		mov		_bffive, rax
		mov		[rax].ABF._sign, 0
		mov		rcx, [rax].ABF._numerator
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 5
		call	CreateBI
		mov		_biten, rax
		mov		[rax].ABI._sign, 0
		mov		rcx, [rax].ABI._num
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 10
		call	CreateBD
		mov		_bdten, rax
		mov		[rax].ABD._sign, 0
		mov		rcx, [rax].ABD._num
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 10
		call	CreateBF
		mov		_bften, rax
		mov		[rax].ABF._sign, 0
		mov		rcx, [rax].ABF._numerator
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 10
		call	CreateBI
		mov		_bitemp, rax
		mov		[rax].ABI._sign, 0
		mov		rcx, [rax].ABI._num
		mov		qword ptr[rcx], 1
		mov		qword ptr[rcx+8], 0
		call	CreateBD
		mov		_bdtemp, rax
		call	CreateBF
		mov		_bftemp, rax
		mov		rcx, 64
		call	malloc
		mov		_t3, rax
		mov		qword ptr[rax], 6
		mov		rbx, 23025851 
		mov		qword ptr[rax+8], rbx
		mov		rcx, rax
		call	CreateBDTake
		mov		_bdln, rax
		mov		[rax].ABD._decimalplace, 7
		mov		_bdln, rax

		EXITFUNC _one, _one, _one, _one, _one, _one, _one, _one, _one, _one, _zero, _zero, _zero, _zero, _zero

		RET

	InitVals ENDP

		
	end
