/**
 * \file BI.h
 * \brief This file contains the native and the managed versions of the classes as well as the math functions.
 *
 * The native classes are used internally in the managed classes as a container for the values they contain.  The managed classes call the assembly language functions directly.  The native classes are passed
 * directly into the assembly functions, which understand them fine, also, the return values from the assembly routines can be handled on the native side as if they were created with the new 
 * operator (in other words they are destroyed without any problems)
 *
 */


#pragma once
#include "stdafx.h"
#include <string.h>
#include "stdlib.h"
#include <cstdlib>
#include <windows.h>
#include <memory>
#include <stdio.h>
#include <conio.h>
#include <iostream>
#include <string>
#include <tchar.h>
#include <vector>
#include <fstream>
#include <iomanip>
#include <locale>
#include <sstream>
#include <assert.h>
#include <algorithm> 

// This is the value of PI
#define CPI				"3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679"
// This is the value of the natural log base
#define CE				"2.7182818284590452353602874713526624977572470936999595749669676277240766303535475945713821785251664274"
// This is the value to multiply degrees by to convert to Radians
#define CTORAD			"0.0174532927777777777777777777777777778"
// This is the value of sqr(2)
#define CSQR2			"1.41421356237309504880168872420969807857"
// This is the value of PHI
#define CPHI			"1.618033988749894848204586834365638117720"
// This is the value of Lambda
#define CLAMBDA			"0.577215664901532860606512090082402431042"
// this is the value of the golden ratio
#define CGOLDENRATIO	"1.6180339887498948482045868343656381177203091798057628621354486227052604628189024497072072041893911374"
// the is the value 2-sqr(3)
#define C2SQR3			"3.4641016151377545870548926830117"
// This is the value of sqr(3)
#define CSQR3			"1.7320508075688772935274463415059"

#pragma warning(disable:4101 4244)
#ifdef BASE_EXPORTS
#define BASE_API __declspec(dllexport)
#else
#define BASE_API __declspec(dllimport)
#endif

using namespace System;
using namespace System::Text;
using namespace System::IO;
using namespace System::Threading;
using namespace System::Collections;
using namespace System::Numerics;
using namespace System::Windows::Forms;
using namespace System::Runtime::InteropServices;

delegate void SetLabelTextDelegate(System::Windows::Forms::Label^ _label,String^ text);

#pragma managed (push, off)

class CBI;
class CBD;
class CBF;
class CBDI;
class CBFI;

#if _WIN64 || __amd64__ || _x64

#define MYWORD		unsigned __int64
#define MYWORDPTR	unsigned __int64*
#define MYBITS		64
#define MYHIGHBIT	63
#define MYBYTES		8
#define BASE		18446744073709551616
#define MAX			18446744073709551615
#define SQRBASE		4294967296

	extern "C"  MYWORD GetMemCount(void);
	extern "C"  MYWORDPTR BIEuclideanAlg64(CBI*, CBI*);

	extern "C"  MYWORDPTR AMultiply642(MYWORDPTR, MYWORDPTR);

	extern "C"  MYWORDPTR AAnd64(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR AOr64(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR AXor64(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR AAdd64(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR ASub64(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR AMultiply64(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR ADivide64(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR AMod64(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR APow64(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR AModPow64(MYWORDPTR, MYWORDPTR, MYWORDPTR);
	extern "C"  int AE64(MYWORDPTR, MYWORDPTR);
	extern "C"  int ANE64(MYWORDPTR, MYWORDPTR);
	extern "C"  int ALT64(MYWORDPTR, MYWORDPTR);
	extern "C"  int ALTOE64(MYWORDPTR, MYWORDPTR);
	extern "C"  int AGT64(MYWORDPTR, MYWORDPTR);
	extern "C"  int AGTOE64(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR ABitShiftLeft64(MYWORDPTR, MYWORD);
	extern "C"  MYWORDPTR ABitShiftRight64(MYWORDPTR, MYWORD);
	extern "C"  MYWORDPTR AByteShiftLeft64(MYWORDPTR, MYWORD);
	extern "C"  MYWORDPTR AByteShiftRight64(MYWORDPTR, MYWORD);

	extern "C"  MYWORDPTR AAdd64e(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR ASub64e(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR AMultiply64e(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR ADivide64e(MYWORDPTR, MYWORDPTR);	
	extern "C"  MYWORDPTR AMod64e(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR APow64e(MYWORDPTR, MYWORDPTR);
	extern "C"  MYWORDPTR AModPow64e(MYWORDPTR, MYWORDPTR, MYWORDPTR);
	extern "C"  int AE64e(MYWORDPTR, MYWORDPTR);
	extern "C"  int ANE64e(MYWORDPTR, MYWORDPTR);
	extern "C"  int ALT64e(MYWORDPTR, MYWORDPTR);
	extern "C"  int ALTOE64e(MYWORDPTR, MYWORDPTR);
	extern "C"  int AGT64e(MYWORDPTR, MYWORDPTR);
	extern "C"  int AGTOE64e(MYWORDPTR, MYWORDPTR);

	extern "C"  CBI* BIAdd64(CBI*, CBI*);
	extern "C"  CBI* BISub64(CBI*, CBI*);
	extern "C"  CBI* BIMultiply64(CBI*, CBI*);
	extern "C"  CBI* BIDivide64(CBI*, CBI*);
	extern "C"  CBI* BIMod64(CBI*, CBI*);
	extern "C"  CBI* BIPow64(CBI*, CBI*);
	extern "C"  CBI* BIModPow64(CBI*, CBI*, CBI*);
	extern "C"  CBI* BILeftBitShift64(CBI*, MYWORD);
	extern "C"  CBI* BIRightBitShift64(CBI*, MYWORD);
	extern "C"  CBI* BILeftByteShift64(CBI*, MYWORD);
	extern "C"  CBI* BIRightByteShift64(CBI*, MYWORD);
	extern "C"  int BIE64(CBI*, CBI*);
	extern "C"  int BINE64(CBI*, CBI*);
	extern "C"  int BILT64(CBI*, CBI*);
	extern "C"  int BILTOE64(CBI*, CBI*);
	extern "C"  int BIGT64(CBI*, CBI*);
	extern "C"  int BIGTOE64(CBI*, CBI*);

	extern "C"  CBD* BDAdd64(CBD*, CBD*);
	extern "C"  CBD* BDSub64(CBD*, CBD*);
	extern "C"  CBD* BDMultiply64(CBD*, CBD*);
	extern "C"  CBD* BDDivide64(CBD*, CBD*);
	extern "C"  CBD* BDMod64(CBD*, CBD*);
	extern "C"  CBD* BDPow64(CBD*, CBD*);
	extern "C"  CBD* BDModPow64(CBD*, CBD*, CBD*);
	extern "C"  int BDE64(CBD*, CBD*);
	extern "C"  int BDNE64(CBD*, CBD*);
	extern "C"  int BDLT64(CBD*, CBD*);
	extern "C"  int BDLTOE64(CBD*, CBD*);
	extern "C"  int BDGT64(CBD*, CBD*);
	extern "C"  int BDGTOE64(CBD*, CBD*);
	extern "C"  void BDFixTheEnd(CBD*);

	extern "C"  CBF* BFAdd64(CBF*, CBF*);
	extern "C"  CBF* BFSub64(CBF*, CBF*);
	extern "C"  CBF* BFMultiply64(CBF*, CBF*);
	extern "C"  CBF* BFDivide64(CBF*, CBF*);
	extern "C"  CBF* BFMod64(CBF*, CBF*);
	extern "C"  CBF* BFPow64(CBF*, CBF*);
	extern "C"  CBF* BFModPow64(CBF*, CBF*, CBF*);
	extern "C"  int BFE64(CBF*, CBF*);
	extern "C"  int BFNE64(CBF*, CBF*);
	extern "C"  int BFLT64(CBF*, CBF*);
	extern "C"  int BFLTOE64(CBF*, CBF*);
	extern "C"  int BFGT64(CBF*, CBF*);
	extern "C"  int BFGTOE64(CBF*, CBF*);

	extern "C"  CBDI* BDIAdd64(CBDI*, CBDI*);
	extern "C"  CBDI* BDISub64(CBDI*, CBDI*);
	extern "C"  CBDI* BDIMultiply64(CBDI*, CBDI*);
	extern "C"  CBDI* BDIDivide64(CBDI*, CBDI*);
	extern "C"  CBDI* BDIPow64(CBDI*, CBDI*);
	extern "C"  CBDI* BDIToPolar(CBDI*);
	extern "C"  CBDI* BDIConjugate(CBDI*);
	extern "C"  int BDIE64(CBDI*, CBDI*);
	extern "C"  int BDINE64(CBDI*, CBDI*);

	extern "C"  CBFI* BFIAdd64(CBFI*, CBFI*);
	extern "C"  CBFI* BFISub64(CBFI*, CBFI*);
	extern "C"  CBFI* BFIMultiply64(CBFI*, CBFI*);
	extern "C"  CBFI* BFIDivide64(CBFI*, CBFI*);
	extern "C"  CBFI* BFIPow64(CBFI*, CBFI*);
	extern "C"  int BFIE64(CBFI*, CBFI*);
	extern "C"  int BFINE64(CBFI*, CBFI*);

	extern "C" MYWORDPTR GetMem64(MYWORD);
	extern "C" MYWORDPTR Free64(MYWORDPTR);
	extern "C"  void BDChangeHowToHandleRounding(MYWORD);
	extern "C"  void BDChangeDecimalLimit(MYWORD);
	extern "C"  MYWORD BDGetHowToHandleRounding(void);
	extern "C"  MYWORD BDGetDecimalLimit(void);
	extern "C"  MYWORD GetMalloc(void);
	extern "C" void UseMalloc(MYWORD);
	extern "C"  void SetNumberOfAllocations(MYWORD);
	extern "C"  void SetAllocationSize(MYWORD);
	extern "C"  void SetResetCount(MYWORD);
	extern "C"  MYWORD GetAllocationSize(void);
	extern "C"  MYWORD GetTableReqSize(void);
	extern "C"  MYWORD GetAllocReqSize(void);
	extern "C"  MYWORD GetResetCount(void);
	extern "C"  MYWORD GetNumberOfAllocations(void);
	extern "C"  void Initialize(void);

	extern "C"  CBI* BIGCD64(CBI*, CBI*);
	extern "C"  CBI* BILCM64(CBI*, CBI*);
	extern "C"  CBD* BDGCD64(CBD*, CBD*);
	extern "C"  CBD* BDLCM64(CBD*, CBD*);
	extern "C"  CBF* BFGCD64(CBF*, CBF*);
	extern "C"  CBF* BFLCM64(CBF*, CBF*);
	extern "C"  MYWORDPTR BIEuclideanAlg64(CBI*, CBI*);
	extern "C"  MYWORDPTR AsmFactorial(MYWORDPTR);
	extern "C"  CBI* BIFactorial64(CBI*);
	extern "C"  CBD* BDFactorial64(CBD*);
	extern "C"  CBF* BFFactorial64(CBF*);
	extern "C"  CBI* BISqrt64(CBI*, MYWORD);
	extern "C"  CBD* BDSqrt64(CBD*);
	extern "C"  CBF* BFSqrt64(CBF*, MYWORD);
	extern "C"  CBDI* BDISqrt64(CBDI*);
	extern "C"  CBFI* BFISqrt64(CBFI*, MYWORD);
	extern "C"  CBI* BINthRoot64(CBI*, CBI*, MYWORD);
	extern "C"  CBD* BDNthRoot64(CBD*, CBD*);
	extern "C"  CBF* BFNthRoot64(CBF*, CBF*, MYWORD);
	extern "C"  CBDI* BDINthRoot64(CBDI*, CBD*);
	extern "C"  CBFI* BFINthRoot64(CBFI*, CBF*);

	extern "C"  CBD* BDSin64(CBD*);
	extern "C"  CBF* BFSin64(CBF*, MYWORD);
	extern "C"  CBD* BDCos64(CBD*);
	extern "C"  CBF* BFCos64(CBF*, MYWORD);
	extern "C"  CBD* BDTan64(CBD*);
	extern "C"  CBF* BFTan64(CBF*);
	extern "C"  CBD* BDASin64(CBD*, MYWORD);
	extern "C"  CBF* BFASin64(CBF*, MYWORD);
	extern "C"  CBD* BDACos64(CBD*, MYWORD);
	extern "C"  CBF* BFACos64(CBF*, MYWORD);
	extern "C"  CBD* BDATan64(CBD*, MYWORD);
	extern "C"  CBF* BFATan64(CBF*, MYWORD);
	extern "C"  CBD* BDln64(CBD*, MYWORD);
	extern "C"  CBF* BFln64(CBF*, MYWORD);
	extern "C"  CBD* BDLog64(CBD*, MYWORD);
	extern "C"  CBF* BFLog64(CBF*, MYWORD);
	extern "C"  CBD* BDExp64(CBD*, MYWORD);
	extern "C"  CBF* BFExp64(CBF*, MYWORD);
	extern "C"  CBD* BDPower64(CBD*, CBD*, MYWORD);
	extern "C"  CBF* BFPower64(CBF*, CBF*, MYWORD);

	extern "C"  char* AToString64(MYWORDPTR, MYWORD);
	extern "C"  char* BIToString64(CBI*, MYWORD);
	extern "C"  char* BDToString64(CBD*, MYWORD);
	extern "C"  char* BFToString64(CBF*, MYWORD);
	extern "C"  char* BDIToString64(CBDI*, MYWORD);

	extern "C"  CBI* BFItoBI(CBFI*);
	extern "C"  CBD* BFItoBD(CBFI*);
	extern "C"  CBF* BFItoBF(CBFI*);
	extern "C"  CBDI* BFItoBDI(CBFI*);

	extern "C"  CBI* BDItoBI(CBDI*);
	extern "C"  CBD* BDItoBD(CBDI*);
	extern "C"  CBF* BDItoBF(CBDI*);
	extern "C"  CBFI* BDItoBFI(CBDI*);

	extern "C"  CBI* BFtoBI(CBF*);
	extern "C"  CBD* BFtoBD(CBF*);
	extern "C"  CBDI* BFtoBDI(CBF*);
	extern "C"  CBFI* BFtoBFI(CBF*);

	extern "C"  CBI* BDtoBI(CBD*);
	extern "C"  CBF* BDtoBF(CBD*);
	extern "C"  CBDI* BDtoBDI(CBD*);
	extern "C"  CBFI* BDtoBFI(CBD*);

	extern "C"  CBD* BItoBD(CBI*);
	extern "C"  CBF* BItoBF(CBI*);
	extern "C"  CBDI* BItoBDI(CBI*);
	extern "C"  CBFI* BItoBFI(CBI*);

	extern "C"  CBD* BDAdd64e(CBD*, CBD*, MYWORD);

	#define SFree			Free64
	#define SGetMem			GetMem64
	#define UseTheMalloc	UseMalloc

	#define SAdd			AAdd64
	#define SSub			ASub64
	#define SMul			AMultiply64
	#define SDiv			ADivide64
	#define SMod			AMod64
	#define SPow			APow64
	#define SModPow			AModPow64
	#define SE				AE64
	#define SNE				ANE64
	#define SGT				AGT64
	#define SLT				ALT64
	#define SGTOE			AGTOE64
	#define SLTOE			ALTOE64

	#define SAddE			AAdd64e
	#define SSubE			ASub64e
	#define SMulE			AMultiply64e
	#define SDivE			ADivide64e
	#define SModE			AMod64e
	#define SPowE			APow64e
	#define SModPowE		AModPow64e
	#define SEE				AE64e
	#define SNEE			ANE64e
	#define SGTE			AGT64e
	#define SLTE			ALT64e
	#define SGTOEE			AGTOE64e
	#define SLTOEE			ALTOE64e

#else

#define MYWORD		unsigned int
#define MYWORDPTR	unsigned int*
#define MYBITS		32
#define MYHIGHBIT	31
#define MYBYTES		4
#define BASE		4294967296
#define MAX			4294967295
#define SQRBASE		65536

	extern "C"  MYWORDPTR GetMem32(MYWORD);
	extern "C"  void Free32(MYWORDPTR);
	extern "C" void UseMalloc32(MYWORD);

	#define SGetMem			GetMem32
	#define SFree			Free32
	#define UseTheMalloc	UseMalloc32

#endif
enum TType{

	_DUMMY=0,
	_BI=10,
	_CBI=11,
	_ASMBI=15,
	_BD=20,
	_CBD=21,
	_ASMBD=25,
	_BF=30,
	_CBF=31,
	_ASMBF=35,
	_BDI=40,
	_CBDI=41,
	_ASMBDI=45,
	_BFI=50,
	_CBFI=51,
	_ASMBFI=55,
	_MYWORD=100,
	_MYWORDPTR=101,
	_BYTE=110,
	_BYTEPTR=111,
	_VARSTRUCT=200,
	_CVARSTRUCT=201,
	_ASMVARSTRUCT=205,
	_ALGSTATE=210,
	_CALGSTATE=211,
	_ASMALGSTATE=215,
	_DEBUGENTRY=220,
	_CDEBUGENTRY=221,
	_ASMDEBUGENTRY=225,
	_DEBUGSTACK=230,
	_CDEBUGSTACK=231,
	_ASMDEBUGSTACK=235

};

static TType _stype=TType::_DUMMY;
//!This class is the native version of the BI class, as well as the container for all the BI algorithms in assembly language.
/*!
this implementation holds the value in an array of wordlength elements (64 bit on 64 bit systems otherwise 32 bit)
The element at position 0 in the array holds the element count in the array (using sizeof will give incorrect results)
There is also a _sign property of type MYWORD 1==Negative, 0==Positive
*/
public class BASE_API CBI{

	private:

		//!This is the array that contains the value for this instance.  Element 0 in this array holds the number of elements being used in this instance.
		MYWORD _sign;
		//!This is the Sign of the instance, 0==Positive and 1==Negative.
		MYWORDPTR _num;

	public:

		CBI(bool _ts, MYWORDPTR _val, bool _ts2){

			if(_ts2){

				_sign=1;

			}

			else{

				_sign=0;

			}

			_num=_val;

		}

		//!CTOR.
		CBI(){

			int _mb=MYBYTES;

			_num=SGetMem(2*_mb);

			_sign=0;
			_num[0]=1;
			_num[1]=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		\param _ts The value to set the sign to.
		*/
		CBI(MYWORDPTR _val, bool _ts){

			_num=SGetMem((_val[0]+1)*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			if(_ts){

				_sign=1;

			}

			else{

				_sign=0;

			}

			return;

		}
		//!CTOR.
		/*!
		\param _vn The value to set the value to.
		\param _vs The value to set the sign to.
		*/
		CBI(MYWORDPTR _vn, int _vs){

			_num=SGetMem((_vn[0]+1)*MYBYTES);

			for(int x=0;x<=_vn[0];x++){

				_num[x]=_vn[x];

			}

			_sign=_vs;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(MYWORDPTR _val){

			_num=SGetMem((_val[0]+1)*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			_sign=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(short _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;

			short _tval=_val;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

				_sign=0;

			}

			_num[1]=_tval;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(unsigned short _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;
			_num[1]=_val;

			_sign=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(int _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;

			int _tval=_val;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

				_sign=0;

			}

			_num[1]=_tval;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(unsigned int _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;
			_num[1]=_val;

			_sign=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(__int64 _val)
		{

			__int64 _tval=_val;
			_sign=0;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

			}

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(unsigned __int64 _val)
		{

			unsigned __int64 _tval=_val;

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			_sign=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(char _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;

			int _tval=_val;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

				_sign=0;

			}

			_num[1]=_tval;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(unsigned char _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;
			_num[1]=_val;

			_sign=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(long _val)
		{

			__int64 _tval=_val;
			_sign=0;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

			}

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(unsigned long _val)
		{

			unsigned __int64 _tval=_val;

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			_sign=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBI(CBI* _val){

			_num=SGetMem((_val->_num[0]+1)*MYBYTES);

			for(int x=0;x<=_val->_num[0];x++){

				_num[x]=_val->_num[x];

			}

			_sign=_val->_sign;

			return;

		}
		void operator=(CBI* _val){

			SFree(_num);

			_num=SGetMem((_val->_num[0]+1)*MYBYTES);

			for(int x=0;x<=_val->_num[0];x++){

				_num[x]=_val->_num[x];

			}

			_sign=_val->_sign;

			return;

		}
		//!Sets the instance equal to one
		void SetToOne(){

			_num[0]=1;
			_num[1]=1;
			_sign=0;

			return;

		}
		//!Sets the instance equal to zero
		void SetToZero(){

			_num[0]=1;
			_num[1]=0;
			_sign=0;

			return;

		}
		void Resize(int _size){

			SFree(_num);

			_num=SGetMem(_size);

			_num[0]=_size/MYBYTES;

			if(_num[0]==0){

				_num[0]=1;

			}

			else{

				for(int x=1;x<=_num[0];x++){

					_num[x]=0;

				}

			}

			return;

		}
		//!Sets the sign of the value.
		/*!
		\param _newsign The value to set.
		*/
		void SetSign(MYWORD _newsign){

			if(_newsign>0){

				_sign=1;

			}

			else{

				_sign=0;

			}

			return;

		}
		//!Gets the sign of the value.
		/*!
		\returns A MYWORD
		*/
		MYWORD GetSign(){

			return _sign;

		}
		//!Sets the value of a particular element of the array.
		/*!
		\param _val The value to set.
		\param _pos The position in the array.
		*/
		void SetValue(int _pos, MYWORD _val){

			if(_pos>=0 && _pos<=_num[0]){

				_num[_pos]=_val;

			}

			return;

		}
		//!Gets the value of a particular element of the array.
		/*!
		\param _pos The position in the array.
		\returns A MYWORD
		*/
		MYWORD GetValue(int _pos){

			if(_pos>=0 && _pos<=_num[0]){

				return _num[_pos];

			}

			return 0;

		}
		//!Sets the array.
		/*!
		\param _val The array to set.
		*/
		void SetArray(MYWORDPTR _val){

			SFree(_num);

			_num=SGetMem((_val[0]+1)*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			return;

		}
		//!Gets the array.
		/*!
		\returns A MYWORDPTR object
		*/
		MYWORDPTR GetArray(){

			return _num;

		}
		//!Replaces the current value with the new value then deletes the old.
		/*!
		\param _val The value to set.
		*/
		void ReplaceAndDelete(CBI* _val){

			if(_num[0]<_val->_num[0]){

				SFree(_num);
				_num=SGetMem((_val->_num[0]+1)*MYBYTES);

			}

			for(int x=0;x<=_val->_num[0];x++){

				_num[x]=_val->_num[x];

			}

			_sign=_val->_sign;

			delete _val;

			return;

		}
		//!Takes ownership of the array.
		/*!
		\param _val The array.
		*/
		void TakeArray(MYWORDPTR _val){

			SFree(_num);

			_num=_val;

		}
		//!Creates an instance of CBI.
		/*!
		\returns A CBI object
		*/
		static CBI* Create(){

			CBI* _retval=new CBI();

			return _retval;

		}
		//!Creates an instance of CBI.
		/*!
		\param _val The value to copy.
		\returns A CBI object
		*/
		static CBI* CreateFromArrayTake(MYWORDPTR _val){

			CBI* _retval=new CBI(true, _val, true);

			return _retval;

		}
		//!Creates an instance of CBI.
		/*!
		\param _val The value to copy.
		\returns A CBI object
		*/
		static CBI* CreateFromArrayCopy(MYWORDPTR _val){

			CBI* _retval=new CBI(_val, false);

			return _retval;

		}
		//!Creates an instance of CBI.
		/*!
		\param _val The value to copy.
		\param _si The sign to copy.
		\returns A CBI object
		*/
		static CBI* CreateFromArrayAndSign(MYWORDPTR _val, unsigned __int64 _si){

			CBI* _retval;

			if(_si==0){

				_retval=new CBI(_val, false);

			}

			else{

				_retval=new CBI(_val, true);

			}

			return _retval;

		}
		//!Creates an instance of CBI.
		/*!
		\param _val The value to copy.
		\returns A CBI object
		*/
		static CBI* CreateFromCBI(CBI* _val){

			CBI* _retval=new CBI(_val);

			return _retval;

		}
		//!Destroys an instance of CBI.
		/*!
		\param _val The value to destroy.
		*/
		static void Destroy(CBI* _val){

			delete _val;

		}
		//!Takes ownership of the value.
		/*!
		\param _val The value to set.
		*/
		void Take(CBI* _val){

			//SFree(_num);

			//_num=_val->_num;
			//_sign=_val->_sign;

			MYWORDPTR _temp=_num;
			_num=_val->_num;
			_sign=_val->_sign;
			_val->_num=_temp;

			delete _val;

		}
		char* ToString(){

			char* _temp=BIToString64(this, 10);
			//char* _temp;

			return _temp;

		}
		char* ToString(MYWORD _radix){

			char* _temp=BIToString64(this, 10);
			//char* _temp;

			return _temp;

		}
		~CBI(){

			if(_num!=0){

				_num=SFree(_num);

			}

		}
		CBI* operator+(CBI _val){

			CBI* _temp=BIAdd64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBI* operator+=(CBI _val){

			return operator+(_val);

		}
		CBI* operator-(CBI _val){

			CBI* _temp=BISub64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBI* operator-=(CBI _val){

			return operator-(_val);

		}
		CBI* operator*(CBI _val){

			CBI* _temp=BIMultiply64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBI* operator*=(CBI _val){

			return operator*(_val);

		}
		CBI* operator/(CBI _val){

			CBI* _temp=BIDivide64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBI* operator/=(CBI _val){

			return operator/(_val);

		}
		CBI* operator%(CBI _val){

			CBI* _temp=BIMod64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBI* operator%=(CBI _val){

			return operator%(_val);

		}
		CBI* operator&(CBI _val){

			MYWORDPTR _temp=AAnd64(_num, _val._num);

			if(_num[0]<_temp[0]){

				SFree(_num);

				_num=SGetMem((_temp[0]+1)*8);

			}

			for(int x=0;x<=_temp[0];x++){

				_num[0]=_temp[0];

			}

			SFree(_temp);

			return this;

		}
		CBI* operator|(CBI _val){

			MYWORDPTR _temp=AOr64(_num, _val._num);

			if(_num[0]<_temp[0]){

				SFree(_num);

				_num=SGetMem((_temp[0]+1)*8);

			}

			for(int x=0;x<=_temp[0];x++){

				_num[0]=_temp[0];

			}

			SFree(_temp);

			return this;

		}
		CBI* operator^(CBI _val){

			MYWORDPTR _temp=AXor64(_num, _val._num);

			if(_num[0]<_temp[0]){

				SFree(_num);

				_num=SGetMem((_temp[0]+1)*8);

			}

			for(int x=0;x<=_temp[0];x++){

				_num[0]=_temp[0];

			}

			SFree(_temp);

			return this;

		}
		CBI* operator>>(MYWORD _val){

			MYWORDPTR _temp=ABitShiftRight64(_num, _val);
		
			if(_num[0]<_temp[0]){

				SFree(_num);

				_num=SGetMem((_temp[0]+1)*8);

			}

			for(int x=0;x<=_temp[0];x++){

				_num[0]=_temp[0];

			}

			SFree(_temp);

			return this;

		}
		CBI* operator<<(MYWORD _val){

			MYWORDPTR _temp=ABitShiftLeft64(_num, _val);

			if(_num[0]<_temp[0]){

				SFree(_num);

				_num=SGetMem((_temp[0]+1)*8);

			}

			for(int x=0;x<=_temp[0];x++){

				_num[0]=_temp[0];

			}

			SFree(_temp);

			return this;

		}
		bool operator==(CBI _val){

			if(BIE64(this, &_val)==1){

				return true;

			}

			return false;

		}
		bool operator!=(CBI _val){

			return !(operator==(_val));

		}
		bool operator<(CBI _val){

			if(BILT64(this, &_val)==1){

				return true;

			}

			return false;

		}
		bool operator<=(CBI _val){

			return !(_val<this);

		}
		bool operator>(CBI _val){

			return _val<this;

		}
		bool operator>=(CBI _val){

			return !(operator<(_val));

		}
		static CBI* Pow(CBI _base, CBI _exp){

			CBI* _retval=BIPow64(&_base, &_exp);

			return _retval;

		}
		static CBI* ModPow(CBI _base, CBI _exp, CBI _mod){

			CBI* _retval=BIModPow64(&_base, &_exp, &_mod);

			return _retval;

		}
		//!Explicit cast from CBI.
		/*!
		\returns A char
		*/
		operator char(){

			char _temp=0;

			if(_num[0]==1 && _num[1]<256){

				_temp=(char)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBI.
		/*!
		\returns A unsigned char
		*/
		operator unsigned char(){

			if(_num[0]==1 && _num[1]<256){

				return (unsigned char)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBI.
		/*!
		\returns A short
		*/
		operator short(){

			short _temp=0;

			if(_num[0]==1 && _num[1]<65536){

				_temp=(short)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBI.
		/*!
		\returns A unsigned short
		*/
		operator unsigned short(){

			if(_num[0]==1 && _num[1]<65536){

				return (unsigned short)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBI.
		/*!
		\returns A int
		*/
		operator int(){

			int _temp=0;

			if(_num[0]==1 && _num[1]<4294967296){

				_temp=(int)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBI.
		/*!
		\returns A unsigned int
		*/
		operator unsigned int(){

			if(_num[0]==1 && _num[1]<4294967296){

				return (int)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBI.
		/*!
		\returns A long long
		*/
		operator long long(){

			long long _temp=0;

			if(_num[0]==1 && _num[1]<18446744073709551615){

				_temp=(long long)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBI.
		/*!
		\returns A unsigned long long
		*/
		operator unsigned long long(){

			if(_num[0]==1 && _num[1]<18446744073709551615){

				return (unsigned long long)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBI.
		/*!
		\returns A float
		*/
		operator float(){

			float _retval=0.0;
			float _base=1.8446744073709551616;
			float _digit=0.0;
			int _basecount=20;

			if(MYBYTES==4){

				_base=4.294967296;
				_basecount=10;

			}


			for(int x=1;x<20;x++){

				_base=_base*10;

			}

			for(int x=1;x<=_num[0];x++){

				_digit=_base;

				for(int y=1;y<_num[0];y++){

					_digit=_digit*_digit;

				}

				_digit=_digit*_num[x];

				_retval=_retval+_digit;

			}

			return _retval;

		}
		//!Explicit cast from CBI.
		/*!
		\returns A double
		*/
		operator double(){

			double _retval=0.0;
			double _base=1.8446744073709551616;
			double _digit=0.0;
			int _basecount=20;

			if(MYBYTES==4){

				_base=4.294967296;
				_basecount=10;

			}


			for(int x=1;x<20;x++){

				_base=_base*10;

			}

			for(int x=1;x<=_num[0];x++){

				_digit=_base;

				for(int y=1;y<_num[0];y++){

					_digit=_digit*_digit;

				}

				_digit=_digit*_num[x];

				_retval=_retval+_digit;

			}

			return _retval;

		}
		//!Explicit cast from CBI.
		/*!
		\returns A CBD object
		*/
		operator CBD*();
		//!Explicit cast from CBI.
		/*!
		\returns A CBF object
		*/
		operator CBF*();
		//!Explicit cast from CBI.
		/*!
		\returns A CBDI object
		*/
		operator CBDI*();
		//!Explicit cast from CBI.
		/*!
		\returns A CBFI object
		*/
		operator CBFI*();
		//!Gives the absolute value of the instance
		/*!
		\returns A CBI* object
		*/
		CBI* ABS(){

			CBI* _retval=new CBI(_num);

			_retval->_sign=0;

			return _retval;

		}
		//!Returns the Least Common Multiple of the 2 instances
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A CBI* object
		*/
		static CBI* LCM(CBI _val1, CBI _val2){

			CBI* _retval=new CBI();
			CBI* _temp=GCD(_val1, _val2);

			_retval=_val1*_val2;

			_retval=*_retval/_temp;

			delete _temp;

			return _retval;

		}
		//!Returns the Greatest Common Divisor of the 2 instances
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A CBI* object
		*/
		static CBI* GCD(CBI _val1, CBI _val2){

			CBI* _a;
			CBI* _b;

			if(_val1>=_val2){

				_a=new CBI(_val1);
				_b=new CBI(_val2);

			}

			else{

				_a=new CBI(_val2);
				_b=new CBI(_val1);

			}

			CBI* _r=new CBI();

			while(_b>0){

				_r=*_a%*_b;
				_a=_b;
				_b=_r;

			}

			delete _b;
			delete _r;

			return _a;

		}

};

//!This class is the native version of the BD class, as well as the container for all the BD algorithms in assembly language
/*!
this implementation holds the value in an array of wordlength elements (64 bit on 64 bit systems otherwise 32 bit)
The element at position 0 in the array holds the element count in the array (using sizeof will give incorrect results)
the number is represented in the array as if it were an integer, the decimalplace property (of type MYWORD) keeps track of where it goes.
There is also a _sign property of type MYWORD 1==Negative, 0==Positive, and lastly there is an adjustment property of type MYWORD
The adjustment property is to keep the rounding errors to a minimum.  What it does is temporarily raises the decimal limit (for this instance only)
so that you can run the object through a series of calculations without accumalating massive rounding errors.  The idea is to set the adjustment
high enough so that the rounding errors never reach your resolution, in practice, raising it by 3 places will typically allow a few hundred operations
without loosing precision.  Once the object is out of its set of calculations, you should set it back to 0 and run the MakeFinal() method
*/
public class BASE_API CBD{

	private:

		//!This is the sign for this instance, 0==Positive and 1==Negative
		MYWORD _sign;
		//!This keeps track of where the decimalplace is for this instance.
		MYWORD _decimalplace;
		//!This is the array that contains the value for this instance.  Element 0 contains the number of elements being used for this instance.  The value is held as an integer and has no concept of the decimal place.
		MYWORDPTR _num;
		//!This is for use for when you have a bunch of operations to perform on this instance.  It is here to help keep the rounding errors to a minimum.
		/*!
		     The way to correctly use this variable, before you begin a series of calculations that will use this instance, you set this variable to how many extra digits youd like it to keep for the 
			 decimal place.  The problem that occurs whenever using this class (or float or double) is that if you run a series of calculations at a certain decimal limit, the rounding errors will start
			 to accumalate.  This can be avoided with a little thought.  First calculate how many extra digits youd need to keep in order to prevent the rounding errors, there are two ways.

				1:	If you count it as if every single operation is going to cause a rounding error, then you simply count the number of operations and use the base 10 log.  In other words, if you figure on
				100 calculations, you would set the _adj variable to 2.  This would tell the program to keep 2 extra digits to the right of the decimal place and would keep the instance precise at the decimal limit.

				2:  If you cant averages, your thoughts should be along the lines of half the time the class will round up just because the digit at decimal limit+1 is greater than 4.  So if you only had
				10 operations you could increase the decimal limit for this instance by 1 (that is figuring the current instance has a value of 5, which is the average, at decimallimit + 1, so if 1/2
				the time it is rounding up it would take an average of 10 operations to cause decimalimit+1 to round up enough times that it would affect the decimallimit digit).  With this thinking in mind
				if you set the _adj variable to 2, it should take an average of 20 * 10 or 200 calculations before the decimal limit digit is affected, setting it to 3 should give you about 2000 calculations
				and so on.
		*/
		MYWORD _adj;


	public:

		CBD(bool _ts, MYWORDPTR _val, bool _ts2){

			if(_ts2){

				_sign=1;

			}

			else{

				_sign=0;

			}

			_num=_val;
			_decimalplace=0;
			_adj=0;

		}

		//!CTOR.
		CBD(){

			_num=SGetMem(2*MYBYTES);

			_sign=0;
			_decimalplace=0;
			_num[0]=1;
			_num[1]=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		\param _ts The value to set the sign to.
		*/
		CBD(MYWORDPTR _val, bool _ts){

			_num=SGetMem((_val[0]+1)*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			if(_ts){

				_sign=1;

			}

			else{

				_sign=0;

			}

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _vn The value to set the value to.
		\param _vs The value to set the sign to.
		*/
		CBD(MYWORDPTR _vn, int _vs){

			_num=SGetMem((_vn[0]+1)*MYBYTES);

			for(int x=0;x<=_vn[0];x++){

				_num[x]=_vn[x];

			}

			_sign=_vs;

			_decimalplace=0;
			_adj=0;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(MYWORDPTR _val){

			_num=SGetMem((_val[0]+1)*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			_sign=0;

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(short _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;

			short _tval=_val;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

				_sign=0;

			}

			_num[1]=_tval;

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(unsigned short _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;
			_num[1]=_val;

			_sign=0;

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(int _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;

			int _tval=_val;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

				_sign=0;

			}

			_num[1]=_tval;

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(unsigned int _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;
			_num[1]=_val;

			_sign=0;

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(__int64 _val)
		{

			__int64 _tval=_val;
			_sign=0;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

			}

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(unsigned __int64 _val)
		{

			unsigned __int64 _tval=_val;

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			_sign=0;

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(char _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;

			int _tval=_val;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

				_sign=0;

			}

			_num[1]=_tval;

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(unsigned char _val)
		{

			_num=SGetMem(3*MYBYTES);

			_num[0]=1;
			_num[1]=_val;

			_sign=0;

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(long _val)
		{

			__int64 _tval=_val;
			_sign=0;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

			}

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(unsigned long _val)
		{

			unsigned __int64 _tval=_val;

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			_sign=0;

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(CBD* _val){

			_num=SGetMem((_val->_num[0]+1)*MYBYTES);

			for(int x=0;x<=_val->_num[0];x++){

				_num[x]=_val->_num[x];

			}

			_sign=_val->_sign;

			_decimalplace=_val->_decimalplace;
			_adj=_val->_adj;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBD(CBI* _val){

			_num=SGetMem((_val->GetValue(0)+1)*MYBYTES);

			for(int x=0;x<=_val->GetValue(0);x++){

				_num[x]=_val->GetValue(x);

			}

			_sign=_val->GetSign();

			_decimalplace=0;
			_adj=0;

			return;

		}
		//!Assignment.
		/*!
		\param _val The value to assign.
		*/
		void operator=(CBD* _val){

			SFree(_num);

			_num=SGetMem((_val->_num[0]+1)*MYBYTES);

			for(int x=0;x<=_val->_num[0];x++){

				_num[x]=_val->_num[x];

			}

			_sign=_val->_sign;

			_decimalplace=_val->_decimalplace;
			_adj=_val->_adj;

			return;

		}
		//!Sets the current instance equal to one.
		void SetToOne(){

			_num[0]=1;
			_num[1]=1;
			_sign=0;
			_decimalplace=0;
			return;

		}
		//!Sets the current instance equal to zero.
		void SetToZero(){

			_num[0]=1;
			_num[1]=0;
			_sign=0;
			_decimalplace=0;

			return;

		}
		void Resize(int _size){

			SFree(_num);

			_num=SGetMem(_size);

			_num[0]=_size/MYBYTES;

			if(_num[0]==0){

				_num[0]=1;

			}

			else{

				for(int x=1;x<=_num[0];x++){

					_num[x]=0;

				}

			}

			return;

		}
		//!Sets the sign of the instance.
		/*!
		\param _newsign The value to set the sign.
		*/
		void SetSign(MYWORD _newsign){

			if(_newsign>0){

				_sign=1;

			}

			else{

				_sign=0;

			}

			return;

		}
		//!Gets the sign of the instance.
		/*!
		\returns A MYWORD
		*/
		MYWORD GetSign(){

			return _sign;

		}
		//!Sets the value of a particular element of the array.
		/*!
		\param _val The value to set.
		\param _pos The position in the array.
		*/
		void SetValue(int _pos, MYWORD _val){

			if(_pos>=0 && _pos<=_num[0]){

				_num[_pos]=_val;

			}

			return;

		}
		//!Gets the value of a particular element of the array.
		/*!
		\param _pos The position in the array.
		\returns A MYWORD
		*/
		MYWORD GetValue(int _pos){

			if(_pos>=0 && _pos<=_num[0]){

				return _num[_pos];

			}

			return 0;

		}
		//!Sets the array.
		/*!
		\param _val The array to set.
		*/
		void SetArray(MYWORDPTR _val){

			SFree(_num);

			_num=SGetMem((_val[0]+1)*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			return;

		}
		//!Gets the array.
		/*!
		\returns A MYWORDPTR object
		*/
		MYWORDPTR GetArray(){

			return _num;

		}
		//!Sets the decimal place of the instance.
		/*!
		\param _val The value to set.
		*/
		void SetDecimalPlace(MYWORD _val){

			_decimalplace=_val;

		}
		//!Gets the decimal place of the instance.
		/*!
		\returns A MYWORD
		*/
		MYWORD GetDecimalPlace(){

			return _decimalplace;

		}
		//!Gets the adjustment for the instance.
		/*!
		\returns A MYWORD
		*/
		MYWORD GetAdjustment(){

			return _adj;

		}
		//!Sets the adjustment for the instance.
		/*!
		\param _val The value to set.
		*/
		void SetAdjustment(MYWORD _val){

			_adj=_val;

		}
		//!Replaces the current values then deletes the old.
		/*!
		\param _val The value to set.
		*/
		void ReplaceAndDelete(CBD* _val){

			if(_num[0]<_val->_num[0]){

				SFree(_num);
				_num=SGetMem((_val->_num[0]+1)*MYBYTES);

			}

			for(int x=0;x<=_val->_num[0];x++){

				_num[x]=_val->_num[x];

			}

			_sign=_val->_sign;

			_decimalplace=_val->_decimalplace;
			_adj=_val->_adj;

			delete _val;

			return;

		}
		//!Takes ownership of an array.
		/*!
		\param _val The array to take.
		*/
		void TakeArray(MYWORDPTR _val){

			SFree(_num);

			_num=_val;

		}
		//!Creates an instance of CBD.
		/*!
		\returns A CBD object
		*/
		static CBD* Create(){

			CBD* _retval=new CBD();

			return _retval;

		}
		//!Creates an instance of CBD.
		/*!
		\param _val The value to copy.
		\returns A CBD object
		*/
		static CBD* CreateFromArrayTake(MYWORDPTR _val){

			CBD* _retval=new CBD(true, _val, false);

			return _retval;

		}
		//!Creates an instance of CBD.
		/*!
		\param _val The value to copy.
		\returns A CBD object
		*/
		static CBD* CreateFromArrayCopy(MYWORDPTR _val){

			CBD* _retval=new CBD(_val, false);

			return _retval;

		}
		//!Creates an instance of CBD.
		/*!
		\param _val The value to copy.
		\param _si The sign to copy.
		\returns A CBD object
		*/
		static CBD* CreateFromArrayAndSign(MYWORDPTR _val, unsigned __int64 _si){

			CBD* _retval;

			if(_si==0){

				_retval=new CBD(_val, false);

			}

			else{

				_retval=new CBD(_val, true);

			}

			return _retval;

		}
		//!Creates an instance of CBD.
		/*!
		\param _val The value to copy.
		\returns A CBD object
		*/
		static CBD* CreateFromCBD(CBD* _val){

			CBD* _retval=new CBD(_val);

			return _retval;

		}
		//!Destroys an instance of CBD.
		/*!
		\param _val The vlue to destroy.
		*/
		static void Destroy(CBD* _val){

			delete _val;

		}
		void Take(CBD* _val){

			SFree(_num);

			_num=_val->_num;
			_sign=_val->_sign;
			_decimalplace=_val->_decimalplace;

		}
		char* ToString(){

			char* _temp=BDToString64(this, 10);
			//char* _temp;

			return _temp;

		}
		char* ToString(MYWORD _radix){

			char* _temp=BDToString64(this, 10);
			//char* _temp;

			return _temp;

		}
		~CBD(){

			if(_num!=0){

				_num=SFree(_num);

			}

		}
		CBD* operator+(CBD _val){

			CBD* _temp=BDAdd64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			_decimalplace=_temp->_decimalplace;
			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBD* operator+=(CBD _val){

			return operator+(_val);

		}
		CBD* operator-(CBD _val){

			CBD* _temp=BDSub64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			_decimalplace=_temp->_decimalplace;
			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBD* operator-=(CBD _val){

			return operator-(_val);

		}
		CBD* operator*(CBD _val){

			CBD* _temp=BDMultiply64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			_decimalplace=_temp->_decimalplace;
			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBD* operator*=(CBD _val){

			return operator*(_val);

		}
		CBD* operator/(CBD _val){

			CBD* _temp=BDDivide64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			_decimalplace=_temp->_decimalplace;
			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBD* operator/=(CBD _val){

			return operator/(_val);

		}
		CBD* operator%(CBD _val){

			CBD* _temp=BDMod64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			_decimalplace=_temp->_decimalplace;
			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBD* operator%=(CBD _val){

			return operator%(_val);

		}
		bool operator==(CBD _val){

			if(BDE64(this, &_val)==1){

				return true;

			}

			return false;

		}
		bool operator!=(CBD _val){

			return !(operator==(_val));

		}
		bool operator<(CBD _val){

			if(BDLT64(this, &_val)==1){

				return true;

			}

			return false;

		}
		bool operator<=(CBD _val){

			return !(_val<this);

		}
		bool operator>(CBD _val){

			return _val<this;

		}
		bool operator>=(CBD _val){

			return !(operator<(_val));

		}
		static CBD* Pow(CBD _base, CBD _exp){

			CBD* _retval=BDPow64(&_base, &_exp);

			return _retval;

		}
		static CBD* ModPow(CBD _base, CBD _exp, CBD _mod){

			CBD* _retval=BDModPow64(&_base, &_exp, &_mod);

			return _retval;

		}
		//!Explicit cast from CBD.
		/*!
		\returns A char
		*/
		operator char(){

			char _temp=0;

			if(_num[0]==1 && _num[1]<256){

				_temp=(char)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBD.
		/*!
		\returns A unsigned char
		*/
		operator unsigned char(){

			if(_num[0]==1 && _num[1]<256){

				return (unsigned char)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBD.
		/*!
		\returns A short
		*/
		operator short(){

			short _temp=0;

			if(_num[0]==1 && _num[1]<65536){

				_temp=(short)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBD.
		/*!
		\returns A unsigned short
		*/
		operator unsigned short(){

			if(_num[0]==1 && _num[1]<65536){

				return (unsigned short)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBD.
		/*!
		\returns A int
		*/
		operator int(){

			int _temp=0;

			if(_num[0]==1 && _num[1]<4294967296){

				_temp=(int)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBD.
		/*!
		\returns A unsigned int
		*/
		operator unsigned int(){

			if(_num[0]==1 && _num[1]<4294967296){

				return (int)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBD.
		/*!
		\returns A long long
		*/
		operator long long(){

			long long _temp=0;

			if(_num[0]==1 && _num[1]<18446744073709551615){

				_temp=(long long)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBD.
		/*!
		\returns A unsigned long long
		*/
		operator unsigned long long(){

			if(_num[0]==1 && _num[1]<18446744073709551615){

				return (unsigned long long)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBD.
		/*!
		\returns A float
		*/
		operator float(){

			float _retval=0.0;
			float _base=1.8446744073709551616;
			float _digit=0.0;
			int _basecount=20;

			if(MYBYTES==4){

				_base=4.294967296;
				_basecount=10;

			}


			for(int x=1;x<20;x++){

				_base=_base*10;

			}

			for(int x=1;x<=_num[0];x++){

				_digit=_base;

				for(int y=1;y<_num[0];y++){

					_digit=_digit*_digit;

				}

				_digit=_digit*_num[x];

				_retval=_retval+_digit;

			}

			return _retval;

		}
		//!Explicit cast from CBD.
		/*!
		\returns A double
		*/
		operator double(){

			double _retval=0.0;
			double _base=1.8446744073709551616;
			double _digit=0.0;
			int _basecount=20;

			if(MYBYTES==4){

				_base=4.294967296;
				_basecount=10;

			}


			for(int x=1;x<20;x++){

				_base=_base*10;

			}

			for(int x=1;x<=_num[0];x++){

				_digit=_base;

				for(int y=1;y<_num[0];y++){

					_digit=_digit*_digit;

				}

				_digit=_digit*_num[x];

				_retval=_retval+_digit;

			}

			return _retval;

		}
		//!Explicit cast from CBD.
		/*!
		\returns A CBI object
		*/
		operator CBI*();
		//!Explicit cast from CBD.
		/*!
		\returns A CBF object
		*/
		operator CBF*();
		//!Explicit cast from CBD.
		/*!
		\returns A CBDI object
		*/
		operator CBDI*();
		//!Explicit cast from CBD.
		/*!
		\returns A CBFI object
		*/
		operator CBFI*();
		CBD* ABS(){

			CBD* _retval=new CBD(_num);

			_retval->_sign=0;

			return _retval;

		}
		static CBD* LCM(CBD _val1, CBD _val2){

			CBD* _retval=new CBD();
			CBD* _temp=GCD(_val1, _val2);

			_retval=_val1*_val2;

			_retval=*_retval/_temp;

			delete _temp;

			return _retval;

		}
		static CBD* GCD(CBD _val1, CBD _val2){

			CBD* _a;
			CBD* _b;

			if(_val1>=_val2){

				_a=new CBD(_val1);
				_b=new CBD(_val2);

			}

			else{

				_a=new CBD(_val2);
				_b=new CBD(_val1);

			}

			CBD* _r=new CBD();

			while(_b>0){

				_r=*_a%*_b;
				_a=_b;
				_b=_r;

			}

			delete _b;
			delete _r;

			return _a;

		}
		//!This methods sole purpose is to finalize the value of this instance after setting _adj value for a seriese of calculations by setting _adj=0 and calling the FixThe End routine
		void MakeFinal(){

			_adj=0;
			BDFixTheEnd(this);

		}

};
//!This class is the native version of the BF class, as well as the container for all the BF algorithms in assembly language
/*!
this implementation holds the numerator value in one array of wordlength elements (64 bit on 64 bit systems otherwise 32 bit) and the denominator in another
The element at position 0 in the arrays hold the element count in that array (using sizeof will give incorrect results)
There is also a _sign property of type MYWORD 1==Negative, 0==Positive
*/
public class BASE_API CBF{

	private:

		//!This is the sign of the instance, 0==Positive and 1==Negative.
		MYWORD _sign;
		//!This is the array that contains the value of the numerator.  The element at position 0 contains the number of elements the instance is using.
		MYWORDPTR _num;
		//!This is the array that contains the value of the denominator.  The element at position 0 contains the number of elements the instance is using.
		MYWORDPTR _den;

	public:

		//!CTOR.
		/*!
		\param _bs1 Not Used.
		\param _nn The value to set the value to.
		\param _bs2 Not Used.
		*/
		CBF(bool _bs1, MYWORDPTR _nn, MYWORDPTR _nd, bool _bs2){

			_num=_nn;
			_den=_nd;
			_sign=0;

		}
		//!CTOR.
		CBF(){

			_num=SGetMem(2*MYBYTES);
			_den=SGetMem(2*MYBYTES);

			_sign=0;
			_num[0]=1;
			_num[1]=0;
			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		\param _ts The value to set the sign to.
		*/
		CBF(MYWORDPTR _val, bool _ts){

			_num=SGetMem((_val[0]+1)*MYBYTES);
			_den=SGetMem(2*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			if(_ts){

				_sign=1;

			}

			else{

				_sign=0;

			}

			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the value to.
		*/
		CBF(MYWORDPTR _val){

			_num=SGetMem((_val[0]+1)*MYBYTES);
			_den=SGetMem(2*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			_sign=0;

			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!CTOR.
		/*!
		\param _newnum The value to set the numerator to.
		\param _newden The value to set the denominator to.
		*/
		CBF(MYWORDPTR _newnum, MYWORDPTR _newden){

			_num=SGetMem((_newnum[0]+1)*MYBYTES);
			_den=SGetMem((_newden[0]+1)*MYBYTES);

			for(int x=0;x<=_newnum[0];x++){

				_num[x]=_newnum[x];

			}

			for(int x=0;x<=_newden[0];x++){

				_den[x]=_newden[x];

			}

			_sign=0;

			return;

		}
		//!CTOR.
		/*!
		\param _vn The value to set the value to.
		\param _vs The value to set the sign to.
		*/
		CBF(MYWORDPTR _vn, int _vs){

			_num=SGetMem((_vn[0]+1)*MYBYTES);

			for(int x=0;x<=_vn[0];x++){

				_num[x]=_vn[x];

			}

			if(_vs>0){

				_sign=1;

			}

			else{

				_sign=0;

			}

			_den=SGetMem(3*MYBYTES);

			_den[0]=1;
			_den[1]=1;

		}
		//!CTOR.
		/*!
		\param _vn The value to set the numerator to.
		\param _vd The value to set the denominator to.
		\param _vs The value to set the sign to.
		*/
		CBF(MYWORDPTR _vn, MYWORDPTR _vd, int _vs){

			_num=SGetMem((_vn[0]+1)*MYBYTES);

			for(int x=0;x<=_vn[0];x++){

				_num[x]=_vn[x];

			}

			_den=SGetMem((_vd[0]+1)*MYBYTES);

			for(int x=0;x<=_vd[0];x++){

				_den[x]=_vd[x];

			}

			if(_vs>0){

				_sign=1;

			}

			else{

				_sign=0;

			}

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(short _val)
		{

			_num=SGetMem(3*MYBYTES);
			_den=SGetMem(2*MYBYTES);

			_num[0]=1;

			short _tval=_val;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

				_sign=0;

			}

			_num[1]=_tval;

			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(unsigned short _val)
		{

			_num=SGetMem(3*MYBYTES);
			_den=SGetMem(2*MYBYTES);

			_num[0]=1;
			_num[1]=_val;
			_den[0]=1;
			_den[1]=1;

			_sign=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(int _val)
		{

			_num=SGetMem(3*MYBYTES);
			_den=SGetMem(2*MYBYTES);

			_num[0]=1;

			int _tval=_val;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

				_sign=0;

			}

			_num[1]=_tval;

			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(unsigned int _val)
		{

			_num=SGetMem(3*MYBYTES);
			_den=SGetMem(2*MYBYTES);

			_num[0]=1;
			_num[1]=_val;

			_sign=0;

			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(__int64 _val)
		{

			__int64 _tval=_val;
			bool _neg=false;

			if(_val<0){

				_tval=_tval*-1;
				_neg=true;

			}

			else{

			}

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			if(_neg){

				_sign=1;

			}

			else{

				_sign=0;

			}

			_den=SGetMem(2*MYBYTES);

			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(unsigned __int64 _val)
		{

			unsigned __int64 _tval=_val;

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			_sign=0;
			_den=SGetMem(2*MYBYTES);

			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(char _val)
		{

			_num=SGetMem(3*MYBYTES);
			_den=SGetMem(2*MYBYTES);

			_num[0]=1;

			int _tval=_val;

			if(_val<0){

				_tval=_tval*-1;
				_sign=1;

			}

			else{

				_sign=0;

			}

			_num[1]=_tval;

			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(unsigned char _val)
		{

			_num=SGetMem(3*MYBYTES);
			_den=SGetMem(2*MYBYTES);

			_num[0]=1;
			_num[1]=_val;

			_sign=0;

			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(long _val)
		{

			__int64 _tval=_val;
			bool _neg=false;

			if(_val<0){

				_tval=_tval*-1;
				_neg=true;

			}

			else{

			}

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			if(_neg){

				_sign=1;

			}

			else{

				_sign=0;

			}

			_den=SGetMem(2*MYBYTES);

			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(unsigned long _val)
		{

			unsigned __int64 _tval=_val;

			if(MYBYTES>4){

				_num=SGetMem(3*MYBYTES);

				_num[0]=1;
				_num[1]=_tval;

			}

			else{

				_num=SGetMem(4*MYBYTES);
				_num[0]=2;
				_num[1]=_tval&65535;
				_tval=_tval>>16;
				_num[2]=_tval;

			}

			_sign=0;

			_den=SGetMem(2*MYBYTES);

			_den[0]=1;
			_den[1]=1;
			
			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(CBF* _val){

			_num=SGetMem((_val->_num[0]+1)*MYBYTES);
			_den=SGetMem((_val->_den[0]+1)*MYBYTES);

			for(int x=0;x<=_val->_num[0];x++){

				_num[x]=_val->_num[x];

			}

			_sign=_val->_sign;

			for(int x=0;x<=_val->_den[0];x++){

				_den[x]=_val->_den[x];

			}

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(CBD* _val){

			_num=SGetMem((_val->GetValue(0)+1)*MYBYTES);

			for(int x=0;x<=_val->GetValue(0);x++){

				_num[x]=_val->GetValue(x);

			}

			if(_val->GetDecimalPlace()>0){

				MYWORDPTR _ten;
				MYWORDPTR _exp;

				_ten=SGetMem(2*MYBYTES);
				_exp=SGetMem(2*MYBYTES);

				_ten[0]=1;
				_ten[1]=10;

				_exp[0]=1;
				_exp[1]=_val->GetDecimalPlace();

				_den=APow64(_ten, _exp);

				SFree(_exp);
				SFree(_ten);

			}

			else{

				_den=SGetMem(2*MYBYTES);

				_den[0]=1;
				_den[1]=1;

			}

		}
		//!CTOR.
		/*!
		\param _val The value to set the numerator to.
		*/
		CBF(CBI* _val){

			_num=SGetMem((_val->GetValue(0)+1)*MYBYTES);

			for(int x=0;x<=_val->GetValue(0);x++){

				_num[x]=_val->GetValue(x);

			}

			_den=SGetMem(2*MYBYTES);

			_den[0]=1;
			_den[1]=1;

		}
		//!Assignment.
		/*!
		\param _val The value to assign.
		*/
		void operator=(CBF* _val){

			SFree(_num);
			SFree(_den);

			_num=SGetMem((_val->_num[0]+1)*MYBYTES);
			_den=SGetMem((_val->_den[0]+1)*MYBYTES);

			for(int x=0;x<=_val->_num[0];x++){

				_num[x]=_val->_num[x];

			}

			for(int x=0;x<=_val->_den[0];x++){

				_den[x]=_val->_den[x];

			}

			_sign=_val->_sign;

			return;

		}
		//!Sets the instance equal to one.
		void SetToOne(){

			_num[0]=1;
			_num[1]=1;
			_sign=0;
			_den[0]=1;
			_den[1]=1;

			return;

		}
		//!Sets the instance equal to zero.
		void SetToZero(){

			_num[0]=1;
			_num[1]=0;
			_sign=0;
			_den[0]=1;
			_den[1]=1;

			return;

		}
		void TakeNewNumerator(MYWORDPTR _val){

			SFree(_num);
			_num=_val;

			return;

		}
		void TakeNewDenominator(MYWORDPTR _val){

			SFree(_den);
			_den=_val;

			return;

		}
		void Resize(int _size){

			SFree(_num);

			_num=SGetMem(_size);

			_den[0]=1;
			_den[1]=1;

			_num[0]=_size/MYBYTES;

			if(_num[0]==0){

				_num[0]=1;

			}

			else{

				for(int x=1;x<=_num[0];x++){

					_num[x]=0;

				}

			}

			return;

		}
		void Resize(int _numsize, int _densize){

			SFree(_num);
			SFree(_den);

			_num=SGetMem(_numsize);
			_den=SGetMem(_densize);

			_num[0]=_numsize/MYBYTES;
			_den[0]=_densize/MYBYTES;

			if(_num[0]==0){

				_num[0]=1;

			}

			else{

				for(int x=1;x<=_num[0];x++){

					_num[x]=0;

				}

			}

			if(_den[0]==0){

				_den[0]=1;

			}

			else{

				for(int x=1;x<=_den[0];x++){

					_den[x]=0;

				}

			}

			return;

		}
		void SetSign(MYWORD _newsign){

			if(_newsign>0){

				_sign=1;

			}

			else{

				_sign=0;

			}

			return;

		}
		MYWORD GetSign(){

			return _sign;

		}
		void SetNumeratorValue(int _pos, MYWORD _val){

			if(_pos>=0 && _pos<=_num[0]){

				_num[_pos]=_val;

			}

			return;

		}
		MYWORD GetNumeratorValue(int _pos){

			if(_pos>=0 && _pos<=_num[0]){

				return _num[_pos];

			}

			return 0;

		}
		void SetNumerator(MYWORDPTR _val){

			SFree(_num);

			_num=_val;

			return;

		}
		MYWORDPTR GetNumerator(){

			return _num;

		}
		void SetDenominatorValue(int _pos, MYWORD _val){

			if(_pos>=0 && _pos<=_den[0]){

				_den[_pos]=_val;

			}

			return;

		}
		MYWORD GetDenominatorValue(int _pos){

			if(_pos>=0 && _pos<=_den[0]){

				return _den[_pos];

			}

			return 0;

		}
		void SetDenominator(MYWORDPTR _val){

			SFree(_den);

			_den=_val;

			return;

		}
		MYWORDPTR GetDenominator(){

			return _den;

		}
		void ReplaceAndDelete(CBF* _val){

			if(_num[0]<=_val->_num[0]){

				SFree(_num);
				_num=SGetMem((_val->_num[0]+1)*MYBYTES);

			}

			for(int x=0;x<=_val->_num[0];x++){

				_num[x]=_val->_num[x];

			}

			if(_den[0]<=_val->_den[0]){

				SFree(_den);
				_den=SGetMem((_val->_den[0]+1)*MYBYTES);

			}

			for(int x=0;x<=_val->_den[0];x++){

				_den[x]=_val->_den[x];

			}

			_sign=_val->_sign;

			delete _val;

			return;

		}
		//!Creates an instance of CBF.
		/*!
		\returns A CBF object
		*/
		static CBF* Create(){

			CBF* _retval=new CBF();

			return _retval;

		}
		//!Creates an instance of CBF.
		/*!
		\param _val The numerator to copy.
		\returns A CBF object
		*/
		static CBF* CreateFromArrayTake(MYWORDPTR _val){

			CBF* _retval=new CBF(_val);

			SFree(_retval->_num);
			_retval->_num=_val;

			return _retval;

		}
		//!Creates an instance of CBF.
		/*!
		\param _newnum The numerator to copy.
		\param _newden The denominator to copy.
		\returns A CBF object
		*/
		static CBF* CreateFromArrayTakeBoth(MYWORDPTR _newnum, MYWORDPTR _newden){

			CBF* _retval=new CBF(true, _newnum, _newden, true);

			return _retval;

		}
		//!Creates an instance of CBF.
		/*!
		\param _val The numerator to copy.
		\returns A CBF object
		*/
		static CBF* CreateFromArrayCopy(MYWORDPTR _val){

			CBF* _retval=new CBF(_val, false);

			return _retval;

		}
		//!Creates an instance of CBF.
		/*!
		\param _newnum The numerator to copy.
		\param _newden The denominator to copy.
		\returns A CBF object
		*/
		static CBF* CreateFromArrayCopyBoth(MYWORDPTR _newnum, MYWORDPTR _newden){

			CBF* _retval=new CBF(_newnum, _newden);

			_retval->_sign=0;

			return _retval;

		}
		//!Creates an instance of CBF.
		/*!
		\param _val The numerator to copy.
		\param _si The sign to copy.
		\returns A CBF object
		*/
		static CBF* CreateFromArrayAndSign(MYWORDPTR _val, unsigned __int64 _si){

			CBF* _retval;

			if(_si==0){

				_retval=new CBF(_val, false);

			}

			else{

				_retval=new CBF(_val, true);

			}

			return _retval;

		}
		//!Creates an instance of CBF.
		/*!
		\param _newnum The numerator to copy.
		\param _newden The denominator to copy.
		\param _si The sign to copy.
		\returns A CBF object
		*/
		static CBF* CreateFromArrayAndSignBoth(MYWORDPTR _newnum, MYWORDPTR _newden, unsigned __int64 _si){

			CBF* _retval;

			if(_si==0){

				_retval=new CBF(_newnum, _newden, false);

			}

			else{

				_retval=new CBF(_newnum, _newden, true);

			}

			return _retval;

		}
		//!Creates an instance of CBF.
		/*!
		\param _val The value to copy.
		\returns A CBF object
		*/
		static CBF* CreateFromCBF(CBF* _val){

			CBF* _retval=new CBF(_val);

			return _retval;

		}
		//!Destroys an instance of CBDI.
		/*!
		\param _val The instance to destroy.
		*/
		static void Destroy(CBF* _val){

			delete _val;

		}
		void Take(CBF* _val){

			SFree(_num);
			SFree(_den);

			_num=_val->_num;
			_den=_val->_den;
			_sign=_val->_sign;

		}
		char* ToString(){

			char* _temp=BFToString64(this, 10);
			//char* _temp;

			return _temp;

		}
		char* ToString(MYWORD _radix){

			char* _temp=BFToString64(this, 10);
			//char* _temp;

			return _temp;

		}
		~CBF(){

			SFree(_num);
			SFree(_den);

		}
		CBF* operator+(CBF _val){

			CBF* _temp=BFAdd64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			if(_den[0]<_temp->_den[0]){

				SFree(_den);

				_den=SGetMem((_temp->_den[0]+1)*8);

			}

			for(int x=0;x<=_temp->_den[0];x++){

				_den[0]=_temp->_den[0];

			}

			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBF* operator+=(CBF _val){

			return operator+(_val);

		}
		CBF* operator-(CBF _val){

			CBF* _temp=BFSub64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			if(_den[0]<_temp->_den[0]){

				SFree(_den);

				_den=SGetMem((_temp->_den[0]+1)*8);

			}

			for(int x=0;x<=_temp->_den[0];x++){

				_den[0]=_temp->_den[0];

			}

			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBF* operator-=(CBF _val){

			return operator-(_val);

		}
		CBF* operator*(CBF _val){

			CBF* _temp=BFMultiply64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			if(_den[0]<_temp->_den[0]){

				SFree(_den);

				_den=SGetMem((_temp->_den[0]+1)*8);

			}

			for(int x=0;x<=_temp->_den[0];x++){

				_den[0]=_temp->_den[0];

			}

			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBF* operator*=(CBF _val){

			return operator*(_val);

		}
		CBF* operator/(CBF _val){

			CBF* _temp=BFDivide64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			if(_den[0]<_temp->_den[0]){

				SFree(_den);

				_den=SGetMem((_temp->_den[0]+1)*8);

			}

			for(int x=0;x<=_temp->_den[0];x++){

				_den[0]=_temp->_den[0];

			}

			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBF* operator/=(CBF _val){

			return operator/(_val);

		}
		CBF* operator%(CBF _val){

			CBF* _temp=BFMod64(this, &_val);

			if(_num[0]<_temp->_num[0]){

				SFree(_num);

				_num=SGetMem((_temp->_num[0]+1)*8);

			}

			for(int x=0;x<=_temp->_num[0];x++){

				_num[0]=_temp->_num[0];

			}

			if(_den[0]<_temp->_den[0]){

				SFree(_den);

				_den=SGetMem((_temp->_den[0]+1)*8);

			}

			for(int x=0;x<=_temp->_den[0];x++){

				_den[0]=_temp->_den[0];

			}

			_sign=_temp->_sign;

			delete _temp;

			return this;

		}
		CBF* operator%=(CBF _val){

			return operator%(_val);

		}
		bool operator==(CBF _val){

			if(BFE64(this, &_val)==1){

				return true;

			}

			return false;

		}
		bool operator!=(CBF _val){

			return !(operator==(_val));

		}
		bool operator<(CBF _val){

			if(BFLT64(this, &_val)==1){

				return true;

			}

			return false;

		}
		bool operator<=(CBF _val){

			return !(_val<this);

		}
		bool operator>(CBF _val){

			return _val<this;

		}
		bool operator>=(CBF _val){

			return !(operator<(_val));

		}
		static CBF* Pow(CBF _base, CBF _exp){

			CBF* _retval=BFPow64(&_base, &_exp);

			return _retval;

		}
		static CBF* ModPow(CBF _base, CBF _exp, CBF _mod){

			CBF* _retval=BFModPow64(&_base, &_exp, &_mod);

			return _retval;

		}
		//!Explicit cast from CBF.
		/*!
		\returns A char
		*/
		operator char(){

			char _temp=0;

			if(_num[0]==1 && _num[1]<256){

				_temp=(char)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBF.
		/*!
		\returns A unsigned char
		*/
		operator unsigned char(){

			if(_num[0]==1 && _num[1]<256){

				return (unsigned char)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBF.
		/*!
		\returns A short
		*/
		operator short(){

			short _temp=0;

			if(_num[0]==1 && _num[1]<65536){

				_temp=(short)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBF.
		/*!
		\returns A unsigned short
		*/
		operator unsigned short(){

			if(_num[0]==1 && _num[1]<65536){

				return (unsigned short)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBF.
		/*!
		\returns A int
		*/
		operator int(){

			int _temp=0;

			if(_num[0]==1 && _num[1]<4294967296){

				_temp=(int)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBF.
		/*!
		\returns A unsigned int
		*/
		operator unsigned int(){

			if(_num[0]==1 && _num[1]<4294967296){

				return (int)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBF.
		/*!
		\returns A long long
		*/
		operator long long(){

			long long _temp=0;

			if(_num[0]==1 && _num[1]<18446744073709551615){

				_temp=(long long)_num[1];

			}

			if(_sign==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBF.
		/*!
		\returns A unsigned long long
		*/
		operator unsigned long long(){

			if(_num[0]==1 && _num[1]<18446744073709551615){

				return (unsigned long long)_num[1];

			}

			return 0;

		}
		//!Explicit cast from CBF.
		/*!
		\returns A float
		*/
		operator float(){

			float _retval=0.0;
			float _base=1.8446744073709551616;
			float _digit=0.0;
			int _basecount=20;

			if(MYBYTES==4){

				_base=4.294967296;
				_basecount=10;

			}


			for(int x=1;x<20;x++){

				_base=_base*10;

			}

			for(int x=1;x<=_num[0];x++){

				_digit=_base;

				for(int y=1;y<_num[0];y++){

					_digit=_digit*_digit;

				}

				_digit=_digit*_num[x];

				_retval=_retval+_digit;

			}

			return _retval;

		}
		//!Explicit cast from CBF.
		/*!
		\returns A double
		*/
		operator double(){

			double _retval=0.0;
			double _base=1.8446744073709551616;
			double _digit=0.0;
			int _basecount=20;

			if(MYBYTES==4){

				_base=4.294967296;
				_basecount=10;

			}


			for(int x=1;x<20;x++){

				_base=_base*10;

			}

			for(int x=1;x<=_num[0];x++){

				_digit=_base;

				for(int y=1;y<_num[0];y++){

					_digit=_digit*_digit;

				}

				_digit=_digit*_num[x];

				_retval=_retval+_digit;

			}

			return _retval;

		}
		//!Explicit cast from CBF.
		/*!
		\returns A CBD object
		*/
		operator CBD*();
		//!Explicit cast from CBF.
		/*!
		\returns A CBI object
		*/
		operator CBI*();
		//!Explicit cast from CBF.
		/*!
		\returns A CBDI object
		*/
		operator CBDI*();
		//!Explicit cast from CBF.
		/*!
		\returns A CBFI object
		*/
		operator CBFI*();
		CBF* ABS(){

			CBF* _retval=new CBF(_num);

			_retval->_sign=0;

			return _retval;

		}
		static CBF* LCM(CBF _val1, CBF _val2){

			CBF* _retval=new CBF();
			CBF* _temp=GCD(_val1, _val2);

			_retval=_val1*_val2;

			_retval=*_retval/_temp;

			delete _temp;

			return _retval;

		}
		static CBF* GCD(CBF _val1, CBF _val2){

			CBF* _a;
			CBF* _b;

			if(_val1>=_val2){

				_a=new CBF(_val1);
				_b=new CBF(_val2);

			}

			else{

				_a=new CBF(_val2);
				_b=new CBF(_val1);

			}

			CBF* _r=new CBF();

			while(_b>0){

				_r=*_a%*_b;
				_a=_b;
				_b=_r;

			}

			delete _b;
			delete _r;

			return _a;

		}

};
//!This class is the native version of the BDI class, as well as the container for all the BDI algorithms in assembly language
/*!
This class uses 2 instances of the CBD class (one for the real part and one for the imaginary part, or one for the angle and one for the mod if in polar form)
There is also a _polarform property of type MYWORD to keep track of which form the instance is in (1==Polar form 0==not)
*/
public class BASE_API CBDI{

	private:

		//!This variable simply keeps track if the instance is in polar form or not.  If it is, the _real part contains the angle and the imaginary part contains the phase.
		MYWORD _polarform;
		//!This is the real part of the instance.  It is kept in a CBD* object.
		CBD* _real;
		//!This is the imaginary part of the instance.  It is kept in a CBD* object.
		CBD* _im;
		CBDI(CBD* _r, CBD* _i, bool _duh){

			_polarform=0;
			_real=_r;
			_im=_i;

		}
		CBDI(CBD* _val, bool _duh){

			_polarform=0;
			_real=_val;
			_im=new CBD();

		}

	public:

		//!CTOR.
		CBDI(){

			_polarform=0;
			_real=new CBD();
			_im=new CBD();

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(MYWORDPTR _val){

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

		}
		//!CTOR.
		/*!
		\param _newreal The value to set the real to.
		\param _newim The value to set the imaginary to.
		*/
		CBDI(MYWORDPTR _newreal, MYWORDPTR _newim){

			_polarform=0;
			_real=new CBD(_newreal);
			_im=new CBD(_newim);

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(short _val)
		{

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(unsigned short _val)
		{

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(int _val)
		{

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(unsigned int _val)
		{

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(__int64 _val)
		{

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(unsigned __int64 _val)
		{

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(char _val)
		{

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(unsigned char _val)
		{

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(long _val)
		{

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(unsigned long _val)
		{

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(CBDI* _val){

			_polarform=_val->_polarform;
			_real=new CBD(_val->_real);
			_im=new CBD(_val->_im);

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(CBI* _val){

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(CBD* _val){

			_polarform=0;
			_real=new CBD(_val);
			_im=new CBD();

		}
		//!CTOR.
		/*!
		\param _r The value to set the real to.
		\param _i The value to set the imaginary to.
		*/
		CBDI(CBD* _r, CBD* _i){

			_polarform=0;
			_real=new CBD(_r);
			_im=new CBD(_i);

		}
		//!CTOR.
		/*!
		\param _val The value to set the real to.
		*/
		CBDI(CBF* _val){

			_polarform=0;
			if(_val->GetDenominatorValue(0)!=1 || _val->GetDenominatorValue(1)!=1){

				CBI* _number=new CBI(_val->GetNumerator());
				CBI* _div=new CBI(_val->GetDenominator());
				CBI* _mod=new CBI();
				CBI* _temp=new CBI();
				CBI* _ten=new CBI();

				_ten->SetValue(0, 1);
				_ten->SetValue(1, 10);

				_real=new CBD(_val->GetNumerator());

				SFree(_mod->GetArray());
				_mod->SetArray(AMod64(_number->GetArray(), _div->GetArray()));

				SFree(_temp->GetArray());
				_temp->SetArray(ASub64(_number->GetArray(), _mod->GetArray()));

				SFree(_real->GetArray());
				_real->SetArray(ADivide64(_temp->GetArray(), _div->GetArray()));

				bool _done=false;

				if(_mod->GetValue(0)==1 && _mod->GetValue(1)==0){

					_done=true;

				}

				while(_real->GetDecimalPlace()<BDGetDecimalLimit() && !_done){

					SFree(_temp->GetArray());
					_temp->SetArray(AMultiply64(_mod->GetArray(), _ten->GetArray()));

					SFree(_mod->GetArray());
					_mod->SetArray(AMod64(_temp->GetArray(), _div->GetArray()));

					SFree(_number->GetArray());
					_number->SetArray(ASub64(_temp->GetArray(), _mod->GetArray()));

					SFree(_temp->GetArray());
					_temp->SetArray(ADivide64(_number->GetArray(), _div->GetArray()));
					SFree(_number->GetArray());

					_number->SetArray(AMultiply64(_real->GetArray(), _ten->GetArray()));
					SFree(_real->GetArray());

					_real->SetArray(AAdd64(_number->GetArray(), _temp->GetArray()));
					_real->SetDecimalPlace(_real->GetDecimalPlace()+1);

				}

				SFree(_temp->GetArray());

				_temp->SetArray(AMultiply64(_mod->GetArray(), _ten->GetArray()));

				SFree(_mod->GetArray());
				_mod->SetArray(AMod64(_temp->GetArray(), _div->GetArray()));

				SFree(_number->GetArray());
				_number->SetArray(ASub64(_temp->GetArray(), _mod->GetArray()));

				SFree(_mod->GetArray());
				_mod->SetArray(ADivide64(_number->GetArray(), _div->GetArray()));

				_temp->SetValue(0, 0);
				_temp->SetValue(1, 0);
				_number->SetValue(0, 1);
				_number->SetValue(1, 1);

				if(_mod->GetValue(0)==1 && _mod->GetValue(1)>4){

					SFree(_temp->GetArray());
					_temp->SetArray(AAdd64(_real->GetArray(), _number->GetArray()));

				}

				delete _number;
				delete _ten;
				delete _div;
				delete _mod;

				if(_temp->GetValue(0)!=0 || _temp->GetValue(1)!=0){

					SFree(_real->GetArray());

					_real->SetArray(SGetMem((_temp->GetValue(0)+1)*MYBYTES));

					for(int x=0;x<=_temp->GetValue(0);x++){

						_real->SetValue(x, _temp->GetValue(x));

					}

				}

				delete _temp;

			}

			else{

				_real=new CBD(_val->GetNumerator());

			}

			_real->SetSign(_val->GetSign());

			_im=new CBD();

		}
		//!CTOR.
		/*!
		\param _vn The value to set the real to.
		\param _vs The value to set the real sign to.
		*/
		CBDI(MYWORDPTR _vn, int _vs){

			_polarform=0;
			_real=new CBD(_vn, _vs);
			_im=new CBD();
			
		}
		//!CTOR.
		/*!
		\param _vn The value to set the real to.
		\param _vs The value to set the real sign to.
		\param _vn The value to set the imaginary to.
		\param _vs The value to set the imaginary sign to.
		*/
		CBDI(MYWORDPTR _vn, MYWORDPTR _vs, int _in, int _is){

			_polarform=0;
			if(_in==1){

				_real=new CBD(_vn, true);

			}

			else{

				_real=new CBD(_vn, false);

			}

			if(_is==1){

				_im=new CBD(_vs, true);

			}

			else{

				_im=new CBD(_vs, false);

			}

		}
		void Take(CBDI* _val){

			delete _real;
			delete _im;

			_polarform=_val->_polarform;
			_real=_val->_real;
			_im=_val->_im;

		}
		void operator=(CBDI* _val){

			delete _real;
			delete _im;

			_polarform=_val->_polarform;
			_real=new CBD(_val->_real);
			_im=new CBD(_val->_im);

			return;

		}
		void SetToOne(){

			_polarform=0;
			_real->SetToOne();
			_im->SetToZero();

			return;

		}
		void SetToZero(){

			_polarform=0;
			_real->SetToZero();
			_im->SetToZero();

			return;

		}
		void Resize(int _size){

			_real->Resize(_size);

			return;

		}
		void SetReal(CBD* _newval){

			delete _real;
			_real=_newval;

			return;

		}
		CBD* GetReal(){

			return _real;

		}
		void SetImaginary(CBD* _newval){

			delete _im;
			_im=_newval;

			return;

		}
		CBD* GetImaginary(){

			return _im;

		}
		void SetPolarForm(MYWORD _val){

			if(_val==0){

				_polarform=0;

			}

			else{

				_polarform=1;

			}

		}
		MYWORD GetPolarForm(){

			return _polarform;

		}
		void ReplaceAndDelete(CBDI* _val){

			delete _real;
			delete _im;

			_polarform=_val->_polarform;
			_real=new CBD(_val->_real);
			_im=new CBD(_val->_im);

			delete _val;

			return;

		}
		//!Creates an instance of CBDI.
		/*!
		\returns A CBDI object
		*/
		static CBDI* Create(){

			CBDI* _retval=new CBDI();

			return _retval;

		}
		//!Creates an instance of CBDI.
		/*!
		\param _newreal The value to copy to real.
		\returns A CBDI object
		*/
		static CBDI* CreateFromArrayTake(MYWORDPTR _newreal){

			CBDI* _retval=new CBDI(_newreal, true);

			return _retval;

		}
		//!Creates an instance of CBDI.
		/*!
		\param _r The value to copy to real.
		\param _i The value to copy to imaginary.
		\returns A CBDI object
		*/
		static CBDI* CreateFromCBDTakeBoth(CBD* _r, CBD* _i){

			CBDI* _retval=new CBDI(_r, _i, true);

			return _retval;

		}
		//!Creates an instance of CBDI.
		/*!
		\param _val The value to copy to real.
		\returns A CBDI object
		*/
		static CBDI* CreateFromArrayCopy(MYWORDPTR _val){

			CBDI* _retval=new CBDI(_val);

			return _retval;

		}
		//!Creates an instance of CBDI.
		/*!
		\param _r The value to copy to real.
		\param _i The value to copy to imaginary.
		\returns A CBDI object
		*/
		static CBDI* CreateFromCBDCopyBoth(CBD* _r, CBD* _i){

			CBDI* _retval=new CBDI(_r, _i);

			return _retval;

		}
		//!Creates an instance of CBDI.
		/*!
		\param _newreal The value to copy to real.
		\param _newim The value to copy to imaginary.
		\returns A CBDI object
		*/
		static CBDI* CreateFromArrayAndSign(MYWORDPTR _newreal, MYWORDPTR _newim){

			CBDI* _retval=new CBDI(_newreal, _newim);

			return _retval;

		}
		//!Creates an instance of CBDI.
		/*!
		\param _val The value to copy.
		\returns A CBDI object
		*/
		static CBDI* CreateFromCBDI(CBDI* _val){

			CBDI* _retval=new CBDI(_val);

			return _retval;

		}
		//!Destroys an instance of CBDI.
		/*!
		\param _val The value to destroy.
		*/
		static void Destroy(CBDI* _val){

			delete _val;

		}
		char* ToString(){

			char* _temp=BDIToString64(this, 10);
			//char* _temp;

			return _temp;

		}
		char* ToString(MYWORD _radix){

			char* _temp=BDIToString64(this, 10);
			//char* _temp;

			return _temp;

		}
		~CBDI(){

			delete _real;
			delete _im;

		}
		CBDI* operator+(CBDI _val){

			CBDI* _temp=BDIAdd64(this, &_val);

			delete _real;

			_real=new CBD(_temp->_real);

			delete _im;

			_im=new CBD(_temp->_im);

			delete _temp;

			return this;

		}
		CBDI* operator+=(CBDI _val){

			return operator+(_val);

		}
		CBDI* operator-(CBDI _val){

			CBDI* _temp=BDISub64(this, &_val);

			delete _real;

			_real=new CBD(_temp->_real);

			delete _im;

			_im=new CBD(_temp->_im);

			delete _temp;

			return this;

		}
		CBDI* operator-=(CBDI _val){

			return operator-(_val);

		}
		CBDI* operator*(CBDI _val){

			CBDI* _temp=BDIMultiply64(this, &_val);

			delete _real;

			_real=new CBD(_temp->_real);

			delete _im;

			_im=new CBD(_temp->_im);

			delete _temp;

			return this;

		}
		CBDI* operator*=(CBDI _val){

			return operator*(_val);

		}
		CBDI* operator/(CBDI _val){

			CBDI* _temp=BDIDivide64(this, &_val);

			delete _real;

			_real=new CBD(_temp->_real);

			delete _im;

			_im=new CBD(_temp->_im);

			delete _temp;

			return this;

		}
		CBDI* operator/=(CBDI _val){

			return operator/(_val);

		}
		bool operator==(CBDI _val){

			if(BDIE64(this, &_val)==1){

				return true;

			}

			return false;

		}
		bool operator!=(CBDI _val){

			return !(operator==(_val));

		}
		static CBDI* Pow(CBDI _base, CBDI _exp){

			CBDI* _retval=BDIPow64(&_base, &_exp);

			return _retval;

		}
		//!Explicit cast from CBDI.
		/*!
		\returns A char
		*/
		operator char(){

			char _temp=0;

			if(_real->GetValue(0)==1 && _real->GetValue(1)<256){

				_temp=(char)_real->GetValue(1);

			}

			if(_real->GetSign()==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBDI.
		/*!
		\returns A unsigned char
		*/
		operator unsigned char(){

			if(_real->GetValue(0)==1 && _real->GetValue(1)<256){

				return (unsigned char)_real->GetValue(1);

			}

			return 0;

		}
		//!Explicit cast from CBDI.
		/*!
		\returns A short
		*/
		operator short(){

			short _temp=0;

			if(_real->GetValue(0)==1 && _real->GetValue(1)<65536){

				_temp=(short)_real->GetValue(1);

			}

			if(_real->GetSign()==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBDI.
		/*!
		\returns A unsigned short
		*/
		operator unsigned short(){

			if(_real->GetValue(0)==1 && _real->GetValue(1)<65536){

				return (unsigned short)_real->GetValue(1);

			}

			return 0;

		}
		//!Explicit cast from CBDI.
		/*!
		\returns A int
		*/
		operator int(){

			int _temp=0;

			if(_real->GetValue(0)==1 && _real->GetValue(1)<4294967296){

				_temp=(int)_real->GetValue(1);

			}

			if(_real->GetSign()==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBDI.
		/*!
		\returns A unsigned int
		*/
		operator unsigned int(){

			if(_real->GetValue(0)==1 && _real->GetValue(1)<4294967296){

				return (int)_real->GetValue(1);

			}

			return 0;

		}
		//!Explicit cast from CBDI.
		/*!
		\returns A long long
		*/
		operator long long(){

			long long _temp=0;

			if(_real->GetValue(0)==1 && _real->GetValue(1)<18446744073709551615){

				_temp=(long long)_real->GetValue(1);

			}

			if(_real->GetSign()==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast from CBDI.
		/*!
		\returns A unsigned long long
		*/
		operator unsigned long long(){

			if(_real->GetValue(0)==1 && _real->GetValue(1)<18446744073709551615){

				return (unsigned long long)_real->GetValue(1);

			}

			return 0;

		}
		//!Explicit cast from CBDI.
		/*!
		\returns A float
		*/
		operator float(){

			float _retval=0.0;
			float _base=1.8446744073709551616;
			float _digit=0.0;
			int _basecount=20;

			if(MYBYTES==4){

				_base=4.294967296;
				_basecount=10;

			}


			for(int x=1;x<20;x++){

				_base=_base*10;

			}

			for(int x=1;x<=_real->GetValue(0);x++){

				_digit=_base;

				for(int y=1;y<_real->GetValue(0);y++){

					_digit=_digit*_digit;

				}

				_digit=_digit*_real->GetValue(x);

				_retval=_retval+_digit;

			}

			return _retval;

		}
		//!Explicit cast from CBDI.
		/*!
		\returns A double
		*/
		operator double(){

			double _retval=0.0;
			double _base=1.8446744073709551616;
			double _digit=0.0;
			int _basecount=20;

			if(MYBYTES==4){

				_base=4.294967296;
				_basecount=10;

			}


			for(int x=1;x<20;x++){

				_base=_base*10;

			}

			for(int x=1;x<=_real->GetValue(0);x++){

				_digit=_base;

				for(int y=1;y<_real->GetValue(0);y++){

					_digit=_digit*_digit;

				}

				_digit=_digit*_real->GetValue(x);

				_retval=_retval+_digit;

			}

			return _retval;

		}
		//!Explicit cast from CBDI.
		/*!
		\returns A CBD object
		*/
		operator CBD*();
		//!Explicit cast from CBDI.
		/*!
		\returns A CBF object
		*/
		operator CBF*();
		//!Explicit cast from CBDI.
		/*!
		\returns A CBI object
		*/
		operator CBI*();
		//!Explicit cast from CBDI.
		/*!
		\returns A CBFI object
		*/
		operator CBFI*();

};
//!This class is the native version of the BFI class, as well as the container for all the BFI algorithms in assembly language
/*!
This class uses 2 instances of the CBF class (one for the real part and one for the imaginary part, or one for the angle and one for the mod if in polar form)
There is also a _polarform property of type MYWORD to keep track of which form the instance is in (1==Polar form 0==not)
*/
public class BASE_API CBFI{

	private:

		//!This variable simply keeps track if the instance is in polar form or not.  If it is, the _real part contains the angle and the imaginary part contains the phase.		MYWORD _polarform;
		MYWORD _polarform;
		//!This is the real part of the instance.  It is kept in a CBF* object.
		CBF* _real;
		//!This is the imaginary part of the instance.  It is kept in a CBF* object.
		CBF* _im;

		CBFI(CBF* _val, bool _duh){

			_real=_val;
			_im=new CBF();
			_polarform=0;

		}
		CBFI(CBF* _nr, CBF* _ni, bool _duh){

			_real=_nr;
			_im=_ni;
			_polarform=0;

		}

	public:

		//!CTOR.
		CBFI(){

			_real=new CBF();
			_im=new CBF();

			_polarform=0;
			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(MYWORDPTR _val){

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

		}
		//!CTOR.
		/*!
		\param _newreal The value to set the real to.
		\param _newim The value to set the imaginary to.
		*/
		CBFI(MYWORDPTR _newreal, MYWORDPTR _newim){

			_real=new CBF(_newreal);
			_im=new CBF(_newim);
			_polarform=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(short _val)
		{

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(unsigned short _val)
		{

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(int _val)
		{

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(unsigned int _val)
		{

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(__int64 _val)
		{

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(unsigned __int64 _val)
		{

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(char _val)
		{

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(unsigned char _val)
		{

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(long _val)
		{

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(unsigned long _val)
		{

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(CBFI* _val){

			_real=new CBF(_val->_real);
			_im=new CBF(_val->_im);
			_polarform=_val->_polarform;

			return;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(CBI* _val){

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(CBD* _val){

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(CBF* _val){

			_real=new CBF(_val);
			_im=new CBF();
			_polarform=0;

		}
		//!CTOR.
		/*!
		\param _nr The value to set the real.
		\param _ni The value to set the imaginary.
		*/
		CBFI(CBF* _nr, CBF* _ni){

			_real=new CBF(_nr);
			_im=new CBF(_ni);
			_polarform=0;

		}
		//!CTOR.
		/*!
		\param _val The value to set the instance to.
		*/
		CBFI(CBDI* _val){

			CBI* _ten=new CBI();
			CBI* _exp=new CBI();

			_ten->SetValue(0, 1);
			_ten->SetValue(1, 10);
			_exp->SetValue(0, 1);

			_real=new CBF(_val->GetReal()->GetArray());
			_real->SetSign(_val->GetReal()->GetSign());
			_im=new CBF(_val->GetImaginary()->GetArray());
			_im->SetSign(_val->GetImaginary()->GetSign());

			if(_val->GetReal()->GetDecimalPlace()>0){

				_exp->SetValue(1, _val->GetReal()->GetDecimalPlace());

				SFree(_real->GetDenominator());

				_real->SetDenominator(APow64(_ten->GetArray(), _exp->GetArray()));

			}

			if(_val->GetImaginary()->GetDecimalPlace()>0){

				_exp->SetValue(1, _val->GetImaginary()->GetDecimalPlace());

				SFree(_im->GetDenominator());

				_im->SetDenominator(APow64(_ten->GetArray(), _exp->GetArray()));

			}

			_polarform=_val->GetPolarForm();

			delete _ten;
			delete _exp;

		}
		//!CTOR.
		/*!
		\param _vn The value to set the real numerator.
		\param _vs The value to set the real sign.
		*/
		CBFI(MYWORDPTR _vn, int _vs){

			_real=new CBF(_vn, _vs);
			_im=new CBF();
			_polarform=0;

		}
		//!CTOR.
		/*!
		\param _vn The value to set the real numerator.
		\param _vd The value to set the real denominator.
		\param _vs The value to set the real sign.
		*/
		CBFI(MYWORDPTR _vn, MYWORDPTR _vd, int _vs){

			_real=new CBF(_vn, _vd, _vs);
			_im=new CBF();
			_polarform=0;

		}
		//!CTOR.
		/*!
		\param _vn The value to set the real numerator.
		\param _vs The value to set the real sign.
		\param _in The value to set the imaginary numerator.
		\param _is The value to set the imaginary sign.
		*/
		CBFI(MYWORDPTR _vn, int _vs, MYWORDPTR _in, int _is){

			_real=new CBF(_vn, _vs);
			_im=new CBF(_in, _is);
			_polarform=0;

		}
		//!CTOR.
		/*!
		\param _vn The value to set the real numerator.
		\param _vd The value to set the real denominator.
		\param _vs The value to set the real sign.
		\param _in The value to set the imaginary numerator.
		\param _id The value to set the imaginary denominator.
		\param _is The value to set the imaginary sign.
		*/
		CBFI(MYWORDPTR _vn, MYWORDPTR _vd, int _vs, MYWORDPTR _in, MYWORDPTR _id, int _is){

			_real=new CBF(_vn, _vd, _vs);
			_im=new CBF(_in, _id, _is);
			_polarform=0;

		}
		//!Takes possession of an object.
		/*!
		\param _val The value to take.
		*/
		void Take(CBFI* _val){

			delete _real;
			delete _im;

			_real=new CBF(_val->_real);
			_im=new CBF(_val->_im);
			_polarform=_val->_polarform;

			delete _val;

		}
		//!Assignment.
		/*!
		\param _val The value to set the instance.
		*/
		void operator=(CBFI* _val){

			delete _real;
			delete _im;

			_real=new CBF(_val->_real);
			_im=new CBF(_val->_im);
			_polarform=_val->_polarform;

			delete _val;

			return;

		}
		//!Sets the instance to one.
		void SetToOne(){

			_real->SetToOne();
			_im->SetToZero();
			_polarform=0;

			return;

		}
		//!Sets the instance to zero.
		void SetToZero(){

			_real->SetToZero();
			_im->SetToZero();
			_polarform=0;

			return;

		}
		void Resize(int _size){

			_real->Resize(_size);

			return;

		}
		//!Sets the Real variable.
		/*!
		\param _val The value to set the variable.
		*/
		void SetReal(CBF* _newval){

			delete _real;
			_real=_newval;

			return;

		}
		//!Gets the Real variable.
		/*!
		\returns A CBF object
		*/
		CBF* GetReal(){

			return _real;

		}
		//!Sets the Imaginary variable.
		/*!
		\param _newval The value to set the variable.
		*/
		void SetImaginary(CBF* _newval){

			delete _im;
			_im=_newval;

			return;

		}
		//!Gets the Imaginary variable.
		/*!
		\returns A CBF object
		*/
		CBF* GetImaginary(){

			return _im;

		}
		//!Gets the polarform variable.
		/*!
		\returns A MYWORD
		*/
		MYWORD GetPolarForm(){

			return _polarform;

		}
		//!Sets the polarform variable.
		/*!
		\param _val The value to set the variable.
		*/
		void SetPolarForm(MYWORD _val){

			_polarform=_val;

		}
		//!Creates an instance of CBFI.
		/*!
		\returns A CBFI object
		*/
		static CBFI* Create(){

			CBFI* _retval=new CBFI();

			return _retval;

		}
		//!Creates an instance of CBFI from a MYWORDPTR object(take).
		/*!
		\param _newreal The real value.
		\returns A CBFI object
		*/
		static CBFI* CreateFromArrayTake(MYWORDPTR _newreal){

			CBFI* _retval=new CBFI(_newreal, true);

			return _retval;

		}
		//!Creates an instance of CBFI from a MYWORDPTR object (copy).
		/*!
		\param _val The real value.
		\returns A CBFI object
		*/
		static CBFI* CreateFromArrayCopy(MYWORDPTR _val){

			CBFI* _retval=new CBFI(_val);

			return _retval;

		}
		//!Creates an instance of CBFI from 2 MYWORDPTR objects.
		/*!
		\param _newreal The real value.
		\param _newim The imaginary value.
		\returns A CBFI object
		*/
		static CBFI* CreateFromArrayAndSign(MYWORDPTR _newreal, MYWORDPTR _newim){

			CBFI* _retval=new CBFI(_newreal, _newim);

			return _retval;

		}
		//!Creates an instance of CBFI from a CBFI object.
		/*!
		\param _val The value.
		\returns A CBFI object
		*/
		static CBFI* CreateFromCBFI(CBFI* _val){

			CBFI* _retval=new CBFI(_val);

			return _retval;

		}
		//!Creates an instance of CBFI from 2 CBF objects(take).
		/*!
		\param _r The real value.
		\param _i The imaginary value.
		\returns A CBFI object
		*/
		static CBFI* CreateFromCBFTakeBoth(CBF* _r, CBF* _i){

			CBFI* _retval=new CBFI(_r, _i, true);

			return _retval;

		}
		//!Creates an instance of CBFI from 2 CBF objects (copy).
		/*!
		\param _r The real value.
		\param _i The imaginary value.
		\returns A CBFI object
		*/
		static CBFI* CreateFromCBFCopyBoth(CBF* _r, CBF* _i){

			CBFI* _retval=new CBFI(_r, _i);

			return _retval;

		}
		//!Destroys an instance of CBFI.
		/*!
		\param _val The value to destroy.
		*/
		static void Destroy(CBFI* _val){

			delete _val;

		}
		~CBFI(){

			delete _real;
			delete _im;

		}
		//!Adds 2 instances.
		/*!
		\param _val The value to add.
		\returns A CBFI object
		*/
		CBFI* operator+(CBFI _val){

			CBFI* _temp=BFIAdd64(this, &_val);

			delete _real;

			_real=new CBF(_temp->_real);

			delete _im;

			_im=new CBF(_temp->_im);

			delete _temp;

			return this;

		}
		//!Adds 2 instances.
		/*!
		\param _val The value to add.
		\returns A CBFI object
		*/
		CBFI* operator+=(CBFI _val){

			return operator+(_val);

		}
		//!Subtracts 2 instances.
		/*!
		\param _val The value to subtract.
		\returns A CBFI object
		*/
		CBFI* operator-(CBFI _val){

			CBFI* _temp=BFISub64(this, &_val);

			delete _real;

			_real=new CBF(_temp->_real);

			delete _im;

			_im=new CBF(_temp->_im);

			delete _temp;

			return this;

		}
		//!Subtracts 2 instances.
		/*!
		\param _val The value to subtract.
		\returns A CBFI object
		*/
		CBFI* operator-=(CBFI _val){

			return operator-(_val);

		}
		//!Multiplies 2 instances.
		/*!
		\param _val The value to Multiply.
		\returns A CBFI object
		*/
		CBFI* operator*(CBFI _val){

			CBFI* _temp=BFIMultiply64(this, &_val);

			delete _real;

			_real=new CBF(_temp->_real);

			delete _im;

			_im=new CBF(_temp->_im);

			delete _temp;

			return this;

		}
		//!Multiplies 2 instances.
		/*!
		\param _val The value to Multiply.
		\returns A CBFI object
		*/
		CBFI* operator*=(CBFI _val){

			return operator*(_val);

		}
		//!Divides 2 instances.
		/*!
		\param _val The value to divide.
		\returns A CBFI object
		*/
		CBFI* operator/(CBFI _val){

			CBFI* _temp=BFIDivide64(this, &_val);

			delete _real;

			_real=new CBF(_temp->_real);

			delete _im;

			_im=new CBF(_temp->_im);

			delete _temp;

			return this;

		}
		//!Divides 2 instances.
		/*!
		\param _val The value to divide.
		\returns A CBFI object
		*/
		CBFI* operator/=(CBFI _val){

			return operator/(_val);

		}
		//!Checks for equality.
		/*!
		\param _val The value to compare against.
		\returns A bool
		*/
		bool operator==(CBFI _val){

			if(BFIE64(this, &_val)==1){

				return true;

			}

			return false;

		}
		//!Checks for Inequality.
		/*!
		\param _val The value to compare against.
		\returns A bool
		*/
		bool operator!=(CBFI _val){

			return !(operator==(_val));

		}
		//!Calculates the POW.
		/*!
		\param _base The base value.
		\param _exp The exponent.
		\returns A CBFI object
		*/
		static CBFI* Pow(CBFI _base, CBFI _exp){

			CBFI* _retval=BFIPow64(&_base, &_exp);

			return _retval;

		}
		//!Explicit cast From CBFI.
		/*!
		\returns A char
		*/
		operator char(){

			char _temp=0;

			if(_real->GetNumeratorValue(0)==1 && _real->GetNumeratorValue(1)<256){

				_temp=(char)_real->GetNumeratorValue(1);

			}

			if(_real->GetSign()==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast From CBFI.
		/*!
		\returns A unsigned char
		*/
		operator unsigned char(){

			if(_real->GetNumeratorValue(0)==1 && _real->GetNumeratorValue(1)<256){

				return (unsigned char)_real->GetNumeratorValue(1);

			}

			return 0;

		}
		//!Explicit cast From CBFI.
		/*!
		\returns A short
		*/
		operator short(){

			short _temp=0;

			if(_real->GetNumeratorValue(0)==1 && _real->GetNumeratorValue(1)<65536){

				_temp=(short)_real->GetNumeratorValue(1);

			}

			if(_real->GetSign()==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast From CBFI.
		/*!
		\returns A unsigned short
		*/
		operator unsigned short(){

			if(_real->GetNumeratorValue(0)==1 && _real->GetNumeratorValue(1)<65536){

				return (unsigned short)_real->GetNumeratorValue(1);

			}

			return 0;

		}
		//!Explicit cast From CBFI.
		/*!
		\returns A int
		*/
		operator int(){

			int _temp=0;

			if(_real->GetNumeratorValue(0)==1 && _real->GetNumeratorValue(1)<4294967296){

				_temp=(int)_real->GetNumeratorValue(1);

			}

			if(_real->GetSign()==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast From CBFI.
		/*!
		\returns A unsigned int
		*/
		operator unsigned int(){

			if(_real->GetNumeratorValue(0)==1 && _real->GetNumeratorValue(1)<4294967296){

				return (int)_real->GetNumeratorValue(1);

			}

			return 0;

		}
		//!Explicit cast From CBFI.
		/*!
		\returns A long long
		*/
		operator long long(){

			long long _temp=0;

			if(_real->GetNumeratorValue(0)==1 && _real->GetNumeratorValue(1)<18446744073709551615){

				_temp=(long long)_real->GetNumeratorValue(1);

			}

			if(_real->GetSign()==1){

				_temp=_temp*-1;

			}

			return _temp;

		}
		//!Explicit cast From CBFI.
		/*!
		\returns A unsigned long long
		*/
		operator unsigned long long(){

			if(_real->GetNumeratorValue(0)==1 && _real->GetNumeratorValue(1)<18446744073709551615){

				return (unsigned long long)_real->GetNumeratorValue(1);

			}

			return 0;

		}
		//!Explicit cast From CBFI.
		/*!
		\returns A float
		*/
		operator float(){

			float _retval=0.0;
			float _base=1.8446744073709551616;
			float _digit=0.0;
			int _basecount=20;

			if(MYBYTES==4){

				_base=4.294967296;
				_basecount=10;

			}


			for(int x=1;x<20;x++){

				_base=_base*10;

			}

			for(int x=1;x<=_real->GetNumeratorValue(0);x++){

				_digit=_base;

				for(int y=1;y<_real->GetNumeratorValue(0);y++){

					_digit=_digit*_digit;

				}

				_digit=_digit*_real->GetNumeratorValue(x);

				_retval=_retval+_digit;

			}

			return _retval;

		}
		//!Explicit cast From CBFI.
		/*!
		\returns A double
		*/
		operator double(){

			double _retval=0.0;
			double _base=1.8446744073709551616;
			double _digit=0.0;
			int _basecount=20;

			if(MYBYTES==4){

				_base=4.294967296;
				_basecount=10;

			}


			for(int x=1;x<20;x++){

				_base=_base*10;

			}

			for(int x=1;x<=_real->GetNumeratorValue(0);x++){

				_digit=_base;

				for(int y=1;y<_real->GetNumeratorValue(0);y++){

					_digit=_digit*_digit;

				}

				_digit=_digit*_real->GetNumeratorValue(x);

				_retval=_retval+_digit;

			}

			return _retval;

		}
		//!Explicit cast From CBFI.
		/*!
		\returns A CBD object
		*/
		operator CBD*();
		//!Explicit cast From CBFI.
		/*!
		\returns A CBF object
		*/
		operator CBF*();
		//!Explicit cast From CBFI.
		/*!
		\returns A CBDI object
		*/
		operator CBDI*();
		//!Explicit cast From CBFI.
		/*!
		\returns A CBI object
		*/
		operator CBI*();

};

extern BASE_API bool UseExp;
extern BASE_API bool _test;
BASE_API bool UseExp=false;
BASE_API void SetExp(bool _val)
{

	UseExp=_val;
	return;
}
BASE_API void UsingMalloc(MYWORD _val){

	UseMalloc(_val);

	return;
	
}
BASE_API MYWORDPTR GetMem(MYWORD bytes){

	MYWORDPTR _retval=SGetMem(bytes);

	__int64 _error=GetLastError();

	if(_error!=0){

		int a=4;
		int b=5;
		int c=a*b;

	}

	return _retval;

}
BASE_API void FreeMem(MYWORDPTR var){

	Free64(var);

	//if(_error!=0){

	//	int a=4;
	//	int b=5;
	//	int c=a*b;

	//}

}
BASE_API MYWORD GetHowToHandleRounding(){

	return BDGetHowToHandleRounding();

}
BASE_API MYWORD GetDecimalLimit(){

	return BDGetDecimalLimit();

}
BASE_API void SetHowToHandleRounding(MYWORD _val){

	BDChangeHowToHandleRounding(_val);

}
BASE_API void SetDecimalLimit(MYWORD _val){

	BDChangeDecimalLimit(_val);

}

#pragma managed(pop)

#pragma managed(push, on)

//!This is the namespace for most of my work.
namespace MyMath {

	//!This enumeration defines the trace levels for debugging
	[Guid("77F99007-4124-43C1-B43A-7E3F7BD76F6A")]
	public enum class TraceLevel:int{

		None=5,
		Basic=10,
		Standard=20,
		Debug=35

	};

	//!This enumeration defines the message level for a particular message being sent to the trace class
	[Guid("20549C89-B2FD-4274-9391-EF75EE68BF35")]
	public enum class MessageLevel:int{

		None=0,
		Information=10,
		Warning=20,
		Error=35

	};

	//!This enumeration defines the radix to be used in ToString
	[Guid("3F3397EC-086C-49D8-8502-66F2F6006F7D")]
	public enum class Radix : int{

		_Undefined=-1,
		_Binary=2,
		_OctaDecimal=8,
		_Decimal=10,
		_HexaDecimal=32,
		_Base32=32,
		_Base64=64

	};

	//!This enumeration defines how the BD class handles rounding
	[Guid("75A38F6F-4B99-40C5-A232-2BD242C0C7A2")]
	public enum class Rounding : int{

		_Undefined=-1,
		_FiveAndBetter=5,
		_DropRemaining=8,
		_RoundUp=10,

	};

	//!This class is a container for trace messages used for debugging
	[Guid("3CA50579-60C7-488C-9DCD-18337F281A99")]
	public ref class MessageInfo{

		private:

			//these hold the values passed in (and the return value)

			MYWORDPTR _v1;
			MYWORDPTR _v2;
			MYWORDPTR _v3;
			MYWORDPTR _r;

			//this holds the ref number passed in
			unsigned int _ref;

			//this holds the message
			String^ _message;
			TraceLevel _level;
			Microsoft::Office::Interop::Word::WdColor _thecolor;
			static Microsoft::Office::Interop::Word::WdColor _infocolor;
			static Microsoft::Office::Interop::Word::WdColor _warningcolor;
			static Microsoft::Office::Interop::Word::WdColor _errorcolor;

			bool _bold;
			bool _underline;
			bool _italic;
			bool _used;
			static bool _init=false;

		public:

			//NativeMessage* _natmes;

			void Init();
			MessageInfo();
			MessageInfo(MessageInfo% _val);
			MessageInfo(MessageInfo^ _val);
			MessageInfo(String^ _me, MYWORDPTR _val);
			MessageInfo(String^ _me, MYWORDPTR _va, String^ _st);
			MessageInfo(String^ _me, MYWORDPTR _va1, MYWORDPTR _va2, MYWORDPTR _va3, String^ _st1, String^ _st2, String^ _st3);
			MessageInfo(MyMath::MessageLevel _level, String^ _message);
			//MessageInfo(NativeMessage* _natmes2);
			static MessageInfo GetInfo(MessageInfo^ _val){

				MessageInfo _retval=*gcnew MessageInfo();

				//_retval._natmes->SetPtr(_val->_natmes->GetPtr());
				//_retval._natmes->SetIndex(_val->_natmes->GetIndex());
				//_retval._natmes->SetMes(_val->_natmes->GetMes());

				//_val->_natmes->SetUsed(true);

				return _retval;

			}
			static MessageInfo^ CopyInfo(MessageInfo^ _val){

				MessageInfo^ _retval=gcnew MessageInfo();

				//_retval->_natmes->SetPtr(_val->_natmes->GetPtr());
				//_retval->_natmes->SetIndex(_val->_natmes->GetIndex());
				//_retval->_natmes->SetMes(_val->_natmes->GetMes());

				//_val->_natmes->SetUsed(true);

				return _retval;

			}


			~MessageInfo(){

				this->!MessageInfo();

			}
			!MessageInfo(){

				delete[] _v1;
				delete[] _v2;
				delete[] _v3;
				delete[] _r;

			}
			MessageInfo^ operator=(MessageInfo^ _val);
			MessageInfo^ operator=(MessageInfo% _val);
			static bool operator==(MessageInfo^ _val1, MessageInfo^ _val2);
			static bool operator!=(MessageInfo^ _val1, MessageInfo^ _val2);
			property String^ Message{
				String^ get(){return _message;}
				void set(String^ value){_message=value;}
			}
			property TraceLevel Level{
				TraceLevel get(){return _level;}
				void set(TraceLevel value){_level=value;}
			}
			property Microsoft::Office::Interop::Word::WdColor Color{
				Microsoft::Office::Interop::Word::WdColor get(){return _thecolor;}
				void set(Microsoft::Office::Interop::Word::WdColor value){_thecolor=value;}
			}
			property bool Bold{
				bool get(){return _bold;}
				void set(bool value){_bold=value;}
			}
			property bool Underline{
				bool get(){return _underline;}
				void set(bool value){_bold=value;}
			}
			property bool Italic{
				bool get(){return _italic;}
				void set(bool value){_italic=value;}
			}
			property bool Used{
				bool get(){return _used;}
				void set(bool value){_used=value;}
			}
			property MYWORDPTR V1{
				MYWORDPTR get(){return _v1;}
				void set(MYWORDPTR _val){_v1=_val;}
			}
			property MYWORDPTR V2{
				MYWORDPTR get(){return _v2;}
				void set(MYWORDPTR _val){_v2=_val;}
			}
			property MYWORDPTR V3{
				MYWORDPTR get(){return _v3;}
				void set(MYWORDPTR _val){_v3=_val;}
			}
			property MYWORDPTR R{
				MYWORDPTR get(){return _r;}
				void set(MYWORDPTR _val){_r=_val;}
			}
			property unsigned int Ref{
				unsigned int get(){return _ref;}
				void set(unsigned int _val){_ref=_val;}
			}

		};

	//!This class contains the message pump for trace messages
	[Guid("99FD0F32-54B9-46C1-852B-ECDC92ADD188")]
	public ref class Trace{

	private:

		//static NativeQue* _myq2=new NativeQue();
		static System::Collections::Queue^ _myq=gcnew System::Collections::Queue(1000,10);
		static System::Collections::Queue^ _stringq=gcnew System::Collections::Queue(1000,10);

		static System::Threading::Thread^ _mythread=gcnew System::Threading::Thread(gcnew System::Threading::ThreadStart(&MyMath::Trace::DoMessage));
		static System::Threading::Thread^ _doq=gcnew System::Threading::Thread(gcnew System::Threading::ThreadStart(&MyMath::Trace::MoveMessage));
		static System::Threading::Thread^ _saveit=gcnew System::Threading::Thread(gcnew System::Threading::ThreadStart(&MyMath::Trace::SaveIt));
		
		static bool _runpump=true;
		static bool _isready=false;
		static bool _holdthreads=false;
		static bool _imout=false;
		static bool _stillactive=true;
		static bool _endit=false;
		static bool _runpump2=true;
		static bool _messagewait=false;
		static bool _endnow=false;
		static bool _initialized=false;

		static int _entrycount=0;

		static Form^ _form=gcnew Form();
		static System::Windows::Forms::Label^  _message = (gcnew System::Windows::Forms::Label());;
		static System::Windows::Forms::Label^  _count = (gcnew System::Windows::Forms::Label());;
		static System::Windows::Forms::Label^  _lbl = (gcnew System::Windows::Forms::Label());;
		static System::Windows::Forms::Button^ btnEndNow = (gcnew System::Windows::Forms::Button());

		static void DoMessage();
		static void SetLabelText(System::Windows::Forms::Label^ _label,String^ _mes);
		static System::Void btnEndNow_Click(System::Object^  sender, System::EventArgs^  e);

	public:

		static property bool EndIt{
			bool get(){return _endit;}
			void set(bool value){_endit=value;}
		}
		static property unsigned long EntryCount{
			unsigned long get(){return _entrycount;}
		}
		static property bool StillActive{
			bool get(){return _stillactive;}
		}
		static property bool WaitState{
			bool get(){return _messagewait;}
			void set(bool value){_messagewait=value;}
		}
		//!
		/// Overall value for setting class wide tracing
		/// 
		static TraceLevel TheTraceLevel=TraceLevel::Basic;
		//!
		/// whether trace is active or not: Default=false
		/// 
		static bool _trace=false;
		//static Trace();
		static Trace(){

			_runpump=false;
			_runpump2=false;
			_stillactive=false;

			//NativeQue::NInit();

			//if(!_initialized){

			//	_form->SuspendLayout();

			//	_message->Location = System::Drawing::Point(490, 106);
			//	_message->Name = L"_message";
			//	_message->Size = System::Drawing::Size(406, 13);
			//	_message->TabIndex = 206;
			//	_message->Text = L"Please Wait while I finish processing the entries into Word, this could take a few minutes";
			//	_message->TextAlign = System::Drawing::ContentAlignment::MiddleCenter;

			//	_lbl->Location = System::Drawing::Point(490, 184);
			//	_lbl->Name = L"lbl";
			//	_lbl->Size = System::Drawing::Size(406, 13);
			//	_lbl->TabIndex = 206;
			//	_lbl->Text = L"Entries Left To Process";
			//	_lbl->TextAlign = System::Drawing::ContentAlignment::MiddleCenter;

			//	_count->Location = System::Drawing::Point(490, 197);
			//	_count->Name = L"_count";
			//	_count->Size = System::Drawing::Size(406, 13);
			//	_count->TabIndex = 206;
			//	_count->Text = L"0";
			//	_count->TextAlign = System::Drawing::ContentAlignment::MiddleCenter;

			//	btnEndNow->Location = System::Drawing::Point(657, 252);
			//	btnEndNow->Name = L"btnEndNow";
			//	btnEndNow->Size = System::Drawing::Size(75, 23);
			//	btnEndNow->TabIndex = 207;
			//	btnEndNow->Text = L"End Now";
			//	btnEndNow->UseVisualStyleBackColor = true;
			//	btnEndNow->Click += gcnew System::EventHandler(&Trace::btnEndNow_Click);

			//	_form->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			//	_form->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			//	_form->ClientSize = System::Drawing::Size(1461, 726);
			//	_form->Controls->Add(btnEndNow);
			//	_form->Controls->Add(_message);
			//	_form->Controls->Add(_lbl);
			//	_form->Controls->Add(_count);

			//	_form->Name = L"_form";
			//	_form->Text = L"Please Wait";
			//	_form->ResumeLayout(false);

			//	_initialized=true;

			//}

			//_mythread->Name="TraceMain";
			//_mythread->Start();

			//_doq->Name="DoQ";
			//_doq->Start();

			//_saveit->Name="SaveIt";
			//_saveit->Start();

		}
		~Trace(){

			_runpump2=false;

		}
		static void StartTrace(TraceLevel tracelevel);
		//!Routine to send trace message.
		/*!
		
		\param level Describes the trace level to use
		\param _message Message to post to trace file
		\returns void
		*/
		static void SendTraceMessage(int level, String^ _message, Microsoft::Office::Interop::Word::WdColor _thecolor,bool _bold, bool _underline, bool _italic);
		static void SendTraceMessage(String^ _message);
		static void SendTraceMessage(int level, int _sel, String^ _message);
		static void SendTraceMessage(MessageInfo^ _mi);
		static void EndTracing();
		static Microsoft::Office::Interop::Word::WdColor GetAColor();
		static void Mes(MYWORDPTR _val1, MYWORDPTR _val2, MYWORDPTR _val3){
			
			MyMath::MessageInfo^ _mess=gcnew MyMath::MessageInfo();

			_mess->Used=false;


		}
		static void MoveMessage();
		static void Save2(String^ _me);
		static void SaveIt();

	};

	//!This class is a container for accessing the global values in the assembly language code
	[Guid("5D9FDD39-E103-4C24-9EDB-BAFE98EE1A26")]
	public ref class OtherCrap{

		private:

			static MyMath::Radix _base=Radix::_Decimal;
			static MyMath::Radix _stringformat=Radix::_Decimal;
			static bool _usecapitols=false;
			static bool _usestable=true;
			static bool _usecommas=false;

		public:

			static MYWORD MemCount(){

				return GetMemCount();

			}

			static int GetVal(String^ _val, MyMath::Radix _rad){

				array<String^,1>^ _strings;

				if((int)_rad==64){

					_strings=gcnew array<String^,1>{"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z","0","1","2","3","4","5","6","7","8","9","+","/"};

				}

				else if((int)_rad==32){

					_strings=gcnew array<String^,1>{"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z","2","3","4","5","6","7","="};

				}

				switch((int)_rad){

					case 64:

						for(int x=0;x<63;x++){

							if(_val==_strings[x]){

								return x;
								break;

							}

						}
						
						break;

					case 32:

						for(int x=0;x<33;x++){

							if(_val==_strings[x]){

								return x;
								break;

							}

						}

						break;

					case 16:

						if(_val=="F" || _val=="f"){

							return 15;

						}

						else if(_val=="E" || _val=="e"){

							return 14;

						}

						else if(_val=="D" || _val=="d"){

							return 13;

						}

						else if(_val=="C" || _val=="c"){

							return 12;

						}

						else if(_val=="B" || _val=="b"){

							return 11;

						}

						else if(_val=="A" || _val=="a"){

							return 10;

						}

						else{

							return System::Convert::ToInt32(_val);

						}

						break;

					case 10:

						if(_val=="9"){

							return 9;

						}

						else if(_val=="8"){

							return 8;

						}

						else if(_val=="7"){

							return 7;

						}

						else if(_val=="6"){

							return 6;

						}

						else if(_val=="5"){

							return 5;

						}

						else if(_val=="4"){

							return 4;

						}

						else if(_val=="3"){

							return 3;

						}

						else if(_val=="2"){

							return 2;

						}

						else if(_val=="1"){

							return 1;

						}

						else if(_val=="0"){

							return 0;

						}

						else{

							return 99;

						}

						break;

					case 8:

						if(_val=="7"){

							return 7;

						}

						else if(_val=="6"){

							return 6;

						}

						else if(_val=="5"){

							return 5;

						}

						else if(_val=="4"){

							return 4;

						}

						else if(_val=="3"){

							return 3;

						}

						else if(_val=="2"){

							return 2;

						}

						else if(_val=="1"){

							return 1;

						}

						else if(_val=="0"){

							return 0;

						}

						else{

							return 99;

						}

						break;

					case 2:

						if(_val=="1"){

							return 1;

						}

						else if(_val=="0"){

							return 0;

						}

						else{

							return 99;

						}

						break;

				}

			}
			static String^ GetString(int _val, MyMath::Radix _rad){

				switch((int)_rad){

					case 64:

						switch(_val){

							case 63:

								return "/";
								break;

							case 62:

								return "+";
								break;

							case 61:

								return "9";
								break;

							case 60:

								return "8";
								break;

							case 59:

								return "7";
								break;

							case 58:

								return "6";
								break;

							case 57:

								return "5";
								break;

							case 56:

								return "4";
								break;

							case 55:

								return "3";
								break;

							case 54:

								return "2";
								break;

							case 53:

								return "1";
								break;

							case 52:

								return "0";
								break;

							case 51:

								return "z";
								break;

							case 50:

								return "y";
								break;

							case 49:

								return "x";
								break;

							case 48:

								return "w";
								break;

							case 47:

								return "v";
								break;

							case 46:

								return "u";
								break;

							case 45:

								return "t";
								break;

							case 44:

								return "s";
								break;

							case 43:

								return "r";
								break;

							case 42:

								return "q";
								break;

							case 41:

								return "p";
								break;

							case 40:

								return "o";
								break;

							case 39:

								return "n";
								break;

							case 38:

								return "m";
								break;

							case 37:

								return "l";
								break;

							case 36:

								return "k";
								break;

							case 35:

								return "j";
								break;

							case 34:

								return "i";
								break;

							case 33:

								return "h";
								break;

							case 32:

								return "g";
								break;

							case 31:

								return "f";
								break;

							case 30:

								return "e";
								break;

							case 29:

								return "d";
								break;

							case 28:

								return "c";
								break;

							case 27:

								return "b";
								break;

							case 26:

								return "a";
								break;

							case 25:

								return "Z";
								break;

							case 24:

								return "Y";
								break;

							case 23:

								return "X";
								break;

							case 22:

								return "W";
								break;

							case 21:

								return "V";
								break;

							case 20:

								return "U";
								break;

							case 19:

								return "T";
								break;

							case 18:

								return "S";
								break;

							case 17:

								return "R";
								break;

							case 16:

								return "Q";
								break;

							case 15:

								return "P";
								break;

							case 14:

								return "O";
								break;

							case 13:

								return "N";
								break;

							case 12:

								return "M";
								break;

							case 11:

								return "L";
								break;

							case 10:

								return "K";
								break;

							case 9:

								return "J";
								break;

							case 8:

								return "I";
								break;

							case 7:

								return "H";
								break;

							case 6:

								return "G";
								break;

							case 5:

								return "F";
								break;

							case 4:

								return "E";
								break;

							case 3:

								return "D";
								break;

							case 2:

								return "C";
								break;

							case 1:

								return "B";
								break;

							case 0:

								return "A";
								break;

						}

						break;

					case 32:

						switch(_val){

							case 31:

								return "7";
								break;

							case 30:

								return "6";
								break;

							case 29:

								return "5";
								break;

							case 28:

								return "4";
								break;

							case 27:

								return "3";
								break;

							case 26:

								return "2";
								break;

							case 25:

								return "Z";
								break;

							case 24:

								return "Y";
								break;

							case 23:

								return "X";
								break;

							case 22:

								return "W";
								break;

							case 21:

								return "V";
								break;

							case 20:

								return "U";
								break;

							case 19:

								return "T";
								break;

							case 18:

								return "S";
								break;

							case 17:

								return "R";
								break;

							case 16:

								return "Q";
								break;

							case 15:

								return "P";
								break;

							case 14:

								return "O";
								break;

							case 13:

								return "N";
								break;

							case 12:

								return "M";
								break;

							case 11:

								return "L";
								break;

							case 10:

								return "K";
								break;

							case 9:

								return "J";
								break;

							case 8:

								return "I";
								break;

							case 7:

								return "H";
								break;

							case 6:

								return "G";
								break;

							case 5:

								return "F";
								break;

							case 4:

								return "E";
								break;

							case 3:

								return "D";
								break;

							case 2:

								return "C";
								break;

							case 1:

								return "B";
								break;

							case 0:

								return "A";
								break;

						}

						break;

					case 16:

						switch(_val){

							case 15:

								return "F";
								break;

							case 14:

								return "E";
								break;

							case 13:

								return "D";
								break;

							case 12:

								return "C";
								break;

							case 11:

								return "B";
								break;

							case 10:

								return "A";
								break;

							case 9:

								return "9";
								break;

							case 8:

								return "8";
								break;

							case 7:

								return "7";
								break;

							case 6:

								return "6";
								break;

							case 5:

								return "5";
								break;

							case 4:

								return "4";
								break;

							case 3:

								return "3";
								break;

							case 2:

								return "2";
								break;

							case 1:

								return "1";
								break;

							case 0:

								return "0";
								break;

						}

						break;

					case 10:

						switch(_val){

							case 9:

								return "9";
								break;

							case 8:

								return "8";
								break;

							case 7:

								return "7";
								break;

							case 6:

								return "6";
								break;

							case 5:

								return "5";
								break;

							case 4:

								return "4";
								break;

							case 3:

								return "3";
								break;

							case 2:

								return "2";
								break;

							case 1:

								return "1";
								break;

							case 0:

								return "0";
								break;

						}

						break;

					case 8:

						switch(_val){

							case 7:

								return "7";
								break;

							case 6:

								return "6";
								break;

							case 5:

								return "5";
								break;

							case 4:

								return "4";
								break;

							case 3:

								return "3";
								break;

							case 2:

								return "2";
								break;

							case 1:

								return "1";
								break;

							case 0:

								return "0";
								break;

						}

						break;

					case 2:

						switch(_val){

							case 1:

								return "1";
								break;

							case 0:

								return "0";
								break;

						}

						break;

				}

			}
			static String^ RemoveCommas(String^ _val){

				String^ _retval="";

				for(int x=_val->Length-1;x>=0;x--){

					if(_val->Substring(x,1)!=","){

						_retval=_val->Substring(x,1)+_retval;

					}

				}

				return _retval;

			}
			static String^ DoCommas(String^ _val){

				String^ _retval=_val;

				int _pos=0;
				int _count=0;
				int _mod=_val->Length%3;
				int _offset=0;

				if(_val->Substring(_val->Length-1, 1)=="-"){

					_offset=1;

				}

				while(((_count*3)+_mod+_count+_offset)<_retval->Length){

					_retval=_retval->Substring(0, ((_count*3)+_mod+_count+_offset)-1) + "," + _retval->Substring(((_count*3)+_mod+_count+_offset), _retval->Length-((_count*3)+_mod+_count)-1);

					_count+=1;

				}

				return _retval;

			}

			/*!
			\brief Use commas is usefull in some instances when working with long strings of numbers.  If set to true will insert commas every third digit from the left
			*/
			static property bool UseCommas{
				bool get(){return _usecommas;}
				void set(bool _val){_usecommas=_val;}
			}
			/*!
			\brief Specifies whether capitol letters will be used in the radix forms that use letters
			*/
			static property bool UseCapitols{
				bool get(){return _usecapitols;}
				void set(bool _val){_usecapitols=_val;}
			}
			/*!
			\brief NOT USED ANYMORE: value specifying whether to use the stable versions or the experimental versions of the algorithms
			*/
			static property bool UseStable{
				bool get(){return _usestable;}
				void set(bool _val){_usestable=_val;}
			}
			/*!
			\brief Specifies whether to use my internal memory management routines or use malloc
			*/
			static property bool UseMalloc{
				bool get(){
					
					if(GetMalloc()==1){
						
						return true;
					
					}
				
					else{
					
						return false;
					
					}
				
				}
				void set(bool _val){
									
					if(_val){
					
						UsingMalloc(1);
					
					}
				
					else{
					
						UsingMalloc(0);
					
					}
				
				}
			}
			/*!
			\brief Specifies the default radix for ToString
			*/
			static property MyMath::Radix Base{
				MyMath::Radix get(){return _base;}
				void set(MyMath::Radix _val){_base=_val;}
			}
			/*!
			\brief NOT USED ANYMORE
			*/
			static property MyMath::Radix StringFormat{
				MyMath::Radix get(){return _stringformat;}
				void set(MyMath::Radix _val){_stringformat=_val;}
			}
			/*!
			\brief This value specifies how to handle rounding in the BD and BDI classes
			*/
			static property MyMath::Rounding HowToHandleRounding{
				MyMath::Rounding get(){return (MyMath::Rounding)BDGetHowToHandleRounding();}
				void set(MyMath::Rounding _val){BDChangeHowToHandleRounding((int)_val);}
			}
			/*!
			\brief This value specifies how many digits to the right of the decimal point to keep
			*/
			static property MYWORD DecimalLimit{
				MYWORD get(){return BDGetDecimalLimit();}
				void set(MYWORD _val){BDChangeDecimalLimit(_val);}
			}
			/*!
			///<value>this property controls the block size returned in a request in the memory management routine
			*/
			static property MYWORD AllocationBlockSize{
				MYWORD get(){return GetAllocationSize();}
				void set(MYWORD _val){SetAllocationSize(_val);}
			}
			/*!
			///<value>this property controls the number of blocks it requests when reserving memory from the system,
			*/
			static property MYWORD NumberOfAllocations{
				MYWORD get(){return GetNumberOfAllocations();}
				void set(MYWORD _val){SetNumberOfAllocations(_val);}
			}
			/*!
			///<value>this property controls an inner structure of the memory allocation routine
			*/
			static property MYWORD ResetCount{
				MYWORD get(){return GetResetCount();}
				void set(MYWORD _val){SetResetCount(_val);}
			}
			/*!
			///<value>this property (get only) will tell you the size of memory requested from the system for the structure that keeps track of the memory requests
			*/
			static property MYWORD TableRequestSize{
				MYWORD get(){return GetTableReqSize();}
			}
			/*!
			///<value>this property (get only) will tell you the size of the memory requested from the system to be handed out in the memory request routine
			*/
			static property MYWORD AllocationRequestSize{
				MYWORD get(){return GetAllocReqSize();}
			}

	};

	//!This class is a container for accessing the new algorithms in the assembly code for debugging
	[Guid("6C02DE36-7472-47E6-B6F6-85019C308C08")]
	public ref class Other{

		private:

		public:

			static MYWORDPTR EAdd(MYWORDPTR _val1, MYWORDPTR _val2){

				return AAdd64e(_val1, _val2);

			}
			static MYWORDPTR ESub(MYWORDPTR _val1, MYWORDPTR _val2){

				return ASub64e(_val1, _val2);

			}
			static MYWORDPTR EMul(MYWORDPTR _val1, MYWORDPTR _val2){

				return AMultiply64e(_val1, _val2);

			}
			static MYWORDPTR EDiv(MYWORDPTR _val1, MYWORDPTR _val2){

				return ADivide64e(_val1, _val2);

			}
			static MYWORDPTR EMod(MYWORDPTR _val1, MYWORDPTR _val2){

				return AMod64e(_val1, _val2);

			}
			static MYWORDPTR EPow(MYWORDPTR _base, MYWORDPTR _exp){

				return APow64e(_base, _exp);

			}
			static MYWORDPTR EModPow(MYWORDPTR _base, MYWORDPTR _exponent, MYWORDPTR _modulus){

				return AModPow64e(_base, _exponent, _modulus);

			}
			static MYWORDPTR AAdd(MYWORDPTR _val1, MYWORDPTR _val2){

				return AAdd64(_val1, _val2);

			}
			static MYWORDPTR ASub(MYWORDPTR _val1, MYWORDPTR _val2){

				return ASub64(_val1, _val2);

			}
			static MYWORDPTR AMul(MYWORDPTR _val1, MYWORDPTR _val2){

				return AMultiply64(_val1, _val2);

			}
			static MYWORDPTR ADiv(MYWORDPTR _val1, MYWORDPTR _val2){

				return ADivide64(_val1, _val2);

			}
			static MYWORDPTR AMod(MYWORDPTR _val1, MYWORDPTR _val2){

				return AMod64(_val1, _val2);

			}
			static MYWORDPTR APow(MYWORDPTR _base, MYWORDPTR _exp){

				return APow64(_base, _exp);

			}
			static MYWORDPTR AModPow(MYWORDPTR _base, MYWORDPTR _exponent, MYWORDPTR _modulus){

				return AModPow64(_base, _exponent, _modulus);

			}
			static MYWORD AE(MYWORDPTR _v1, MYWORDPTR _v2){

				return AE64(_v1, _v2);

			}
			static MYWORD ANE(MYWORDPTR _v1, MYWORDPTR _v2){

				return ANE64(_v1, _v2);

			}
			static MYWORD ALT(MYWORDPTR _v1, MYWORDPTR _v2){

				return ALT64(_v1, _v2);

			}
			static MYWORD ALTOE(MYWORDPTR _v1, MYWORDPTR _v2){

				return ALTOE64(_v1, _v2);

			}
			static MYWORD AGT(MYWORDPTR _v1, MYWORDPTR _v2){

				return AGT64(_v1, _v2);

			}
			static MYWORD AGTOE(MYWORDPTR _v1, MYWORDPTR _v2){

				return AGTOE64(_v1, _v2);

			}
			static MYWORDPTR RetMem(MYWORD _req){

				return GetMem(_req);

			}
			static void RetFree(MYWORDPTR _var){

				FreeMem(_var);

			}
			static void Init(){

				Initialize();

			}
			static MYWORDPTR NewMul(MYWORDPTR _val1, MYWORDPTR _val2){

				return AMultiply642(_val1, _val2);

			}
			static MYWORD IE64(CBI* _val1, CBI* _val2){

				return BIE64(_val1, _val2);

			}

	};

	ref class BD;
	ref class BF;
	ref class BDI;
	ref class BFI;

	//!This class is the BigInteger class and only capable of integer values.  There are no classes that have implicit converions to BI, only explicit
	/*!
	This class uses the CBI native class as a container in that the array and sign value are held in an instance of that class internally.
	The managed class does directly call the assembly language functions passing in the CBI class as arguments.
	*/
	[Guid("EA1DB517-0003-494D-ABAE-E1F84FE155FC")]
	public ref class BI : public IComparable
	{

	protected:

		//!This is the container for the value of the instance.  We use the CBI* type so that we can access the assembly language functions by passing this object in.
		CBI* _cbval;
		//static Trace^ _trace=gcnew Trace();

	private:

		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isdone=false;
		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isinit=false;
		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isiniting=false;
		void ProcessString(String^ _tval){

			String^ _val=OtherCrap::RemoveCommas(_tval);

			BI^ _retval=gcnew BI();
			//BI^ _tret=gcnew BI();
			//MYWORDPTR _tret=GetMem64(24);

			//_tret[0]=1;

			String^ _sbase="";
			String^ _exp="";
			String^ _dbase="";

			bool _bneg=false;
			bool _eneg=false;
			bool _dec=false;
			bool _doexp=false;
			bool _doden=false;

			int _deccount=0;

			Radix _thebase=MyMath::Radix::_Decimal;

			for(int x=0;x<_val->Length;x++){

				if(_val->Substring(x,1)=="a" || _val->Substring(x,1)=="A" || _val->Substring(x,1)=="b" || _val->Substring(x,1)=="B" ||  _val->Substring(x,1)=="c" || _val->Substring(x,1)=="C" ||
						_val->Substring(x,1)=="d" || _val->Substring(x,1)=="D" ||  _val->Substring(x,1)=="e" || _val->Substring(x,1)=="E" ||  _val->Substring(x,1)=="f" || _val->Substring(x,1)=="F"){

							if((int)_thebase<16){

								_thebase=MyMath::Radix::_HexaDecimal;

							}

				}

				else if(_val->Substring(x,1)=="G" || _val->Substring(x,1)=="H" || _val->Substring(x,1)=="I" || _val->Substring(x,1)=="J" || _val->Substring(x,1)=="K" || _val->Substring(x,1)=="L" ||
						_val->Substring(x,1)=="M" || _val->Substring(x,1)=="N" || _val->Substring(x,1)=="O" || _val->Substring(x,1)=="P" || _val->Substring(x,1)=="Q" || _val->Substring(x,1)=="R" ||
						_val->Substring(x,1)=="S" || _val->Substring(x,1)=="T" || _val->Substring(x,1)=="U" || _val->Substring(x,1)=="V" || _val->Substring(x,1)=="W" || _val->Substring(x,1)=="X"  ||
						_val->Substring(x,1)=="Y" || _val->Substring(x,1)=="Z"){

							if((int)_thebase<32){

								_thebase=MyMath::Radix::_Base32;

							}

				}

				else if(_val->Substring(x,1)=="g" || _val->Substring(x,1)=="h" || _val->Substring(x,1)=="i" || _val->Substring(x,1)=="j" || _val->Substring(x,1)=="k" || _val->Substring(x,1)=="l" ||
					_val->Substring(x,1)=="m" || _val->Substring(x,1)=="n" || _val->Substring(x,1)=="o" || _val->Substring(x,1)=="p" || _val->Substring(x,1)=="q" || _val->Substring(x,1)=="r"  ||
					_val->Substring(x,1)=="s" || _val->Substring(x,1)=="t" || _val->Substring(x,1)=="u" || _val->Substring(x,1)=="v" || _val->Substring(x,1)=="w" || _val->Substring(x,1)=="x"  ||
					_val->Substring(x,1)=="y" || _val->Substring(x,1)=="z"){

						_thebase=MyMath::Radix::_Base64;
						break;

				}

			}

			for(int x=0;x<_val->Length;x++){

				if(_dec && ! _doexp){

					_deccount+=1;

				}

				if(_val->Substring(x,1)=="-"){

					if(_doexp){

						_eneg=true;

					}

					else{

						_bneg=true;

					}

				}

				else if(_val->Substring(x,1)=="."){

					_dec=true;
					_deccount+=1;

				}

				else if(_val->Substring(x,1)=="e" || _val->Substring(x,1)=="E"){

					_doexp=true;

				}

				else if(_val->Substring(x,1)=="/"){

					_doden=true;

				}

				else if(_val->Substring(x,1)==" "){

				}

				else{

					if(_doexp){

						_exp=_exp+_val->Substring(x,1);

					}

					else if(_doden){

						_dbase=_dbase+_val->Substring(x,1);

					}

					else{

						_sbase=_sbase+_val->Substring(x,1);

					}

				}

			}

			BI^ _base=gcnew BI();
			BI^ _digit=gcnew BI();
			BI^ _pow=gcnew BI();
			BI^ _mult=gcnew BI();

			_base->_cbval->SetValue(0, 1);
			_base->_cbval->SetValue(1, (int)_thebase);

			_mult->_cbval->SetValue(0, 1);
			_mult->_cbval->SetValue(1, (int)_thebase);

			_retval->_cbval->SetValue(0, 1);

			//MYWORDPTR _base=GetMem64(24);
			//MYWORDPTR _digit=GetMem64(24);
			//MYWORDPTR _pow=GetMem64(24);
			//MYWORDPTR _mult=GetMem64(24);

			//_base[0]=1;
			//_base[1]=(int)_thebase;

			//_mult[0]=1;
			//_mult[1]=(int)_thebase;

			//_digit[0]=1;


			if(_sbase->Length<18 && (int)_thebase==10){

				_retval->_cbval->SetValue(1, System::Convert::ToUInt64(_sbase));

			}

			else{

				_retval->_cbval->SetValue(1, 0);

				for(int x=0;x<_sbase->Length;x++){
					
					_digit->_cbval->SetValue(1, System::Convert::ToUInt64(_sbase->Substring(x,1)));
					_retval=_retval*_mult;
					_retval=_retval+_digit;

				}

			}

			if(_doden && _dbase->Length<18 && (int)_thebase==10){

				_digit->_cbval->SetValue(1, System::Convert::ToUInt64(_dbase));
				_digit->_cbval->SetValue(1, 1);

			}

			else{

				_digit->_cbval->SetValue(1, 0);
				_digit->_cbval->SetValue(1, 1);

				for(int x=0;x<_dbase->Length;x++){

					_digit->_cbval->SetValue(1,System::Convert::ToUInt64(_dbase->Substring(x,1)));
					//Free64(_pow);
					//_pow=AMultiply64(_tret, _mult);
					//Free64(_tret);
					//_tret=AAdd64(_pow, _digit);
					////_tret=_tret*_mult;
					////_tret=_tret+_digit;

				}

			}

			if(_doexp){
						
				_digit->_cbval->SetValue(1,System::Convert::ToUInt64(_exp));

				if(_deccount>0){

					if(_eneg){

						_digit->_cbval->SetValue(1, _digit->_cbval->GetValue(1)+_deccount);

						for(int x=0;x<_digit->_cbval->GetValue(1);x++){

							_retval=_retval/_mult;

						}

					}

					else{
						
						if(_deccount<=_digit->_cbval->GetValue(1)){

							_digit->_cbval->SetValue(1, _digit->_cbval->GetValue(1)-_deccount);

							if(_digit->_cbval->GetValue(1)>0){

								for(int x=0;x<=_digit->_cbval->GetValue(1);x++){

									_retval=_retval*_mult;

								}

							}

						}

						else{

							_digit->_cbval->SetValue(1, _deccount-_digit->_cbval->GetValue(1));

							for(int x=0;x<=_digit->_cbval->GetValue(1);x++){

								_retval=_retval/_mult;

							}

						}

					}

				}

				else{

					if(_eneg){

						for(int x=0;x<=_digit->_cbval->GetValue(1);x++){

							_retval=_retval/_mult;

						}

					}

					else{

						for(int x=0;x<_digit->_cbval->GetValue(1);x++){

							_retval=_retval*_mult;

						}

					}

				}

			}

			else{

				if(_deccount>0){

					for(int x=0;x<_deccount;x++){

						_retval=_retval/_mult;

					}

				}

			}

			if(_doden){

				//_retval=gcnew BI(ADivide64(_retval->_cbval->GetArray(), _tret));

			}

			if(_bneg){

				_retval->_cbval->SetSign(1);

			}

			else{

				_retval->_cbval->SetSign(0);

			}

			//Free64(_tret);
			//Free64(_pow);
			//Free64(_digit);
			//Free64(_mult);
			//Free64(_base);

			CBI* _temp=_retval->_cbval;
			_retval->_cbval=_cbval;
			_cbval=_temp;

			//delete _cbval;

			//_cbval=new CBI(_retval->_cbval);

		}
		void Init(){

			if(!_isdone){

				_isdone=true;

			}
			//InitIt();

		}
		bool MillerRabbin(int _cert){

			BI^ primenumber=gcnew BI(_cbval->GetArray());

			BI^ s=gcnew BI(2);
			BI^ r=gcnew BI(1);
			BI^ y=gcnew BI(1);
			BI^ j=gcnew BI(1);

			long i;
			BI^ nminusone=gcnew BI(1);
			BI^ sminusone=gcnew BI(1);
			BI^ temp=gcnew BI(1);
			bool isprime,blndone,found;
			System::Random ^thisrandom=gcnew System::Random();
			
			isprime=true;
			blndone=false;
			found=false;
			nminusone=primenumber-1;

			while (blndone==false){

				temp=Pow(gcnew BI(2),s);

				if (nminusone%temp==0){

					r=nminusone/temp;

					if(r%2==1){
				
						blndone=true;
						found=true;

					}

				}

				if (nminusone<temp){

					blndone=true;

				}

				if(blndone==false){

					s=s+1;

				}

			}

			if(found==false){

				r=nminusone/2;

				if(r%2!=1){

					s=gcnew BI(1);
					temp=gcnew BI(2);
					r=nminusone/temp;

				}

			}

			if (r==0){

				r=nminusone;
				s=gcnew BI();

			}

			sminusone=s-1;

			for (i=1;i<=_cert;i++){

				y=gcnew BI(thisrandom->Next(1000));
				//y=BI::Random(primenumber-1);

				y=BI::ModPow(y,r,primenumber);

				if (y!=1 && y!=nminusone){

					j=gcnew BI(1);

					while (j<=sminusone && y!=nminusone){

						y=BI::ModPow(y,2,primenumber);

						if (y==1){

							isprime=false;
							return isprime;

						}

						j=j+1;

					}

					if (y!=nminusone){

						j=sminusone+1;

					}

				}

			}

			return isprime;

		}

	protected:

	public:

		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (short _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (unsigned short _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ ( int _val ){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (unsigned int _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (long _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (unsigned long _val){

			BI^ _retval=gcnew BI((unsigned __int64)_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (__int64 _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (unsigned __int64 _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (char _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (unsigned char _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (float _val){

			BI^ _retval=gcnew BI();

			_retval->ProcessString(_val.ToString());

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (double _val){

			BI^ _retval=gcnew BI();

			_retval->ProcessString(_val.ToString());

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (String^ _val){

			BI^ _retval=gcnew BI();

			_retval->ProcessString(_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (BigInteger^ _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Implicit cast to BI.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static operator BI^ (MYWORDPTR _val){

			BI^ _retval=gcnew BI(_val);

			FreeMem(_val);

			return _retval;

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A char
		*/
		static explicit operator char(BI^ _val){

			char _retval=0;

			if(_val->_cbval->GetValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetValue(1)%256;
					
				}

				else{

					_retval=_val->_cbval->GetValue(1)%128;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A unsigned char
		*/
		static explicit operator unsigned char(BI^ _val){

			if(_val->_cbval->GetValue(0)>0){

				return _val->_cbval->GetValue(1)%256;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A short
		*/
		static explicit operator short(BI^ _val){

			short _retval=0;

			if(_val->_cbval->GetValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetValue(1)%65536;
					
				}

				else{

					_retval=_val->_cbval->GetValue(1)%32768;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A unsigned short
		*/
		static explicit operator unsigned short(BI^ _val){

			if(_val->_cbval->GetValue(0)>0){

				return _val->_cbval->GetValue(1)%65536;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A int
		*/
		static explicit operator int(BI^ _val){

			int _retval=0;

			if(_val->_cbval->GetValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetValue(1)%4294967296;
					
				}

				else{

					_retval=_val->_cbval->GetValue(1)%2147483648;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A unsigned int
		*/
		static explicit operator unsigned int(BI^ _val){

			if(_val->_cbval->GetValue(0)>0){

				return _val->_cbval->GetValue(1)%4294967296;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A long
		*/
		static explicit operator long(BI^ _val){

			long _retval=0;

			if(_val->_cbval->GetValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetValue(1);
					
				}

				else{

					_retval=_val->_cbval->GetValue(1);
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A unsigned long
		*/
		static explicit operator unsigned long(BI^ _val){

			if(_val->_cbval->GetValue(0)>0){

				return _val->_cbval->GetValue(1);

			}

			else{

				return 0;

			}

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A __int64
		*/
		static explicit operator __int64(BI^ _val){

			__int64 _retval=0;

			if(_val->_cbval->GetValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetValue(1);
					
				}

				else{

					_retval=_val->_cbval->GetValue(1);
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A unsigned __int64
		*/
		static explicit operator unsigned __int64(BI^ _val){

			if(_val->_cbval->GetValue(0)>0){

				return _val->_cbval->GetValue(1);

			}

			else{

				return 0;

			}

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A string object
		*/
		static explicit operator String^(BI^ _val){

			return _val->ToString();

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A BigInteger object
		*/
		static explicit operator BigInteger^(BI^ _val){

			System::Numerics::BigInteger^ _base=gcnew System::Numerics::BigInteger(2);
			System::Numerics::BigInteger^ _mult=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _retval=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _digit=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _pow=gcnew System::Numerics::BigInteger();

			_base=System::Numerics::BigInteger::Pow(*_base, 64);

			for(int x=1;x<=_val->Number[0];x++){

				_pow=System::Numerics::BigInteger::Pow(*_base, x-1);
				_digit=gcnew System::Numerics::BigInteger(_val->Number[x]);
				_mult=*_digit**_pow;
				_retval=*_retval+*_mult;

			}

			if(_val->Sign==1){

				_retval=*_retval**gcnew System::Numerics::BigInteger(-1);

			}

			return _retval;

		}
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A MYWORDPTR object
		*/
		static explicit operator MYWORDPTR(BI^ _val){

			return _val->_cbval->GetArray();

		}
		static explicit operator BD^(BI^ _val);
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static explicit operator BF^(BI^ _val);
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static explicit operator BDI^(BI^ _val);
		//!Explicit cast from BI.
		/*!
		\param _val The value to cast from.
		\returns A BFI object
		*/
		static explicit operator BFI^(BI^ _val);
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(String^ _val){

			ProcessString(_val);
				Init();

		}
		//!CTOR.
		BI(){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();

		}
		//!CTOR.
		BI(bool _nn){

			_cbval=new CBI();

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(array<MYWORD,1>^ _val){	

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			int _len=_val[0]+1;

			MYWORDPTR _num=SGetMem(_len*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			_cbval=new CBI(_num, false);

			SFree(_num);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(__int64 _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			try{

			_cbval->SetValue(1, _val);
			int _pos=1;

			}

			catch(...){


			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(char _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			_cbval->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(unsigned char _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();

			_cbval->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(short _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			_cbval->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(unsigned short _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();
			_cbval->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(int _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			_cbval->SetValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(unsigned int _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();
			_cbval->SetValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(float _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();

			String^ _temp1=_val.ToString();
			String^ _ret="";

			for(int x=0;x<_temp1->Length;x++){

				_ret=_ret+_temp1->Substring(x,1);

			}

			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(unsigned __int64 _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();
			_cbval->SetValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param rhs The value to assign to the instance.
		*/
		BI(BI% rhs){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(rhs._cbval->GetSign()==1){

				_cbval=new CBI(rhs._cbval->GetArray(), true);

			}

			else{

				_cbval=new CBI(rhs._cbval->GetArray(), false);

			}

		}
		//!CTOR.
		/*!
		\param rhs The value to assign to the instance.
		*/
		BI(BI% rhs, bool _nn){


			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(rhs._cbval->GetSign()==1){

				_cbval=new CBI(rhs._cbval->GetArray(), true);

			}

			else{

				_cbval=new CBI(rhs._cbval->GetArray(), false);

			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(BI^ _val, bool _nn){


			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(_val->_cbval->GetSign()==1){

				_cbval=new CBI(_val->_cbval->GetArray(), true);

			}

			else{

				_cbval=new CBI(_val->_cbval->GetArray(), false);

			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(BI^ _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(_val->_cbval->GetSign()==1){

				_cbval=new CBI(_val->_cbval->GetArray(), true);

			}

			else{

				_cbval=new CBI(_val->_cbval->GetArray(), false);

			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(Decimal _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();
			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(double _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBI();

			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(BigInteger^ _val){

			System::Numerics::BigInteger^ _tval=gcnew System::Numerics::BigInteger();
			_tval=_val;

			int _pos=1;
			MYWORD _t1=0;
			MYWORD _t2=0;
			bool _sign=false;

			if(_tval->Sign==-1){

				System::Numerics::BigInteger^ _negone=gcnew System::Numerics::BigInteger(-1);
				_tval=*_tval**_negone;
				_sign=true;

			}

			array<System::Byte,1>^ _tarray=_tval->ToByteArray();

			int _count=_tarray->Length/MYBYTES;

			if(_tarray->Length%MYBYTES!=0){

				_count+=1;

			}

			_count+=1;

			MYWORDPTR _num;

			_num=SGetMem(_count*MYBYTES);

			_count-=1;

			_num[0]=_count;

		#if _WIN64 || __amd64__ || _x64

			for(int x=0;x<_tarray->Length;x++){

				if(x%MYBYTES==0){

					_num[_pos]=_tarray[x];
					_t1=_tarray[x];

				}

				else if(x%MYBYTES==1){

					_t2=_tarray[x];
					_t2=_t2<<8;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==2){

					_t2=_tarray[x];
					_t2=_t2<<16;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==3){

					_t2=_tarray[x];
					_t2=_t2<<24;
					_t1=_t1+_t2;
					
					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==4){

					_t2=_tarray[x];
					_t2=_t2<<32;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==5){

					_t2=_tarray[x];
					_t2=_t2<<40;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==6){

					_t2=_tarray[x];
					_t2=_t2<<48;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==7){

					_t2=_tarray[x];
					_t2=_t2<<56;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

					_pos+=1;
					_t1=0;

				}

			}

		#else

			for(int x=0;x<_tarray->Length;x++){

				if(x%MYBYTES==0){

					_num[_pos]=_tarray[x];
					_t1=_tarray[x];

				}

				else if(x%MYBYTES==1){

					_t2=_tarray[x];
					_t2=_t2<<8;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==2){

					_t2=_tarray[x];
					_t2=_t2<<16;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==3){

					_t2=_tarray[x];
					_t2=_t2<<24;
					_t1=_t1+_t2;
					
					_num[_pos]=_t1;
					_pos+=1;
					_t1=0;

				}

			}

		#endif

			while(_num[_num[0]]==0 && _num[0]>1){

				_num[0]=_num[0]-1;

			}

			_cbval=new CBI(_num, _sign);

			SFree(_num);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(MYWORDPTR _val, bool _s){

			_cbval=new CBI(_val, _s);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(MYWORDPTR _val){

			_cbval=new CBI(_val, false);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BI(CBI* _val){

			_cbval=new CBI(_val);

		}

		~BI(){

			this->!BI();

		}
		!BI(){

			delete _cbval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BI object
		*/
		BI^ operator=(BI^ _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BI object
		*/
		BI^ operator=(System::Decimal _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BI object
		*/
		BI^ operator=(double _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BI object
		*/
		BI^ operator=(BI% rhs){

			BI^ _retval=gcnew BI(rhs);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BI object
		*/
		BI^ operator=(__int64 _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BI object
		*/
		BI^ operator=(unsigned __int64 _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BI object
		*/
		BI^ operator=(BigInteger^ _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BI object
		*/
		BI^ operator=(String^ _val){

			BI^ _retval=gcnew BI(_val);

			return _retval;

		}
		//!Equals.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator==(BI^ _val1, BI^ _val2){

			if(BIE64(_val1->_cbval,_val2->_cbval)==1){

				return true;

			}

			return false;

		}
		//!NE.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator!=(BI^ _val1, BI^ _val2){

			return !(_val1==_val2);

		}
		//!LT.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator<(BI^ _val1, BI^ _val2){

			bool _retval=false;

			if(BILT64(_val1->_cbval,_val2->_cbval)==1){

				return true;

			}

			return false;

		}
		//!LTOE.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator<=(BI^ _val1, BI^ _val2){

			if(_val2<_val1){

				return false;

			}

			else{

				return true;

			}

		}
		//!GT.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator>(BI^ _val1, BI^ _val2){

			if(_val2<_val1){

				return true;

			}

			else{

				return false;

			}

		}
		//!GTOE.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator>=(BI^ _val1, BI^ _val2){

			if(_val1<_val2){

				return false;

			}

			else{

				return true;

			}

		}
		//!Adds 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator+=(BI^ _val1, BI^ _val2){

			return _val1+_val2;
		}
		//!Adds 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator+(BI^ _val1, BI^ _val2){

			CBI* _temp=BIAdd64(_val1->_cbval, _val2->_cbval);
			BI^ _retval=gcnew BI(_temp);

			delete _temp;

			while(_retval->_cbval->GetValue(_retval->_cbval->GetValue(0))==0 && _retval->_cbval->GetValue(0)>1){

				_retval->_cbval->SetValue(0,_retval->_cbval->GetValue(0)-1);

			}

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Subtracts 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator-=(BI^ _val1, BI^ _val2){

			return _val1-_val2;

		}
		//!Subtracts 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator-(BI^ _val1, BI^ _val2){

			CBI* _temp=BISub64(_val1->_cbval, _val2->_cbval);
			BI^ _retval=gcnew BI(_temp);

			delete _temp;

			while(_retval->_cbval->GetValue(_retval->_cbval->GetValue(0))==0 && _retval->_cbval->GetValue(0)>1){

				_retval->_cbval->SetValue(0, _retval->_cbval->GetValue(0)-1);

			}

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Multiplies 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator*=(BI^ _val1, BI^ _val2){

			return _val1*_val2;

		}
		//!Multiplies 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator*(BI^ _val1, BI^ _val2){

			CBI* _temp=BIMultiply64(_val1->_cbval, _val2->_cbval);
			BI^ _retval=gcnew BI(_temp);

			delete _temp;

			while(_retval->_cbval->GetValue(_retval->_cbval->GetValue(0))==0 && _retval->_cbval->GetValue(0)>1){

				_retval->_cbval->SetValue(0, _retval->_cbval->GetValue(0)-1);

			}

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Divides 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator/=(BI^ _val1, BI^ _val2){

			return _val1/_val2;

		}
		//!Divides 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator/(BI^ _val1, BI^ _val2){

			CBI* _temp=BIDivide64(_val1->_cbval, _val2->_cbval);
			BI^ _retval=gcnew BI(_temp);

			delete _temp;

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Modular Reduction.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator%=(BI^ _val1, BI^ _val2){

			return _val1%_val2;

		}
		//!Modular Reduction.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator%(BI^ _val1, BI^ _val2){

			CBI* _temp=BIMod64(_val1->_cbval, _val2->_cbval);
			BI^ _retval=gcnew BI(_temp);

			delete _temp;
	
			while(_retval->_cbval->GetValue(_retval->_cbval->GetValue(0))==0 && _retval->_cbval->GetValue(0)>1){

				_retval->_cbval->SetValue(0, _retval->_cbval->GetValue(0)-1);

			}

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Bitwise AND.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator&(BI^ _val1, BI^ _val2){

			BI^ _retval=gcnew BI(AAnd64(_val1->_cbval->GetArray(), _val2->_cbval->GetArray()));

			return _retval;

		}
		//!Bitwise OR.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator|(BI^ _val1, BI^ _val2){

			BI^ _retval=gcnew BI(AOr64(_val1->_cbval->GetArray(), _val2->_cbval->GetArray()));

			return _retval;

		}
		//!Bitwise Exclusive OR.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BI object
		*/
		static BI^ operator^(BI^ _val1, BI^ _val2){

			BI^ _retval=gcnew BI(AXor64(_val1->_cbval->GetArray(), _val2->_cbval->GetArray()));

			return _retval;

		}
		//!Bit shift left.
		/*!
		\param _val1 The value to shift.
		\param _val2 How many bits to shift.
		\returns A BI object
		*/
		static BI^ operator<<(BI^ _val1, MYWORD _val2){

			BI^ _retval=gcnew BI(ABitShiftLeft64(_val1->_cbval->GetArray(), _val2));

			return _retval;

		}
		//!Bit shift right.
		/*!
		\param _val1 The value to shift.
		\param _val2 How many bits to shift.
		\returns A BI object
		*/
		static BI^ operator>>(BI^ _val1, MYWORD _val2){

			BI^ _retval=gcnew BI(ABitShiftRight64(_val1->_cbval->GetArray(), _val2));

			return _retval;

		}
		//!Calculates the Pow.
		/*!
		\param _base The base value.
		\param _exp The exponent.
		\returns A BI object
		*/
		static BI^ Pow(BI^ _base, BI^ _exp){

			CBI* _temp=BIPow64(_base->_cbval, _exp->_cbval);
			BI^ _retval=gcnew BI(_temp);

			delete _temp;

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Modular Exponentiation.
		/*!
		\param _base The base value.
		\param _exponent The exponent.
		\param _modulus The Modulus.
		\returns A BI object
		*/
		static BI^ ModPow(BI^ _base, BI^ _exponent, BI^ _modulus){

			CBI* _temp=BIModPow64(_base->_cbval, _exponent->_cbval, _modulus->_cbval);
			BI^ _retval=gcnew BI(_temp);

			delete _temp;

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Negates the instance.
		void Negate(){

			if(_cbval->GetSign()==1){

				_cbval->SetSign(0);

			}

			else{

				_cbval->SetSign(1);

			}

		}
		//!Implemented for IComparable.
		/*!
		\param _val The object to compare against.
		\returns A int
		*/
		virtual int CompareTo(Object^ _val){

			BI^ _temp = dynamic_cast<BI^>(_val);

			int _retval=1;

			return _retval;

		}
		//!Sets the instance equal to zero.
		void SetToZero(){

			_cbval->SetToZero();

		}
		//!Sets the instance equal to one.
		void SetToOne(){

			_cbval->SetToOne();

		}

#pragma region Properties

		/*!
		\brief This is the native version of the BI class (CBI*).  It is used within the managed class as a container
		\returns Returns a CBI* object 
		*/
		property CBI* Val{
			CBI* get(){return _cbval;}
			void set(CBI* _val){_cbval->ReplaceAndDelete(_val);}
		}
		/*!
		\brief This is the native array that holds the value for this instance of the class
		\returns Returns a MYWORDPTR 
		*/
		property MYWORDPTR Number{
			MYWORDPTR get(){return _cbval->GetArray();}
			void set(MYWORDPTR value){_cbval->SetArray(value);}
		}
		/*!
		\brief This gives you access to the individual elements within the native array
		\returns Returns a MYWORD 
		*/
		property MYWORD Number[int]{
			MYWORD get(int _index){return _cbval->GetValue(_index);}
			void set(int _index, MYWORD _val){_cbval->SetValue(_index, _val);}
		}
		/*!
		\brief This is the value that represents the sign of the instance 1==Negative 0==Positive
		\returns Returns a MYWORD 
		*/
		property MYWORD Sign{
			MYWORD get(){return _cbval->GetSign();}
			void set(MYWORD _val){_cbval->SetSign(_val);}
		}
		/*!
		\brief A boolean value to tell you if this number is Negative
		\returns Returns a boolean 
		*/
		property bool IsNegative{
			bool get(){if(_cbval->GetSign()==1){
				return true;}return false;}
		}
		/*!
		\brief A boolean value to tell you if this number is equal to zero
		\returns Returns a boolean 
		*/
		property bool IsZero{
			bool get(){
				
				if(_cbval->GetValue(0)<1 || (_cbval->GetValue(0)==1 && _cbval->GetValue(1)==0)){

					return true;
				
				}
				
				for(int x=1;x<=_cbval->GetValue(0);x++){

					if(_cbval->GetValue(x)!=0){

						return false;
						break;

					}

				}

				return true;
			}
		}
		/*!
		\brief A boolean value to tell you if this number is equal to one
		\returns Returns a boolean 
		*/
		property bool IsOne{
			bool get(){if(_cbval->GetValue(0)==1 && _cbval->GetValue(1)==1){
				return true;}return false;
			}
		}
		/*!
		\brief A boolean value to tell you if this number is odd
		\returns Returns a boolean 
		*/
		property bool IsOdd{
			bool get(){if(_cbval->GetValue(1)%2==1){
				return true;}return false;}
		}

#pragma endregion properties

		MYWORD IntVal(){

			MYWORD _retval=0;

			if(_cbval->GetValue(0)==1){

				_retval=_cbval->GetValue(1);

			}

			else if(_cbval->GetValue(0)>=2){

				_retval=_cbval->GetValue(2)*MAX+_cbval->GetValue(1);
				_retval=_retval+1;

			}

			MYWORD _mod=0;

			_mod=_mod-1;

			_retval=_retval%_mod;

			return _retval;

		}
		//!Tests the primality of the instance.
		/*!
		\param _cert The number of cycles to run the algorithm.
		\returns A bool
		*/
		bool IsPrime(int _cert){

			return MillerRabbin(_cert);

		}
		//!Returns the string representation of the value.
		/*!
		\returns A string object
			*/
		virtual String^ ToString()override{

			return ToString(OtherCrap::Base);

		}
		//!Returns the string representation of the value.
		/*!
		\param _tbase The radix to use to calculate the string.
		\returns A string object
		*/
		String^ ToString(MyMath::Radix _tbase){

			String^ _retval="";

			if((int)_tbase==10){

				System::Numerics::BigInteger^ _bbase=gcnew System::Numerics::BigInteger(2);
				System::Numerics::BigInteger^ _pow=gcnew System::Numerics::BigInteger();
				System::Numerics::BigInteger^ _mult=gcnew System::Numerics::BigInteger();
				System::Numerics::BigInteger^ _digit=gcnew System::Numerics::BigInteger();
				System::Numerics::BigInteger^ _bb=gcnew System::Numerics::BigInteger();
				System::Numerics::BigInteger^ _no=gcnew System::Numerics::BigInteger(-1);

				_bbase=System::Numerics::BigInteger::Pow(*_bbase, MYBITS);

				for(int x=1;x<=_cbval->GetValue(0);x++){

					_digit=gcnew System::Numerics::BigInteger(_cbval->GetValue(x));
					_pow=System::Numerics::BigInteger::Pow(*_bbase,(x-1));
					_mult=*_digit**_pow;
					_bb=*_bb+*_mult;

				}

				if(_bb->Sign<0){

					_bb=*_bb*_bb->MinusOne;

				}

				_retval=_bb->ToString();

				if(_cbval->GetSign()==1){

					_retval="-" + _retval;

				}

			}

			else{

				int _bits=(int)_tbase-1;
				int _lshift=0;
				int _rshift=0;
				int _offset=0;
				int _byte=1;
				int _topbit=0;
				unsigned __int64 _tbyte1=0;
				unsigned __int64 _tbyte2=0;
				unsigned __int64 _mask1=_bits;
				unsigned __int64 _mask2=_bits;
				unsigned __int64 _temp1=0;
				unsigned __int64 _temp2=0;

				if(_bits==63){

					_lshift=6;

				}

				else if(_bits==31){

					_lshift=5;

				}

				else if(_bits==15){

					_lshift=4;

				}

				else{

					_lshift=3;

				}

				_topbit=_lshift;

				while(_byte<=_cbval->GetValue(0)){

					while(_topbit<=MYBITS){

						_temp1=_cbval->GetValue(_byte) & _mask1;
						_rshift=_topbit-_lshift;
						_temp1=_temp1>>_rshift;

						_retval=OtherCrap::GetString(_temp1, _tbase) + _retval;

						_mask1=_mask1<<_lshift;
						_topbit+=_lshift;

					}

					if(_mask1==0){

						_mask1=_lshift;
						_topbit=_lshift;
						_byte+=1;

					}

					else{

						_temp1=_cbval->GetValue(_byte) & _mask1;
						_rshift=_topbit-_lshift;
						_temp1=_temp1>>_rshift;
						_byte+=1;

						if(_byte<=_cbval->GetValue(0)){

							_mask2=_lshift;
							_rshift=_lshift-(_topbit-MYBITS);
							_mask2=_mask2>>_rshift;
							_temp2=_cbval->GetValue(_byte) & _mask2;
							_temp1=_temp1+_temp2;

						}

						_retval=OtherCrap::GetString(_temp1, _tbase);
						_mask1=_lshift;
						_rshift=_lshift-_rshift;

					}

				}

				if(_cbval->GetSign()==1){

					_retval="-" + _retval;

				}

			}

			return _retval;

		}
		//!Returns the absolute value of the object.
		/*!
		\returns A BI object
		*/
		BI^ ABS(){

			BI^ _temp=gcnew BI(_cbval->GetArray());

			_temp->_cbval->SetSign(0);

			return _temp;

		}
		//!Calculates the Least Common Multiple.
		/*!
		\param _val1 The first value.
		\param _val The second value.
		\returns A BI object
		*/
		static BI^ LCM(BI^ _val1, BI^ _val2){

			//BI^ _retval=gcnew BI(BILCM64(_val1->_cbval, _val2->_cbval));

			BI^ _retval=gcnew BI();

			_retval=_val1*_val2;

			_retval=_retval/GCD(_val1, _val2);

			return _retval;

		}
		//!Calculates the Greatest Common Divisor.
		/*!
		\param _val1 The first value.
		\param _val The second value.
		\returns A BI object
		*/
		static BI^ GCD(BI^ _val1, BI^ _val2){

			//BI^ _retval=gcnew BI(BIGCD64(_val1->_cbval, _val2->_cbval));

			//return _retval;

			BI^ _a;
			BI^ _b;

			if(_val1>=_val2){

				_a=gcnew BI(_val1);
				_b=gcnew BI(_val2);

			}

			else{

				_a=gcnew BI(_val2);
				_b=gcnew BI(_val1);

			}

			BI^ _r=gcnew BI();

			while(_b>0){

				_r=_a%_b;
				_a=_b;
				_b=_r;

			}

			return _a;

		}
		MYWORD operator[](int _pos){

			return _cbval->GetValue(_pos);

		}
					
	};
	//!This class is the BigDecimal class which is capable of holding all real numbers.  Only the BI class has an implicit converion to BD 
	/*!
	this class uses an instance of the CBD class as a container only.  It accesses the assembly language routines directly, just passing in the CBD objects as paramaters
	*/
	[Guid("E54300B0-6988-452B-85E8-3EE1A810BB8C")]
	public ref class BD : public IComparable
	{

	protected:

		//!This is the container for the value of the instance.  We use the CBD* type so that we can access the assembly language functions by passing this object in.
		CBD* _cbval;
		//static Trace^ _trace=gcnew Trace();

	private:

		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isdone=false;
		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isinit=false;
		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isiniting=false;
		void ProcessString(String^ _tval){

			String^ _val=OtherCrap::RemoveCommas(_tval);

			BD^ _retval=gcnew BD();
			BD^ _tret=gcnew BD();

			String^ _sbase="";
			String^ _exp="";
			String^ _dbase="";

			bool _bneg=false;
			bool _eneg=false;
			bool _dec=false;
			bool _doexp=false;
			bool _doden=false;

			int _deccount=0;

			Radix _thebase=MyMath::Radix::_Decimal;

			for(int x=0;x<_val->Length;x++){

				if(_val->Substring(x,1)=="a" || _val->Substring(x,1)=="A" || _val->Substring(x,1)=="b" || _val->Substring(x,1)=="B" ||  _val->Substring(x,1)=="c" || _val->Substring(x,1)=="C" ||
						_val->Substring(x,1)=="d" || _val->Substring(x,1)=="D" ||  _val->Substring(x,1)=="e" || _val->Substring(x,1)=="E" ||  _val->Substring(x,1)=="f" || _val->Substring(x,1)=="F"){

							if((int)_thebase<16){

								_thebase=MyMath::Radix::_HexaDecimal;

							}

				}

				else if(_val->Substring(x,1)=="G" || _val->Substring(x,1)=="H" || _val->Substring(x,1)=="I" || _val->Substring(x,1)=="J" || _val->Substring(x,1)=="K" || _val->Substring(x,1)=="L" ||
						_val->Substring(x,1)=="M" || _val->Substring(x,1)=="N" || _val->Substring(x,1)=="O" || _val->Substring(x,1)=="P" || _val->Substring(x,1)=="Q" || _val->Substring(x,1)=="R" ||
						_val->Substring(x,1)=="S" || _val->Substring(x,1)=="T" || _val->Substring(x,1)=="U" || _val->Substring(x,1)=="V" || _val->Substring(x,1)=="W" || _val->Substring(x,1)=="X"  ||
						_val->Substring(x,1)=="Y" || _val->Substring(x,1)=="Z"){

							if((int)_thebase<32){

								_thebase=MyMath::Radix::_Base32;

							}

				}

				else if(_val->Substring(x,1)=="g" || _val->Substring(x,1)=="h" || _val->Substring(x,1)=="i" || _val->Substring(x,1)=="j" || _val->Substring(x,1)=="k" || _val->Substring(x,1)=="l" ||
					_val->Substring(x,1)=="m" || _val->Substring(x,1)=="n" || _val->Substring(x,1)=="o" || _val->Substring(x,1)=="p" || _val->Substring(x,1)=="q" || _val->Substring(x,1)=="r"  ||
					_val->Substring(x,1)=="s" || _val->Substring(x,1)=="t" || _val->Substring(x,1)=="u" || _val->Substring(x,1)=="v" || _val->Substring(x,1)=="w" || _val->Substring(x,1)=="x"  ||
					_val->Substring(x,1)=="y" || _val->Substring(x,1)=="z"){

						_thebase=MyMath::Radix::_Base64;
						break;

				}

			}

			for(int x=0;x<_val->Length;x++){

				if(_dec && ! _doexp){

					_deccount+=1;

				}

				if(_val->Substring(x,1)=="-"){

					if(_doexp){

						_eneg=true;

					}

					else{

						_bneg=true;

					}

				}

				else if(_val->Substring(x,1)=="."){

					_dec=true;
					//_deccount+=1;

				}

				else if(_val->Substring(x,1)=="e" || _val->Substring(x,1)=="E"){

					_doexp=true;

				}

				else if(_val->Substring(x,1)=="/"){

					_doden=true;

				}

				else if(_val->Substring(x,1)==" "){

				}

				else{

					if(_doexp){

						_exp=_exp+_val->Substring(x,1);

					}

					else{

						if(_doden){

							_dbase=_dbase+_val->Substring(x,1);

						}

						else{

							_sbase=_sbase+_val->Substring(x,1);

						}

					}

				}

			}

			BD^ _base=gcnew BD();
			BD^ _digit=gcnew BD();
			BD^ _pow=gcnew BD();
			BD^ _mult=gcnew BD();

			_base->_cbval->SetValue(0, 1);
			_base->_cbval->SetValue(1, (int)_thebase);

			_mult->_cbval->SetValue(0, 1);
			_mult->_cbval->SetValue(1, (int)_thebase);

			_retval->_cbval->SetValue(0, 1);

			if(_sbase->Length<18 && (int)_thebase==10){

				_retval->_cbval->SetValue(1, System::Convert::ToUInt64(_sbase));

			}

			else{

				_retval->_cbval->SetValue(1, 0);

				for(int x=0;x<_sbase->Length;x++){

					_digit->_cbval->SetValue(1, System::Convert::ToUInt64(_sbase->Substring(x,1)));
					_retval=_retval*_mult;
					_retval=_retval+_digit;

				}

			}

			if(_doden && _dbase->Length<18 && (int)_thebase==10){

				_tret->_cbval->SetValue(1, System::Convert::ToUInt64(_dbase));

			}

			else{

				_tret->_cbval->SetValue(1, 0);

				for(int x=0;x<_dbase->Length;x++){

					_digit->_cbval->SetValue(1, System::Convert::ToUInt64(_dbase->Substring(x,1)));
					_tret=_tret*_mult;
					_tret=_tret+_digit;

				}

			}

			if(_doexp){
						
				_digit->_cbval->SetValue(1, System::Convert::ToUInt64(_exp));

				if(_deccount>0){

					if(_eneg){

						_deccount=_deccount+_digit->_cbval->GetValue(1);

					}

					else{

						if(_deccount<=_digit->_cbval->GetValue(1)){

							_digit->_cbval->SetValue(1, _digit->_cbval->GetValue(1)-_deccount);

							if(_digit->_cbval->GetValue(1)>0){

								for(int x=0;x<=_digit->_cbval->GetValue(0);x++){

									_retval=_retval*_mult;

								}

							}

							_deccount=0;

						}

						else{

							_deccount=_deccount-_digit->_cbval->GetValue(1);

						}

					}

				}

				else{

					if(_eneg){

						for(int x=0;x<=_digit->_cbval->GetValue(0);x++){

							_retval=_retval/_mult;

						}

					}

					else{

						for(int x=0;x<_digit->_cbval->GetValue(0);x++){

							_retval=_retval*_mult;

						}

					}

				}

			}

			else{

			}

			if(_bneg){

				_retval->_cbval->SetSign(1);

			}

			else{

				_retval->_cbval->SetSign(0);

			}

			if(_doden){

				_retval==gcnew BD(BDDivide64(_retval->_cbval, _tret->_cbval));

			}

			delete _cbval;

			_cbval=new CBD(_retval->_cbval);
			_cbval->SetDecimalPlace(_deccount);

		}
		void Init(){

			if(!_isdone){

				_isdone=true;

			}
			//InitIt();

		}
		bool MillerRabbin(int _cert){

			BD^ primenumber=gcnew BD(_cbval);

			BD^ s=gcnew BD(2);
			BD^ r=gcnew BD(1);
			BD^ y=gcnew BD(1);
			BD^ j=gcnew BD(1);

			long i;
			BD^ nminusone=gcnew BD(1);
			BD^ sminusone=gcnew BD(1);
			BD^ temp=gcnew BD(1);
			bool isprime,blndone,found;
			System::Random ^thisrandom=gcnew System::Random();
			
			isprime=true;
			blndone=false;
			found=false;
			nminusone=primenumber-1;

			while (blndone==false){

				temp=Pow(gcnew BD(2),s);

				if (nminusone%temp==0){

					r=nminusone/temp;

					if(r%2==1){
				
						blndone=true;
						found=true;

					}

				}

				if (nminusone<temp){

					blndone=true;

				}

				if(blndone==false){

					s=s+1;

				}

			}

			if(found==false){

				r=nminusone/2;

				if(r%2!=1){

					s=gcnew BD(1);
					temp=gcnew BD(2);
					r=nminusone/temp;

				}

			}

			if (r==0){

				r=nminusone;
				s=gcnew BD();

			}

			sminusone=s-1;

			for (i=1;i<=_cert;i++){

				y=gcnew BD(thisrandom->Next(1000));
				//y=BD::Random(primenumber-1);

				y=BD::ModPow(y,r,primenumber);

				if (y!=1 && y!=nminusone){

					j=gcnew BD(1);

					while (j<=sminusone && y!=nminusone){

						y=BD::ModPow(y,2,primenumber);

						if (y==1){

							isprime=false;
							return isprime;

						}

						j=j+1;

					}

					if (y!=nminusone){

						j=sminusone+1;

					}

				}

			}

			return isprime;

		}

	public:

		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (short _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (unsigned short _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ ( int _val ){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (unsigned int _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (long _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (unsigned long _val){

			BD^ _retval=gcnew BD((unsigned __int64)_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (__int64 _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (unsigned __int64 _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (char _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (unsigned char _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (float _val){

			BD^ _retval=gcnew BD();

			_retval->ProcessString(_val.ToString());

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (double _val){

			BD^ _retval=gcnew BD();

			_retval->ProcessString(_val.ToString());

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (String^ _val){

			BD^ _retval=gcnew BD();

			_retval->ProcessString(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (BigInteger^ _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (MYWORDPTR _val){

			BD^ _retval=gcnew BD(_val);

			FreeMem(_val);

			_val=0;

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (BI^ _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!implicit cast to BD.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static operator BD^ (CBI* _val){

			BD^ _retval=gcnew BD(_val);

			delete _val;

			return _retval;

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A char
		*/
		static explicit operator char(BD^ _val){

			char _retval=0;

			if(_val->_cbval->GetValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetValue(1)%256;
					
				}

				else{

					_retval=_val->_cbval->GetValue(1)%128;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A unsigned char
		*/
		static explicit operator unsigned char(BD^ _val){

			if(_val->_cbval->GetValue(0)>0){

				return _val->_cbval->GetValue(1)%256;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A short
		*/
		static explicit operator short(BD^ _val){

			short _retval=0;

			if(_val->_cbval->GetValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetValue(1)%65536;
					
				}

				else{

					_retval=_val->_cbval->GetValue(1)%32768;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A unsigned short
		*/
		static explicit operator unsigned short(BD^ _val){

			if(_val->_cbval->GetValue(0)>0){

				return _val->_cbval->GetValue(1)%65536;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A int
		*/
		static explicit operator int(BD^ _val){

			int _retval=0;

			if(_val->_cbval->GetValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetValue(1)%4294967296;
					
				}

				else{

					_retval=_val->_cbval->GetValue(1)%2147483648;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A unsigned int
		*/
		static explicit operator unsigned int(BD^ _val){

			if(_val->_cbval->GetValue(0)>0){

				return _val->_cbval->GetValue(1)%4294967296;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A long
		*/
		static explicit operator long(BD^ _val){

			long _retval=0;

			if(_val->_cbval->GetValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetValue(1);
					
				}

				else{

					_retval=_val->_cbval->GetValue(1);
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A unsigned long
		*/
		static explicit operator unsigned long(BD^ _val){

			if(_val->_cbval->GetValue(0)>0){

				return _val->_cbval->GetValue(1);

			}

			else{

				return 0;

			}

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A __int64
		*/
		static explicit operator __int64(BD^ _val){

			__int64 _retval=0;

			if(_val->_cbval->GetValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetValue(1);
					
				}

				else{

					_retval=_val->_cbval->GetValue(1);
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A unsigned __int64
		*/
		static explicit operator unsigned __int64(BD^ _val){

			if(_val->_cbval->GetValue(0)>0){

				return _val->_cbval->GetValue(1);

			}

			else{

				return 0;

			}

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A string object
		*/
		static explicit operator String^(BD^ _val){

			return _val->ToString();

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A BigInteger object
		*/
		static explicit operator BigInteger^(BD^ _val){

			System::Numerics::BigInteger^ _base=gcnew System::Numerics::BigInteger(2);
			System::Numerics::BigInteger^ _mult=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _retval=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _digit=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _pow=gcnew System::Numerics::BigInteger();

			_base=System::Numerics::BigInteger::Pow(*_base, 64);

			for(int x=1;x<=_val->Number[0];x++){

				_pow=System::Numerics::BigInteger::Pow(*_base, x-1);
				_digit=gcnew System::Numerics::BigInteger(_val->Number[x]);
				_mult=*_digit**_pow;
				_retval=*_retval+*_mult;

			}

			if(_val->Sign==1){

				_retval=*_retval**gcnew System::Numerics::BigInteger(-1);

			}

			return _retval;

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A MYWORDPTR object
		*/
		static explicit operator MYWORDPTR(BD^ _val){

			return _val->_cbval->GetArray();

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static explicit operator BI^(BD^ _val){

			BI^ _retval=gcnew BI(_val->_cbval->GetArray());

			_retval->Sign=_val->_cbval->GetSign();

			return _retval;

		}
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static explicit operator BF^(BD^ _val);
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static explicit operator BDI^(BD^ _val);
		//!Explicit cast from BD.
		/*!
		\param _val The value to cast from.
		\returns A BFI object
		*/
		static explicit operator BFI^(BD^ _val);

		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(String^ _val){

			ProcessString(_val);
			Init();

		}
		//!CTOR.
		BD(){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();

		}
		//!CTOR.
		BD(bool _nn){

			_cbval=new CBD();

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(array<MYWORD,1>^ _val){	

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			int _len=_val[0]+1;

			MYWORDPTR _num=SGetMem(_len*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			_cbval=new CBD(_num, false);

			SFree(_num);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(__int64 _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			try{

			_cbval->SetValue(1, _val);
			int _pos=1;

			}

			catch(...){


			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(char _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			_cbval->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(unsigned char _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();

			_cbval->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(short _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			_cbval->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(unsigned short _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();
			_cbval->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(int _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			_cbval->SetValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(unsigned int _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();
			_cbval->SetValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(float _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();

			String^ _temp1=_val.ToString();
			String^ _ret="";

			for(int x=0;x<_temp1->Length;x++){

				_ret=_ret+_temp1->Substring(x,1);

			}

			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(unsigned __int64 _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();
			_cbval->SetValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param rhs The value to assign to the instance.
		*/
		BD(BD% rhs){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(rhs._cbval->GetSign()==1){

				_cbval=new CBD(rhs._cbval);

			}

			else{

				_cbval=new CBD(rhs._cbval);

			}

		}
		//!CTOR.
		/*!
		\param rhs The value to assign to the instance.
		*/
		BD(BD% rhs, bool _nn){


			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(rhs._cbval->GetSign()==1){

				_cbval=new CBD(rhs._cbval);

			}

			else{

				_cbval=new CBD(rhs._cbval);

			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(BD^ _val, bool _nn){


			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(_val->_cbval->GetSign()==1){

				_cbval=new CBD(_val->_cbval);

			}

			else{

				_cbval=new CBD(_val->_cbval);

			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(BD^ _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(_val->_cbval->GetSign()==1){

				_cbval=new CBD(_val->_cbval);

			}

			else{

				_cbval=new CBD(_val->_cbval);

			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(Decimal _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();
			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(double _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBD();

			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(BigInteger^ _val){

			System::Numerics::BigInteger^ _tval=gcnew System::Numerics::BigInteger();
			_tval=_val;

			int _pos=1;
			MYWORD _t1=0;
			MYWORD _t2=0;
			bool _sign=false;

			if(_tval->Sign==-1){

				System::Numerics::BigInteger^ _negone=gcnew System::Numerics::BigInteger(-1);
				_tval=*_tval**_negone;
				_sign=true;

			}

			array<System::Byte,1>^ _tarray=_tval->ToByteArray();

			int _count=_tarray->Length/MYBYTES;

			if(_tarray->Length%MYBYTES!=0){

				_count+=1;

			}

			_count+=1;

			MYWORDPTR _num;

			_num=SGetMem(_count*MYBYTES);

			_count-=1;

			_num[0]=_count;

		#if _WIN64 || __amd64__ || _x64

			for(int x=0;x<_tarray->Length;x++){

				if(x%MYBYTES==0){

					_num[_pos]=_tarray[x];
					_t1=_tarray[x];

				}

				else if(x%MYBYTES==1){

					_t2=_tarray[x];
					_t2=_t2<<8;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==2){

					_t2=_tarray[x];
					_t2=_t2<<16;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==3){

					_t2=_tarray[x];
					_t2=_t2<<24;
					_t1=_t1+_t2;
					
					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==4){

					_t2=_tarray[x];
					_t2=_t2<<32;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==5){

					_t2=_tarray[x];
					_t2=_t2<<40;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==6){

					_t2=_tarray[x];
					_t2=_t2<<48;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==7){

					_t2=_tarray[x];
					_t2=_t2<<56;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

					_pos+=1;
					_t1=0;

				}

			}

		#else

			for(int x=0;x<_tarray->Length;x++){

				if(x%MYBYTES==0){

					_num[_pos]=_tarray[x];
					_t1=_tarray[x];

				}

				else if(x%MYBYTES==1){

					_t2=_tarray[x];
					_t2=_t2<<8;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==2){

					_t2=_tarray[x];
					_t2=_t2<<16;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==3){

					_t2=_tarray[x];
					_t2=_t2<<24;
					_t1=_t1+_t2;
					
					_num[_pos]=_t1;
					_pos+=1;
					_t1=0;

				}

			}

		#endif

			while(_num[_num[0]]==0 && _num[0]>1){

				_num[0]=_num[0]-1;

			}

			_cbval=new CBD(_num, _sign);

			SFree(_num);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(MYWORDPTR _val, bool _s){

			_cbval=new CBD(_val, _s);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(MYWORDPTR _val){

			_cbval=new CBD(_val, false);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(CBD* _val){

			_cbval=new CBD(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(BI^ _val){

			_cbval=new CBD(_val->Number);
			_cbval->SetSign(_val->Sign);
			_cbval->SetDecimalPlace(0);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BD(CBI* _val){

			_cbval=new CBD(_val);

		}

		~BD(){

			this->!BD();

		}
		!BD(){

			delete _cbval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BD object
		*/
		BD^ operator=(BD^ _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BD object
		*/
		BD^ operator=(System::Decimal _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BD object
		*/
		BD^ operator=(double _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param rhs The value to assign to the instance.
		\returns A BD object
		*/
		BD^ operator=(BD% rhs){

			BD^ _retval=gcnew BD(rhs);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BD object
		*/
		BD^ operator=(__int64 _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BD object
		*/
		BD^ operator=(unsigned __int64 _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BD object
		*/
		BD^ operator=(BigInteger^ _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BD object
		*/
		BD^ operator=(String^ _val){

			BD^ _retval=gcnew BD(_val);

			return _retval;

		}
		//!equals.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator==(BD^ _val1, BD^ _val2){

			if(BDE64(_val1->_cbval,_val2->_cbval)==1){

				return true;

			}

			return false;

		}
		//!NE.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator!=(BD^ _val1, BD^ _val2){

			return !(_val1==_val2);

		}
		//!LT.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator<(BD^ _val1, BD^ _val2){

			bool _retval=false;

			if(BDLT64(_val1->_cbval,_val2->_cbval)==1){

				return true;

			}

			return false;

		}
		//!LTOE.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator<=(BD^ _val1, BD^ _val2){

			if(_val2<_val1){

				return false;

			}

			else{

				return true;

			}

		}
		//!GT.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator>(BD^ _val1, BD^ _val2){

			if(_val2<_val1){

				return true;

			}

			else{

				return false;

			}

		}
		//!GTOE.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator>=(BD^ _val1, BD^ _val2){

			if(_val1<_val2){

				return false;

			}

			else{

				return true;

			}

		}
		//!Adds 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ operator+=(BD^ _val1, BD^ _val2){

			return _val1+_val2;
		}
		//!Adds 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ operator+(BD^ _val1, BD^ _val2){

			CBD* _temp=BDAdd64(_val1->_cbval, _val2->_cbval);
			BD^ _retval=gcnew BD(_temp);

			delete _temp;

			while(_retval->_cbval->GetValue(_retval->_cbval->GetValue(0))==0 && _retval->_cbval->GetValue(0)>1){

				_retval->_cbval->SetValue(0,_retval->_cbval->GetValue(0)-1);

			}

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Subtracts 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ operator-=(BD^ _val1, BD^ _val2){

			return _val1-_val2;

		}
		//!Subtracts 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ operator-(BD^ _val1, BD^ _val2){

			CBD* _temp=BDSub64(_val1->_cbval, _val2->_cbval);
			BD^ _retval=gcnew BD(_temp);

			delete _temp;

			while(_retval->_cbval->GetValue(_retval->_cbval->GetValue(0))==0 && _retval->_cbval->GetValue(0)>1){

				_retval->_cbval->SetValue(0, _retval->_cbval->GetValue(0)-1);

			}

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Multiplies 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ operator*=(BD^ _val1, BD^ _val2){

			return _val1*_val2;

		}
		//!Multiplies 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ operator*(BD^ _val1, BD^ _val2){

			CBD* _temp=BDMultiply64(_val1->_cbval, _val2->_cbval);
			BD^ _retval=gcnew BD(_temp);

			delete _temp;

			while(_retval->_cbval->GetValue(_retval->_cbval->GetValue(0))==0 && _retval->_cbval->GetValue(0)>1){

				_retval->_cbval->SetValue(0, _retval->_cbval->GetValue(0)-1);

			}

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Divides 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ operator/=(BD^ _val1, BD^ _val2){

			return _val1/_val2;

		}
		//!Divides 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ operator/(BD^ _val1, BD^ _val2){

			CBD* _temp=BDDivide64(_val1->_cbval, _val2->_cbval);
			BD^ _retval=gcnew BD(_temp);

			delete _temp;

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Modular Reduction.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ operator%=(BD^ _val1, BD^ _val2){

			return _val1%_val2;

		}
		//!Modular Reduction.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ operator%(BD^ _val1, BD^ _val2){

			CBD* _temp=BDMod64(_val1->_cbval, _val2->_cbval);
			BD^ _retval=gcnew BD(_temp);

			delete _temp;
	
			while(_retval->_cbval->GetValue(_retval->_cbval->GetValue(0))==0 && _retval->_cbval->GetValue(0)>1){

				_retval->_cbval->SetValue(0, _retval->_cbval->GetValue(0)-1);

			}

			if(_retval->_cbval->GetValue(0)==1 && _retval->_cbval->GetValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Calculates the pow.
		/*!
		\param _base The base value.
		\param _exp The exponent.
		\returns A BD object
		*/
		static BD^ Pow(BD^ _base, BD^ _exp);
		//!Modular Exponentiation.
		/*!
		\param _base The base value.
		\param _exponent The exponent.
		\param _modulus The modulus.
		\returns A BD object
		*/
		static BD^ ModPow(BD^ _base, BD^ _exponent, BD^ _modulus);
		//!Negates the instance.
		void Negate(){

			if(_cbval->GetSign()==1){

				_cbval->SetSign(0);

			}

			else{

				_cbval->SetSign(1);

			}

		}
		//!Implemented for IComparable.
		/*!
		\param _val The object to compare against.
		\returns A int
		*/
		virtual int CompareTo(Object^ _val){

			BD^ _temp = dynamic_cast<BD^>(_val);

			int _retval=1;

			return _retval;

		}
		//!Sets the instance equal to zero.
		void SetToZero(){

			_cbval->SetToZero();

		}
		//!Sets the instance equal to one.
		void SetToOne(){

			_cbval->SetToOne();

		}

#pragma region Properties

		/*!
		\brief This is the native version of the BD class, it is used within the managed class as a container
		\returns Returns a CBD* object 
		*/
		property CBD* Val{
			CBD* get(){return _cbval;}
			void set(CBD* _val){_cbval->ReplaceAndDelete(_val);}
		}
		/*!
		\brief The native array containing the value of this instance
		\returns Returns a MYWORDPTR 
		*/
		property MYWORDPTR Number{
			MYWORDPTR get(){return _cbval->GetArray();}
			void set(MYWORDPTR value){_cbval->SetArray(value);}
		}
		/*!
		\brief This gives you access to the individual elements of the array
		\returns Returns a MYWORD 
		*/
		property MYWORD Number[int]{
			MYWORD get(int _index){return _cbval->GetValue(_index);}
			void set(int _index, MYWORD _val){_cbval->SetValue(_index, _val);}
		}
		/*!
		\brief This represents the sign of the number 1==Negative 0==Positive
		\returns Returns a MYWORD 
		*/
		property MYWORD Sign{
			MYWORD get(){return _cbval->GetSign();}
			void set(MYWORD _val){_cbval->SetSign(_val);}
		}
		/*!
		\brief A boolean value to tell you if this number is Negative
		*/
		property bool IsNegative{
			bool get(){if(_cbval->GetSign()==1){
				return true;}return false;}
		}
		/*!
		\brief A boolean value to tell you if this number is equal to zero
		\returns Returns a boolean 
		*/
		property bool IsZero{
			bool get(){
				
				if(_cbval->GetValue(0)<1 || (_cbval->GetValue(0)==1 && _cbval->GetValue(1)==0 && _cbval->GetDecimalPlace()==0)){

					return true;
				
				}
				
				for(int x=1;x<=_cbval->GetValue(0);x++){

					if(_cbval->GetValue(x)!=0){

						return false;
						break;

					}

				}

				return true;
			}
		}
		/*!
		\brief A boolean value to tell you if this number is equal to one
		\returns Returns a boolean 
		*/
		property bool IsOne{
			bool get(){if(_cbval->GetValue(0)==1 && _cbval->GetValue(1)==1 && _cbval->GetDecimalPlace()==0){
				return true;}return false;
			}
		}
		/*!
		\brief A boolean value to tell you if this number is odd
		\returns Returns a boolean 
		*/
		property bool IsOdd{
			bool get(){if(_cbval->GetValue(1)%2==1){
				return true;}return false;}
		}
		/*!
		\brief This property specifies the current decimal place (counted in digits from the right)
		\returns Returns a MYWORD 
		*/
		property MYWORD DecimalPlace{
			MYWORD get(){return _cbval->GetDecimalPlace();}
			void set(MYWORD _val){_cbval->SetDecimalPlace(_val);}
		}
		/*!
		\brief This property sets the number of decimal places to keep (how many digits to the right of the decimal point)
		\returns Returns a MYWORD 
		*/
		static property MYWORD DecimalLimit{
			MYWORD get(){return GetDecimalLimit();}
			void set(MYWORD _val){SetDecimalLimit(_val);}
		}
		/*!
		\brief The adjustment is a value to temporarily increase the decimal limit for a set of calculations (this helps to reduce rounding errors)
		The theory behind this property is if you increase the decimal limit for this object before doing a series of calculations, you should reduce the rounding errors for this object.
		if you figure that on average you should only be rounding up 1/2 the time that means if you increase the decimal limity by 1 it would take an average of 10 calculations
		before you accumalte an error, if you increase it by 2 digits, it should take an average of 20 calculations and from here it should go up by a factor of 10 per digit
		at the end of your calculations, set this property to zero, and fire the MakeFinal() method.
		\returns Returns a MYWORD 
		*/
		property MYWORD Adjustment{
			MYWORD get(){return _cbval->GetAdjustment();}
			void set(MYWORD _val){_cbval->SetAdjustment(_val);}
		}

#pragma endregion properties

		//!Tests the instance for primality.
		/*!
		\param _cert The number of times to run through the algorithm.
		\returns A bool
		*/
		bool IsPrime(int _cert){

			return MillerRabbin(_cert);

		}
		//!Returns the string value of the instance.
		/*!
		\returns A string object
		*/
		virtual String^ ToString()override{

			return ToString(OtherCrap::Base);

		}
		//!Returns the string value of the instance.
		/*!
		\param _tbase The radix to use to calculate the string.
		\returns A string object
		*/
		String^ ToString(MyMath::Radix _tbase){

			String^ _retval="";

			if((int)_tbase==10){

				System::Numerics::BigInteger^ _bbase=gcnew System::Numerics::BigInteger(2);
				System::Numerics::BigInteger^ _pow=gcnew System::Numerics::BigInteger();
				System::Numerics::BigInteger^ _mult=gcnew System::Numerics::BigInteger();
				System::Numerics::BigInteger^ _digit=gcnew System::Numerics::BigInteger();
				System::Numerics::BigInteger^ _bb=gcnew System::Numerics::BigInteger();
				System::Numerics::BigInteger^ _no=gcnew System::Numerics::BigInteger(-1);

				_bbase=System::Numerics::BigInteger::Pow(*_bbase, MYBITS);

				for(int x=1;x<=_cbval->GetValue(0);x++){

					_digit=gcnew System::Numerics::BigInteger(_cbval->GetValue(x));
					_pow=System::Numerics::BigInteger::Pow(*_bbase,(x-1));
					_mult=*_digit**_pow;
					_bb=*_bb+*_mult;

				}

				if(_bb->Sign<0){

					_bb=*_bb*_bb->MinusOne;

				}

				_retval=_bb->ToString();

			}

			else{

				int _bits=(int)_tbase-1;
				int _lshift=0;
				int _rshift=0;
				int _offset=0;
				int _byte=1;
				int _topbit=0;
				unsigned __int64 _tbyte1=0;
				unsigned __int64 _tbyte2=0;
				unsigned __int64 _mask1=_bits;
				unsigned __int64 _mask2=_bits;
				unsigned __int64 _temp1=0;
				unsigned __int64 _temp2=0;

				if(_bits==63){

					_lshift=6;

				}

				else if(_bits==31){

					_lshift=5;

				}

				else if(_bits==15){

					_lshift=4;

				}

				else{

					_lshift=3;

				}

				_topbit=_lshift;

				while(_byte<=_cbval->GetValue(0)){

					while(_topbit<=MYBITS){

						_temp1=_cbval->GetValue(_byte) & _mask1;
						_rshift=_topbit-_lshift;
						_temp1=_temp1>>_rshift;

						_retval=OtherCrap::GetString(_temp1, _tbase) + _retval;

						_mask1=_mask1<<_lshift;
						_topbit+=_lshift;

					}

					if(_mask1==0){

						_mask1=_lshift;
						_topbit=_lshift;
						_byte+=1;

					}

					else{

						_temp1=_cbval->GetValue(_byte) & _mask1;
						_rshift=_topbit-_lshift;
						_temp1=_temp1>>_rshift;
						_byte+=1;

						if(_byte<=_cbval->GetValue(0)){

							_mask2=_lshift;
							_rshift=_lshift-(_topbit-MYBITS);
							_mask2=_mask2>>_rshift;
							_temp2=_cbval->GetValue(_byte) & _mask2;
							_temp1=_temp1+_temp2;

						}

						_retval=OtherCrap::GetString(_temp1, _tbase);
						_mask1=_lshift;
						_rshift=_lshift-_rshift;

					}

				}

			}

			if(_cbval->GetDecimalPlace()>0){

				while(_cbval->GetDecimalPlace()>=_retval->Length){

					_retval="0"+_retval;

				}

				String^ _intp=_retval->Substring(0, _retval->Length-_cbval->GetDecimalPlace());
				String^ _decp=_retval->Substring(_retval->Length-_cbval->GetDecimalPlace(),  _retval->Length-(_retval->Length-_cbval->GetDecimalPlace()));

				_retval=_intp + "." + _decp;

			}

			if(_cbval->GetSign()==1){

				_retval="-" + _retval;

			}

			return _retval;

		}
		//!Returns the absolute value of the instance.
		/*!
		\returns A BD object
		*/
		BD^ ABS(){

			BD^ _temp=gcnew BD(_cbval->GetArray());

			_temp->_cbval->SetSign(0);

			return _temp;

		}
		//!Calculates the Least Common Multiple.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ LCM(BD^ _val1, BD^ _val2){

			//BD^ _retval=gcnew BD(BDLCM64(_val1->_cbval, _val2->_cbval));

			BD^ _retval=gcnew BD();

			_retval=_val1*_val2;

			_retval=_retval/GCD(_val1, _val2);

			return _retval;

		}
		//!Calculates the Greatest Common Divisor.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BD object
		*/
		static BD^ GCD(BD^ _val1, BD^ _val2){

			//BD^ _retval=gcnew BD(BDGCD64(_val1->_cbval, _val2->_cbval));

			//return _retval;

			BD^ _a;
			BD^ _b;

			if(_val1>=_val2){

				_a=gcnew BD(_val1);
				_b=gcnew BD(_val2);

			}

			else{

				_a=gcnew BD(_val2);
				_b=gcnew BD(_val1);

			}

			BD^ _r=gcnew BD();

			while(_b>0){

				_r=_a%_b;
				_a=_b;
				_b=_r;

			}

			return _a;

		}
		//!Used to bring the decimal point to within the decimal limit after increasing the _adj value for a series of calculations.
		void MakeFinal(){

			Adjustment=0;

			BDFixTheEnd(_cbval);

		}
					
	};
	//!This class is the BigFraction class capable of holding all real numbers.  Since I pretty much figured out how to eleminate rounding errors, this class is being deprecated, though it is still fully functional.  The BI and BD classes have implicit conversions to BF
	/*!
	This class uses an instance of the CBF class internally as a container.  Again, it calls the assembly routines directly just passing in the CBF objects as paramaters.
	*/
	[Guid("8551475E-FCCB-42EB-B43F-8BF5A93C3418")]
	public ref class BF : public IComparable
	{

	protected:
		
		//!This is the container for the value of the instance.  We use the CBF* type so that we can access the assembly language functions by passing this object in.
		CBF* _cbval;

	private:

		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isdone=false;
		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isinit=false;
		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isiniting=false;
		void ProcessString(String^ _val){

			BI^ _retval=gcnew BI();
			BI^ _dretval=gcnew BI();

			String^ _sbase="";
			String^ _exp="";
			String^ _dbase="";

			bool _bneg=false;
			bool _eneg=false;
			bool _dec=false;
			bool _doexp=false;
			bool _doden=false;

			int _deccount=0;

			Radix _thebase=MyMath::Radix::_Decimal;

			for(int x=0;x<_val->Length;x++){

				if(_val->Substring(x,1)=="a" || _val->Substring(x,1)=="A" || _val->Substring(x,1)=="b" || _val->Substring(x,1)=="B" ||  _val->Substring(x,1)=="c" || _val->Substring(x,1)=="C" ||
						_val->Substring(x,1)=="d" || _val->Substring(x,1)=="D" ||  _val->Substring(x,1)=="e" || _val->Substring(x,1)=="E" ||  _val->Substring(x,1)=="f" || _val->Substring(x,1)=="F"){

							if((int)_thebase<16){

								_thebase=MyMath::Radix::_HexaDecimal;

							}

				}

				else if(_val->Substring(x,1)=="G" || _val->Substring(x,1)=="H" || _val->Substring(x,1)=="I" || _val->Substring(x,1)=="J" || _val->Substring(x,1)=="K" || _val->Substring(x,1)=="L" ||
						_val->Substring(x,1)=="M" || _val->Substring(x,1)=="N" || _val->Substring(x,1)=="O" || _val->Substring(x,1)=="P" || _val->Substring(x,1)=="Q" || _val->Substring(x,1)=="R" ||
						_val->Substring(x,1)=="S" || _val->Substring(x,1)=="T" || _val->Substring(x,1)=="U" || _val->Substring(x,1)=="V" || _val->Substring(x,1)=="W" || _val->Substring(x,1)=="X"  ||
						_val->Substring(x,1)=="Y" || _val->Substring(x,1)=="Z"){

							if((int)_thebase<32){

								_thebase=MyMath::Radix::_Base32;

							}

				}

				else if(_val->Substring(x,1)=="g" || _val->Substring(x,1)=="h" || _val->Substring(x,1)=="i" || _val->Substring(x,1)=="j" || _val->Substring(x,1)=="k" || _val->Substring(x,1)=="l" ||
					_val->Substring(x,1)=="m" || _val->Substring(x,1)=="n" || _val->Substring(x,1)=="o" || _val->Substring(x,1)=="p" || _val->Substring(x,1)=="q" || _val->Substring(x,1)=="r"  ||
					_val->Substring(x,1)=="s" || _val->Substring(x,1)=="t" || _val->Substring(x,1)=="u" || _val->Substring(x,1)=="v" || _val->Substring(x,1)=="w" || _val->Substring(x,1)=="x"  ||
					_val->Substring(x,1)=="y" || _val->Substring(x,1)=="z"){

						_thebase=MyMath::Radix::_Base64;
						break;

				}

			}

			for(int x=0;x<_val->Length;x++){

				if(_dec && ! _doexp){

					_deccount+=1;

				}

				if(_val->Substring(x,1)=="-"){

					if(_doexp){

						_eneg=true;

					}

					else{

						_bneg=true;

					}

				}

				else if(_val->Substring(x,1)=="."){

					_dec=true;
					_deccount+=1;

				}

				else if(_val->Substring(x,1)=="e" || _val->Substring(x,1)=="E"){

					_doexp=true;

				}

				else if(_val->Substring(x,1)=="/"){

					_doden=true;

				}

				else if(_val->Substring(x,1)==" "){

				}

				else{

					if(_doexp){

						_exp=_exp+_val->Substring(x,1);

					}

					else if(_doden){

						_dbase=_dbase+_val->Substring(x,1);

					}

					else{

						_sbase=_sbase+_val->Substring(x,1);

					}

				}

			}

			BI^ _base=gcnew BI();
			BI^ _digit=gcnew BI();
			BI^ _pow=gcnew BI();
			BI^ _mult=gcnew BI();

			_base->Number[0]= 1;
			_base->Number[1]= (int)_thebase;

			_mult->Number[0]= 1;
			_mult->Number[1]= (int)_thebase;

			_retval->Number[0]= 1;

			if(_sbase->Length<18 && (int)_thebase==10){

				_retval->Number[1]= System::Convert::ToUInt64(_sbase);

			}

			else{

				_retval->Number[1]= 0;

				for(int x=0;x<_sbase->Length;x++){

					_digit->Number[1]= System::Convert::ToUInt64(_sbase->Substring(x,1));
					_retval=_retval*_mult;
					_retval=_retval+_digit;

				}

			}

			if(_doden){

				if(_dbase->Length<18 && (int)_thebase==10){

					_dretval->Number[1]= System::Convert::ToUInt64(_dbase);

				}

				else{

					_dretval->Number[1]= 0;

					for(int x=0;x<_dbase->Length;x++){

						_digit->Number[1]= System::Convert::ToUInt64(_dbase->Substring(x,1));
						_dretval=_dretval*_mult;
						_dretval=_dretval+_digit;

					}

				}

			}

			if(_doexp){
						
				_digit->Number[1]= System::Convert::ToUInt64(_exp);

				if(_deccount>0){

					if(_eneg){

						_digit->Number[1]= _digit->Number[1]+_deccount;

						for(int x=0;x<_digit->Number[1];x++){

							_retval=_retval/_mult;

						}

					}

					else{
						
						if(_deccount<=_digit->Number[1]){

							_digit->Number[1]= _digit->Number[1]-_deccount;

							if(_digit->Number[1]>0){

								for(int x=0;x<=_digit->Number[0];x++){

									_retval=_retval*_mult;

								}

							}

						}

						else{

							_digit->Number[1]= _deccount-_digit->Number[1];

							for(int x=0;x<=_digit->Number[0];x++){

								_retval=_retval/_mult;

							}

						}

					}

				}

				else{

					if(_eneg){

						for(int x=0;x<=_digit->Number[0];x++){

							_retval=_retval/_mult;

						}

					}

					else{

						for(int x=0;x<_digit->Number[0];x++){

							_retval=_retval*_mult;

						}

					}

				}

			}

			else{

				if(_deccount>0){

					for(int x=0;x<_deccount;x++){

						_retval=_retval/_mult;

					}

				}

			}

			if(_bneg){

				_retval->Sign=1;

			}

			else{

				_retval->Sign=0;

			}

			delete _cbval;

			_cbval=new CBF(_retval->Number, _dretval->Number,_retval->Sign);

		}
		void Init(){

			if(!_isdone){

				_isdone=true;

			}
			//InitIt();

		}
		bool MillerRabbin(int _cert){

			BF^ primenumber=gcnew BF(_cbval->GetNumerator());

			BF^ s=gcnew BF(2);
			BF^ r=gcnew BF(1);
			BF^ y=gcnew BF(1);
			BF^ j=gcnew BF(1);

			long i;
			BF^ nminusone=gcnew BF(1);
			BF^ sminusone=gcnew BF(1);
			BF^ temp=gcnew BF(1);
			bool isprime,blndone,found;
			System::Random ^thisrandom=gcnew System::Random();
			
			isprime=true;
			blndone=false;
			found=false;
			nminusone=primenumber-1;

			while (blndone==false){

				temp=Pow(gcnew BF(2),s);

				if (nminusone%temp==0){

					r=nminusone/temp;

					if(r%2==1){

						
						blndone=true;
						found=true;

					}

				}

				if (nminusone<temp){

					blndone=true;

				}

				if(blndone==false){

					s=s+1;

				}

			}

			if(found==false){

				r=nminusone/2;

				if(r%2!=1){

					s=gcnew BF(1);
					temp=gcnew BF(2);
					r=nminusone/temp;

				}

			}

			if (r==0){

				r=nminusone;
				s=gcnew BF();

			}

			sminusone=s-1;

			for (i=1;i<=_cert;i++){

				y=thisrandom->Next(5000);
				//y=BF::Random(primenumber-1);

				y=BF::ModPow(y,r,primenumber);

				if (y!=1 && y!=nminusone){

					j=gcnew BF(1);

					while (j<=sminusone && y!=nminusone){

						y=BF::ModPow(y,2,primenumber);

						if (y==1){

							isprime=false;
							return isprime;

						}

						j=j+1;

					}

					if (y!=nminusone){

						j=sminusone+1;

					}

				}

			}

			return isprime;

		}

	protected:


	public:

		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (short _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (unsigned short _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ ( int _val ){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (unsigned int _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (long _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (unsigned long _val){

			BF^ _retval=gcnew BF((unsigned __int64)_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (__int64 _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (unsigned __int64 _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (char _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (unsigned char _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (float _val){

			BF^ _retval=gcnew BF();

			_retval->ProcessString(_val.ToString());

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (double _val){

			BF^ _retval=gcnew BF();

			_retval->ProcessString(_val.ToString());

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (String^ _val){

			BF^ _retval=gcnew BF();

			_retval->ProcessString(_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (BigInteger^ _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Implicit cast to BF.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static operator BF^ (MYWORDPTR _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A char
		*/
		static explicit operator char(BF^ _val){

			char _retval=0;

			if(_val->_cbval->GetNumeratorValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetNumeratorValue(1)%256;
					
				}

				else{

					_retval=_val->_cbval->GetNumeratorValue(1)%128;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A unsigned char
		*/
		static explicit operator unsigned char(BF^ _val){

			if(_val->_cbval->GetNumeratorValue(0)>0){

				return _val->_cbval->GetNumeratorValue(1)%256;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A short
		*/
		static explicit operator short(BF^ _val){

			short _retval=0;

			if(_val->_cbval->GetNumeratorValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetNumeratorValue(1)%65536;
					
				}

				else{

					_retval=_val->_cbval->GetNumeratorValue(1)%32768;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A unsigned short
		*/
		static explicit operator unsigned short(BF^ _val){

			if(_val->_cbval->GetNumeratorValue(0)>0){

				return _val->_cbval->GetNumeratorValue(1)%65536;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A int
		*/
		static explicit operator int(BF^ _val){

			int _retval=0;

			if(_val->_cbval->GetNumeratorValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetNumeratorValue(1)%4294967296;
					
				}

				else{

					_retval=_val->_cbval->GetNumeratorValue(1)%2147483648;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A unsigned int
		*/
		static explicit operator unsigned int(BF^ _val){

			if(_val->_cbval->GetNumeratorValue(0)>0){

				return _val->_cbval->GetNumeratorValue(1)%4294967296;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A long
		*/
		static explicit operator long(BF^ _val){

			long _retval=0;

			if(_val->_cbval->GetNumeratorValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetNumeratorValue(1);
					
				}

				else{

					_retval=_val->_cbval->GetNumeratorValue(1);
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A unsigned long
		*/
		static explicit operator unsigned long(BF^ _val){

			if(_val->_cbval->GetNumeratorValue(0)>0){

				return _val->_cbval->GetNumeratorValue(1);

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A __int64
		*/
		static explicit operator __int64(BF^ _val){

			__int64 _retval=0;

			if(_val->_cbval->GetNumeratorValue(0)>0){

				if(_val->_cbval->GetSign()==0){

					_retval=_val->_cbval->GetNumeratorValue(1);
					
				}

				else{

					_retval=_val->_cbval->GetNumeratorValue(1);
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A unsigned __int64
		*/
		static explicit operator unsigned __int64(BF^ _val){

			if(_val->_cbval->GetNumeratorValue(0)>0){

				return _val->_cbval->GetNumeratorValue(1);

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A string object
		*/
		static explicit operator String^(BF^ _val){

			return _val->ToString();

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A BigInteger object
		*/
		static explicit operator BigInteger^(BF^ _val){

			System::Numerics::BigInteger^ _base=gcnew System::Numerics::BigInteger(2);
			System::Numerics::BigInteger^ _mult=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _retval=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _digit=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _pow=gcnew System::Numerics::BigInteger();

			_base=System::Numerics::BigInteger::Pow(*_base, 64);

			for(int x=1;x<=_val->_cbval->GetNumeratorValue(0);x++){

				_pow=System::Numerics::BigInteger::Pow(*_base, x-1);
				_digit=gcnew System::Numerics::BigInteger(_val->_cbval->GetNumeratorValue(x));
				_mult=*_digit**_pow;
				_retval=*_retval+*_mult;

			}

			if(_val->Sign==1){

				_retval=*_retval**gcnew System::Numerics::BigInteger(-1);

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static explicit operator BI^(BF^ _val){

			BI^ _retval=gcnew BI();

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static explicit operator BD^(BF^ _val){

			BD^ _retval=gcnew BD();

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static explicit operator BDI^(BF^ _val);
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A BFI object
		*/
		static explicit operator BFI^(BF^ _val);

		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(String^ _val){

			ProcessString(_val);
				Init();

		}
		//!CTOR.
		BF(){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();

		}
		//!CTOR.
		BF(bool _nn){

			_cbval=new CBF();

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(array<MYWORD,1>^ _val){	

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			int _len=_val[0]+1;

			MYWORDPTR _num=SGetMem(_len*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			_cbval=new CBF(_num, false);

			SFree(_num);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(__int64 _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			try{

			_cbval->SetNumeratorValue(1, _val);
			int _pos=1;

			}

			catch(...){


			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(char _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			_cbval->SetNumeratorValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(unsigned char _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();

			_cbval->SetNumeratorValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(short _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			_cbval->SetNumeratorValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(unsigned short _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();
			_cbval->SetNumeratorValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(int _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();

			if(_val<0){

				_cbval->SetSign(1);
				_val=_val*-1;

			}

			_cbval->SetNumeratorValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(unsigned int _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();
			_cbval->SetNumeratorValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(float _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();

			String^ _temp1=_val.ToString();
			String^ _ret="";

			for(int x=0;x<_temp1->Length;x++){

				_ret=_ret+_temp1->Substring(x,1);

			}

			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(unsigned __int64 _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();
			_cbval->SetNumeratorValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param rhs The value to assign to the instance.
		*/
		BF(BF% rhs){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(rhs._cbval->GetSign()==1){

				_cbval=new CBF(rhs._cbval->GetNumerator(), true);

			}

			else{

				_cbval=new CBF(rhs._cbval->GetNumerator(), false);

			}

		}
		//!CTOR.
		/*!
		\param rhs The value to assign to the instance.
		*/
		BF(BF% rhs, bool _nn){


			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(rhs._cbval->GetSign()==1){

				_cbval=new CBF(rhs._cbval->GetNumerator(), true);

			}

			else{

				_cbval=new CBF(rhs._cbval->GetNumerator(), false);

			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(BF^ _val, bool _nn){


			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(_val->_cbval->GetSign()==1){

				_cbval=new CBF(_val->_cbval->GetNumerator(), true);

			}

			else{

				_cbval=new CBF(_val->_cbval->GetNumerator(), false);

			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(BF^ _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF(_val->_cbval->GetNumerator(), _val->_cbval->GetDenominator());
			_cbval->SetSign(_val->_cbval->GetSign());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(Decimal _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();
			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(double _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBF();

			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(BigInteger^ _val){

			System::Numerics::BigInteger^ _tval=gcnew System::Numerics::BigInteger();
			_tval=_val;

			int _pos=1;
			MYWORD _t1=0;
			MYWORD _t2=0;
			bool _sign=false;

			if(_tval->Sign==-1){

				System::Numerics::BigInteger^ _negone=gcnew System::Numerics::BigInteger(-1);
				_tval=*_tval**_negone;
				_sign=true;

			}

			array<System::Byte,1>^ _tarray=_tval->ToByteArray();

			int _count=_tarray->Length/MYBYTES;

			if(_tarray->Length%MYBYTES!=0){

				_count+=1;

			}

			_count+=1;

			MYWORDPTR _num;

			_num=SGetMem(_count*MYBYTES);

			_count-=1;

			_num[0]=_count;

		#if _WIN64 || __amd64__ || _x64

			for(int x=0;x<_tarray->Length;x++){

				if(x%MYBYTES==0){

					_num[_pos]=_tarray[x];
					_t1=_tarray[x];

				}

				else if(x%MYBYTES==1){

					_t2=_tarray[x];
					_t2=_t2<<8;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==2){

					_t2=_tarray[x];
					_t2=_t2<<16;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==3){

					_t2=_tarray[x];
					_t2=_t2<<24;
					_t1=_t1+_t2;
					
					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==4){

					_t2=_tarray[x];
					_t2=_t2<<32;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==5){

					_t2=_tarray[x];
					_t2=_t2<<40;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==6){

					_t2=_tarray[x];
					_t2=_t2<<48;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==7){

					_t2=_tarray[x];
					_t2=_t2<<56;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

					_pos+=1;
					_t1=0;

				}

			}

		#else

			for(int x=0;x<_tarray->Length;x++){

				if(x%MYBYTES==0){

					_num[_pos]=_tarray[x];
					_t1=_tarray[x];

				}

				else if(x%MYBYTES==1){

					_t2=_tarray[x];
					_t2=_t2<<8;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==2){

					_t2=_tarray[x];
					_t2=_t2<<16;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==3){

					_t2=_tarray[x];
					_t2=_t2<<24;
					_t1=_t1+_t2;
					
					_num[_pos]=_t1;
					_pos+=1;
					_t1=0;

				}

			}

		#endif

			while(_num[_num[0]]==0 && _num[0]>1){

				_num[0]=_num[0]-1;

			}

			_cbval=new CBF(_num, _sign);

			SFree(_num);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(MYWORDPTR _val, bool _s){

			_cbval=new CBF(_val, _s);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(MYWORDPTR _val){

			_cbval=new CBF(_val, false);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(CBF* _val){

			_cbval=new CBF(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(BI^ _val){

			_cbval=new CBF(_val->Val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(CBI* _val){

			_cbval=new CBF(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(BD^ _val){

			_cbval=new CBF(_val->Val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to the instance.
		*/
		BF(CBD* _val){

			_cbval=new CBF(_val);

		}
		//!CTOR.
		/*!
		\param _newnum The value to assign to the numerator.
		\param _newden The value to assign to the denominator.
		*/
		BF(MYWORDPTR _newnum, MYWORDPTR _newden){

			_cbval=new CBF(_newnum, _newden);

			_cbval->SetSign(0);

		}

		~BF(){

			this->!BF();

		}
		!BF(){

			try{

				delete _cbval;

			}

			catch(...){

				String^ _td="gahd";

				_td=_td+_td;

			}

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BF object
		*/
		BF^ operator=(BF^ _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BF object
		*/
		BF^ operator=(System::Decimal _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BF object
		*/
		BF^ operator=(double _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param rhs The value to assign to the instance.
		\returns A BF object
		*/
		BF^ operator=(BF% rhs){

			BF^ _retval=gcnew BF(rhs);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BF object
		*/
		BF^ operator=(__int64 _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BF object
		*/
		BF^ operator=(unsigned __int64 _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BF object
		*/
		BF^ operator=(BigInteger^ _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to the instance.
		\returns A BF object
		*/
		BF^ operator=(String^ _val){

			BF^ _retval=gcnew BF(_val);

			return _retval;

		}
		//!Equals.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator==(BF^ _val1, BF^ _val2){

			if(BFE64(_val1->_cbval,_val2->_cbval)==1){

				return true;

			}

			return false;

		}
		//!NE.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator!=(BF^ _val1, BF^ _val2){

			return !(_val1==_val2);

		}
		//!LT.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator<(BF^ _val1, BF^ _val2){

			bool _retval=false;

			if(BFLT64(_val1->_cbval,_val2->_cbval)==1){

				return true;

			}

			return false;

		}
		//!LTOE.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator<=(BF^ _val1, BF^ _val2){

			if(_val2<_val1){

				return false;

			}

			else{

				return true;

			}

		}
		//!GT.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator>(BF^ _val1, BF^ _val2){

			if(_val2<_val1){

				return true;

			}

			else{

				return false;

			}

		}
		//!GTOE.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator>=(BF^ _val1, BF^ _val2){

			if(_val1<_val2){

				return false;

			}

			else{

				return true;

			}

		}
		//!Adds 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ operator+=(BF^ _val1, BF^ _val2){

			return _val1+_val2;
		}
		//!Adds 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ operator+(BF^ _val1, BF^ _val2){

			CBF* _temp=BFAdd64(_val1->_cbval, _val2->_cbval);
			BF^ _retval=gcnew BF(_temp);

			delete _temp;

			while(_retval->_cbval->GetNumeratorValue(_retval->_cbval->GetNumeratorValue(0))==0 && _retval->_cbval->GetNumeratorValue(0)>1){

				_retval->_cbval->SetNumeratorValue(0,_retval->_cbval->GetNumeratorValue(0)-1);

			}

			if(_retval->_cbval->GetNumeratorValue(0)==1 && _retval->_cbval->GetNumeratorValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Subtracts 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ operator-=(BF^ _val1, BF^ _val2){

			return _val1-_val2;

		}
		//!Subtracts 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ operator-(BF^ _val1, BF^ _val2){

			CBF* _temp=BFSub64(_val1->_cbval, _val2->_cbval);
			BF^ _retval=gcnew BF(_temp);

			delete _temp;

			while(_retval->_cbval->GetNumeratorValue(_retval->_cbval->GetNumeratorValue(0))==0 && _retval->_cbval->GetNumeratorValue(0)>1){

				_retval->_cbval->SetNumeratorValue(0, _retval->_cbval->GetNumeratorValue(0)-1);

			}

			if(_retval->_cbval->GetNumeratorValue(0)==1 && _retval->_cbval->GetNumeratorValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Multiplies 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ operator*=(BF^ _val1, BF^ _val2){

			return _val1*_val2;

		}
		//!Multiplies 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ operator*(BF^ _val1, BF^ _val2){

			CBF* _temp=BFMultiply64(_val1->_cbval, _val2->_cbval);
			BF^ _retval=gcnew BF(_temp);

			delete _temp;

			while(_retval->_cbval->GetNumeratorValue(_retval->_cbval->GetNumeratorValue(0))==0 && _retval->_cbval->GetNumeratorValue(0)>1){

				_retval->_cbval->SetNumeratorValue(0, _retval->_cbval->GetNumeratorValue(0)-1);

			}

			if(_retval->_cbval->GetNumeratorValue(0)==1 && _retval->_cbval->GetNumeratorValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Divides 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ operator/=(BF^ _val1, BF^ _val2){

			return _val1/_val2;

		}
		//!Divides 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ operator/(BF^ _val1, BF^ _val2){

			CBF* _temp=BFDivide64(_val1->_cbval, _val2->_cbval);
			BF^ _retval=gcnew BF(_temp);

			delete _temp;

			if(_retval->_cbval->GetNumeratorValue(0)==1 && _retval->_cbval->GetNumeratorValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Calculates the modulus.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ operator%=(BF^ _val1, BF^ _val2){

			return _val1%_val2;

		}
		//!Calculates the modulus.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ operator%(BF^ _val1, BF^ _val2){

			CBF* _temp=BFMod64(_val1->_cbval, _val2->_cbval);
			BF^ _retval=gcnew BF(_temp);

			delete _temp;
	
			while(_retval->_cbval->GetNumeratorValue(_retval->_cbval->GetNumeratorValue(0))==0 && _retval->_cbval->GetNumeratorValue(0)>1){

				_retval->_cbval->SetNumeratorValue(0, _retval->_cbval->GetNumeratorValue(0)-1);

			}

			if(_retval->_cbval->GetNumeratorValue(0)==1 && _retval->_cbval->GetNumeratorValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Exponentiation.
		/*!
		\param _base The base value.
		\param _exp The exponent.
		\returns A BF object
		*/
		static BF^ Pow(BF^ _base, BF^ _exp){

			CBF* _temp=BFPow64(_base->_cbval, _exp->_cbval);
			BF^ _retval=gcnew BF(_temp);

			delete _temp;

			if(_retval->_cbval->GetNumeratorValue(0)==1 && _retval->_cbval->GetNumeratorValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Modular Exponentiation.
		/*!
		\param _base The base value.
		\param _exponent The exponent.
		\param _modulus The Modulus.
		\returns A BF object
		*/
		static BF^ ModPow(BF^ _base, BF^ _exponent, BF^ _modulus){

			CBF* _temp=BFModPow64(_base->_cbval, _exponent->_cbval, _modulus->_cbval);
			BF^ _retval=gcnew BF(_temp);

			delete _temp;

			if(_retval->_cbval->GetNumeratorValue(0)==1 && _retval->_cbval->GetNumeratorValue(1)==0){

				_retval->_cbval->SetSign(0);

			}

			return _retval;

		}
		//!Negates the instance.
		void Negate(){

			if(_cbval->GetSign()==1){

				_cbval->SetSign(0);

			}

			else{

				_cbval->SetSign(1);

			}

		}
		//!Implemented for IComparable.
		/*!
		\param _val The object to compare to.
		\returns A int
		*/
		virtual int CompareTo(Object^ _val){

			BF^ _temp = dynamic_cast<BF^>(_val);

			int _retval=1;

			return _retval;

		}
		//!Sets the instance equal to zero.
		void SetToZero(){

			_cbval->SetToZero();

		}
		//!Sets the instance equal to one.
		void SetToOne(){

			_cbval->SetToOne();

		}

#pragma region Properties

		/*!
		\brief This property contains the native version of the BF class, it is used in the managed class as a container
		\returns Returns a CBF* object 
		*/
		property CBF* Val{
			CBF* get(){return _cbval;}
			void set(CBF* _val){_cbval->ReplaceAndDelete(_val);}
		}
		/*!
		\brief This property represents the native array containing the value for the numerator
		\returns Returns a MYWORDPTR 
		*/
		property MYWORDPTR Numerator{
			MYWORDPTR get(){return _cbval->GetNumerator();}
			void set(MYWORDPTR value){_cbval->SetNumerator(value);}
		}
		/*!
		\brief This gives you access to the individual elements of the numerator array
		\returns Returns a MYWORD 
		*/
		property MYWORD Numerator[int]{
			MYWORD get(int _index){return _cbval->GetNumeratorValue(_index);}
			void set(int _index, MYWORD _val){_cbval->SetNumeratorValue(_index, _val);}
		}
		/*!
		\brief This property represents the native array containing the value for the denominator
		\returns Returns a MYWORDPTR 
		*/
		property MYWORDPTR Denominator{
			MYWORDPTR get(){return _cbval->GetDenominator();}
			void set(MYWORDPTR value){_cbval->SetDenominator(value);}
		}
		/*!
		\brief This gives you access to the individual elements of the denominator array
		\returns Returns a MYWORD 
		*/
		property MYWORD Denominator[int]{
			MYWORD get(int _index){return _cbval->GetDenominatorValue(_index);}
			void set(int _index, MYWORD _val){_cbval->SetDenominatorValue(_index, _val);}
		}
		/*!
		\brief This property is the sign for the object 1==Negative 0==Positive
		\returns Returns a MYWORD 
		*/
		property MYWORD Sign{
			MYWORD get(){return _cbval->GetSign();}
			void set(MYWORD _val){_cbval->SetSign(_val);}
		}
		/*!
		\brief This tells you if the object is Negative
		\returns Returns a boolean 
		*/
		property bool IsNegative{
			bool get(){if(_cbval->GetSign()==1){
				return true;}return false;}
		}
		/*!
		\brief This property tells you if the object is equal to zero
		\returns Returns a boolean 
		*/
		property bool IsZero{
			bool get(){
				
				if((_cbval->GetNumeratorValue(0)<1 || (_cbval->GetNumeratorValue(0)==1 && _cbval->GetNumeratorValue(1)==0)) && (_cbval->GetDenominatorValue(0)<1 || (_cbval->GetDenominatorValue(0)==1 && _cbval->GetDenominatorValue(1)==0))){

					return true;
				
				}
				
				for(int x=1;x<=_cbval->GetNumeratorValue(0);x++){

					if(_cbval->GetNumeratorValue(x)!=0){

						return false;
						break;

					}

				}

				return true;
			}
		}
		/*!
		\brief This tells you if the object is equal to one
		\returns Returns a boolean 
		*/
		property bool IsOne{
			bool get(){if((_cbval->GetNumeratorValue(0)==1 && _cbval->GetNumeratorValue(1)==1) && (_cbval->GetDenominatorValue(0)==1 && _cbval->GetDenominatorValue(1)==1)) {
				return true;}return false;
			}
		}
		/*!
		\brief This tells you if the object is odd
		\returns Returns a boolean 
		*/
		property bool IsOdd{
			bool get(){if((_cbval->GetNumeratorValue(1)%2==1) && (_cbval->GetDenominatorValue(0)==1 && _cbval->GetDenominatorValue(1)==1)){
				return true;}return false;}
		}

#pragma endregion properties

		//!Tests the instance for primality.
		/*!
		\param _cert The number of cycles to run the algorithm.
		\returns A bool
		*/
		bool IsPrime(int _cert){

			return MillerRabbin(_cert);

		}
		//!Returns the string representation of the instance.
		/*!
		\returns A string object
		*/
		virtual String^ ToString()override{

			return ToString(OtherCrap::Base);

		}
		//!Returns the string representation of the instance.
		/*!
		\param _tbase The radix to use for calculating the string.
		\returns A string object
		*/
		String^ ToString(MyMath::Radix _tbase){

			BI^ _n=gcnew BI(_cbval->GetNumerator(), 0);
			BI^ _d=gcnew BI(_cbval->GetDenominator(), 0);

			String^ _retval=_n->ToString();
			
			if(_cbval->GetDenominatorValue(1)!=1 || _cbval->GetDenominatorValue(1)!=1){
				
				_retval=_retval + " / " + _d->ToString();

			}

			if(_cbval->GetSign()==1){

				_retval="-" + _retval;

			}

			return _retval;

		}
		//!Returns the absloute value of the instance.
		/*!
		\returns A BF object
		*/
		BF^ ABS(){

			BF^ _temp=gcnew BF(_cbval->GetNumerator());

			_temp->_cbval->SetSign(0);

			return _temp;

		}
		//!Calculates the Least Common Multiple.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ LCM(BF^ _val1, BF^ _val2){

			BF^ _retval=gcnew BF();

			_retval=_val1*_val2;

			_retval=_retval/GCD(_val1, _val2);

			return _retval;

		}
		//!Calculates the Greatest Common Divisor.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BF object
		*/
		static BF^ GCD(BF^ _val1, BF^ _val2){

			BF^ _a;
			BF^ _b;

			if(_val1>=_val2){

				_a=gcnew BF(_val1);
				_b=gcnew BF(_val2);

			}

			else{

				_a=gcnew BF(_val2);
				_b=gcnew BF(_val1);

			}

			BF^ _r=gcnew BF();

			while(_b>0){

				_r=_a%_b;
				_a=_b;
				_b=_r;

			}

			return _a;

		}
					
	};
	//!This class is the complex form of the BD class capable of holding any number.  All classes except BFI have implicit conversions to BDI
	[Guid("92D627C1-0646-44A4-944B-EED393C58024")]
	public ref class BDI : public IComparable
	{

	protected:

		//!This is the container for the value of the instance.  We use the CBDI* type so that we can access the assembly language functions by passing this object in.
		CBDI* _cbval;

	private:

		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isdone=false;
		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isinit=false;
		/*!
		   /deprecated This value is no longer used.
		*/
		static bool _isiniting=false;
		void ProcessString(String^ _val){

			//BDI^ _retval=gcnew BDI();
			//BDI^ _tret=gcnew BDI();

			//String^ _sbase="";
			//String^ _exp="";
			//String^ _dbase="";

			//bool _bneg=false;
			//bool _eneg=false;
			//bool _dec=false;
			//bool _doexp=false;
			//bool _doden=false;

			//int _deccount=0;

			//Radix _thebase=MyMath::Radix::_Decimal;

			//for(int x=0;x<_val->Length;x++){

			//	if(_val->Substring(x,1)=="a" || _val->Substring(x,1)=="A" || _val->Substring(x,1)=="b" || _val->Substring(x,1)=="B" ||  _val->Substring(x,1)=="c" || _val->Substring(x,1)=="C" ||
			//			_val->Substring(x,1)=="d" || _val->Substring(x,1)=="D" ||  _val->Substring(x,1)=="e" || _val->Substring(x,1)=="E" ||  _val->Substring(x,1)=="f" || _val->Substring(x,1)=="F"){

			//				if((int)_thebase<16){

			//					_thebase=MyMath::Radix::_HexaDecimal;

			//				}

			//	}

			//	else if(_val->Substring(x,1)=="G" || _val->Substring(x,1)=="H" || _val->Substring(x,1)=="I" || _val->Substring(x,1)=="J" || _val->Substring(x,1)=="K" || _val->Substring(x,1)=="L" ||
			//			_val->Substring(x,1)=="M" || _val->Substring(x,1)=="N" || _val->Substring(x,1)=="O" || _val->Substring(x,1)=="P" || _val->Substring(x,1)=="Q" || _val->Substring(x,1)=="R" ||
			//			_val->Substring(x,1)=="S" || _val->Substring(x,1)=="T" || _val->Substring(x,1)=="U" || _val->Substring(x,1)=="V" || _val->Substring(x,1)=="W" || _val->Substring(x,1)=="X"  ||
			//			_val->Substring(x,1)=="Y" || _val->Substring(x,1)=="Z"){

			//				if((int)_thebase<32){

			//					_thebase=MyMath::Radix::_Base32;

			//				}

			//	}

			//	else if(_val->Substring(x,1)=="g" || _val->Substring(x,1)=="h" || _val->Substring(x,1)=="i" || _val->Substring(x,1)=="j" || _val->Substring(x,1)=="k" || _val->Substring(x,1)=="l" ||
			//		_val->Substring(x,1)=="m" || _val->Substring(x,1)=="n" || _val->Substring(x,1)=="o" || _val->Substring(x,1)=="p" || _val->Substring(x,1)=="q" || _val->Substring(x,1)=="r"  ||
			//		_val->Substring(x,1)=="s" || _val->Substring(x,1)=="t" || _val->Substring(x,1)=="u" || _val->Substring(x,1)=="v" || _val->Substring(x,1)=="w" || _val->Substring(x,1)=="x"  ||
			//		_val->Substring(x,1)=="y" || _val->Substring(x,1)=="z"){

			//			_thebase=MyMath::Radix::_Base64;
			//			break;

			//	}

			//}

			//for(int x=0;x<_val->Length;x++){

			//	if(_dec && ! _doexp){

			//		_deccount+=1;

			//	}

			//	if(_val->Substring(x,1)=="-"){

			//		if(_doexp){

			//			_eneg=true;

			//		}

			//		else{

			//			_bneg=true;

			//		}

			//	}

			//	else if(_val->Substring(x,1)=="."){

			//		_dec=true;
			//		_deccount+=1;

			//	}

			//	else if(_val->Substring(x,1)=="e" || _val->Substring(x,1)=="E"){

			//		_doexp=true;

			//	}

			//	else if(_val->Substring(x,1)=="/"){

			//		_doden=true;

			//	}

			//	else if(_val->Substring(x,1)==" "){

			//	}

			//	else{

			//		if(_doexp){

			//			_exp=_exp+_val->Substring(x,1);

			//		}

			//		else{

			//			if(_doden){

			//				_dbase=_dbase+_val->Substring(x,1);

			//			}

			//			else{

			//				_sbase=_sbase+_val->Substring(x,1);

			//			}

			//		}

			//	}

			//}

			//BDI^ _base=gcnew BDI();
			//BDI^ _digit=gcnew BDI();
			//BDI^ _pow=gcnew BDI();
			//BDI^ _mult=gcnew BDI();

			//_base->_cbval->GetReal()->SetValue(0, 1);
			//_base->_cbval->GetReal()->SetValue(1, (int)_thebase);

			//_mult->_cbval->GetReal()->SetValue(0, 1);
			//_mult->_cbval->GetReal()->SetValue(1, (int)_thebase);

			//_retval->_cbval->GetReal()->SetValue(0, 1);

			//if(_sbase->Length<18 && (int)_thebase==10){

			//	_retval->_cbval->GetReal()->SetValue(1, System::Convert::ToUInt64(_sbase));

			//}

			//else{

			//	_retval->_cbval->GetReal()->SetValue(1, 0);

			//	for(int x=0;x<_sbase->Length;x++){

			//		//_digit->_cbval->GetReal()->SetValue(1, System::Convert::ToUInt64(_sbase->Substring(x,1)));
			//		//_retval=_retval*_mult;
			//		//_retval=_retval+_digit;

			//	}

			//}

			//if(_doden && _dbase->Length<18 && (int)_thebase==10){

			//	//_tret->_cbval->GetReal()->SetValue(1, System::Convert::ToUInt64(_dbase));

			//}

			//else{

			//	_tret->_cbval->GetReal()->SetValue(1, 0);

			//	for(int x=0;x<_dbase->Length;x++){

			//		//_digit->_cbval->GetReal()->SetValue(1, System::Convert::ToUInt64(_dbase->Substring(x,1)));
			//		//_tret=_tret*_mult;
			//		//_tret=_tret+_digit;

			//	}

			//}

			//if(_doexp){
			//			
			//	_digit->_cbval->GetReal()->SetValue(1, System::Convert::ToUInt64(_exp));

			//	if(_deccount>0){

			//		if(_eneg){

			//			_digit->_cbval->GetReal()->SetValue(1, _digit->_cbval->GetReal()->GetValue(1)+_deccount);

			//			for(int x=0;x<_digit->_cbval->GetReal()->GetValue(1);x++){

			//				//_retval=_retval/_mult;

			//			}

			//		}

			//		else{
			//			
			//			if(_deccount<=_digit->_cbval->GetReal()->GetValue(1)){

			//				_digit->_cbval->GetReal()->SetValue(1, _digit->_cbval->GetReal()->GetValue(1)-_deccount);

			//				if(_digit->_cbval->GetReal()->GetValue(1)>0){

			//					for(int x=0;x<=_digit->_cbval->GetReal()->GetValue(0);x++){

			//						//_retval=_retval*_mult;

			//					}

			//				}

			//			}

			//			else{

			//				_digit->_cbval->GetReal()->SetValue(1, _deccount-_digit->_cbval->GetReal()->GetValue(1));

			//				for(int x=0;x<=_digit->_cbval->GetReal()->GetValue(0);x++){

			//					//_retval=_retval/_mult;

			//				}

			//			}

			//		}

			//	}

			//	else{

			//		if(_eneg){

			//			for(int x=0;x<=_digit->_cbval->GetReal()->GetValue(0);x++){

			//				//_retval=_retval/_mult;

			//			}

			//		}

			//		else{

			//			for(int x=0;x<_digit->_cbval->GetReal()->GetValue(0);x++){

			//				//_retval=_retval*_mult;

			//			}

			//		}

			//	}

			//}

			//else{

			//	if(_deccount>0){

			//		for(int x=0;x<_deccount;x++){

			//			//_retval=_retval/_mult;

			//		}

			//	}

			//}

			//if(_bneg){

			//	_retval->_cbval->GetReal()->SetSign(1);

			//}

			//else{

			//	_retval->_cbval->GetReal()->SetSign(0);

			//}

			//if(_doden){

			//	//_retval=gcnew BDI(BDIDivide64(_retval->_cbval, _tret->_cbval));

			//}
			//delete _cbval;

			////_cbval=new CBDI(_retval->_cbval);
			//_cbval=new CBDI();

		}
		void Init(){

			if(!_isdone){

				_isdone=true;

			}
			//InitIt();

		}

	protected:

	public:

		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (short _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (unsigned short _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ ( int _val ){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (unsigned int _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (long _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (unsigned long _val){

			BDI^ _retval=gcnew BDI((unsigned __int64)_val);

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (__int64 _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (unsigned __int64 _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (char _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (unsigned char _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (float _val){

			BDI^ _retval=gcnew BDI();

			_retval->ProcessString(_val.ToString());

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (double _val){

			BDI^ _retval=gcnew BDI();

			_retval->ProcessString(_val.ToString());

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (String^ _val){

			BDI^ _retval=gcnew BDI();

			_retval->ProcessString(_val);

			return _retval;

		}
		//!Implicit cast to BDI.
		/*!
		\param _val The value to cast from.
		\returns A BDI object
		*/
		static operator BDI^ (BigInteger^ _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A char
		*/
		static explicit operator char(BDI^ _val){

			char _retval=0;

			if(_val->_cbval->GetReal()->GetValue(0)>0){

				if(_val->_cbval->GetReal()->GetSign()==0){

					_retval=_val->_cbval->GetReal()->GetValue(1)%256;
					
				}

				else{

					_retval=_val->_cbval->GetReal()->GetValue(1)%128;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A unsigned char
		*/
		static explicit operator unsigned char(BDI^ _val){

			if(_val->_cbval->GetReal()->GetValue(0)>0){

				return _val->_cbval->GetReal()->GetValue(1)%256;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A short
		*/
		static explicit operator short(BDI^ _val){

			short _retval=0;

			if(_val->_cbval->GetReal()->GetValue(0)>0){

				if(_val->_cbval->GetReal()->GetSign()==0){

					_retval=_val->_cbval->GetReal()->GetValue(1)%65536;
					
				}

				else{

					_retval=_val->_cbval->GetReal()->GetValue(1)%32768;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A unsigned short
		*/
		static explicit operator unsigned short(BDI^ _val){

			if(_val->_cbval->GetReal()->GetValue(0)>0){

				return _val->_cbval->GetReal()->GetValue(1)%65536;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A int
		*/
		static explicit operator int(BDI^ _val){

			int _retval=0;

			if(_val->_cbval->GetReal()->GetValue(0)>0){

				if(_val->_cbval->GetReal()->GetSign()==0){

					_retval=_val->_cbval->GetReal()->GetValue(1)%4294967296;
					
				}

				else{

					_retval=_val->_cbval->GetReal()->GetValue(1)%2147483648;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A unsigned int
		*/
		static explicit operator unsigned int(BDI^ _val){

			if(_val->_cbval->GetReal()->GetValue(0)>0){

				return _val->_cbval->GetReal()->GetValue(1)%4294967296;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A long
		*/
		static explicit operator long(BDI^ _val){

			long _retval=0;

			if(_val->_cbval->GetReal()->GetValue(0)>0){

				if(_val->_cbval->GetReal()->GetSign()==0){

					_retval=_val->_cbval->GetReal()->GetValue(1);
					
				}

				else{

					_retval=_val->_cbval->GetReal()->GetValue(1);
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A unsigned long
		*/
		static explicit operator unsigned long(BDI^ _val){

			if(_val->_cbval->GetReal()->GetValue(0)>0){

				return _val->_cbval->GetReal()->GetValue(1);

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A __int64
		*/
		static explicit operator __int64(BDI^ _val){

			__int64 _retval=0;

			if(_val->_cbval->GetReal()->GetValue(0)>0){

				if(_val->_cbval->GetReal()->GetSign()==0){

					_retval=_val->_cbval->GetReal()->GetValue(1);
					
				}

				else{

					_retval=_val->_cbval->GetReal()->GetValue(1);
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A unsigned __int64
		*/
		static explicit operator unsigned __int64(BDI^ _val){

			if(_val->_cbval->GetReal()->GetValue(0)>0){

				return _val->_cbval->GetReal()->GetValue(1);

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A string object
		*/
		static explicit operator String^(BDI^ _val){

			return _val->ToString();

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A BigInteger object
		*/
		static explicit operator BigInteger^(BDI^ _val){

			System::Numerics::BigInteger^ _base=gcnew System::Numerics::BigInteger(2);
			System::Numerics::BigInteger^ _mult=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _retval=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _digit=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _pow=gcnew System::Numerics::BigInteger();

			_base=System::Numerics::BigInteger::Pow(*_base, 64);

			for(int x=1;x<=_val->_cbval->GetReal()->GetValue(0);x++){

				_pow=System::Numerics::BigInteger::Pow(*_base, x-1);
				_digit=gcnew System::Numerics::BigInteger(_val->_cbval->GetReal()->GetValue(x));
				_mult=*_digit**_pow;
				_retval=*_retval+*_mult;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_retval=*_retval**gcnew System::Numerics::BigInteger(-1);

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A BI object
		*/
		static explicit operator BI^(BDI^ _val){

			BI^ _retval=gcnew BI();

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A BD object
		*/
		static explicit operator BD^(BDI^ _val){

			BD^ _retval=gcnew BD();

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A BF object
		*/
		static explicit operator BF^(BDI^ _val){

			BF^ _retval=gcnew BF();

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast from.
		\returns A BFI object
		*/
		static explicit operator BFI^(BDI^ _val);
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(String^ _val){

			//ProcessString(_val);
			//	Init();

		}
		//!CTOR.
		BDI(){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();

		}
		//!CTOR.
		BDI(bool _nn){

			//_cbval=new CBDI();

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(array<MYWORD,1>^ _val){	

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//int _len=_val[0]+1;

			//MYWORDPTR _num=SGetMem(_len*MYBYTES);

			//for(int x=0;x<=_val[0];x++){

			//	_num[x]=_val[x];

			//}

			//_cbval=new CBDI(_num, (int)0);

			//SFree(_num);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(__int64 _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();

			//if(_val<0){

			//	_cbval->GetReal()->SetSign(1);
			//	_val=_val*-1;

			//}

			//try{

			//	_cbval->GetReal()->SetValue(1, _val);
			//	int _pos=1;

			//}

			//catch(...){


			//}

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(char _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();

			//if(_val<0){

			//	_cbval->GetReal()->SetSign(1);
			//	_val=_val*-1;

			//}

			//_cbval->GetReal()->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(unsigned char _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();

			//_cbval->GetReal()->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(short _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();

			//if(_val<0){

			//	_cbval->GetReal()->SetSign(1);
			//	_val=_val*-1;

			//}

			//_cbval->GetReal()->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(unsigned short _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();
			//_cbval->GetReal()->SetValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(int _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();

			//if(_val<0){

			//	_cbval->GetReal()->SetSign(1);
			//	_val=_val*-1;

			//}

			//_cbval->GetReal()->SetValue(1, _val);
			//int _pos=1;

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(unsigned int _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();
			//_cbval->GetReal()->SetValue(1, _val);
			//int _pos=1;

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(float _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();

			//String^ _temp1=_val.ToString();
			//String^ _ret="";

			//for(int x=0;x<_temp1->Length;x++){

			//	_ret=_ret+_temp1->Substring(x,1);

			//}

			//ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(unsigned __int64 _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();
			//_cbval->GetReal()->SetValue(1, _val);
			//int _pos=1;

		}
		//!CTOR.
		/*!
		\param rhs The value to assign.
		*/
		BDI(BDI% rhs){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//if(rhs._cbval->GetReal()->GetSign()==1){

			//	_cbval=new CBDI(rhs._cbval->GetReal()->GetArray(), rhs._cbval->GetImaginary()->GetArray());

			//}

			//else{

			//	_cbval=new CBDI(rhs._cbval->GetReal()->GetArray(), rhs._cbval->GetImaginary()->GetArray());

			//}

		}
		//!CTOR.
		/*!
		\param rhs The value to assign.
		*/
		BDI(BDI% rhs, bool _nn){


			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//if(rhs._cbval->GetReal()->GetSign()==1){

			//	_cbval=new CBDI(rhs._cbval->GetReal()->GetArray(), rhs._cbval->GetImaginary()->GetArray());

			//}

			//else{

			//	_cbval=new CBDI(rhs._cbval->GetReal()->GetArray(), rhs._cbval->GetImaginary()->GetArray());

			//}

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(BDI^ _val, bool _nn){


			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//if(_val->_cbval->GetReal()->GetSign()==1){

			//	_cbval=new CBDI(_val->_cbval->GetReal()->GetArray(), _val->_cbval->GetImaginary()->GetArray());

			//}

			//else{

			//	_cbval=new CBDI(_val->_cbval->GetReal()->GetArray(), _val->_cbval->GetImaginary()->GetArray());

			//}

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(BDI^ _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//if(_val->_cbval->GetReal()->GetSign()==1){

			//	_cbval=new CBDI(_val->_cbval->GetReal()->GetArray(), _val->_cbval->GetImaginary()->GetArray());

			//}

			//else{

			//	_cbval=new CBDI(_val->_cbval->GetReal()->GetArray(), _val->_cbval->GetImaginary()->GetArray());

			//}

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(Decimal _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();
			//ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(double _val){

			//while(_isiniting){

			//	System::Threading::Thread::Sleep(1000);

			//}

			//if(!_isinit){

			//	_isiniting=true;
			//	_isinit=true;

			//	Init();
			//	_isiniting=false;

			//}

			//_cbval=new CBDI();

			//ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(BigInteger^ _val){

		//	System::Numerics::BigInteger^ _tval=gcnew System::Numerics::BigInteger();
		//	_tval=_val;

		//	int _pos=1;
		//	MYWORD _t1=0;
		//	MYWORD _t2=0;
		//	bool _sign=false;

		//	if(_tval->Sign==-1){

		//		System::Numerics::BigInteger^ _negone=gcnew System::Numerics::BigInteger(-1);
		//		_tval=*_tval**_negone;
		//		_sign=true;

		//	}

		//	array<System::Byte,1>^ _tarray=_tval->ToByteArray();

		//	int _count=_tarray->Length/MYBYTES;

		//	if(_tarray->Length%MYBYTES!=0){

		//		_count+=1;

		//	}

		//	_count+=1;

		//	MYWORDPTR _num;

		//	_num=SGetMem(_count*MYBYTES);

		//	_count-=1;

		//	_num[0]=_count;

		//#if _WIN64 || __amd64__ || _x64

		//	for(int x=0;x<_tarray->Length;x++){

		//		if(x%MYBYTES==0){

		//			_num[_pos]=_tarray[x];
		//			_t1=_tarray[x];

		//		}

		//		else if(x%MYBYTES==1){

		//			_t2=_tarray[x];
		//			_t2=_t2<<8;
		//			_t1=_t1+_t2;

		//			_num[_pos]=_t1;

		//		}

		//		else if(x%MYBYTES==2){

		//			_t2=_tarray[x];
		//			_t2=_t2<<16;
		//			_t1=_t1+_t2;

		//			_num[_pos]=_t1;

		//		}

		//		else if(x%MYBYTES==3){

		//			_t2=_tarray[x];
		//			_t2=_t2<<24;
		//			_t1=_t1+_t2;
		//			
		//			_num[_pos]=_t1;

		//		}

		//		else if(x%MYBYTES==4){

		//			_t2=_tarray[x];
		//			_t2=_t2<<32;
		//			_t1=_t1+_t2;

		//			_num[_pos]=_t1;

		//		}

		//		else if(x%MYBYTES==5){

		//			_t2=_tarray[x];
		//			_t2=_t2<<40;
		//			_t1=_t1+_t2;

		//			_num[_pos]=_t1;

		//		}

		//		else if(x%MYBYTES==6){

		//			_t2=_tarray[x];
		//			_t2=_t2<<48;
		//			_t1=_t1+_t2;

		//			_num[_pos]=_t1;

		//		}

		//		else if(x%MYBYTES==7){

		//			_t2=_tarray[x];
		//			_t2=_t2<<56;
		//			_t1=_t1+_t2;

		//			_num[_pos]=_t1;

		//			_pos+=1;
		//			_t1=0;

		//		}

		//	}

		//#else

		//	for(int x=0;x<_tarray->Length;x++){

		//		if(x%MYBYTES==0){

		//			_num[_pos]=_tarray[x];
		//			_t1=_tarray[x];

		//		}

		//		else if(x%MYBYTES==1){

		//			_t2=_tarray[x];
		//			_t2=_t2<<8;
		//			_t1=_t1+_t2;

		//			_num[_pos]=_t1;

		//		}

		//		else if(x%MYBYTES==2){

		//			_t2=_tarray[x];
		//			_t2=_t2<<16;
		//			_t1=_t1+_t2;

		//			_num[_pos]=_t1;

		//		}

		//		else if(x%MYBYTES==3){

		//			_t2=_tarray[x];
		//			_t2=_t2<<24;
		//			_t1=_t1+_t2;
		//			
		//			_num[_pos]=_t1;
		//			_pos+=1;
		//			_t1=0;

		//		}

		//	}

		//#endif

		//	while(_num[_num[0]]==0 && _num[0]>1){

		//		_num[0]=_num[0]-1;

		//	}

		//	_cbval=new CBDI(_num);

		//	SFree(_num);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(MYWORDPTR _val, bool _s){

			//_cbval=new CBDI(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(MYWORDPTR _val){

			//_cbval=new CBDI(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(CBDI* _val){

			_cbval=new CBDI(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(BI^ _val){

			//_cbval=new CBDI(_val->Val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(CBI* _val){

			//_cbval=new CBDI(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(BD^ _val){

			//_cbval=new CBDI(_val->Val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(CBD* _val){

			//_cbval=new CBDI(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(BF^ _val){

			//_cbval=new CBDI(_val->Val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign.
		*/
		BDI(CBF* _val){

			//_cbval=new CBDI(_val->_num);

		}

		~BDI(){

			this->!BDI();

		}
		!BDI(){

			delete _cbval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign.
		\returns A BDI object
		*/
		BDI^ operator=(BDI^ _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign.
		\returns A BDI object
		*/
		BDI^ operator=(System::Decimal _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign.
		\returns A BDI object
		*/
		BDI^ operator=(double _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param rhs The value to assign.
		\returns A BDI object
		*/
		BDI^ operator=(BDI% rhs){

			BDI^ _retval=gcnew BDI(rhs);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign.
		\returns A BDI object
		*/
		BDI^ operator=(__int64 _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign.
		\returns A BDI object
		*/
		BDI^ operator=(unsigned __int64 _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign.
		\returns A BDI object
		*/
		BDI^ operator=(BigInteger^ _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign.
		\returns A BDI object
		*/
		BDI^ operator=(String^ _val){

			BDI^ _retval=gcnew BDI(_val);

			return _retval;

		}
		//!Compares 2 instances for equality.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator==(BDI^ _val1, BDI^ _val2){

			if(BDIE64(_val1->_cbval,_val2->_cbval)==1){

				return true;

			}

			return false;

		}
		//!compares 2 instances for inequality.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator!=(BDI^ _val1, BDI^ _val2){

			return !(_val1==_val2);

		}
		//!Adds 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BDI object
		*/
		static BDI^ operator+=(BDI^ _val1, BDI^ _val2){

			return _val1+_val2;
		}
		//!Adds 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BDI object
		*/
		static BDI^ operator+(BDI^ _val1, BDI^ _val2){

			CBDI* _temp=BDIAdd64(_val1->_cbval, _val2->_cbval);
			BDI^ _retval=gcnew BDI(_temp);

			delete _temp;

			while(_retval->_cbval->GetReal()->GetValue(_retval->_cbval->GetReal()->GetValue(0))==0 && _retval->_cbval->GetReal()->GetValue(0)>1){

				_retval->_cbval->GetReal()->SetValue(0,_retval->_cbval->GetReal()->GetValue(0)-1);

			}

			if(_retval->_cbval->GetReal()->GetValue(0)==1 && _retval->_cbval->GetReal()->GetValue(1)==0){

				_retval->_cbval->GetReal()->SetSign(0);

			}

			return _retval;

		}
		//!Subtracts 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BDI object
		*/
		static BDI^ operator-=(BDI^ _val1, BDI^ _val2){

			return _val1-_val2;

		}
		//!Subtracts 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BDI object
		*/
		static BDI^ operator-(BDI^ _val1, BDI^ _val2){

			CBDI* _temp=BDISub64(_val1->_cbval, _val2->_cbval);
			BDI^ _retval=gcnew BDI(_temp);

			delete _temp;

			while(_retval->_cbval->GetReal()->GetValue(_retval->_cbval->GetReal()->GetValue(0))==0 && _retval->_cbval->GetReal()->GetValue(0)>1){

				_retval->_cbval->GetReal()->SetValue(0, _retval->_cbval->GetReal()->GetValue(0)-1);

			}

			if(_retval->_cbval->GetReal()->GetValue(0)==1 && _retval->_cbval->GetReal()->GetValue(1)==0){

				_retval->_cbval->GetReal()->SetSign(0);

			}

			return _retval;

		}
		//!Multiplies 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BDI object
		*/
		static BDI^ operator*=(BDI^ _val1, BDI^ _val2){

			return _val1*_val2;

		}
		//!Multiplies 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BDI object
		*/
		static BDI^ operator*(BDI^ _val1, BDI^ _val2){

			CBDI* _temp=BDIMultiply64(_val1->_cbval, _val2->_cbval);
			BDI^ _retval=gcnew BDI(_temp);

			delete _temp;

			while(_retval->_cbval->GetReal()->GetValue(_retval->_cbval->GetReal()->GetValue(0))==0 && _retval->_cbval->GetReal()->GetValue(0)>1){

				_retval->_cbval->GetReal()->SetValue(0, _retval->_cbval->GetReal()->GetValue(0)-1);

			}

			if(_retval->_cbval->GetReal()->GetValue(0)==1 && _retval->_cbval->GetReal()->GetValue(1)==0){

				_retval->_cbval->GetReal()->SetSign(0);

			}

			return _retval;

		}
		//!Divides 2 instances.
		/*!
		\param _val1 The dividend value.
		\param _val2 The divisor value.
		\returns A BDI object
		*/
		static BDI^ operator/=(BDI^ _val1, BDI^ _val2){

			return _val1/_val2;

		}
		//!Divides 2 instances.
		/*!
		\param _val1 The dividend value.
		\param _val2 The divisor value.
		\returns A BDI object
		*/
		static BDI^ operator/(BDI^ _val1, BDI^ _val2){

			CBDI* _temp=BDIDivide64(_val1->_cbval, _val2->_cbval);
			BDI^ _retval=gcnew BDI(_temp);

			delete _temp;

			if(_retval->_cbval->GetReal()->GetValue(0)==1 && _retval->_cbval->GetReal()->GetValue(1)==0){

				_retval->_cbval->GetReal()->SetSign(0);

			}

			return _retval;

		}
		//!Calculates the pow.
		/*!
		\param _base The _base value.
		\param _exp The exponent.
		\returns A BDI object
		*/
		static BDI^ Pow(BDI^ _base, BDI^ _exp){

			CBDI* _temp=BDIPow64(_base->_cbval, _exp->_cbval);
			BDI^ _retval=gcnew BDI(_temp);

			delete _temp;

			if(_retval->_cbval->GetReal()->GetValue(0)==1 && _retval->_cbval->GetReal()->GetValue(1)==0){

				_retval->_cbval->GetReal()->SetSign(0);

			}

			return _retval;

		}
		//!Negates the instance.
		void Negate(){

			if(_cbval->GetReal()->GetSign()==1){

				_cbval->GetReal()->SetSign(0);

			}

			else{

				_cbval->GetReal()->SetSign(1);

			}

		}
		//!Implemented for IComparable.
		/*!
		\param _val The object to compare against.
		\returns A int
		*/
		virtual int CompareTo(Object^ _val){

			BDI^ _temp = dynamic_cast<BDI^>(_val);

			int _retval=1;

			return _retval;

		}
		//!Sets the instance equal to zero.
		void SetToZero(){

			_cbval->SetToZero();

		}
		//!Sets the instance equal to one.
		void SetToOne(){

			_cbval->SetToOne();

		}

#pragma region Properties

		/*!
		\brief This is the native version of the BDI class, the managed class uses it as a container
		\returns Returns a CBDI* object 
		*/
		property CBDI* Val{
			CBDI* get(){return _cbval;}
			void set(CBDI* _val){_cbval->Take(_val);}
		}
		/*!
		\brief This property is used to set the global decimal limit for all BD and BDI objects
		\returns Returns a MYWORD 
		*/
		static property MYWORD DecimalLimit{
			MYWORD get(){return GetDecimalLimit();}
			void set(MYWORD _val){SetDecimalLimit(_val);}
		}
		/*!
		\brief This property tells the class how to handle rounding.  Global to all BD and BDI objects
		\returns Returns a MYWORD 
		*/
		static property MYWORD HowToHandleRounding{
			MYWORD get(){return GetHowToHandleRounding();}
			void set(MYWORD _val){SetHowToHandleRounding(_val);}
		}
		/*!
		\brief This is the real part of the object
		\returns Returns a CBD* object 
		*/
		property CBD* Real{
			CBD* get(){return _cbval->GetReal();}
			void set(CBD* value){_cbval->SetReal(value);}
		}
		/*!
		\brief This native array containing the value of the real part of this object
		\returns Returns a MYWORDPTR 
		*/
		property MYWORDPTR RealNumber{
			MYWORDPTR get(){return _cbval->GetReal()->GetArray();}
			void set(MYWORDPTR value){_cbval->GetReal()->SetArray(value);}
		}
		/*!
		\brief This gives you access to the individual elements of the array
		\returns Returns a MYWORD 
		*/
		property MYWORD RealNumber[int]{
			MYWORD get(int _index){return _cbval->GetReal()->GetValue(_index);}
			void set(int _index, MYWORD _val){_cbval->GetReal()->SetValue(_index, _val);}
		}
		/*!
		\brief This is the imaginary part of the object
		\returns Returns a CBD* object 
		*/
		property CBD* Imaginary{
			CBD* get(){return _cbval->GetImaginary();}
			void set(CBD* value){_cbval->SetImaginary(value);}
		}
		/*!
		\brief This native array containing the value of the imaginary part of this object
		\returns Returns a MYWORDPTR 
		*/
		property MYWORDPTR ImaginaryNumber{
			MYWORDPTR get(){return _cbval->GetImaginary()->GetArray();}
			void set(MYWORDPTR value){_cbval->GetImaginary()->SetArray(value);}
		}
		/*!
		\brief This native array containing the value of the imaginary part of this object
		\returns Returns a MYWORDPTR 
		*/
		property MYWORD ImaginaryNumber[int]{
			MYWORD get(int _index){return _cbval->GetImaginary()->GetValue(_index);}
			void set(int _index, MYWORD _val){_cbval->GetImaginary()->SetValue(_index, _val);}
		}
		/*!
		\brief This is the sign of the real part of this object 1==Negative 0==Positive
		\returns Returns a MYWORD 
		*/
		property MYWORD RealSign{
			MYWORD get(){return _cbval->GetReal()->GetSign();}
			void set(MYWORD _val){_cbval->GetReal()->SetSign(_val);}
		}
		/*!
		\brief This is the sign of the imaginary part of this object 1==Negative 0==Positive
		\returns Returns a MYWORD 
		*/
		property MYWORD ImaginarySign{
			MYWORD get(){return _cbval->GetImaginary()->GetSign();}
			void set(MYWORD _val){_cbval->GetImaginary()->SetSign(_val);}
		}
		/*!
		\brief This is the decimal point of the real part of this object
		\returns Returns a MYWORD 
		*/
		property MYWORD RealDecimalPlace{
			MYWORD get(){return _cbval->GetReal()->GetDecimalPlace();}
			void set(MYWORD _val){_cbval->GetReal()->SetDecimalPlace(_val);}
		}
		/*!
		\brief This is the decimal point of the imaginary part of this object
		\returns Returns a MYWORD 
		*/
		property MYWORD ImaginaryDecimalPlace{
			MYWORD get(){return _cbval->GetImaginary()->GetDecimalPlace();}
			void set(MYWORD _val){_cbval->GetImaginary()->SetDecimalPlace(_val);}
		}

#pragma endregion properties

		//!Creates the string equivalent of the value.
		/*!
		\returns A string object
		*/
		virtual String^ ToString()override{

			return ToString(OtherCrap::Base);

		}
		//!Creates the string equivalent of the value.
		/*!
		\param _tbase The base to use for the calculation.
		\returns A string object
		*/
		String^ ToString(MyMath::Radix _tbase){

			BD^ _r=gcnew BD(_cbval->GetReal()->GetArray());
			BD^ _i=gcnew BD(_cbval->GetImaginary()->GetArray());

			String^ _retval="( ";
			
			if(_cbval->GetReal()->GetSign()==1){
				
				_retval=_retval + "-";
				
			}
			
			_retval=_retval + _r->ToString();

			if(_cbval->GetImaginary()->GetSign()==1){

				_retval=_retval + " - ";

			}

			else{

				_retval=_retval + " + ";

			}

			_retval=_retval + _i->ToString() + " )";

			return _retval;

		}
		//!Returns the absolute value of the instance.
		/*!
		\returns A BDI object
		*/
		BDI^ ABS(){

			BDI^ _temp=gcnew BDI(_cbval->GetReal()->GetArray());

			_temp->_cbval->GetReal()->SetSign(0);

			return _temp;

		}
					
	};
	//!This class is the complex form of the BF class capable of holding all numbers.  All classes have implicit conversions to BFI
	[Guid("526BB557-EAE1-4A67-BF1C-8B0A18D95E47")]
	public ref class BFI : public IComparable
	{

	protected:

		//!This is the container for the value of the instance.  We use the CBFI* type so that we can access the assembly language functions by passing this object in.
		CBFI* _cbval;

	private:

		/*!
		/deprecated This value is no longer used.
		*/
		static bool _isdone=false;
		/*!
		deprecated
		*/
		static bool _isinit=false;
		/*!
		   /deprecated 
		*/
		static bool _isiniting=false;
		void ProcessString(String^ _val){

			BF^ _retval=gcnew BF();
			BF^ _tret=gcnew BF();

			String^ _sbase="";
			String^ _exp="";
			String^ _dbase="";

			bool _bneg=false;
			bool _eneg=false;
			bool _dec=false;
			bool _doexp=false;
			bool _doden=false;

			int _deccount=0;

			Radix _thebase=MyMath::Radix::_Decimal;

			for(int x=0;x<_val->Length;x++){

				if(_val->Substring(x,1)=="a" || _val->Substring(x,1)=="A" || _val->Substring(x,1)=="b" || _val->Substring(x,1)=="B" ||  _val->Substring(x,1)=="c" || _val->Substring(x,1)=="C" ||
						_val->Substring(x,1)=="d" || _val->Substring(x,1)=="D" ||  _val->Substring(x,1)=="e" || _val->Substring(x,1)=="E" ||  _val->Substring(x,1)=="f" || _val->Substring(x,1)=="F"){

							if((int)_thebase<16){

								_thebase=MyMath::Radix::_HexaDecimal;

							}

				}

				else if(_val->Substring(x,1)=="G" || _val->Substring(x,1)=="H" || _val->Substring(x,1)=="I" || _val->Substring(x,1)=="J" || _val->Substring(x,1)=="K" || _val->Substring(x,1)=="L" ||
						_val->Substring(x,1)=="M" || _val->Substring(x,1)=="N" || _val->Substring(x,1)=="O" || _val->Substring(x,1)=="P" || _val->Substring(x,1)=="Q" || _val->Substring(x,1)=="R" ||
						_val->Substring(x,1)=="S" || _val->Substring(x,1)=="T" || _val->Substring(x,1)=="U" || _val->Substring(x,1)=="V" || _val->Substring(x,1)=="W" || _val->Substring(x,1)=="X"  ||
						_val->Substring(x,1)=="Y" || _val->Substring(x,1)=="Z"){

							if((int)_thebase<32){

								_thebase=MyMath::Radix::_Base32;

							}

				}

				else if(_val->Substring(x,1)=="g" || _val->Substring(x,1)=="h" || _val->Substring(x,1)=="i" || _val->Substring(x,1)=="j" || _val->Substring(x,1)=="k" || _val->Substring(x,1)=="l" ||
					_val->Substring(x,1)=="m" || _val->Substring(x,1)=="n" || _val->Substring(x,1)=="o" || _val->Substring(x,1)=="p" || _val->Substring(x,1)=="q" || _val->Substring(x,1)=="r"  ||
					_val->Substring(x,1)=="s" || _val->Substring(x,1)=="t" || _val->Substring(x,1)=="u" || _val->Substring(x,1)=="v" || _val->Substring(x,1)=="w" || _val->Substring(x,1)=="x"  ||
					_val->Substring(x,1)=="y" || _val->Substring(x,1)=="z"){

						_thebase=MyMath::Radix::_Base64;
						break;

				}

			}

			for(int x=0;x<_val->Length;x++){

				if(_dec && ! _doexp){

					_deccount+=1;

				}

				if(_val->Substring(x,1)=="-"){

					if(_doexp){

						_eneg=true;

					}

					else{

						_bneg=true;

					}

				}

				else if(_val->Substring(x,1)=="."){

					_dec=true;
					_deccount+=1;

				}

				else if(_val->Substring(x,1)=="e" || _val->Substring(x,1)=="E"){

					_doexp=true;

				}

				else if(_val->Substring(x,1)=="/"){

					_doden=true;

				}

				else if(_val->Substring(x,1)==" "){

				}

				else{

					if(_doexp){

						_exp=_exp+_val->Substring(x,1);

					}

					else{

						if(_doden){

							_dbase=_dbase+_val->Substring(x,1);

						}

						else{

							_sbase=_sbase+_val->Substring(x,1);

						}

					}

				}

			}

			BF^ _base=gcnew BF();
			BF^ _digit=gcnew BF();
			BF^ _pow=gcnew BF();
			BF^ _mult=gcnew BF();

			_base->Numerator[0]= 1;
			_base->Numerator[1]= (int)_thebase;
			_base->Denominator[0]= 1;
			_base->Denominator[1]= 1;

			_mult->Numerator[0]= 1;
			_mult->Numerator[1]= (int)_thebase;
			_mult->Denominator[0]= 1;
			_mult->Denominator[1]= 1;

			_retval->Numerator[0]= 1;
			_retval->Denominator[0]= 1;
			_retval->Denominator[1]= 1;

			_tret->Numerator[0]= 1;
			_tret->Denominator[0]= 1;
			_tret->Denominator[1]= 1;

			_digit->Denominator[0]= 1;
			_digit->Denominator[1]= 1;

			if(_sbase->Length<18 && (int)_thebase==10){

				_retval->Numerator[1]= System::Convert::ToUInt64(_sbase);

			}

			else{

				_retval->Numerator[1]= 0;

				for(int x=0;x<_sbase->Length;x++){

					//_digit->Numerator[1]= System::Convert::ToUInt64(_sbase->Substring(x,1));
					//_retval=_retval*_mult;
					//_retval=_retval+_digit;

				}

			}

			if(_doden && _dbase->Length<18 && (int)_thebase==10){

				//_tret->Numerator[1]= System::Convert::ToUInt64(_dbase);

			}

			else{

				_tret->Numerator[1]= 0;

				for(int x=0;x<_dbase->Length;x++){

					//_digit->Numerator[1]= System::Convert::ToUInt64(_dbase->Substring(x,1));
					//_tret=_tret*_mult;
					//_tret=_tret+_digit;

				}

			}

			if(_doexp){
						
				_digit->Numerator[1]= System::Convert::ToUInt64(_exp);

				if(_deccount>0){

					if(_eneg){

						_digit->Numerator[1]= _digit->Numerator[1]+_deccount;

						for(int x=0;x<_digit->Numerator[0];x++){

							//_retval=_retval/_mult;

						}

					}

					else{
						
						if(_deccount<=_digit->Numerator[1]){

							_digit->Numerator[1]= _digit->Numerator[1]-_deccount;

							if(_digit->Numerator[1]>0){

								for(int x=0;x<=_digit->Numerator[0];x++){

									//_retval=_retval*_mult;

								}

							}

						}

						else{

							_digit->Numerator[1]= _deccount-_digit->Numerator[1];

							for(int x=0;x<=_digit->Numerator[0];x++){

								//_retval=_retval/_mult;

							}

						}

					}

				}

				else{

					if(_eneg){

						for(int x=0;x<=_digit->Numerator[0];x++){

							//_retval=_retval/_mult;

						}

					}

					else{

						for(int x=0;x<_digit->Numerator[0];x++){

							//_retval=_retval*_mult;

						}

					}

				}

			}

			else{

				if(_deccount>0){

					for(int x=0;x<_deccount;x++){

						//_retval=_retval/_mult;

					}

				}

			}

			if(_bneg){

				_retval->Sign=1;

			}

			else{

				_retval->Sign=0;

			}

			delete _cbval;

			//_cbval=new CBFI(_retval->Numerator, _tret->Numerator);
			_cbval=new CBFI();

		}
		void Init(){

			if(!_isdone){

				_isdone=true;

			}
			//InitIt();

		}

	protected:

	public:

		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (short _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (unsigned short _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ ( int _val ){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (unsigned int _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (long _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (unsigned long _val){

			BFI^ _retval=gcnew BFI((unsigned __int64)_val);

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (__int64 _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (unsigned __int64 _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (char _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (unsigned char _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (float _val){

			BFI^ _retval=gcnew BFI();

			_retval->ProcessString(_val.ToString());

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (double _val){

			BFI^ _retval=gcnew BFI();

			_retval->ProcessString(_val.ToString());

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (String^ _val){

			BFI^ _retval=gcnew BFI();

			_retval->ProcessString(_val);

			return _retval;

		}
		//!Implicit cast to BFI.
		/*!
		\param _val The value to cast.
		\returns A BFI object .
		*/
		static operator BFI^ (BigInteger^ _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A char.
		*/
		static explicit operator char(BFI^ _val){

			char _retval=0;

			if(_val->_cbval->GetReal()->GetNumeratorValue(0)>0){

				if(_val->_cbval->GetReal()->GetSign()==0){

					_retval=_val->_cbval->GetReal()->GetNumeratorValue(1)%256;
					
				}

				else{

					_retval=_val->_cbval->GetReal()->GetNumeratorValue(1)%128;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A unsigned char.
		*/
		static explicit operator unsigned char(BFI^ _val){

			if(_val->_cbval->GetReal()->GetNumeratorValue(0)>0){

				return _val->_cbval->GetReal()->GetNumeratorValue(1)%256;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A short.
		*/
		static explicit operator short(BFI^ _val){

			short _retval=0;

			if(_val->_cbval->GetReal()->GetNumeratorValue(0)>0){

				if(_val->_cbval->GetReal()->GetSign()==0){

					_retval=_val->_cbval->GetReal()->GetNumeratorValue(1)%65536;
					
				}

				else{

					_retval=_val->_cbval->GetReal()->GetNumeratorValue(1)%32768;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A unsigned short.
		*/
		static explicit operator unsigned short(BFI^ _val){

			if(_val->_cbval->GetReal()->GetNumeratorValue(0)>0){

				return _val->_cbval->GetReal()->GetNumeratorValue(1)%65536;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A int.
		*/
		static explicit operator int(BFI^ _val){

			int _retval=0;

			if(_val->_cbval->GetReal()->GetNumeratorValue(0)>0){

				if(_val->_cbval->GetReal()->GetSign()==0){

					_retval=_val->_cbval->GetReal()->GetNumeratorValue(1)%4294967296;
					
				}

				else{

					_retval=_val->_cbval->GetReal()->GetNumeratorValue(1)%2147483648;
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A unsigned int.
		*/
		static explicit operator unsigned int(BFI^ _val){

			if(_val->_cbval->GetReal()->GetNumeratorValue(0)>0){

				return _val->_cbval->GetReal()->GetNumeratorValue(1)%4294967296;

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A long.
		*/
		static explicit operator long(BFI^ _val){

			long _retval=0;

			if(_val->_cbval->GetReal()->GetNumeratorValue(0)>0){

				if(_val->_cbval->GetReal()->GetSign()==0){

					_retval=_val->_cbval->GetReal()->GetNumeratorValue(1);
					
				}

				else{

					_retval=_val->_cbval->GetReal()->GetNumeratorValue(1);
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A unsigned long.
		*/
		static explicit operator unsigned long(BFI^ _val){

			if(_val->_cbval->GetReal()->GetNumeratorValue(0)>0){

				return _val->_cbval->GetReal()->GetNumeratorValue(1);

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A __int64 .
		*/
		static explicit operator __int64(BFI^ _val){

			__int64 _retval=0;

			if(_val->_cbval->GetReal()->GetNumeratorValue(0)>0){

				if(_val->_cbval->GetReal()->GetSign()==0){

					_retval=_val->_cbval->GetReal()->GetNumeratorValue(1);
					
				}

				else{

					_retval=_val->_cbval->GetReal()->GetNumeratorValue(1);
					_retval=_retval*-1;

				}

			}

			else{

				return 0;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_retval=_retval*-1;

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A unsigned __int64.
		*/
		static explicit operator unsigned __int64(BFI^ _val){

			if(_val->_cbval->GetReal()->GetNumeratorValue(0)>0){

				return _val->_cbval->GetReal()->GetNumeratorValue(1);

			}

			else{

				return 0;

			}

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A string object.
		*/
		static explicit operator String^(BFI^ _val){

			return _val->ToString();

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A BigInteger object.
		*/
		static explicit operator BigInteger^(BFI^ _val){

			System::Numerics::BigInteger^ _base=gcnew System::Numerics::BigInteger(2);
			System::Numerics::BigInteger^ _mult=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _retval=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _digit=gcnew System::Numerics::BigInteger();
			System::Numerics::BigInteger^ _pow=gcnew System::Numerics::BigInteger();

			_base=System::Numerics::BigInteger::Pow(*_base, 64);

			for(int x=1;x<=_val->_cbval->GetReal()->GetNumeratorValue(0);x++){

				_pow=System::Numerics::BigInteger::Pow(*_base, x-1);
				_digit=gcnew System::Numerics::BigInteger(_val->_cbval->GetReal()->GetNumeratorValue(x));
				_mult=*_digit**_pow;
				_retval=*_retval+*_mult;

			}

			if(_val->RealSign==1){

				_retval=*_retval**gcnew System::Numerics::BigInteger(-1);

			}

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A BI object.
		*/
		static explicit operator BI^(BFI^ _val){

			BI^ _retval=gcnew BI();

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A BD object.
		*/
		static explicit operator BD^(BFI^ _val){

			BD^ _retval=gcnew BD();

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A BF object.
		*/
		static explicit operator BF^(BFI^ _val){

			BF^ _retval=gcnew BF();

			return _retval;

		}
		//!Explicit cast.
		/*!
		\param _val The value to cast.
		\returns A BDI object.
		*/
		static explicit operator BDI^(BFI^ _val){

			BDI^ _retval=gcnew BDI();

			return _retval;

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(String^ _val){

			ProcessString(_val);
				Init();

		}
		//!CTOR.
		BFI(){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();

		}
		//!CTOR.
		BFI(bool _nn){

			_cbval=new CBFI();

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(array<MYWORD,1>^ _val){	

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			int _len=_val[0]+1;

			MYWORDPTR _num=SGetMem(_len*MYBYTES);

			for(int x=0;x<=_val[0];x++){

				_num[x]=_val[x];

			}

			_cbval=new CBFI(_num, (int)0);

			SFree(_num);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(__int64 _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();

			if(_val<0){

				_cbval->GetReal()->SetSign(1);
				_val=_val*-1;

			}

			try{

			_cbval->GetReal()->SetNumeratorValue(1, _val);
			int _pos=1;

			}

			catch(...){


			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(char _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();

			if(_val<0){

				_cbval->GetReal()->SetSign(1);
				_val=_val*-1;

			}

			_cbval->GetReal()->SetNumeratorValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(unsigned char _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();

			_cbval->GetReal()->SetNumeratorValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(short _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();

			if(_val<0){

				_cbval->GetReal()->SetSign(1);
				_val=_val*-1;

			}

			_cbval->GetReal()->SetNumeratorValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(unsigned short _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();
			_cbval->GetReal()->SetNumeratorValue(1, _val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(int _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();

			if(_val<0){

				_cbval->GetReal()->SetSign(1);
				_val=_val*-1;

			}

			_cbval->GetReal()->SetNumeratorValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(unsigned int _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();
			_cbval->GetReal()->SetNumeratorValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(float _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();

			String^ _temp1=_val.ToString();
			String^ _ret="";

			for(int x=0;x<_temp1->Length;x++){

				_ret=_ret+_temp1->Substring(x,1);

			}

			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(unsigned __int64 _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();
			_cbval->GetReal()->SetNumeratorValue(1, _val);
			int _pos=1;

		}
		//!CTOR.
		/*!
		\param rhs The value to assign to this instance.
		*/
		BFI(BFI% rhs){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(rhs._cbval->GetReal()->GetSign()==1){

				_cbval=new CBFI(rhs._cbval->GetReal()->GetNumerator(), rhs._cbval->GetImaginary()->GetNumerator());

			}

			else{

				_cbval=new CBFI(rhs._cbval->GetReal()->GetNumerator(), rhs._cbval->GetImaginary()->GetNumerator());

			}

		}
		//!CTOR.
		/*!
		\param rhs The value to assign to this instance.
		*/
		BFI(BFI% rhs, bool _nn){


			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(rhs._cbval->GetReal()->GetSign()==1){

				_cbval=new CBFI(rhs._cbval->GetReal()->GetNumerator(), rhs._cbval->GetImaginary()->GetNumerator());

			}

			else{

				_cbval=new CBFI(rhs._cbval->GetReal()->GetNumerator(), rhs._cbval->GetImaginary()->GetNumerator());

			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(BFI^ _val, bool _nn){


			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_cbval=new CBFI(_val->_cbval->GetReal()->GetNumerator(), _val->_cbval->GetImaginary()->GetNumerator());

			}

			else{

				_cbval=new CBFI(_val->_cbval->GetReal()->GetNumerator(), _val->_cbval->GetImaginary()->GetNumerator());

			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(BFI^ _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			if(_val->_cbval->GetReal()->GetSign()==1){

				_cbval=new CBFI(_val->_cbval->GetReal()->GetNumerator(), _val->_cbval->GetImaginary()->GetNumerator());

			}

			else{

				_cbval=new CBFI(_val->_cbval->GetReal()->GetNumerator(), _val->_cbval->GetImaginary()->GetNumerator());

			}

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(Decimal _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();
			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(double _val){

			while(_isiniting){

				System::Threading::Thread::Sleep(1000);

			}

			if(!_isinit){

				_isiniting=true;
				_isinit=true;

				Init();
				_isiniting=false;

			}

			_cbval=new CBFI();

			ProcessString(_val.ToString());

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(BigInteger^ _val){

			System::Numerics::BigInteger^ _tval=gcnew System::Numerics::BigInteger();
			_tval=_val;

			int _pos=1;
			MYWORD _t1=0;
			MYWORD _t2=0;
			bool _sign=false;

			if(_tval->Sign==-1){

				System::Numerics::BigInteger^ _negone=gcnew System::Numerics::BigInteger(-1);
				_tval=*_tval**_negone;
				_sign=true;

			}

			array<System::Byte,1>^ _tarray=_tval->ToByteArray();

			int _count=_tarray->Length/MYBYTES;

			if(_tarray->Length%MYBYTES!=0){

				_count+=1;

			}

			_count+=1;

			MYWORDPTR _num;

			_num=SGetMem(_count*MYBYTES);

			_count-=1;

			_num[0]=_count;

		#if _WIN64 || __amd64__ || _x64

			for(int x=0;x<_tarray->Length;x++){

				if(x%MYBYTES==0){

					_num[_pos]=_tarray[x];
					_t1=_tarray[x];

				}

				else if(x%MYBYTES==1){

					_t2=_tarray[x];
					_t2=_t2<<8;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==2){

					_t2=_tarray[x];
					_t2=_t2<<16;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==3){

					_t2=_tarray[x];
					_t2=_t2<<24;
					_t1=_t1+_t2;
					
					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==4){

					_t2=_tarray[x];
					_t2=_t2<<32;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==5){

					_t2=_tarray[x];
					_t2=_t2<<40;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==6){

					_t2=_tarray[x];
					_t2=_t2<<48;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==7){

					_t2=_tarray[x];
					_t2=_t2<<56;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

					_pos+=1;
					_t1=0;

				}

			}

		#else

			for(int x=0;x<_tarray->Length;x++){

				if(x%MYBYTES==0){

					_num[_pos]=_tarray[x];
					_t1=_tarray[x];

				}

				else if(x%MYBYTES==1){

					_t2=_tarray[x];
					_t2=_t2<<8;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==2){

					_t2=_tarray[x];
					_t2=_t2<<16;
					_t1=_t1+_t2;

					_num[_pos]=_t1;

				}

				else if(x%MYBYTES==3){

					_t2=_tarray[x];
					_t2=_t2<<24;
					_t1=_t1+_t2;
					
					_num[_pos]=_t1;
					_pos+=1;
					_t1=0;

				}

			}

		#endif

			while(_num[_num[0]]==0 && _num[0]>1){

				_num[0]=_num[0]-1;

			}

			_cbval=new CBFI(_num);

			SFree(_num);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(MYWORDPTR _val, bool _s){

			_cbval=new CBFI(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(MYWORDPTR _val){

			_cbval=new CBFI(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(CBFI* _val){

			_cbval=new CBFI(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(BI^ _val){

			_cbval=new CBFI(_val->Val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(CBI* _val){

			_cbval=new CBFI(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(BD^ _val){

			_cbval=new CBFI(_val->Val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(CBD* _val){

			_cbval=new CBFI(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(BF^ _val){

			_cbval=new CBFI(_val->Val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(CBF* _val){

			_cbval=new CBFI(_val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(BDI^ _val){

			_cbval=new CBFI(_val->Val);

		}
		//!CTOR.
		/*!
		\param _val The value to assign to this instance.
		*/
		BFI(CBDI* _val){

			_cbval=new CBFI(_val);

		}

		~BFI(){

			this->!BFI();

		}
		!BFI(){

			delete _cbval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to this instance.
		\returns A BFI object
		*/
		BFI^ operator=(BFI^ _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to this instance.
		\returns A BFI object
		*/
		BFI^ operator=(System::Decimal _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to this instance.
		\returns A BFI object
		*/
		BFI^ operator=(double _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param rhs The value to assign to this instance.
		\returns A BFI object
		*/
		BFI^ operator=(BFI% rhs){

			BFI^ _retval=gcnew BFI(rhs);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to this instance.
		\returns A BFI object
		*/
		BFI^ operator=(__int64 _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to this instance.
		\returns A BFI object
		*/
		BFI^ operator=(unsigned __int64 _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to this instance.
		\returns A BFI object
		*/
		BFI^ operator=(BigInteger^ _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!Assignment.
		/*!
		\param _val The value to assign to this instance.
		\returns A BFI object
		*/
		BFI^ operator=(String^ _val){

			BFI^ _retval=gcnew BFI(_val);

			return _retval;

		}
		//!compares 2 instances for equality.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator==(BFI^ _val1, BFI^ _val2){

			if(BFIE64(_val1->_cbval,_val2->_cbval)==1){

				return true;

			}

			return false;

		}
		//!compares 2 instances for inequality.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A bool
		*/
		static bool operator!=(BFI^ _val1, BFI^ _val2){

			return !(_val1==_val2);

		}
		//!adds 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BFI Object
		*/
		static BFI^ operator+=(BFI^ _val1, BFI^ _val2){

			return _val1+_val2;
		}
		//!adds 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BFI Object
		*/
		static BFI^ operator+(BFI^ _val1, BFI^ _val2){

			CBFI* _temp=BFIAdd64(_val1->_cbval, _val2->_cbval);
			BFI^ _retval=gcnew BFI(_temp);

			delete _temp;

			while(_retval->_cbval->GetReal()->GetNumeratorValue(_retval->_cbval->GetReal()->GetNumeratorValue(0))==0 && _retval->_cbval->GetReal()->GetNumeratorValue(0)>1){

				_retval->_cbval->GetReal()->SetNumeratorValue(0,_retval->_cbval->GetReal()->GetNumeratorValue(0)-1);

			}

			if(_retval->_cbval->GetReal()->GetNumeratorValue(0)==1 && _retval->_cbval->GetReal()->GetNumeratorValue(1)==0){

				_retval->_cbval->GetReal()->SetSign(0);

			}

			return _retval;

		}
		//!subtracts 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BFI Object
		*/
		static BFI^ operator-=(BFI^ _val1, BFI^ _val2){

			return _val1-_val2;

		}
		//!subtracts 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BFI Object
		*/
		static BFI^ operator-(BFI^ _val1, BFI^ _val2){

			CBFI* _temp=BFISub64(_val1->_cbval, _val2->_cbval);
			BFI^ _retval=gcnew BFI(_temp);

			delete _temp;

			while(_retval->_cbval->GetReal()->GetNumeratorValue(_retval->_cbval->GetReal()->GetNumeratorValue(0))==0 && _retval->_cbval->GetReal()->GetNumeratorValue(0)>1){

				_retval->_cbval->GetReal()->SetNumeratorValue(0, _retval->_cbval->GetReal()->GetNumeratorValue(0)-1);

			}

			if(_retval->_cbval->GetReal()->GetNumeratorValue(0)==1 && _retval->_cbval->GetReal()->GetNumeratorValue(1)==0){

				_retval->_cbval->GetReal()->SetSign(0);

			}

			return _retval;

		}
		//!Multiplies 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BFI Object
		*/
		static BFI^ operator*=(BFI^ _val1, BFI^ _val2){

			return _val1*_val2;

		}
		//!Multiplies 2 instances.
		/*!
		\param _val1 The first value.
		\param _val2 The second value.
		\returns A BFI Object
		*/
		static BFI^ operator*(BFI^ _val1, BFI^ _val2){

			CBFI* _temp=BFIMultiply64(_val1->_cbval, _val2->_cbval);
			BFI^ _retval=gcnew BFI(_temp);

			delete _temp;

			while(_retval->_cbval->GetReal()->GetNumeratorValue(_retval->_cbval->GetReal()->GetNumeratorValue(0))==0 && _retval->_cbval->GetReal()->GetNumeratorValue(0)>1){

				_retval->_cbval->GetReal()->SetNumeratorValue(0, _retval->_cbval->GetReal()->GetNumeratorValue(0)-1);

			}

			if(_retval->_cbval->GetReal()->GetNumeratorValue(0)==1 && _retval->_cbval->GetReal()->GetNumeratorValue(1)==0){

				_retval->_cbval->GetReal()->SetSign(0);

			}

			return _retval;

		}
		//!Divides 2 instances.
		/*!
		\param _val1 The dividend.
		\param _val2 The divisor.
		\returns A BFI Object
		*/
		static BFI^ operator/=(BFI^ _val1, BFI^ _val2){

			return _val1/_val2;

		}
		//!Divides 2 instances.
		/*!
		\param _val1 The dividend.
		\param _val2 The divisor.
		\returns A BFI Object
		*/
		static BFI^ operator/(BFI^ _val1, BFI^ _val2){

			CBFI* _temp=BFIDivide64(_val1->_cbval, _val2->_cbval);
			BFI^ _retval=gcnew BFI(_temp);

			delete _temp;

			if(_retval->_cbval->GetReal()->GetNumeratorValue(0)==1 && _retval->_cbval->GetReal()->GetNumeratorValue(1)==0){

				_retval->_cbval->GetReal()->SetSign(0);

			}

			return _retval;

		}
		//!Calculates the Pow root of a number.
		/*!
		\param _base The value to calculate the power of.
		\param _exp The exponent to use.
		\returns A BFI Object
		*/
		static BFI^ Pow(BFI^ _base, BFI^ _exp){

			CBFI* _temp=BFIPow64(_base->_cbval, _exp->_cbval);
			BFI^ _retval=gcnew BFI(_temp);

			delete _temp;

			if(_retval->_cbval->GetReal()->GetNumeratorValue(0)==1 && _retval->_cbval->GetReal()->GetNumeratorValue(1)==0){

				_retval->_cbval->GetReal()->SetSign(0);

			}

			return _retval;

		}
		//!Negate This instance.
		void Negate(){

			if(_cbval->GetReal()->GetSign()==1){

				_cbval->GetReal()->SetSign(0);

			}

			else{

				_cbval->GetReal()->SetSign(1);

			}

		}
		//!Implemented for IComparable
		/*!
		\param _val The value to compare against.
		\returns an int.
		*/
		virtual int CompareTo(Object^ _val){

			BFI^ _temp = dynamic_cast<BFI^>(_val);

			int _retval=1;

			return _retval;

		}
		//!Sets this instance equal to zero.
		void SetToZero(){

			_cbval->SetToZero();

		}
		//!Sets this instance equal to one.
		void SetToOne(){

			_cbval->SetToOne();

		}

#pragma region Properties

		/*!
		\brief This is the native version of the CBFI class. the managed side uses it as a container
		\returns Returns a CBFI* object 
		*/
		property CBFI* Val{
			CBFI* get(){return _cbval;}
			void set(CBFI* _val){_cbval->Take(_val);}
		}
		/*!
		\brief This property gives access to the real side of the object
		\returns Returns a CBF* 
		*/
		property CBF* Real{
			CBF* get(){return _cbval->GetReal();}
			void set(CBF* value){_cbval->SetReal(value);}
		}
		/*!
		\brief This is the native array containing the value of the real numerator
		\returns Returns a MYWORDPTR 
		*/
		property MYWORDPTR RealNumerator{
			MYWORDPTR get(){return _cbval->GetReal()->GetNumerator();}
			void set(MYWORDPTR value){_cbval->GetReal()->SetNumerator(value);}
		}
		/*!
		\brief This gives access to the individual elements of the real numerator
		\returns Returns a MYWORD 
		*/
		property MYWORD RealNumerator[int]{
			MYWORD get(int _index){return _cbval->GetReal()->GetNumeratorValue(_index);}
			void set(int _index, MYWORD _val){_cbval->GetReal()->SetNumeratorValue(_index, _val);}
		}
		/*!
		\brief This is the native array containing the value of the real denominator
		\returns Returns a MYWORDPTR 
		*/
		property MYWORDPTR RealDenominator{
			MYWORDPTR get(){return _cbval->GetReal()->GetDenominator();}
			void set(MYWORDPTR value){_cbval->GetReal()->SetDenominator(value);}
		}
		/*!
		\brief This gives access to the individual elements of the real denominator
		\returns Returns a MYWORD 
		*/
		property MYWORD RealDenominator[int]{
			MYWORD get(int _index){return _cbval->GetReal()->GetDenominatorValue(_index);}
			void set(int _index, MYWORD _val){_cbval->GetReal()->SetDenominatorValue(_index, _val);}
		}
		/*!
		\brief This property gives access to the imaginary side of the object
		\returns Returns a CBF* 
		*/
		property CBF* Imaginary{
			CBF* get(){return _cbval->GetImaginary();}
			void set(CBF* value){_cbval->SetImaginary(value);}
		}
		/*!
		\brief This is the native array containing the value of the imaginary numerator
		\returns Returns a MYWORDPTR 
		*/
		property MYWORDPTR ImaginaryNumerator{
			MYWORDPTR get(){return _cbval->GetImaginary()->GetNumerator();}
			void set(MYWORDPTR value){_cbval->GetImaginary()->SetNumerator(value);}
		}
		/*!
		\brief This gives access to the individual elements of the imaginary numerator
		\returns Returns a MYWORD 
		*/
		property MYWORD ImaginaryNumerator[int]{
			MYWORD get(int _index){return _cbval->GetImaginary()->GetNumeratorValue(_index);}
			void set(int _index, MYWORD _val){_cbval->GetImaginary()->SetNumeratorValue(_index, _val);}
		}
		/*!
		\brief This is the native array containing the value of the imaginary denominator
		\returns Returns a MYWORDPTR 
		*/
		property MYWORDPTR ImaginaryDenominator{
			MYWORDPTR get(){return _cbval->GetImaginary()->GetDenominator();}
			void set(MYWORDPTR value){_cbval->GetImaginary()->SetDenominator(value);}
		}
		/*!
		\brief This gives access to the individual elements of the imaginary denominator
		\returns Returns a MYWORD 
		*/
		property MYWORD ImaginaryDenominator[int]{
			MYWORD get(int _index){return _cbval->GetImaginary()->GetDenominatorValue(_index);}
			void set(int _index, MYWORD _val){_cbval->GetImaginary()->SetDenominatorValue(_index, _val);}
		}
		/*!
		\brief This represents the sign of the real part of the object 1==Negative 0==Positive
		\returns Returns a MYWORD 
		*/
		property MYWORD RealSign{
			MYWORD get(){return _cbval->GetReal()->GetSign();}
			void set(MYWORD _val){_cbval->GetReal()->SetSign(_val);}
		}
		/*!
		\brief This represents the sign of the imaginary part of the object 1==Negative 0==Positive
		\returns Returns a MYWORD 
		*/
		property MYWORD ImaginarySign{
			MYWORD get(){return _cbval->GetImaginary()->GetSign();}
			void set(MYWORD _val){_cbval->GetImaginary()->SetSign(_val);}
		}

#pragma endregion properties

		//!Produces the string for this value.
		/*!
		\returns A String Object
		*/
		virtual String^ ToString()override{

			return ToString(OtherCrap::Base);

		}
		//!Produces the string for this value.
		/*!
		\param _tbase The base to use to calculate the string value.
		\returns A string Object
		*/
		String^ ToString(MyMath::Radix _tbase){

			BF^ _r=gcnew BF(_cbval->GetReal());
			BF^ _i=gcnew BF(_cbval->GetImaginary());

			String^ _retval="( ";
			
			if(_cbval->GetReal()->GetSign()==1){
				
				_retval=_retval + "-";
				
			}
			
			_retval=_retval + _r->ToString();

			if(_cbval->GetImaginary()->GetSign()==1){

				_retval=_retval + " - ";

			}

			else{

				_retval=_retval + " + ";

			}

			_retval=_retval + _i->ToString() + " )";

			return _retval;
		}
		//!Returns the absolute value for the instance.
		/*!
		\returns A BFI Object
		*/
		BFI^ ABS(){

			BFI^ _temp=gcnew BFI(_cbval->GetReal()->GetNumerator());

			_temp->_cbval->GetReal()->SetSign(0);

			return _temp;

		}
					
	};
	//!This namespace is going to be the container for all the math stuff
	namespace MathStuff{

		//!This class is Container class for most of the math functions to be used by the library
		[Guid("0896EE27-16C6-49AC-B2C4-8581B32D35EB")]
		public ref class Math{

		private:


			static BI^ SinRecursive(BI^ _val, int _current){

				BI^ _retval=gcnew BI();
				BI^ _temp=gcnew BI();

				if(_current>1){

					_temp=BI::Pow(_val, _current)/Factorial((BI^)_current);

					if(((_current+1)/2)%2==0){

						_retval=_temp-SinRecursive(_val, _current-2);

					}

					else{

						_retval=_temp+SinRecursive(_val, _current-2);

					}

				}

				else{

					_retval=_val;

				}

				return _retval;

			}
			static BD^ SinRecursive(BD^ _val, int _current){

				BD^ _retval=gcnew BD();
				BD^ _temp=gcnew BD();

				if(_current>1){

					_temp=BD::Pow(_val, _current)/Factorial((BD^)_current);

					if(((_current+1)/2)%2==0){

						_retval=_temp-SinRecursive(_val, _current-2);

					}

					else{

						_retval=_temp+SinRecursive(_val, _current-2);

					}

				}

				else{

					_retval=_val;

				}

				return _retval;

			}
			static BF^ SinRecursive(BF^ _val, int _current){

				BF^ _retval=gcnew BF();
				BF^ _temp=gcnew BF();

				if(_current>1){

					_temp=BF::Pow(_val, _current)/Factorial((BF^)_current);

					if(((_current+1)/2)%2==0){

						_retval=_temp-SinRecursive(_val, _current-2);

					}

					else{

						_retval=_temp+SinRecursive(_val, _current-2);

					}

				}

				else{

					_retval=_val;

				}

				return _retval;

			}
			static BI^ CosRecursive(BI^ _val, int _current){

				BI^ _retval=gcnew BI();
				BI^ _temp=gcnew BI();

				if(_current>0){

					_temp=BI::Pow(_val,_current)/Factorial((BI^)_current);

					if((_current/2)%2==1){

						_retval=_temp-CosRecursive(_val, _current-2);

					}

					else{

						_retval=_temp+CosRecursive(_val, _current-2);

					}

				}

				else{

					_retval=1;

				}

				return _retval;

			}
			static BD^ CosRecursive(BD^ _val, int _current){

				BD^ _retval=gcnew BD();
				BD^ _temp=gcnew BD();

				if(_current>0){

					_temp=BD::Pow(_val,_current)/Factorial((BD^)_current);

					if((_current/2)%2==1){

						_retval=CosRecursive(_val, _current-2)-_temp;

					}

					else{

						_retval=CosRecursive(_val, _current-2)+_temp;

					}

				}

				else{

					_retval=1;

				}

				return _retval;

			}
			static BF^ CosRecursive(BF^ _val, int _current){

				BF^ _retval=gcnew BF();

				if(_current>0){

					if((_current/2)%2==1){

						_retval=BF::Pow(_val,_current)/Factorial((BF^)_current)+CosRecursive(_val, _current-2);

					}

					else{

						_retval=BF::Pow(_val,_current)/Factorial((BF^)_current)-CosRecursive(_val, _current-2);

					}

				}

				else{

					_retval=_val-_retval;

				}

				return _retval;

			}
			static BI^ TanAlg(BI^ _val){

				BI^ _term1=gcnew BI();
				BI^ _term2=gcnew BI();
				BI^ _term3=gcnew BI();

				_term1=BI::Pow(_val, 3)/3;
				_term2=(2*(BI::Pow(_val, 5)))/15;
				_term3=(17*(BI::Pow(_val, 7)))/315;

				BI^ _retval=gcnew BI();

				_retval=_val+_term1+_term2+_term3;

				return _retval;

			}
			static BD^ TanAlg(BD^ _val){

				BD^ _term1=gcnew BD();
				BD^ _term2=gcnew BD();
				BD^ _term3=gcnew BD();

				_term1=BD::Pow(_val, 3)/3;
				_term2=(2*(BD::Pow(_val, 5)))/15;
				_term3=(17*(BD::Pow(_val, 7)))/315;

				BD^ _retval=gcnew BD();

				_retval=_val+_term1+_term2+_term3;

				return _retval;

			}
			static BF^ TanAlg(BF^ _val){

				BF^ _term1=gcnew BF();
				BF^ _term2=gcnew BF();
				BF^ _term3=gcnew BF();

				_term1=BF::Pow(_val, 3)/3;
				_term2=(2*(BF::Pow(_val, 5)))/15;
				_term3=(17*(BF::Pow(_val, 7)))/315;

				BF^ _retval=gcnew BF();

				_retval=_val+_term1+_term2+_term3;

				return _retval;

			}
			static BI^ LogRecursive(BI^ _val, int _current){

				BI^ _retval=gcnew BI();

				if(_current>0){

					_retval=(BI::Pow(_val, _current)/_current)+LogRecursive(_val, _current-2);

				}

				else{

					_retval=_val;

				}

				return _retval;

			}
			static BD^ LogRecursive(BD^ _val, int _current){

				BD^ _retval=gcnew BD();

				if(_current>1){

					_retval=(BD::Pow(_val, _current)/_current)+LogRecursive(_val, _current-2);

				}

				else{

					_retval=_val;

				}

				return _retval;

			}
			static BF^ LogRecursive(BF^ _val, int _current){

				BF^ _retval=gcnew BF();

				if(_current>0){

					_retval=(BF::Pow(_val, _current)/_current)+LogRecursive(_val, _current-2);

				}

				else{

					_retval=_val;

				}

				return _retval;

			}
			static BI^ expRecursive(BI^ _val, int _current){

				BI^ _retval=gcnew BI();

				if(_current>0){

					_retval=(BI::Pow(_val, _current)/Factorial((BI^)_current))+expRecursive(_val, _current-1);

				}

				else{

					_retval=1+_val;

				}

				return _retval;

			}
			static BD^ expRecursive(BD^ _val, int _current){

				BD^ _retval=gcnew BD();

				if(_current>0){

					_retval=(BD::Pow(_val, _current)/Factorial((BD^)_current))+expRecursive(_val, _current-1);

				}

				else{

					_retval=1+_val;

				}

				return _retval;

			}
			static BF^ expRecursive(BF^ _val, int _current){

				BF^ _retval=gcnew BF();

				if(_current>0){

					_retval=(BF::Pow(_val, _current)/Factorial((BF^)_current))+expRecursive(_val, _current-1);

				}

				else{

					_retval=1+_val;

				}

				return _retval;

			}
			static BI^ FactorialRecursive(BI^ _current){

				BI^ _retval=gcnew BI();

				if(_current>2){

					_retval=_current*FactorialRecursive(_current-1);

				}

				else{

					return _current;

				}

				return _retval;

			}
			static BD^ FactorialRecursive(BD^ _current){

				BD^ _retval=gcnew BD();

				if(_current>2){

					_retval=_current*FactorialRecursive(_current-1);

				}

				else{

					return _current;

				}

				return _retval;

			}
			static BF^ FactorialRecursive(BF^ _current){

				BF^ _retval=gcnew BF();

				if(_current>2){

					_retval=_current*FactorialRecursive(_current-1);

				}

				else{

					return _current;

				}

				return _retval;

			}
			static BI^ ArcTanRecursive(BI^ _val, int _current){

				BI^ _retval=gcnew BI();

				if(_current>1){

					if(((_current+1)/2)%2==0){

						_retval=(BI::Pow(_val, _current)/_current)+ArcTanRecursive(_val, _current-2);

					}

					else{

						_retval=(BI::Pow(_val, _current)/_current)-ArcTanRecursive(_val, _current-2);

					}

				}

				else{

					_retval=_val-(BI::Pow(_val, _current)/_current);

				}

				return _retval;

			}
			static BD^ ArcTanRecursive(BD^ _val, int _current){

				BD^ _retval=gcnew BD();
				BD^ _temp=gcnew BD();

				if(_current>1){

					_temp=(BD::Pow(_val, _current)/_current);

					if(((_current+1)/2)%2==0){

						_retval=_temp-ArcTanRecursive(_val, _current-2);

					}

					else{

						_retval=_temp+ArcTanRecursive(_val, _current-2);

					}

				}

				else{

					_retval=_val;

				}

				return _retval;

			}
			static BF^ ArcTanRecursive(BF^ _val, int _current){

				BF^ _retval=gcnew BF();

				if(_current>1){

					if(((_current+1)/2)%2==0){

						_retval=(BF::Pow(_val, _current)/_current)+ArcTanRecursive(_val, _current-2);

					}

					else{

						_retval=(BF::Pow(_val, _current)/_current)-ArcTanRecursive(_val, _current-2);

					}

				}

				else{

					_retval=_val-(BF::Pow(_val, _current)/_current);

				}

				return _retval;

			}
			static BI^ ArcTanAlg(BI^ _val, int _iter){

				BI^ _retval=gcnew BI();
				BI^ _tval=gcnew BI(_val);
				BI^ _twosqr3=gcnew BI("3.4641016151377545870548926830117");
				BI^ _pihalf=gcnew BI("3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679");
				BI^ _pisix=gcnew BI("3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679");
				BI^ _sqr3=gcnew BI("1.7320508075688772935274463415059");

				bool _neg=false;
				bool _pihalves=false;

				_pihalf=_pihalf/2;
				_pisix=_pisix/6;

				int _current=_iter;

				if(_current%2==0){

					_current+=1;

				}

				if(_tval->IsNegative){

					_tval->Sign=0;
					_neg=true;

				}

				if(_tval>1){

					_tval=1/_tval;
					_retval=_pihalf+ArcTan(_tval, _current);

				}

				else if(_tval>_twosqr3){

					_tval=((_tval*_sqr3)-1)/(_sqr3+_tval);
					_retval=_pisix+ArcTan(_tval, _current);

				}

				else{

					_retval=ArcTanRecursive(_tval, _current);

				}

				if(_neg){

					_retval->Sign=1;

				}

				return _retval;

			}
			static BD^ ArcTanAlg(BD^ _val, int _iter){

				BD^ _retval=gcnew BD();
				BD^ _tval=gcnew BD(_val);
				BD^ _twosqr3=gcnew BD(3.4641016151377545870548926830117);
				BD^ _pihalf=gcnew BD(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);
				BD^ _pisix=gcnew BD(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);
				BD^ _sqr3=gcnew BD(1.7320508075688772935274463415059);

				bool _neg=false;
				bool _pihalves=false;

				_pihalf=_pihalf/2;
				_pisix=_pisix/6;

				int _current=_iter;

				if(_current%2==0){

					_current+=1;

				}

				if(_tval->IsNegative){

					_tval->Sign=0;
					_neg=true;

				}

				if(_tval>1){

					_tval=1/_tval;
					_retval=_pihalf+ArcTanAlg(_tval, _current);

				}

				else if(_tval>_twosqr3){

					_tval=((_tval*_sqr3)-1)/(_sqr3+_tval);
					_retval=_pisix+ArcTanAlg(_tval, _current);

				}

				else{

					_retval=ArcTanRecursive(_tval, _current);

				}

				if(_neg){

					_retval->Sign=1;

				}

				return _retval;

			}
			static BF^ ArcTanAlg(BF^ _val, int _iter){

				BF^ _retval=gcnew BF();
				BF^ _tval=gcnew BF(_val);
				BF^ _twosqr3=gcnew BF(3.4641016151377545870548926830117);
				BF^ _pihalf=gcnew BF(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);
				BF^ _pisix=gcnew BF(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);
				BF^ _sqr3=gcnew BF(1.7320508075688772935274463415059);

				bool _neg=false;
				bool _pihalves=false;

				_pihalf=_pihalf/2;
				_pisix=_pisix/6;

				int _current=_iter;

				if(_current%2==0){

					_current+=1;

				}

				if(_tval->IsNegative){

					_tval->Sign=0;
					_neg=true;

				}

				if(_tval>1){

					_tval=1/_tval;
					_retval=_pihalf+ArcTan(_tval, _current);

				}

				else if(_tval>_twosqr3){

					_tval=((_tval*_sqr3)-1)/(_sqr3+_tval);
					_retval=_pisix+ArcTan(_tval, _current);

				}

				else{

					_retval=ArcTanRecursive(_tval, _current);

				}

				if(_neg){

					_retval->Sign=1;

				}

				return _retval;

			}

		public:


			//!Calculates the Square Root of a number.
			/*!
			\param _val The value to calculate the square root of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BI^ Sqr(BI^ _val, int _iter){
			
				bool _neg=false;

				int _count=0;

				BI^ _temp=gcnew BI(_val);
				BI^ _half=gcnew BI(1);
				BI^ _retval=gcnew BI(_val);
				BI^ _current=gcnew BI(10);
				BI^ _last=gcnew BI();
				BI^ _ret=gcnew BI();
				BI^ _la=gcnew BI();

				_half=_half/2;

				if(_temp<0){

					_neg=true;
					_temp=_temp*-1;

				}

				while(_count<_iter){

					_last=_retval;
					_retval=_current+(_temp/_current);
					_retval=_retval*_half;
					_current=_retval;
					_count+=1;

					if(_last==_retval){

						break;

					}

				}

				return _current;


			}
			//!Calculates the Square Root of a number.
			/*!
			\param _val The value to calculate the square root of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ Sqr(BD^ _val, int _iter){
			
				bool _neg=false;

				int _count=0;

				BD^ _temp=gcnew BD(_val);
				BD^ _half=gcnew BD(1);
				BD^ _retval=gcnew BD(_val);
				BD^ _current=gcnew BD(10);
				BD^ _last=gcnew BD();
				BD^ _ret=gcnew BD();
				BD^ _la=gcnew BD();

				_half=_half/2;

				String^ _tsr=_half->ToString();

				if(_temp<0){

					_neg=true;
					_temp=_temp*-1;

				}

				while(_count<_iter){

					_last=_retval;

					_tsr=_last->ToString();

					_retval=_current+(_temp/_current);

					_tsr=_retval->ToString();

					_retval=_retval*_half;

					_tsr=_retval->ToString();

					_current=_retval;

					_tsr=_current->ToString();

					_count+=1;

					if(_last==_retval){

						break;

					}

				}

				return _current;

			}
			//!Calculates the Square Root of a number.
			/*!
			\param _val The value to calculate the square root of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BF Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ Sqr(BF^ _val, int _iter){
			
				bool _neg=false;

				int _count=0;

				BF^ _temp=gcnew BF(_val);
				BF^ _half=gcnew BF(1);
				BF^ _retval=gcnew BF(_val);
				BF^ _current=gcnew BF(10);
				BF^ _last=gcnew BF();
				BF^ _ret=gcnew BF();
				BF^ _la=gcnew BF();

				_half=_half/2;

				if(_temp<0){

					_neg=true;
					_temp=_temp*-1;

				}

				while(_count<_iter){

					_last=_retval;
					_retval=_current+(_temp/_current);
					_retval=_retval*_half;
					_current=_retval;
					_count+=1;

					if(_last==_retval){

						break;

					}

				}

				return _current;


			}
			//!Calculates the Square Root of a number.
			/*!
			\param _val The value to calculate the square root of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BDI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BDI^ Sqr(BDI^ _val, int _iter){
			
				return gcnew BDI(BDISqrt64(_val->Val));
				
			}
			//!Calculates the Square Root of a number.
			/*!
			\param _val The value to calculate the square root of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BFI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BFI^ Sqr(BFI^ _val, int _iter){
			
				return gcnew BFI(BFISqrt64(_val->Val, _iter));

			}
			//!Calculates the Nth Root of a number.
			/*!
			\param _val The value to calculate the Nth root of.
			\param _root The root you are looking for.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BI^ NthRoot(BI^ _val, BI^ _root, int _iter){

				BI^ _retval=gcnew BI(10);
				BI^ _nminusone=gcnew BI(_root);
				BI^ _oneovern=gcnew BI(_root);

				int _count=0;

				_nminusone=_nminusone-1;
				_oneovern=1/_oneovern;

				while(_count<=_iter){

					_retval=_oneovern*((_nminusone*_retval)+(_val/BI::Pow(_retval, _nminusone)));

					_count+=1;

				}

				return _retval;

			}
			//!Calculates the Nth Root of a number.
			/*!
			\param _val The value to calculate the Nth root of.
			\param _root The root you are looking for.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ NthRoot(BD^ _val, BD^ _root, int _iter){

				BD^ _retval=gcnew BD(10);
				BD^ _nminusone=gcnew BD(_root);
				BD^ _oneovern=gcnew BD(_root);

				int _count=0;

				_nminusone=_nminusone-1;
				_oneovern=1/_oneovern;

				while(_count<=_iter){

					_retval=_oneovern*((_nminusone*_retval)+(_val/BD::Pow(_retval, _nminusone)));

					_count+=1;

				}

				return _retval;

			}
			//!Calculates the Nth Root of a number.
			/*!
			\param _val The value to calculate the Nth root of.
			\param _root The root you are looking for.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BF Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ NthRoot(BF^ _val, BF^ _root, int _iter){

				BF^ _retval=gcnew BF(10);
				BF^ _nminusone=gcnew BF(_root);
				BF^ _oneovern=gcnew BF(_root);

				int _count=0;

				_nminusone=_nminusone-1;
				_oneovern=1/_oneovern;

				while(_count<=_iter){

					_retval=_oneovern*((_nminusone*_retval)+(_val/BF::Pow(_retval, _nminusone)));

					_count+=1;

				}

				return _retval;

			}
			//!Calculates the Nth Root of a number.
			/*!
			\param _val The value to calculate the Nth root of.
			\param _root The root you are looking for (of type BD).
			\param _iter The number of iterations the algorithm should go through.
			\returns A BDI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BDI^ NthRoot(BDI^ _val, BD^ _root, int _iter){

				return gcnew BDI(BDINthRoot64(_val->Val, _root->Val));

			}
			//!Calculates the Nth Root of a number.
			/*!
			\param _val The value to calculate the Nth root of.
			\param _root The root you are looking for (of type BF).
			\param _iter The number of iterations the algorithm should go through.
			\returns A BFI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BFI^ NthRoot(BFI^ _val, BF^ _root, int _iter){

				return gcnew BFI(BFINthRoot64(_val->Val, _root->Val));

			}
			//!Calculates the sin of a number given in degrees. (implemented in c++)
			/*!
			\param _val The value to calculate the sin of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ Sin(BD^ _val, int _iter){

				BD^ _retval=gcnew BD();
				BD^ _newangle=gcnew BD(_val);
				BD^ _mult=gcnew BD(0.0174532927777777777777777777777777778);

				bool _neg=false;

				int _current=_iter;

				if(_current%2==0){

					_current+=1;

				}

				_newangle=_newangle%360;

				if(_newangle<0){

					_newangle=_newangle+360;

				}

				if(_newangle>90 && _newangle<=180){

					_newangle=90-(_newangle-90);

				}

				else if(_newangle>180 && _newangle<=270){

					_newangle=_newangle-180;
					_neg=true;

				}

				else if(_newangle>270 && _newangle<=360){

					_newangle=90-(_newangle-270);
					_neg=true;

				}

				if(_newangle>45){

					_newangle=90-_newangle;

					_newangle=_newangle*_mult;

					_current+=1;

					_retval=CosRecursive(_newangle, _current);

				}

				else{

					_newangle=_newangle*_mult;

					_retval=SinRecursive(_newangle, _current);

				}

				if(_neg){

					_retval->Sign=1;

				}

				return _retval;

			}
			//!Calculates the sin of a number given in degrees. (implemented in c++)
			/*!
			\param _val The value to calculate the sin of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BF Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ Sin(BF^ _val, int _iter){

				BF^ _retval=gcnew BF();
				BF^ _newangle=gcnew BF(_val);
				BF^ _mult=gcnew BF(0.0174532927777777777777777777777777778);

				bool _neg=false;

				int _current=_iter;

				if(_current%2==0){

					_current+=1;

				}

				_newangle=_newangle%360;

				if(_newangle<0){

					_newangle=_newangle+360;

				}

				if(_newangle>90 && _newangle<=180){

					_newangle=90-(_newangle-90);

				}

				else if(_newangle>180 && _newangle<=270){

					_newangle=_newangle-180;
					_neg=true;

				}

				else if(_newangle>270 && _newangle<=360){

					_newangle=90-(_newangle-270);
					_neg=true;

				}

				if(_newangle>45){

					_newangle=90-_newangle;

					_newangle=_newangle*_mult;

					_current+=1;

					_retval=CosRecursive(_newangle, _current);

				}

				else{

					_newangle=_newangle*_mult;

					_retval=SinRecursive(_newangle, _current);

				}

				if(_neg){

					_retval->Sign=1;

				}

				return _retval;

			}
			//!Calculates the cos of a number given in degrees. (implemented in c++)
			/*!
			\param _val The value to calculate the cos of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ Cos(BD^ _val, int _iter){

				BD^ _retval=gcnew BD();
				BD^ _newangle=gcnew BD(_val);
				BD^ _mult=gcnew BD(0.0174532927777777777777777777777777778);

				int _current=_iter;

				if(_current%2==1){

					_current+=1;

				}

				bool _neg=false;

				_newangle=_newangle%360;

				if(_newangle<0){

					_newangle=_newangle+360;

				}

				if(_newangle>90 && _newangle<=180){

					_newangle=90-(_newangle-90);
					_neg=true;

				}

				else if(_newangle>180 && _newangle<=270){

					_newangle=_newangle-180;
					_neg=true;

				}

				else if(_newangle>270 && _newangle<=360){

					_newangle=90-(_newangle-270);

				}

				if(_newangle>45){

					_newangle=90-_newangle;

					_newangle=_newangle*_mult;

					_current+=1;

					_retval=SinRecursive(_newangle, _current);

				}

				else{

					_newangle=_newangle*_mult;

					_retval=CosRecursive(_newangle, _current);

				}

				if(_neg){

					_retval->Sign=1;

				}

				return _retval;

			}
			//!Calculates the cos of a number given in degrees. (implemented in c++)
			/*!
			\param _val The value to calculate the cos of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ Cos(BF^ _val, int _iter){

				BF^ _retval=gcnew BF();
				BF^ _newangle=gcnew BF(_val);
				BF^ _mult=gcnew BF(0.0174532927777777777777777777777777778);

				int _current=_iter;

				if(_current%2==1){

					_current+=1;

				}

				bool _neg=false;

				_newangle=_newangle%360;

				if(_newangle<0){

					_newangle=_newangle+360;

				}

				if(_newangle>90 && _newangle<=180){

					_newangle=90-(_newangle-90);
					_neg=true;

				}

				else if(_newangle>180 && _newangle<=270){

					_newangle=_newangle-180;
					_neg=true;

				}

				else if(_newangle>270 && _newangle<=360){

					_newangle=90-(_newangle-270);

				}

				if(_newangle>45){

					_newangle=90-_newangle;

					_newangle=_newangle*_mult;

					_current+=1;

					_retval=SinRecursive(_newangle, _current);

				}

				else{

					_newangle=_newangle*_mult;

					_retval=CosRecursive(_newangle, _current);

				}

				if(_neg){

					_retval->Sign=1;

				}

				return _retval;

			}
			//!Calculates the tan of a number given in degrees. (implemented in c++)
			/*!
			\param _val The value to calculate the tan of.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ Tan(BD^ _val){

				BD^ _newangle=gcnew BD(_val);
				BD^ _retval=gcnew BD();
				BD^ _mult=gcnew BD(0.0174532927777777777777777777777777778);

				bool _neg=false;
				bool _recip=false;

				_newangle=_newangle%180;

				if(_newangle<0){

					_newangle=_newangle+180;

				}

				if(_newangle>90){

					_neg=true;

					_newangle=90-(_newangle-90);

				}

				if(_newangle>45){

					_recip=true;
					_newangle=90-_newangle;

				}

				if(_newangle>22.5){

					_newangle=_newangle/2;

					_retval=(2*Tan(_newangle))/(1-BD::Pow(Tan(_newangle),2));

				}

				else{

					_newangle=_newangle*_mult;

					_retval=TanAlg(_newangle);

				}

				if(_recip){

					_retval=1/_retval;

				}

				if(_neg){

					_retval->Sign=1;

				}

				return _retval;

			}
			//!Calculates the tan of a number given in degrees. (implemented in c++)
			/*!
			\param _val The value to calculate the tan of.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ Tan(BF^ _val){

				BF^ _newangle=gcnew BF(_val);
				BF^ _retval=gcnew BF();
				BF^ _mult=gcnew BF(0.0174532927777777777777777777777777778);

				bool _neg=false;
				bool _recip=false;

				_newangle=_newangle%180;

				if(_newangle<0){

					_newangle=_newangle+180;

				}

				if(_newangle>90){

					_neg=true;

					_newangle=90-(_newangle-90);

				}

				if(_newangle>45){

					_recip=true;
					_newangle=90-_newangle;

				}

				if(_newangle>22.5){

					_newangle=_newangle/2;

					_retval=(2*Tan(_newangle))/(1-BF::Pow(Tan(_newangle),2));

				}

				else{

					_newangle=_newangle*_mult;

					_retval=TanAlg(_newangle);

				}

				if(_recip){

					_retval=1/_retval;

				}

				if(_neg){

					_retval->Sign=1;

				}

				return _retval;

			}
			//!Calculates the base 10 log of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the log of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BI^ Log(BI^ _val, int _iter){

				BD^ _ln=gcnew BD(0.86858896);
				BD^ _m=gcnew BD((BD^)_val);

				String^ _tstring=_val->ToString();

				BD^ _p=gcnew BD(_tstring->Length-1);

				_m->DecimalPlace=_tstring->Length-1;

				_m=Sqr(_m, _iter*10);

				BD^ _x=gcnew BD();

				_x=(_m-1)/(_m+1);

				int _current=_iter;

				if(_current%2==0){

					_current+=1;

				}

				BD^ _retval=gcnew BD();

				_retval=LogRecursive(_x, _current);
				_retval=_retval*2;
				_retval=_retval*_ln;
				_retval=_retval+_p;

				BI^ _ret=gcnew BI((BI^)_retval);

				return _ret;

			}
			//!Calculates the base 10 log of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the log of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ Log(BD^ _val, int _iter){

				BD^ _ln=gcnew BD(0.86858896);
				BD^ _m=gcnew BD((BD^)_val);

				String^ _tstring=_val->ToString();

				BD^ _p=gcnew BD(_tstring->Length-1);

				_m->DecimalPlace=_tstring->Length-1;

				_m=Sqr(_m, _iter*10);

				BD^ _x=gcnew BD();

				_x=(_m-1)/(_m+1);

				int _current=_iter;

				if(_current%2==0){

					_current+=1;

				}

				BD^ _retval=gcnew BD();

				_retval=LogRecursive(_x, _current);
				_retval=_retval*2;
				_retval=_retval*_ln;
				_retval=_retval+_p;

				return _retval;

			}
			//!Calculates the base 10 log of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the log of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BF Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ Log(BF^ _val, int _iter){

				BD^ _ln=gcnew BD(0.86858896);
				BD^ _m=gcnew BD((BD^)_val);

				String^ _tstring=_val->ToString();

				BD^ _p=gcnew BD(_tstring->Length-1);

				_m->DecimalPlace=_tstring->Length-1;

				_m=Sqr(_m, _iter*10);

				BD^ _x=gcnew BD();

				_x=(_m-1)/(_m+1);

				int _current=_iter;

				if(_current%2==0){

					_current+=1;

				}

				BD^ _retval=gcnew BD();

				_retval=LogRecursive(_x, _current);
				_retval=_retval*2;
				_retval=_retval*_ln;
				_retval=_retval+_p;

				BF^ _ret=gcnew BF((BF^)_retval);

				return _ret;

			}
			//!Calculates the ln of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ln of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BI^ ln(BI^ _val, int _iter){

				BD^ _ln=gcnew BD(2.302585);
				BD^ _m=gcnew BD((BD^)_val);

				String^ _tstring=_val->ToString();

				BD^ _p=gcnew BD(_tstring->Length-1);

				_m->DecimalPlace=_tstring->Length-1;

				_m=Sqr(_m, _iter*10);

				BD^ _x=gcnew BD();

				_x=(_m-1)/(_m+1);

				_ln=_ln*_p;

				int _current=_iter;

				if(_current%2==0){

					_current+=1;

				}

				BD^ _retval=gcnew BD();

				_retval=LogRecursive(_x, _current);
				_retval=_retval*2;
				_retval=_retval+_ln;

				BI^ _ret=gcnew BI((BI^)_retval);

				return _ret;

			}
			//!Calculates the ln of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ln of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ ln(BD^ _val, int _iter){

				BD^ _ln=gcnew BD(2.302585);
				BD^ _m=gcnew BD((BD^)_val);

				String^ _tstring=_val->ToString();

				BD^ _p=gcnew BD(_tstring->Length-1);

				_m->DecimalPlace=_tstring->Length-1;

				_m=Sqr(_m, _iter*10);

				BD^ _x=gcnew BD();

				_x=(_m-1)/(_m+1);

				_ln=_ln*_p;

				int _current=_iter;

				if(_current%2==0){

					_current+=1;

				}

				BD^ _retval=gcnew BD();

				_retval=LogRecursive(_x, _current);
				_retval=_retval*4;
				_retval=_retval+_ln;

				return _retval;

			}
			//!Calculates the ln of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ln of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ ln(BF^ _val, int _iter){

				BD^ _ln=gcnew BD(2.302585);
				BD^ _m=gcnew BD((BD^)_val);

				String^ _tstring=_val->ToString();

				BD^ _p=gcnew BD(_tstring->Length-1);

				_m->DecimalPlace=_tstring->Length-1;

				_m=Sqr(_m, _iter*10);

				BD^ _x=gcnew BD();

				_x=(_m-1)/(_m+1);

				_ln=_ln*_p;

				int _current=_iter;

				if(_current%2==0){

					_current+=1;

				}

				BD^ _retval=gcnew BD();

				_retval=LogRecursive(_x, _current);
				_retval=_retval*2;
				_retval=_retval+_ln;

				BF^ _ret=gcnew BF((BF^)_retval);
				return _ret;

			}
			//!Calculates the exponential of a number. (implemented in c++)
			/*!
			\param _val The power to calculate.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ exp(BD^ _val, int _iter){

				BI^ _ifract=gcnew BI(_val->Number);
				BI^ _iint=gcnew BI(_val->Number);
				BI^ _ipow=gcnew BI(10);
				BI^ _imod=gcnew BI();

				bool _neg=false;

				if(_val->Sign==1){

					_neg=true;

				}

				_ipow=BI::Pow(_ipow, _val->DecimalPlace);
				_ifract=_ifract%_ipow;
				_ipow->Number[0]=1;
				_ipow->Number[1]=10;
				_ipow=BI::Pow(_ipow, _val->DecimalPlace-1);
				_imod=_iint%_ipow;
				_iint=_iint-_imod;
				_iint=_iint/_ipow;
				_ipow->Number[0]=1;
				_ipow->Number[1]=10;
				_imod=_iint%_ipow;
				_iint=_iint-_imod;
				_iint=_iint/_ipow;

				if(_imod->Number[1]>4){

					_iint=_iint+1;

					_ipow->Number[0]=1;
					_ipow->Number[1]=10;
					_ipow=BI::Pow(_ipow, _val->DecimalPlace);
					_imod=_iint*_ipow;
					_ifract=_imod-_ifract;

					_neg=!_neg;

				}
				
				BD^ _fract=gcnew BD(_ifract->ToString());
				_fract->DecimalPlace=_val->DecimalPlace;
				_fract->Sign=0;

				if(_neg){

					_fract->Sign=1;

				}

				BD^ _e=gcnew BD(2.7182818284590452353602874713526624977572470936999595749669676277240766303535475945713821785251664274);

				_e=BD::Pow(_e, (BD^)_iint);

				if(_val->Sign==1){

					_e=1/_e;

				}

				BD^ _retval=gcnew BD();

				_retval=expRecursive(_fract, _iter);

				if(_fract->Sign==1){

					_retval=1/_retval;

				}

				_retval=_retval*_e;

				return _retval;

			}
			//!Calculates the ln of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ln of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BF Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ exp(BF^ _val, int _iter){

				BD^ _tnum=gcnew BD((BD^)_val);
				BI^ _ifract=gcnew BI(_tnum->Number);
				BI^ _iint=gcnew BI(_tnum->Number);
				BI^ _ipow=gcnew BI(10);
				BI^ _imod=gcnew BI();

				bool _neg=false;

				if(_tnum->Sign==1){

					_neg=true;

				}

				_ipow=BI::Pow(_ipow, _tnum->DecimalPlace);
				_ifract=_ifract%_ipow;
				_ipow->Number[0]=1;
				_ipow->Number[1]=10;
				_ipow=BI::Pow(_ipow, _tnum->DecimalPlace-1);
				_imod=_iint%_ipow;
				_iint=_iint-_imod;
				_iint=_iint/_ipow;
				_ipow->Number[0]=1;
				_ipow->Number[1]=10;
				_imod=_iint%_ipow;
				_iint=_iint-_imod;
				_iint=_iint/_ipow;

				if(_imod->Number[1]>4){

					_iint=_iint+1;

					_ipow->Number[0]=1;
					_ipow->Number[1]=10;
					_ipow=BI::Pow(_ipow, _tnum->DecimalPlace);
					_imod=_iint*_ipow;
					_ifract=_imod-_ifract;

					_neg=!_neg;

				}
				
				BD^ _fract=gcnew BD(_ifract->ToString());
				_fract->DecimalPlace=_tnum->DecimalPlace;
				_fract->Sign=0;

				if(_neg){

					_fract->Sign=1;

				}

				BD^ _e=gcnew BD(2.7182818284590452353602874713526624977572470936999595749669676277240766303535475945713821785251664274);

				_e=BD::Pow(_e, (BD^)_iint);

				if(_tnum->Sign==1){

					_e=1/_e;

				}

				BD^ _tr=gcnew BD();

				_tr=expRecursive(_fract, _iter);

				if(_fract->Sign==1){

					_tr=1/_tr;

				}

				_tr=_tr*_e;

				BF^ _retval=gcnew BF((BF^)_tr);

				return _retval;

			}
			//!Calculates the Factorial of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the factorial of.
			\returns A BI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BI^ Factorial(BI^ _val){

				BI^ _retval=gcnew BI(_val);

				_retval=_retval*FactorialRecursive(_val-1);

				return _retval;

			}
			//!Calculates the Factorial of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the factorial of.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ Factorial(BD^ _val){

				BD^ _retval=gcnew BD(_val);

				_retval=_retval*FactorialRecursive(_val-1);

				return _retval;

			}
			//!Calculates the Factorial of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the factorial of.
			\returns A BF Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ Factorial(BF^ _val){

				BF^ _retval=gcnew BF(_val);

				_retval=_retval*FactorialRecursive(_val-1);

				return _retval;

			}
			//!Calculates the ArcSin of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ArcSin of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BI^ ArcSin(BI^ _val, int _iter){

				BI^ _tval=gcnew BI(_val);
				BI^ _mult=gcnew BI(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);

				_mult=180/_mult;

				_tval=_tval/(Sqr(1-BI::Pow(_tval, 2), 20));

				return _mult*ArcTanAlg(_tval, _iter);

			}
			//!Calculates the ArcSin of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ArcSin of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ ArcSin(BD^ _val, int _iter){

				BD^ _tval=gcnew BD(_val);
				BD^ _mult=gcnew BD(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);

				_mult=180/_mult;

				_tval=_tval/(Sqr(1-BD::Pow(_tval, 2), 20));

				return _mult*ArcTanAlg(_tval, _iter);

			}
			//!Calculates the ArcSin of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ArcSin of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BF Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ ArcSin(BF^ _val, int _iter){

				BF^ _tval=gcnew BF(_val);
				BF^ _mult=gcnew BF(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);

				_mult=180/_mult;

				_tval=_tval/(Sqr(1-BF::Pow(_tval, 2), 20));

				return _mult*ArcTanAlg(_tval, _iter);

			}
			//!Calculates the ArcCos of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ArcCos of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BI^ ArcCos(BI^ _val, int _iter){

				BI^ _tval=gcnew BI(_val);
				BI^ _mult=gcnew BI(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);

				_mult=180/_mult;

				_tval=(Sqr(1-BI::Pow(_tval, 2), 20))/_tval;

				return _mult*ArcTanAlg(_tval, _iter);

			}
			//!Calculates the ArcCos of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ArcCos of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ ArcCos(BD^ _val, int _iter){

				BD^ _tval=gcnew BD(_val);
				BD^ _mult=gcnew BD(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);

				_mult=180/_mult;

				_tval=(Sqr(1-BD::Pow(_tval, 2), 20))/_tval;

				return _mult*ArcTanAlg(_tval, _iter);

			}
			//!Calculates the ArcCos of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ArcCos of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BF Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ ArcCos(BF^ _val, int _iter){

				BF^ _tval=gcnew BF(_val);
				BF^ _mult=gcnew BF(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);

				_mult=180/_mult;

				_tval=(Sqr(1-BF::Pow(_tval, 2), 20))/_tval;

				return _mult*ArcTanAlg(_tval, _iter);

			}
			//!Calculates the ArcTan of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ArcTan of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BI Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BI^ ArcTan(BI^ _val, int _iter){

				BI^ _mult=gcnew BI(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);

				return _mult*ArcTanAlg(_val, _iter);

			}
			//!Calculates the ArcTan of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ArcTan of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ ArcTan(BD^ _val, int _iter){

				BD^ _mult=gcnew BD(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);

				_mult=180/_mult;

				return _mult*ArcTanAlg(_val, _iter);

			}
			//!Calculates the ArcTan of a number. (implemented in c++)
			/*!
			\param _val The value to calculate the ArcTan of.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BF Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ ArcTan(BF^ _val, int _iter){

				BF^ _mult=gcnew BF(3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679);

				return _mult*ArcTanAlg(_val, _iter);

			}
			//!Calculates the exponential raised to a power of a number. (implemented in c++)
			/*!
			\param _base The value to calculate the Power of.
			\param _exp The exponent to use.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BD Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BD^ Power(BD^ _base, BD^ _exp, int _iter){

				return exp(_exp*ln(_base, _iter), _iter);

			}
			//!Calculates the exponential raised to a power of a number. (implemented in c++)
			/*!
			\param _base The value to calculate the Power of.
			\param _exp The exponent to use.
			\param _iter The number of iterations the algorithm should go through.
			\returns A BF Object
			\deprecated These are kept for now for backward compatibility, but the forms of these functions that start with BI, BD, BF, BDI, or BFI should be used instead.
			*/
			static BF^ Power(BF^ _base, BF^ _exp, int _iter){

				return exp(_exp*ln(_base, _iter), _iter);

			}
			//!Calculates the square root of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the square root of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BD Object
			*/
			static BD^ BDSqr(BD^ _val, MYWORD _cycles){

				BD^ _retval=gcnew BD(BDSqrt64(_val->Val));

				return _retval;

			}
			//!Calculates the Nth root of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the Nth root of.
			\param _root The root you are looking for.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BD Object
			*/
			static BD^ BDNthRoot(BD^ _val, BD^ _root, MYWORD _cycles){

				BD^ _retval=gcnew BD(BDNthRoot64(_val->Val, _root->Val));

				return _retval;

			}
			//!Calculates the sin of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the sin of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BD Object
			*/
			static BD^ BDSin(BD^ _val, MYWORD _cycles){

				BD^ _retval=gcnew BD(BDSin64(_val->Val));

				return _retval;

			}
			//!Calculates the cos of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the cos of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BD Object
			*/
			static BD^ BDCos(BD^ _val, MYWORD _cycles){

				BD^ _retval=gcnew BD(BDCos64(_val->Val));

				return _retval;

			}
			//!Calculates the tan of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the tan of.
			\returns A BD Object
			*/
			static BD^ BDTan(BD^ _val){

				BD^ _retval=gcnew BD(BDTan64(_val->Val));

				return _retval;

			}
			//!Calculates the arcsin of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the arcsin of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BD Object
			*/
			static BD^ BDASin(BD^ _val, MYWORD _cycles){

				BD^ _retval=gcnew BD(BDASin64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the arccos of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the arccos of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BD Object
			*/
			static BD^ BDACos(BD^ _val, MYWORD _cycles){

				BD^ _retval=gcnew BD(BDACos64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the arctan of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the arctan of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BD Object
			*/
			static BD^ BDATan(BD^ _val, MYWORD _cycles){

				BD^ _retval=gcnew BD(BDATan64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the ln of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the ln of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BD Object
			*/
			static BD^ BDln(BD^ _val, MYWORD _cycles){

				BD^ _retval=gcnew BD(BDln64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the log of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the log of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BD Object
			*/
			static BD^ BDLog(BD^ _val, MYWORD _cycles){

				BD^ _retval=gcnew BD(BDLog64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the exponential of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the exponential of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BD Object
			*/
			static BD^ BDExp(BD^ _val, MYWORD _cycles){

				BD^ _retval=gcnew BD(BDExp64(_val->Val, _cycles));

				return _retval;

			}
			//!Uses the exponential function to calculate the base being raised to a non integer exponent. (implemented in assembly language)
			/*!
			\param _base The base to calculate the power of.
			\param _exp The exponent to be used to calculate the power of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BD Object
			*/
			static BD^ BDPower(BD^ _base, BD^ _exp, MYWORD _cycles){

				BD^ _retval=gcnew BD(BDPower64(_base->Val, _exp->Val, _cycles));

				return _retval;

			}



			//!Calculates the square root of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the square root of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BF Object
			*/
			static BF^ BFSqr(BF^ _val, MYWORD _cycles){

				BF^ _retval=gcnew BF(BFSqrt64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the Nth root of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the Nth root of.
			\param _root The root you are looking for.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BF Object
			*/
			static BF^ BFNthRoot(BF^ _val, BF^ _root, MYWORD _cycles){

				BF^ _retval=gcnew BF(BFNthRoot64(_val->Val, _root->Val, _cycles));

				return _retval;

			}
			//!Calculates the sin of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the sin of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BF Object
			*/
			static BF^ BFSin(BF^ _val, MYWORD _cycles){

				BF^ _retval=gcnew BF(BFSin64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the cos of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the cos of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BF Object
			*/
			static BF^ BFCos(BF^ _val, MYWORD _cycles){

				BF^ _retval=gcnew BF(BFCos64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the tan of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the tan of.
			\returns A BF Object
			*/
			static BF^ BFTan(BF^ _val){

				BF^ _retval=gcnew BF(BFTan64(_val->Val));

				return _retval;

			}
			//!Calculates the arcsin of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the arcsin of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BF Object
			*/
			static BF^ BFASin(BF^ _val, MYWORD _cycles){

				BF^ _retval=gcnew BF(BFASin64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the arccos of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the arccos of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BF Object
			*/
			static BF^ BFACos(BF^ _val, MYWORD _cycles){

				BF^ _retval=gcnew BF(BFACos64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the arctan of a number (given in degrees). (implemented in assembly language)
			/*!
			\param _val The value to calculate the arctan of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BF Object
			*/
			static BF^ BFATan(BF^ _val, MYWORD _cycles){

				BF^ _retval=gcnew BF(BFATan64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the ln of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the ln of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BF Object
			*/
			static BF^ BFln(BF^ _val, MYWORD _cycles){

				BF^ _retval=gcnew BF(BFln64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the log of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the log of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BF Object
			*/
			static BF^ BFLog(BF^ _val, MYWORD _cycles){

				BF^ _retval=gcnew BF(BFLog64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the exponential of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the exponential of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BF Object
			*/
			static BF^ BFExp(BF^ _val, MYWORD _cycles){

				BF^ _retval=gcnew BF(BFExp64(_val->Val, _cycles));

				return _retval;

			}
			//!Uses the exponential function to calculate the base being raised to a non integer exponent. (implemented in assembly language)
			/*!
			\param _base The base to calculate the power of.
			\param _exp The exponent to be used to calculate the power of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BF Object
			*/
			static BF^ BDPower(BF^ _base, BF^ _exp, MYWORD _cycles){

				BF^ _retval=gcnew BF(BFPower64(_base->Val, _exp->Val, _cycles));

				return _retval;

			}

			//!Calculates the square root of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the square root of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BDI Object
			*/
			static BDI^ BFSqr(BDI^ _val, MYWORD _cycles){

				BDI^ _retval=gcnew BDI(BDISqrt64(_val->Val));

				return _retval;

			}
			//!Calculates the Nth root of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the Nth root of.
			\param _root The root you are looking for (of type BD).
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BDI Object
			*/
			static BDI^ BFNthRoot(BDI^ _val, BD^ _root, MYWORD _cycles){

				BDI^ _retval=gcnew BDI(BDINthRoot64(_val->Val, _root->Val));

				return _retval;

			}
			//!Calculates the square root of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the square root of.
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BFI Object
			*/
			static BFI^ BFSqr(BFI^ _val, MYWORD _cycles){

				BFI^ _retval=gcnew BFI(BFISqrt64(_val->Val, _cycles));

				return _retval;

			}
			//!Calculates the Nth root of a number. (implemented in assembly language)
			/*!
			\param _val The value to calculate the Nth root of.
			\param _root The root you are looking for (of type BF).
			\param _cycles The number of iterations the algorithm should go through.
			\returns A BFI Object
			*/
			static BFI^ BFNthRoot(BFI^ _val, BF^ _root, MYWORD _cycles){

				BFI^ _retval=gcnew BFI(BFINthRoot64(_val->Val, _root->Val));

				return _retval;

			}



		};

	}

}

#pragma managed(pop)




//#define CPI				"3.1415926535897932384626433832795028841971693993751058209749445923078164062862089986280348253421170679"
//#define CE				"2.7182818284590452353602874713526624977572470936999595749669676277240766303535475945713821785251664274"
//#define CTORAD			"0.0174532927777777777777777777777777778"
//#define CSQR2			"1.41421356237309504880168872420969807857"
//#define CPHI			"1.618033988749894848204586834365638117720"
//#define CLAMBDA			"0.577215664901532860606512090082402431042"
//#define CGOLDENRATIO	"1.6,180,339,887,498,948,482,045,868,343,656,381,177,203,091,798,057,628,621,354,486,227,052,604,628,189,024,497,072,072,041,893,911,374"
//#define C2SQR3			"3.4,641,016,151,377,545,870,548,926,830,117"
//#define CSQR3			"1.7,320,508,075,688,772,935,274,463,415,059"